CREATE TABLE B_CATALOG_IBLOCK
(
	IBLOCK_ID int NOT NULL,
	YANDEX_EXPORT char(1) NOT NULL,
	SUBSCRIPTION char(1) NOT NULL,
	VAT_ID INT NULL,
	PRODUCT_IBLOCK_ID int NOT NULL,
	SKU_PROPERTY_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT PK_B_CATALOG_IBLOCK PRIMARY KEY (IBLOCK_ID)
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT DF_B_CATALOG_IBLOCK_YANDEX_EXPORT DEFAULT 'N' FOR YANDEX_EXPORT
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT DF_B_CATALOG_IBLOCK_SUBSCRIPTION DEFAULT 'N' FOR SUBSCRIPTION
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT DF_B_CATALOG_IBLOCK_VAT_ID DEFAULT '0' FOR VAT_ID
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT DF_B_CATALOG_IBLOCK_PRODUCT_IBLOCK_ID DEFAULT '0' FOR PRODUCT_IBLOCK_ID
GO
ALTER TABLE B_CATALOG_IBLOCK ADD CONSTRAINT DF_B_CATALOG_IBLOCK_SKU_PROPERTY_ID DEFAULT '0' FOR SKU_PROPERTY_ID
GO
CREATE INDEX IXS_CAT_IB_PRODUCT ON B_CATALOG_IBLOCK (PRODUCT_IBLOCK_ID)
GO
CREATE INDEX IXS_CAT_IB_SKU_PROP ON B_CATALOG_IBLOCK (SKU_PROPERTY_ID)
GO

CREATE TABLE B_CATALOG_PRICE
(
	ID int NOT NULL IDENTITY (1, 1),
	PRODUCT_ID int NOT NULL,
	EXTRA_ID int NULL,
	CATALOG_GROUP_ID int NOT NULL,
	PRICE decimal(18,2) NOT NULL,
	CURRENCY char(3) NOT NULL,
	TIMESTAMP_X datetime NOT NULL,
	QUANTITY_FROM int NULL,
	QUANTITY_TO int NULL,
	TMP_ID VARCHAR(40) NULL,
	PRICE_SCALE decimal(26,12) NULL
)
GO
ALTER TABLE B_CATALOG_PRICE ADD CONSTRAINT PK_B_CATALOG_PRICE PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_PRICE ADD CONSTRAINT DF_B_CATALOG_PRICE_TIMESTAMP_X DEFAULT GETDATE() FOR TIMESTAMP_X
GO
CREATE INDEX IX_B_CATALOG_PRICE_1 ON B_CATALOG_PRICE (PRODUCT_ID, CATALOG_GROUP_ID)
GO
CREATE INDEX IX_B_CATALOG_PRICE_2 ON B_CATALOG_PRICE (CATALOG_GROUP_ID)
GO
CREATE INDEX IXS_CAT_PRICE_SCALE ON B_CATALOG_PRICE (PRICE_SCALE)
GO
CREATE TRIGGER B_CATALOG_PRICE_UPDATE on B_CATALOG_PRICE for update as
if (not update(TIMESTAMP_X))
begin
	UPDATE B_CATALOG_PRICE SET
		TIMESTAMP_X = GETDATE()
	FROM
		B_CATALOG_PRICE U,
		INSERTED I
	WHERE
		U.ID = I.ID
end
GO

CREATE TABLE B_CATALOG_PRODUCT
(
	ID int NOT NULL,
	QUANTITY float NOT NULL,
	QUANTITY_TRACE char(1) NOT NULL,
	WEIGHT float NOT NULL,
	TIMESTAMP_X datetime NOT NULL,
	PRICE_TYPE char(1) NOT NULL,
	RECUR_SCHEME_LENGTH int NULL,
	RECUR_SCHEME_TYPE char(1) NOT NULL,
	TRIAL_PRICE_ID int NULL,
	WITHOUT_ORDER char(1) NOT NULL,
	SELECT_BEST_PRICE char(1) NOT NULL,
	VAT_ID INT NULL,
	VAT_INCLUDED CHAR(1) NULL,
	CAN_BUY_ZERO char(1) NOT NULL,
	NEGATIVE_AMOUNT_TRACE char(1) NOT NULL,
	TMP_ID VARCHAR(40) NULL,
	PURCHASING_PRICE decimal(18,2) NULL,
	PURCHASING_CURRENCY char(3) NULL,
	BARCODE_MULTI char(1) NOT NULL,
	QUANTITY_RESERVED decimal(18,2) NULL,
	SUBSCRIBE char(1) NULL,
	WIDTH float NULL,
	LENGTH float NULL,
	HEIGHT float NULL,
	MEASURE int NULL,
	TYPE int NULL,
	AVAILABLE char(1) NULL,
	BUNDLE char(1) NULL
)
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT PK_B_CATALOG_PRODUCT PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_QUANTITY DEFAULT '0' FOR QUANTITY
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_QUANTITY_TRACE DEFAULT 'N' FOR QUANTITY_TRACE
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_WEIGHT DEFAULT '0' FOR WEIGHT
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_PRICE_TYPE DEFAULT 'S' FOR PRICE_TYPE
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_RECUR_SCHEME_TYPE DEFAULT 'D' FOR RECUR_SCHEME_TYPE
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_WITHOUT_ORDER DEFAULT 'N' FOR WITHOUT_ORDER
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_SELECT_BEST_PRICE DEFAULT 'Y' FOR SELECT_BEST_PRICE
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_VAT_ID DEFAULT '0' FOR VAT_ID
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_VAT_INCLUDED DEFAULT 'Y' FOR VAT_INCLUDED
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_TIMESTAMP_X DEFAULT GETDATE() FOR TIMESTAMP_X
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_CAN_BUY_ZERO DEFAULT 'N' FOR CAN_BUY_ZERO
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_NEGATIVE_AMOUNT DEFAULT 'D' FOR NEGATIVE_AMOUNT_TRACE
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_BARCODE_MULTI DEFAULT 'N' FOR BARCODE_MULTI
GO
ALTER TABLE B_CATALOG_PRODUCT ADD CONSTRAINT DF_B_CATALOG_PRODUCT_QUANTITY_RES DEFAULT '0' FOR QUANTITY_RESERVED
GO
CREATE TRIGGER B_CATALOG_PRODUCT_UPDATE on B_CATALOG_PRODUCT for update as
if (not update(TIMESTAMP_X))
begin
	UPDATE B_CATALOG_PRODUCT SET
		TIMESTAMP_X = GETDATE()
	FROM
		B_CATALOG_PRODUCT U,
		INSERTED I
	WHERE
		U.ID = I.ID
end
GO

CREATE TABLE B_CATALOG_EXTRA
(
	ID int NOT NULL IDENTITY (1, 1),
	NAME varchar(50) NOT NULL,
	PERCENTAGE decimal(18,2) NOT NULL
)
GO
ALTER TABLE B_CATALOG_EXTRA ADD CONSTRAINT PK_B_CATALOG_EXTRA PRIMARY KEY (ID)
GO

CREATE TABLE B_CATALOG_GROUP
(
	ID int NOT NULL IDENTITY (1, 1),
	NAME varchar(100) NOT NULL,
	BASE char(1) NOT NULL,
	SORT int NOT NULL,
	XML_ID varchar(255) NULL,
	TIMESTAMP_X datetime NULL,
	MODIFIED_BY int NULL,
	DATE_CREATE datetime NULL,
	CREATED_BY int NULL
)
GO
ALTER TABLE B_CATALOG_GROUP ADD CONSTRAINT PK_B_CATALOG_GROUP PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_GROUP ADD CONSTRAINT DF_B_CATALOG_GROUP_BASE DEFAULT 'N' FOR BASE
GO
ALTER TABLE B_CATALOG_GROUP ADD CONSTRAINT DF_B_CATALOG_GROUP_SORT DEFAULT '100' FOR SORT
GO

CREATE TABLE B_CATALOG_GROUP_LANG
(
	ID int NOT NULL IDENTITY (1, 1),
	CATALOG_GROUP_ID int NOT NULL,
	LANG char(2) NOT NULL,
	NAME varchar(100) NULL
)
GO
ALTER TABLE B_CATALOG_GROUP_LANG ADD CONSTRAINT PK_B_CATALOG_GROUP_LANG PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX IX_CATALOG_GROUP_ID ON B_CATALOG_GROUP_LANG(CATALOG_GROUP_ID, LANG)
GO

CREATE TABLE B_CATALOG_GROUP2GROUP
(
	ID int NOT NULL IDENTITY (1, 1),
	CATALOG_GROUP_ID int NOT NULL,
	GROUP_ID int NOT NULL,
	BUY char(1) NOT NULL
)
GO
ALTER TABLE B_CATALOG_GROUP2GROUP ADD CONSTRAINT PK_B_CATALOG_GROUP2GROUP PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_GROUP2GROUP ADD CONSTRAINT DF_B_CATALOG_GROUP2GROUP_BUY DEFAULT 'Y' FOR BUY
GO
CREATE INDEX IX_B_CATALOG_GROUP2GROUP_1 ON B_CATALOG_GROUP2GROUP(CATALOG_GROUP_ID, GROUP_ID, BUY)
GO

CREATE TABLE B_CATALOG_LOAD
(
	NAME varchar(250) NOT NULL,
	VALUE varchar(2000) NOT NULL,
	TYPE char(1) NOT NULL,
	LAST_USED char(1) NOT NULL
)
GO
ALTER TABLE B_CATALOG_LOAD ADD CONSTRAINT PK_B_CATALOG_LOAD PRIMARY KEY (NAME, TYPE)
GO
ALTER TABLE B_CATALOG_LOAD ADD CONSTRAINT DF_B_CATALOG_LOAD_TYPE DEFAULT 'I' FOR TYPE
GO
ALTER TABLE B_CATALOG_LOAD ADD CONSTRAINT DF_B_CATALOG_LOAD_LAST_USED DEFAULT 'N' FOR LAST_USED
GO

CREATE TABLE B_CATALOG_EXPORT
(
	ID int NOT NULL IDENTITY (1, 1),
	FILE_NAME varchar(100) NOT NULL,
	NAME varchar(250) NOT NULL,
	DEFAULT_PROFILE char(1) NOT NULL,
	IN_MENU char(1) NOT NULL,
	IN_AGENT char(1) NOT NULL,
	IN_CRON char(1) NOT NULL,
	SETUP_VARS text NULL,
	LAST_USE datetime NULL,
	IS_EXPORT char(1) NOT NULL,
	NEED_EDIT char(1) NOT NULL,
	TIMESTAMP_X datetime NULL,
	MODIFIED_BY int NULL,
	DATE_CREATE datetime NULL,
	CREATED_BY int NULL
)
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT PK_B_CATALOG_EXPORT PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_DEFAULT_PROFILE DEFAULT 'N' FOR DEFAULT_PROFILE
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_IN_MENU DEFAULT 'N' FOR IN_MENU
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_IN_AGENT DEFAULT 'N' FOR IN_AGENT
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_IN_CRON DEFAULT 'N' FOR IN_CRON
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_IS_EXPORT DEFAULT 'Y' FOR IS_EXPORT
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_NEED_EDIT DEFAULT 'Y' FOR NEED_EDIT
GO
ALTER TABLE B_CATALOG_EXPORT ADD CONSTRAINT DF_B_CATALOG_EXPORT_TIMESTAMP_X DEFAULT GETDATE() FOR TIMESTAMP_X
GO
CREATE INDEX IX_B_CATALOG_EXPORT_1 ON B_CATALOG_EXPORT(FILE_NAME)
GO
CREATE INDEX IX_B_CATALOG_EXPORT_2 ON B_CATALOG_EXPORT(IS_EXPORT)
GO

CREATE TABLE B_CATALOG_PRODUCT2GROUP
(
	ID int NOT NULL IDENTITY (1, 1),
	PRODUCT_ID int NOT NULL,
	GROUP_ID int NOT NULL,
	ACCESS_LENGTH int NOT NULL,
	ACCESS_LENGTH_TYPE char(1) NOT NULL
)
GO
ALTER TABLE B_CATALOG_PRODUCT2GROUP ADD CONSTRAINT PK_B_CATALOG_PRODUCT2GROUP PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_PRODUCT2GROUP ADD CONSTRAINT DF_B_CATALOG_PRODUCT2GROUP_ACCESS_LENGTH_TYPE DEFAULT 'D' FOR ACCESS_LENGTH_TYPE
GO
CREATE UNIQUE INDEX IX_B_CATALOG_PRODUCT2GROUP_1 ON B_CATALOG_PRODUCT2GROUP(PRODUCT_ID, GROUP_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT
(
	ID int NOT NULL IDENTITY (1, 1),
	XML_ID varchar(255) NULL,
	SITE_ID char(2) NOT NULL,
	TYPE int NOT NULL,
	ACTIVE char(1) NOT NULL,
	ACTIVE_FROM datetime NULL,
	ACTIVE_TO datetime NULL,
	RENEWAL char(1) NOT NULL,
	NAME varchar(255) NULL,
	MAX_USES int NOT NULL,
	COUNT_USES int NOT NULL,
	COUPON varchar(20) NULL,
	SORT int NOT NULL,
	MAX_DISCOUNT decimal(18,4) NULL,
	VALUE_TYPE char(1) NOT NULL,
	VALUE decimal(18,4) NOT NULL,
	CURRENCY char(3) NOT NULL,
	MIN_ORDER_SUM decimal(18,4) NULL,
	TIMESTAMP_X datetime NOT NULL,
	COUNT_PERIOD char(1) NOT NULL,
	COUNT_SIZE int NOT NULL,
	COUNT_TYPE char(1) NOT NULL,
	COUNT_FROM datetime NULL,
	COUNT_TO datetime NULL,
	ACTION_SIZE int NOT NULL,
	ACTION_TYPE char(1) NOT NULL,
	MODIFIED_BY int NULL,
	DATE_CREATE datetime NULL,
	CREATED_BY int NULL,
	PRIORITY int NOT NULL,
	LAST_DISCOUNT char(1) NOT NULL,
	VERSION int NOT NULL,
	NOTES varchar(255) NULL,
	CONDITIONS text NULL,
	UNPACK text NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT PK_B_CATALOG_DISCOUNT PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_TYPE DEFAULT '0' FOR TYPE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_ACTIVE DEFAULT 'Y' FOR ACTIVE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_RENEWAL DEFAULT 'N' FOR RENEWAL
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_MAX_USES DEFAULT '0' FOR MAX_USES
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COUNT_USES DEFAULT '0' FOR COUNT_USES
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_SORT DEFAULT '100' FOR SORT
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_VALUE_TYPE DEFAULT 'P' FOR VALUE_TYPE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_VALUE DEFAULT '0' FOR VALUE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_MIN_ORDER_SUM DEFAULT '0' FOR MIN_ORDER_SUM
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_TIMESTAMP_X DEFAULT GETDATE() FOR TIMESTAMP_X
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COUNT_PERIOD DEFAULT 'U' FOR COUNT_PERIOD
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COUNT_SIZE DEFAULT '0' FOR COUNT_SIZE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COUNT_TYPE DEFAULT 'Y' FOR COUNT_TYPE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_ACTION_SIZE DEFAULT '0' FOR ACTION_SIZE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_ACTION_TYPE DEFAULT 'Y' FOR ACTION_TYPE
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_PRIORITY DEFAULT '1' FOR PRIORITY
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_LAST_DISCOUNT DEFAULT 'Y' FOR LAST_DISCOUNT
GO
ALTER TABLE B_CATALOG_DISCOUNT ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_VERSION DEFAULT '1' FOR VERSION
GO
CREATE INDEX IX_B_CATALOG_DISCOUNT_1 ON B_CATALOG_DISCOUNT(COUPON)
GO
CREATE INDEX IX_B_CATALOG_DISCOUNT_2 ON B_CATALOG_DISCOUNT(ACTIVE, ACTIVE_FROM, ACTIVE_TO)
GO
CREATE INDEX IX_B_CATALOG_DISCOUNT_B ON B_CATALOG_DISCOUNT(SITE_ID, RENEWAL, ACTIVE, ACTIVE_FROM, ACTIVE_TO)
GO
CREATE TRIGGER B_CATALOG_DISCOUNT_UPDATE on B_CATALOG_DISCOUNT for update as
if (not update(TIMESTAMP_X))
begin
	UPDATE B_CATALOG_DISCOUNT SET
		TIMESTAMP_X = GETDATE()
	FROM
		B_CATALOG_DISCOUNT U,
		INSERTED I
	WHERE
		U.ID = I.ID
end
GO

CREATE TABLE B_CATALOG_DISCOUNT_COND
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	ACTIVE char(1) NULL,
	USER_GROUP_ID int NOT NULL,
	PRICE_TYPE_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT_COND ADD CONSTRAINT PK_B_CATALOG_DISCOUNT_COND PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_DISCOUNT_COND ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COND_USER_GROUP_ID DEFAULT -1 FOR USER_GROUP_ID
GO
ALTER TABLE B_CATALOG_DISCOUNT_COND ADD CONSTRAINT DF_B_CATALOG_DISCOUNT_COND_PRICE_TYPE_ID DEFAULT -1 FOR PRICE_TYPE_ID
GO

CREATE TABLE B_CATALOG_DISCOUNT_MODULE
(
  ID int NOT NULL IDENTITY (1, 1),
  DISCOUNT_ID int NOT NULL,
  MODULE_ID varchar(50) NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT_MODULE ADD CONSTRAINT PK_B_CATALOG_DISCOUNT_MODULE PRIMARY KEY (ID)
GO
CREATE INDEX IX_CAT_DSC_MOD ON B_CATALOG_DISCOUNT_MODULE(DISCOUNT_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT2PRODUCT
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	PRODUCT_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT2PRODUCT ADD CONSTRAINT PK_B_CATALOG_DISCOUNT2PRODUCT PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2PRODUCT_1 ON B_CATALOG_DISCOUNT2PRODUCT(PRODUCT_ID, DISCOUNT_ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2PRODUCT_B ON B_CATALOG_DISCOUNT2PRODUCT(DISCOUNT_ID, PRODUCT_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT2GROUP
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	GROUP_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT2GROUP ADD CONSTRAINT PK_B_CATALOG_DISCOUNT2GROUP PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2GROUP_1 ON B_CATALOG_DISCOUNT2GROUP(GROUP_ID, DISCOUNT_ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2GROUP_B ON B_CATALOG_DISCOUNT2GROUP(DISCOUNT_ID, GROUP_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT2CAT
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	CATALOG_GROUP_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT2CAT ADD CONSTRAINT PK_B_CATALOG_DISCOUNT2CAT PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2CAT_1 ON B_CATALOG_DISCOUNT2CAT(CATALOG_GROUP_ID, DISCOUNT_ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2CAT_B ON B_CATALOG_DISCOUNT2CAT(DISCOUNT_ID, CATALOG_GROUP_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT2SECTION
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	SECTION_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT2SECTION ADD CONSTRAINT PK_B_CATALOG_DISCOUNT2SECTION PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2SECTION_1 ON B_CATALOG_DISCOUNT2SECTION(SECTION_ID, DISCOUNT_ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2SECTION_B ON B_CATALOG_DISCOUNT2SECTION(DISCOUNT_ID, SECTION_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT2IBLOCK
(
	ID int NOT NULL IDENTITY (1, 1),
	DISCOUNT_ID int NOT NULL,
	IBLOCK_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISCOUNT2IBLOCK ADD CONSTRAINT PK_B_CATALOG_DISCOUNT2IBLOCK PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2IBLOCK_1 ON B_CATALOG_DISCOUNT2IBLOCK(IBLOCK_ID, DISCOUNT_ID)
GO
CREATE UNIQUE INDEX UX_B_CATALOG_DISCOUNT2IBLOCK_B ON B_CATALOG_DISCOUNT2IBLOCK(DISCOUNT_ID, IBLOCK_ID)
GO

CREATE TABLE B_CATALOG_DISCOUNT_COUPON
(
	ID int not null IDENTITY (1, 1),
	DISCOUNT_ID int not null,
	ACTIVE char(1) not null,
	COUPON varchar(32) not null,
	DATE_APPLY datetime null,
	ONE_TIME char(1) not null,
	TIMESTAMP_X datetime NULL,
	MODIFIED_BY int NULL,
	DATE_CREATE datetime NULL,
	CREATED_BY int NULL,
	DESCRIPTION text null
)
GO

ALTER TABLE B_CATALOG_DISCOUNT_COUPON ADD CONSTRAINT PK_CAT_DISC_COU PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_DISCOUNT_COUPON ADD CONSTRAINT DF_CAT_DISCOU_ACTIVE DEFAULT 'Y' FOR ACTIVE
GO
ALTER TABLE B_CATALOG_DISCOUNT_COUPON ADD CONSTRAINT DF_CAT_DISCOU_ONE_TIME DEFAULT 'Y' FOR ONE_TIME
GO
CREATE UNIQUE INDEX IX_CAT_DISCOU_1 ON B_CATALOG_DISCOUNT_COUPON(DISCOUNT_ID, COUPON)
GO
CREATE INDEX IX_CAT_DISCOU_2 ON B_CATALOG_DISCOUNT_COUPON(COUPON, ACTIVE)
GO

CREATE TABLE B_CATALOG_VAT
(
	ID int NOT NULL IDENTITY(1,1),
	TIMESTAMP_X datetime NULL,
	ACTIVE char(1) NULL,
	C_SORT int NOT NULL,
	NAME varchar(50) NOT NULL,
	RATE decimal(18,2) NOT NULL
)
GO
ALTER TABLE B_CATALOG_VAT ADD CONSTRAINT PK_B_CATALOG_VAT PRIMARY KEY (ID)
GO
CREATE INDEX IX_ACTIVE ON B_CATALOG_VAT (ACTIVE)
GO
ALTER TABLE B_CATALOG_VAT ADD CONSTRAINT DF_CAT_VAT_ACTIVE DEFAULT 'Y' FOR ACTIVE
GO
ALTER TABLE B_CATALOG_VAT ADD CONSTRAINT DF_CAT_VAT_C_SORT DEFAULT '100' FOR C_SORT
GO
ALTER TABLE B_CATALOG_VAT ADD CONSTRAINT DF_CAT_VAT_RATE DEFAULT '0.00' FOR RATE
GO

CREATE TABLE B_CATALOG_DISC_SAVE_RANGE
(
	ID int NOT NULL IDENTITY(1,1),
	DISCOUNT_ID int NOT NULL,
	RANGE_FROM decimal(18,2) NOT NULL,
	TYPE char(1) NOT NULL,
	VALUE decimal(18,2) NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISC_SAVE_RANGE ADD CONSTRAINT PK_B_CAT_DSR PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_DISC_SAVE_RANGE ADD CONSTRAINT DF_B_CAT_DSR_TYPE DEFAULT 'P' FOR TYPE
GO
CREATE INDEX IX_CAT_DSR_DISCOUNT ON B_CATALOG_DISC_SAVE_RANGE(DISCOUNT_ID)
GO
CREATE INDEX IX_CAT_DSR_DISCOUNT2 ON B_CATALOG_DISC_SAVE_RANGE(DISCOUNT_ID, RANGE_FROM)
GO

CREATE TABLE B_CATALOG_DISC_SAVE_GROUP
(
	ID int NOT NULL IDENTITY(1,1),
	DISCOUNT_ID int NOT NULL,
	GROUP_ID int NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISC_SAVE_GROUP ADD CONSTRAINT PK_B_CAT_DSG PRIMARY KEY (ID)
GO
CREATE INDEX IX_CAT_DSG_DISCOUNT ON B_CATALOG_DISC_SAVE_GROUP(DISCOUNT_ID)
GO
CREATE INDEX IX_CAT_DSG_GROUP ON B_CATALOG_DISC_SAVE_GROUP(GROUP_ID)
GO

CREATE TABLE B_CATALOG_DISC_SAVE_USER
(
	ID int NOT NULL IDENTITY(1,1),
	DISCOUNT_ID int NOT NULL,
	USER_ID int NOT NULL,
	ACTIVE_FROM datetime NOT NULL,
	ACTIVE_TO datetime NOT NULL,
	RANGE_FROM decimal(18,2) NOT NULL
)
GO
ALTER TABLE B_CATALOG_DISC_SAVE_USER ADD CONSTRAINT PK_B_CAT_DSU PRIMARY KEY (ID)
GO
CREATE INDEX IX_CAT_DSU_DISCOUNT ON B_CATALOG_DISC_SAVE_USER(DISCOUNT_ID)
GO
CREATE INDEX IX_CAT_DSU_USER ON B_CATALOG_DISC_SAVE_USER(DISCOUNT_ID, USER_ID)
GO

CREATE TABLE B_CATALOG_STORE
(
	ID INT NOT NULL IDENTITY (1, 1),
	TITLE VARCHAR(75) NULL,
	ACTIVE CHAR(1) DEFAULT 'Y' NOT NULL,
	ADDRESS VARCHAR(245) NOT NULL,
	DESCRIPTION TEXT NULL,
	GPS_N VARCHAR(15) NULL,
	GPS_S VARCHAR(15) NULL,
	IMAGE_ID VARCHAR(45) NULL,
	LOCATION_ID INT NULL,
	DATE_MODIFY DATETIME NULL,
	DATE_CREATE DATETIME NULL,
	USER_ID INT NULL,
	MODIFIED_BY INT NULL,
	PHONE VARCHAR(45) NULL,
	XML_ID VARCHAR(255) NULL,
	SCHEDULE VARCHAR(255) NULL,
	SORT INT NOT NULL,
	ISSUING_CENTER CHAR(1) NOT NULL,
	SHIPPING_CENTER CHAR(1) NOT NULL,
	EMAIL VARCHAR(255) NULL,
	SITE_ID CHAR(2) NULL,
)
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT PK_B_CATALOG_STORE PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CATALOG_STORE_GPS_N DEFAULT '0' FOR GPS_N
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CATALOG_STORE_GPS_S DEFAULT '0' FOR GPS_S
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CAT_STORE_SORT DEFAULT '100' FOR SORT
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CAT_STORE_ISSUING_CENTER DEFAULT 'Y' FOR ISSUING_CENTER
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CAT_STORE_SHIPPING_CENTER DEFAULT 'Y' FOR SHIPPING_CENTER
GO
ALTER TABLE B_CATALOG_STORE ADD CONSTRAINT DF_B_CATALOG_STORE_DATE_MODIFY DEFAULT CURRENT_TIMESTAMP FOR DATE_MODIFY
GO

CREATE TABLE B_CATALOG_STORE_PRODUCT
(
	ID INT NOT NULL IDENTITY (1, 1),
	PRODUCT_ID INT NOT NULL,
	AMOUNT decimal(18,2) NOT NULL,
	STORE_ID INT NOT NULL
)
GO
ALTER TABLE B_CATALOG_STORE_PRODUCT ADD CONSTRAINT PK_B_CATALOG_STORE_PRODUCT PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_STORE_PRODUCT ADD CONSTRAINT DF_B_CATALOG_STORE_PRODUCT1 DEFAULT '0' FOR AMOUNT
GO
CREATE INDEX IX_CATALOG_STORE_PRODUCT1 ON B_CATALOG_STORE_PRODUCT(STORE_ID)
GO
CREATE UNIQUE INDEX IX_CATALOG_STORE_PRODUCT2 ON B_CATALOG_STORE_PRODUCT(PRODUCT_ID, STORE_ID)
GO

CREATE TABLE B_CATALOG_STORE_BARCODE
(
	ID INT NOT NULL IDENTITY (1, 1),
	PRODUCT_ID INT NOT NULL,
	BARCODE VARCHAR(100) NULL,
	STORE_ID INT NULL,
	ORDER_ID INT NULL,
	DATE_MODIFY DATETIME NULL,
	DATE_CREATE DATETIME NULL,
	CREATED_BY INT NULL,
	MODIFIED_BY INT NULL
)
GO
ALTER TABLE B_CATALOG_STORE_BARCODE ADD CONSTRAINT PK_B_CATALOG_STORE_BARCODE PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_STORE_BARCODE ADD CONSTRAINT DF_B_CATALOG_STORE_BARCODE DEFAULT CURRENT_TIMESTAMP FOR DATE_MODIFY
GO
CREATE UNIQUE INDEX IX_B_CATALOG_STORE_BARCODE1 ON B_CATALOG_STORE_BARCODE(BARCODE)
GO

CREATE TABLE B_CATALOG_CONTRACTOR
(
	ID INT NOT NULL IDENTITY (1, 1),
	PERSON_TYPE CHAR(1) NOT NULL,
	PERSON_NAME VARCHAR(100) NULL,
	PERSON_LASTNAME VARCHAR(100) NULL,
	PERSON_MIDDLENAME VARCHAR(100) NULL,
	EMAIL VARCHAR(100) NULL,
	PHONE VARCHAR(45) NULL,
	POST_INDEX VARCHAR(45) NULL,
	COUNTRY VARCHAR(45) NULL,
	CITY VARCHAR(45) NULL,
	COMPANY VARCHAR(145) NULL,
	INN VARCHAR(145) NULL,
	KPP VARCHAR(145) NULL,
	ADDRESS VARCHAR(255) NULL,
	DATE_MODIFY DATETIME NULL,
	DATE_CREATE DATETIME NULL,
	CREATED_BY INT NULL,
	MODIFIED_BY INT NULL
)
GO
ALTER TABLE B_CATALOG_CONTRACTOR ADD CONSTRAINT PK_B_CATALOG_CONTRACTOR PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_CONTRACTOR ADD CONSTRAINT DF_B_CATALOG_CONTRACTOR DEFAULT CURRENT_TIMESTAMP FOR DATE_MODIFY
GO

CREATE TABLE B_CATALOG_STORE_DOCS
(
	ID INT NOT NULL IDENTITY (1, 1),
	DOC_TYPE CHAR(1) NOT NULL,
	SITE_ID CHAR(2) NULL,
	CONTRACTOR_ID INT NULL,
	DATE_MODIFY DATETIME NULL,
	DATE_CREATE DATETIME NULL,
	CREATED_BY INT NULL,
	MODIFIED_BY INT NULL,
	CURRENCY CHAR(3) NULL,
	STATUS CHAR(1) NOT NULL,
	DATE_STATUS DATETIME NULL,
	DATE_DOCUMENT DATETIME NULL,
	STATUS_BY INT NULL,
	TOTAL decimal(18,2) NULL,
	COMMENTARY varchar(1000) NULL
)
GO
ALTER TABLE B_CATALOG_STORE_DOCS ADD CONSTRAINT PK_B_CATALOG_STORE_DOCS PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_STORE_DOCS ADD CONSTRAINT DF_B_CATALOG_STORE_DOCS1 DEFAULT 'N' FOR STATUS
GO
ALTER TABLE B_CATALOG_STORE_DOCS ADD CONSTRAINT DF_B_CATALOG_STORE_DOCS2 DEFAULT CURRENT_TIMESTAMP FOR DATE_MODIFY
GO

CREATE TABLE B_CATALOG_DOCS_ELEMENT
(
	ID INT NOT NULL IDENTITY (1, 1),
	DOC_ID INT NOT NULL,
	STORE_FROM INT NULL,
	STORE_TO INT NULL,
	ELEMENT_ID INT NULL,
	AMOUNT decimal(18,2) NULL,
	PURCHASING_PRICE decimal(18,2) NULL
)
GO
ALTER TABLE B_CATALOG_DOCS_ELEMENT ADD CONSTRAINT PK_B_CATALOG_DOCS_ELEMENT PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CATALOG_DOCS_ELEMENT1 ON B_CATALOG_DOCS_ELEMENT(DOC_ID)
GO

CREATE TABLE B_CATALOG_DOCS_BARCODE
(
	ID INT NOT NULL IDENTITY (1, 1),
	DOC_ELEMENT_ID INT NOT NULL,
	BARCODE VARCHAR(100) NOT NULL
)
GO
ALTER TABLE B_CATALOG_DOCS_BARCODE ADD CONSTRAINT PK_B_CATALOG_DOCS_BARCODE PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CATALOG_DOCS_BARCODE1 ON B_CATALOG_DOCS_BARCODE(DOC_ELEMENT_ID)
GO

CREATE TABLE B_CATALOG_MEASURE
(
	ID INT NOT NULL IDENTITY (1, 1),
	CODE INT NOT NULL,
	MEASURE_TITLE VARCHAR(500) NULL,
	SYMBOL_RUS VARCHAR(20) NULL,
	SYMBOL_INTL VARCHAR(20) NULL,
	SYMBOL_LETTER_INTL VARCHAR(20) NULL,
	IS_DEFAULT CHAR(1) NOT NULL
)
GO
ALTER TABLE B_CATALOG_MEASURE ADD CONSTRAINT PK_B_CATALOG_MEASURE PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_MEASURE ADD CONSTRAINT DF_B_CATALOG_MEASURE1 DEFAULT 'N' FOR IS_DEFAULT
GO
CREATE UNIQUE INDEX IX_B_CATALOG_MEASURE1 ON B_CATALOG_MEASURE(CODE)
GO

CREATE TABLE B_CATALOG_MEASURE_RATIO
(
	ID INT NOT NULL IDENTITY (1, 1),
	PRODUCT_ID INT NOT NULL,
	RATIO decimal(18,2) NOT NULL
)
GO
ALTER TABLE B_CATALOG_MEASURE_RATIO ADD CONSTRAINT PK_B_CATALOG_MEASURE_RATIO PRIMARY KEY (ID)
GO
ALTER TABLE B_CATALOG_MEASURE_RATIO ADD CONSTRAINT DF_B_CATALOG_MEASURE_RATIO DEFAULT '1' FOR RATIO
GO
CREATE UNIQUE INDEX IX_B_CATALOG_MEASURE_RATIO ON B_CATALOG_MEASURE_RATIO(PRODUCT_ID, RATIO)
GO

CREATE TABLE B_CATALOG_PRODUCT_SETS
(
	ID int NOT NULL IDENTITY (1, 1),
	TYPE int NOT NULL,
	SET_ID int NOT NULL,
	ACTIVE CHAR(1) NOT NULL,
	OWNER_ID int NOT NULL,
	ITEM_ID int NOT NULL,
	QUANTITY decimal(18,4) NULL,
	MEASURE int NULL,
	DISCOUNT_PERCENT decimal(18,2) NULL,
	SORT int NOT NULL,
	CREATED_BY int NULL,
	DATE_CREATE datetime NULL,
	MODIFIED_BY int NULL,
	TIMESTAMP_X datetime NULL,
	XML_ID varchar(255) NULL
)
GO
ALTER TABLE	B_CATALOG_PRODUCT_SETS ADD CONSTRAINT PK_B_CATALOG_PRODUCT_SETS PRIMARY KEY(ID)
GO
ALTER TABLE B_CATALOG_PRODUCT_SETS ADD CONSTRAINT DF_B_CP_SETS_SORT DEFAULT '100' FOR SORT
GO
CREATE INDEX IX_CAT_PR_SET_TYPE ON B_CATALOG_PRODUCT_SETS(TYPE)
GO
CREATE INDEX IX_CAT_PR_SET_OWNER_ID ON B_CATALOG_PRODUCT_SETS(OWNER_ID)
GO
CREATE INDEX IX_CAT_PR_SET_SET_ID ON B_CATALOG_PRODUCT_SETS(SET_ID)
GO
CREATE INDEX IX_CAT_PR_SET_ITEM_ID ON B_CATALOG_PRODUCT_SETS(ITEM_ID)
GO

CREATE TABLE B_CATALOG_VIEWED_PRODUCT
(
	ID int NOT NULL IDENTITY (1, 1),
	FUSER_ID int NOT NULL,
	DATE_VISIT datetime NULL,
	PRODUCT_ID int NOT NULL,
	ELEMENT_ID int NOT NULL,
	SITE_ID CHAR(2) NOT NULL,
	VIEW_COUNT int NOT NULL,
	RECOMMENDATION VARCHAR(40) NULL
)
GO
ALTER TABLE	B_CATALOG_VIEWED_PRODUCT ADD CONSTRAINT PK_B_CATALOG_VIEWED_PRODUCTS PRIMARY KEY(ID)
GO
ALTER TABLE	B_CATALOG_VIEWED_PRODUCT ADD CONSTRAINT DF_B_VP_VIEW_COUNT DEFAULT '1' FOR VIEW_COUNT
GO
ALTER TABLE B_CATALOG_VIEWED_PRODUCT ADD CONSTRAINT DF_B_VP_ELEMENT_ID DEFAULT '0' FOR ELEMENT_ID
GO
CREATE INDEX IX_CAT_V_PR_FUSER_ID ON B_CATALOG_VIEWED_PRODUCT (FUSER_ID)
GO

CREATE TABLE B_CATALOG_SUBSCRIBE (
	ID INT NOT NULL IDENTITY (1, 1),
	DATE_FROM datetime NOT NULL,
	DATE_TO datetime NULL,
	USER_CONTACT VARCHAR(255) NOT NULL,
	CONTACT_TYPE SMALLINT NOT NULL,
	USER_ID INT NULL,
	ITEM_ID INT NOT NULL,
	NEED_SENDING CHAR(1) NOT NULL,
	SITE_ID CHAR(2) NOT NULL
)
GO
ALTER TABLE	B_CATALOG_SUBSCRIBE ADD CONSTRAINT PK_B_CATALOG_SUBSCRIBE PRIMARY KEY(ID)
GO
ALTER TABLE B_CATALOG_SUBSCRIBE ADD CONSTRAINT DF_B_CATALOG_SUBSCRIBE DEFAULT 'N' FOR NEED_SENDING
GO
CREATE INDEX IX_CAT_SUB_USER_CONTACT ON B_CATALOG_SUBSCRIBE (USER_CONTACT)
GO
CREATE INDEX IX_CAT_SUB_USER_ID ON B_CATALOG_SUBSCRIBE (USER_ID)
GO
CREATE INDEX IX_CAT_SUB_ITEM_ID ON B_CATALOG_SUBSCRIBE (ITEM_ID)
GO

CREATE TABLE B_CATALOG_SUBSCRIBE_ACCESS (
	ID INT NOT NULL IDENTITY (1, 1),
	DATE_FROM datetime NOT NULL,
	USER_CONTACT VARCHAR(255) NOT NULL,
	TOKEN CHAR(6) NOT NULL
)
GO
ALTER TABLE	B_CATALOG_SUBSCRIBE_ACCESS ADD CONSTRAINT PK_B_CATALOG_SUBSCRIBE_ACCESS PRIMARY KEY(ID)
GO
CREATE INDEX IX_CAT_SUB_ACS_USER_CONTACT ON B_CATALOG_SUBSCRIBE_ACCESS (USER_CONTACT)
GO

CREATE TABLE B_CATALOG_ROUNDING
(
	ID int NOT NULL IDENTITY (1, 1),
	CATALOG_GROUP_ID int NOT NULL,
	PRICE decimal(18, 4) NOT NULL,
	ROUND_TYPE int NOT NULL,
	ROUND_PRECISION decimal(18, 4) NOT NULL,
	CREATED_BY int NULL,
	DATE_CREATE datetime NULL,
	MODIFIED_BY int NULL,
	DATE_MODIFY datetime NULL
)
GO
ALTER TABLE B_CATALOG_ROUNDING ADD CONSTRAINT PK_B_B_CATALOG_ROUNDING PRIMARY KEY (ID)
GO
CREATE INDEX IX_CAT_RND_CATALOG_GROUP ON B_CATALOG_ROUNDING (CATALOG_GROUP_ID)
GO