var CrmButtonEditor = function(params) {

	this.init = function (params)
	{
		/* init button colors editing */
		this.colorEditor = new CrmButtonEditColors({
			caller: this,
			context: BX('BUTTON_COLOR_CONTAINER')
		});

		/* init widget editing */
		this.widgetManager = new CrmButtonEditWidgetManager({
			caller: this,
			context: BX('WIDGET_CONTAINER'),
			dictionaryPathEdit: params.dictionaryPathEdit
		});

		/* init location changing */
		this.locationManager = new CrmButtonEditLocation({
			caller: this,
			context: BX('LOCATION_CONTAINER')
		});

		this.initCopy();

		this.mainForm = BX('crm_button_main_form');
		this.subForm = BX('crm_button_sub_form');
		BX.bind(this.mainForm, 'submit', BX.proxy(this.copySubFormDataToMainForm, this));

		this.initButtons();
		this.initToolTips();
	};

	this.initButtons = function()
	{
		var _this = this;
		var buttonCont = BX('BUTTON_COLOR_CONTAINER');
		this.loadedButtonCount = buttonCont.querySelectorAll('[data-b24-crm-button-widget]').length;
		this.blockNode = buttonCont.querySelector('[data-b24-crm-button-block]');
		this.blockInnerNode = buttonCont.querySelector('[data-b24-crm-button-block-inner]');
		BX.bind(this.blockNode, 'mouseover', function (e) {
			_this.showButtons();
		});
		BX.bind(this.blockNode, 'mouseout', function (e) {
			_this.hideButtons();
		});
	};

	this.showButtons = function()
	{
		BX.addClass(this.blockNode, 'b24-crm-button-block-active');
		BX.addClass(this.blockNode, 'b24-crm-button-block-active-' + this.loadedButtonCount);
		BX.removeClass(this.blockInnerNode, 'b24-crm-button-animate-' + this.loadedButtonCount);
	};

	this.hideButtons = function()
	{
		BX.removeClass(this.blockNode, 'b24-crm-button-block-active');
		BX.removeClass(this.blockNode, 'b24-crm-button-block-active-' + this.loadedButtonCount);
		BX.addClass(this.blockInnerNode, 'b24-crm-button-animate-' + this.loadedButtonCount);
	};

	this.copySubFormDataToMainForm = function (e)
	{
		BX.convert.nodeListToArray(this.subForm.elements).forEach(function (element) {

			var name = element.name;
			var value = '';
			var copiedElement = this.mainForm.querySelector('[name="' + name + '"]');
			if(!copiedElement)
			{
				copiedElement = document.createElement('INPUT');
				copiedElement.type = 'hidden';
				copiedElement.name = name;
				this.mainForm.appendChild(copiedElement);
			}

			switch (element.type)
			{
				case 'radio':
					var sourceElement = this.subForm.querySelector('[name="' + name + '"]:checked');
					if(sourceElement)
					{
						value = sourceElement.value;
					}
					break;
				default:
					value = element.value;
			}

			copiedElement.value = value;

		}, this);

		return true;
	};

	this.initToolTips = function()
	{
		this.popupTooltip = {};
		var nodeList = document.body.querySelectorAll(".crm-button-context-help");
		nodeList = BX.convert.nodeListToArray(nodeList);
		var _this = this;
		for (var i = 0; i < nodeList.length; i++)
		{
			if (nodeList[i].getAttribute('context-help') == 'y')
				continue;

			nodeList[i].setAttribute('data-id', i);
			nodeList[i].setAttribute('context-help', 'y');
			BX.bind(nodeList[i], 'mouseover', function(){
				var id = this.getAttribute('data-id');
				var text = this.getAttribute('data-text');

				_this.showTooltip(id, this, text);
			});
			BX.bind(nodeList[i], 'mouseout', function(){
				var id = this.getAttribute('data-id');
				_this.hideTooltip(id);
			});
		}
	};

	this.showTooltip = function(id, bind, text)
	{
		if (this.popupTooltip[id])
			this.popupTooltip[id].close();

		this.popupTooltip[id] = new BX.PopupWindow('bx-crm-site-button-tooltip', bind, {
			lightShadow: true,
			autoHide: false,
			darkMode: true,
			offsetLeft: 0,
			offsetTop: 2,
			bindOptions: {position: "top"},
			zIndex: 200,
			events : {
				onPopupClose : function() {this.destroy()}
			},
			content : BX.create("div", { attrs : { style : "padding-right: 5px; width: 250px;" }, html: text})
		});
		this.popupTooltip[id].setAngle({offset:13, position: 'bottom'});
		this.popupTooltip[id].show();

		return true;
	};

	this.hideTooltip = function(id)
	{
		this.popupTooltip[id].close();
		this.popupTooltip[id] = null;
	};

	this.initCopy = function ()
	{
		var context = BX('SCRIPT_CONTAINER');
		if(!context)
		{
			return;
		}

		var buttonAttribute = 'data-bx-webform-script-copy-btn';
		var copyButton = context.querySelector('[' + buttonAttribute + ']');
		var copyButtonText = context.querySelector('[data-bx-webform-script-copy-text]');
		if(!copyButton || !copyButtonText)
		{
			return;
		}

		BX.clipboard.bindCopyClick(copyButton, {text: copyButtonText, offsetLeft: 30});
	};

	this.init(params);
};

function CrmButtonEditLocation(params)
{
	this.caller = params.caller;
	this.context = params.context;

	this.attributeItem = 'data-bx-crm-button-loc';
	this.attributeVal = 'data-bx-crm-button-loc-val';
	this.className = 'crm-button-edit-sidebar-button-position-block-active-';

	this.init();
}
CrmButtonEditLocation.prototype =
{
	init: function ()
	{
		this.list = this.context.querySelectorAll('[' + this.attributeItem + ']');
		this.list = BX.convert.nodeListToArray(this.list);

		this.list.forEach(function(locNode){
			BX.bind(locNode, 'click', BX.delegate(function () {
				this.list.forEach(this.deactivate, this);
				this.activate(locNode);
			}, this));
		}, this);
	},

	deactivate: function (locNode)
	{
		this.activate(locNode, true);
	},

	activate: function (locNode, isDeActivate)
	{
		isDeActivate = isDeActivate || false;

		var valNode = locNode.querySelector('[' + this.attributeVal + ']');
		var val = valNode.value;

		if(isDeActivate)
		{
			BX.removeClass(locNode, this.className + val);
		}
		else
		{
			BX.addClass(locNode, this.className + val);
		}
	}
};

function CrmButtonEditColors(params)
{
	this.caller = params.caller;
	this.context = params.context;

	this.attributeBlock = 'data-b24-crm-button-block';
	this.attributeBlockBorder = 'data-b24-crm-button-block-border';
	this.attributeCont = 'data-b24-crm-button-block-inner';
	this.attributeIconItem = 'data-b24-crm-button-widget';
	this.classNameActive = 'b24-crm-button-inner-item-active';


	this.colorIconNode = BX('ICON_COLOR');
	this.colorBgNode = BX('BACKGROUND_COLOR');

	this.node = this.context.querySelector('[' + this.attributeBlock + ']');
	this.previewNode = this.node.querySelector('[' + this.attributeCont + ']');
	this.borderNode = this.context.querySelector('[' + this.attributeBlockBorder + ']');

	var colorIconNodeList = this.node.querySelectorAll('[' + this.attributeIconItem + ']');
	colorIconNodeList = BX.convert.nodeListToArray(colorIconNodeList);
	this.previewItems = colorIconNodeList.map(function (iconNode) {
		return {
			'type': iconNode.getAttribute(this.attributeIconItem),
			'node': iconNode,
			'iconNode': iconNode.querySelector('path')
		};
	}, this);

	this.init();
}
CrmButtonEditColors.prototype =
{
	init: function()
	{
		this.initColorPicker();
	},

	changeActiveState: function(type, isActive)
	{
		this.previewItems.forEach(function (item) {

			if(item.type != type)
			{
				return;
			}

			if(isActive)
			{
				BX.addClass(item.node, this.classNameActive);
			}
			else
			{
				BX.removeClass(item.node, this.classNameActive);
			}

		}, this);

		this.previewNode.style.background = this.colorBgNode.value;
		this.previewItems.forEach(function (item) {
			item.iconNode.setAttribute('fill', this.colorIconNode.value);
		}, this);
	},

	updateButtonColors: function()
	{
		var bg = this.colorBgNode.value ? this.colorBgNode.value : '#00AEEF';
		this.previewNode.style.background = bg;
		this.borderNode.style.background = bg;

		this.previewItems.forEach(function (item) {
			item.iconNode.setAttribute(
				'fill',
				this.colorIconNode.value ? this.colorIconNode.value : '#FFFFFF'
			);
		}, this);
	},

	initColorPicker: function()
	{
		this.picker = new window.BXColorPicker({'id': "picker", 'name': 'picker'});
		this.picker.Create();

		var _this = this;

		var clickHandler = function ()
		{
			var element = this;
			element.parentNode.appendChild(_this.picker.pCont);
			_this.picker.oPar.OnSelect = BX.proxy(function (color)
			{
				if(!color)
					color = '';

				element.value = color;
				var colorBox = BX.nextSibling(element);
				if(colorBox)
				{
					colorBox.style.background = color;
				}
				BX.fireEvent(element, 'change');
			}, _this);

			_this.picker.pCont.style.display = '';
			_this.picker.Close();
			_this.picker.Open(element);
			_this.picker.pCont.style.display = 'none';

		};

		var changeHandler = function()
		{
			var colorBox = BX.nextSibling(this);
			if (colorBox)
			{
				colorBox.style.background = this.value;
				_this.updateButtonColors();
			}
		};


		var inputList = this.context.querySelectorAll('[data-web-form-color-picker]');
		inputList = BX.convert.nodeListToArray(inputList);
		for(var i in inputList)
		{
			var inputCtrl = inputList[i];
			var colorBox = BX.nextSibling(inputCtrl);

			BX.bind(colorBox, 'click', BX.proxy(clickHandler, inputCtrl));
			BX.bind(inputCtrl, 'click', clickHandler);
			BX.bind(inputCtrl, "focus", clickHandler);

			BX.bind(inputCtrl, "bxchange", BX.delegate(changeHandler, inputCtrl));
			BX.fireEvent(inputCtrl, 'change');
		}
	}

};

function CrmButtonEditWidgetManager(params)
{
	this.caller = params.caller;
	this.context = params.context;

	this.dictionaryPathEdit = params.dictionaryPathEdit;

	this.attributeItem = 'data-bx-crm-button-item';
	this.attributeActive = 'data-bx-crm-button-item-active';
	this.attributeActiveValue = 'data-bx-crm-button-item-active-val';
	this.attributeSelect = 'data-bx-crm-button-widget-select';
	this.attributeButtonEdit = 'data-bx-crm-button-widget-btn-edit';
	this.attributeSettingsButton = 'data-crm-button-item-settings-btn';
	this.attributeSettings = 'data-crm-button-item-settings';
	this.attributePages = 'data-crm-button-item-pages';
	this.attributePagesList = 'data-crm-button-item-pages-list';
	this.attributePagesPage = 'data-crm-button-item-pages-page';
	this.attributePagesAdd = 'data-crm-button-item-pages-btn-add';
	this.attributePagesDel = 'data-crm-button-item-pages-btn-del';

	this.init();
}
CrmButtonEditWidgetManager.prototype =
{
	init: function()
	{
		this.initPages();
		this.initSelect();
		this.initActivation();

		var settingsBtnNodeList = this.context.querySelectorAll('[' + this.attributeSettingsButton + ']');
		settingsBtnNodeList = BX.convert.nodeListToArray(settingsBtnNodeList);
		settingsBtnNodeList.forEach(function (settingsBtnNode) {
			BX.bind(settingsBtnNode, 'click', BX.proxy(function() {
				var type = settingsBtnNode.getAttribute(this.attributeSettingsButton);
				var settingsNode = this.context.querySelector('[' + this.attributeSettings + '="' + type + '"]');
				BX.toggleClass(settingsNode, 'crm-button-edit-channel-lines-display-options-open');
			}, this));
		}, this);
	},
	initPages: function()
	{
		var pageNodeList = this.context.querySelectorAll('[' + this.attributePagesList + ']');
		pageNodeList = BX.convert.nodeListToArray(pageNodeList);
		pageNodeList.forEach(function (pageNode) {
			this.initPageButtons(pageNode);
		}, this);
	},
	initPageButtons: function(pageNode)
	{
		var pagesBtnAddNode = pageNode.querySelector('[' + this.attributePagesAdd + ']');
		BX.bind(pagesBtnAddNode, 'click', BX.proxy(function(){
			this.onClickPageButton(pagesBtnAddNode, true);
		}, this));

		var pagesBtnDelNode = pageNode.querySelector('[' + this.attributePagesDel + ']');
		BX.bind(pagesBtnDelNode, 'click', BX.proxy(function(){
			this.onClickPageButton(pagesBtnDelNode, false);
		}, this));
	},

	initSelect: function()
	{
		var selectNodeList = this.context.querySelectorAll('[' + this.attributeSelect + ']');
		selectNodeList = BX.convert.nodeListToArray(selectNodeList);
		selectNodeList.forEach(function (selectNode) {

			BX.bind(selectNode, 'change', BX.proxy(function(){
				this.onChangeSelect(selectNode);
			}, this));

			this.onChangeSelect(selectNode);

		}, this);
	},

	initActivation: function()
	{
		var nodeList = this.context.querySelectorAll('[' + this.attributeActive + ']');
		nodeList = BX.convert.nodeListToArray(nodeList);
		nodeList.forEach(function (node) {
			BX.bind(node, 'click', BX.delegate(function (){
				this.toggleActive(node);
			}, this));
		}, this);
	},

	toggleActive: function(node)
	{
		var itemClass = 'crm-button-edit-channel-lines-container-active';
		var activeClassOn = 'crm-button-edit-channel-lines-title-on';
		var activeClassOff = 'crm-button-edit-channel-lines-title-off';

		var type = node.getAttribute(this.attributeActive);
		var itemNode = this.context.querySelector('[' + this.attributeItem + '="' + type + '"]');
		var valueNode = this.context.querySelector('[' + this.attributeActiveValue + '="' + type + '"]');
		var isActive = BX.hasClass(node, activeClassOn);
		if(isActive)
		{
			BX.addClass(node, activeClassOff);
			BX.removeClass(node, activeClassOn);
			BX.removeClass(itemNode, itemClass);

			valueNode.value ='N';
		}
		else
		{
			BX.addClass(node, activeClassOn);
			BX.removeClass(node, activeClassOff);
			BX.addClass(itemNode, itemClass);

			valueNode.value ='Y';
		}
	},

	onClickPageButton: function(node, isAdd)
	{
		if(isAdd)
		{
			var pagesNode = BX.findParent(node, {attribute: this.attributePages});
			var templateNode = pagesNode.querySelector('script');
			var addNode = document.createElement('div');
			addNode.innerHTML = templateNode.innerHTML;
			addNode = addNode.children[0];
			this.initPageButtons(addNode);

			var listNode = BX.findParent(node, {attribute: this.attributePagesList});
			listNode.appendChild(addNode);
		}
		else
		{
			var delNode = BX.findParent(node, {attribute: this.attributePagesPage});
			BX.remove(delNode);
		}
	},

	onChangeSelect: function(node)
	{
		var type = node.getAttribute(this.attributeSelect);
		var isSelected = !!node.value;
		this.caller.colorEditor.changeActiveState(type, isSelected);

		var buttonNode = this.context.querySelector('[' + this.attributeButtonEdit + '="' + type + '"' + ']');
		if(isSelected)
		{
			var path = this.dictionaryPathEdit[type];
			buttonNode.href = path.path.replace(path.id, node.value);
		}
		buttonNode.style.display = isSelected ? 'inline-block' : 'none';
	}
};
