<?if (!defined('B_PROLOG_INCLUDED') || B_PROLOG_INCLUDED !== true) die();

if (!CModule::IncludeModule('crm'))
{
	ShowError(GetMessage('CRM_MODULE_NOT_INSTALLED'));
	return;
}

if (!CModule::IncludeModule('sale'))
{
	ShowError(GetMessage('CRM_SALE_MODULE_NOT_INSTALLED'));
	return;
}

global $USER, $APPLICATION;

$CrmPerms = new CCrmPerms($USER->GetID());
if (!$CrmPerms->HavePerm('CONFIG', BX_CRM_PERM_CONFIG, 'WRITE'))
{
	ShowError(GetMessage('CRM_PERMISSION_DENIED'));
	return;
}

/*
 * PATH_TO_PS_LIST
 * PATH_TO_PS_EDIT
 * PS_ID
 * PS_ID_PAR_NAME
 */

$arParams['PATH_TO_PS_LIST'] = CrmCheckPath('PATH_TO_PS_LIST', $arParams['PATH_TO_PS_LIST'], '');
$arParams['PATH_TO_PS_EDIT'] = CrmCheckPath('PATH_TO_PS_EDIT', $arParams['PATH_TO_PS_EDIT'], '?ps_id=#ps_id#&edit');

$psID = isset($arParams['PS_ID']) ? intval($arParams['PS_ID']) : 0;
$documentRoot = \Bitrix\Main\Application::getDocumentRoot();

if($psID <= 0)
{
	$psIDParName = isset($arParams['PS_ID_PAR_NAME']) ? strval($arParams['PS_ID_PAR_NAME']) : '';

	if(strlen($psIDParName) == 0)
		$psIDParName = 'ps_id';

	$psID = isset($_REQUEST[$psIDParName]) ? (int)$_REQUEST[$psIDParName] : 0;
}

$arPaySys = array();

if ($psID > 0)
{
	$arPaySys = \Bitrix\Sale\PaySystem\Manager::getById($psID);
	$personTypeIds = CSalePaySystem::getPaySystemPersonTypeIds($psID);
	$arPaySys['PARAMS'] = CSalePaySystemAction::getParamsByConsumer('PAYSYSTEM_'.$psID, $personTypeIds[0]);

	if (!$arPaySys)
	{
		ShowError(GetMessage('CRM_PS_NOT_FOUND'));
		@define('ERROR_404', 'Y');
		if($arParams['SET_STATUS_404'] === 'Y')
		{
			CHTTP::SetStatus("404 Not Found");
		}
		return;
	}
}

$arResult['PS_ID'] = $psID;
$arResult['PAY_SYSTEM'] = $arPaySys;

$arResult['FORM_ID'] = 'CRM_PS_EDIT_FORM';
$arResult['GRID_ID'] = 'CRM_PS_EDIT_GRID';
$arResult['BACK_URL'] = CComponentEngine::MakePathFromTemplate(
	$arParams['PATH_TO_PS_LIST'],
	array()
);

if (check_bitrix_sessid())
{
	if ($_SERVER['REQUEST_METHOD'] == 'POST' && (isset($_POST['save']) || isset($_POST['apply'])))
	{
		$psID = isset($_POST['ps_id']) ? (int)$_POST['ps_id'] : 0;
		$personTypeId = isset($_POST['PERSON_TYPE_ID']) ? intval($_POST['PERSON_TYPE_ID']) : 0;
		$handler = $_POST["ACTION_FILE"];
		$errorMessage = '';

		$arActFields = array(
			"PERSON_TYPE_ID" => $personTypeId,
			"ACTION_FILE" => CCrmPaySystem::getActionHandler($handler),
			"NEW_WINDOW" => (($_POST['NEW_WINDOW'] == "Y") ? "Y" : "N" ),
			"HAVE_PREPAY" => "N",
			"HAVE_RESULT" => "N",
			"HAVE_ACTION" => "N",
			"HAVE_PAYMENT" => "N",
			"HAVE_RESULT_RECEIVE" => "N"
		);

		if (isset($_POST['NAME']) && strlen($_POST['NAME']) > 0)
		{
			$arActFields['NAME'] = trim($_POST['NAME']);
			$arActFields['PSA_NAME'] = $arActFields['NAME'];
		}
		else
		{
			$errorMessage .= GetMessage('CRM_PS_ERROR_NO_NAME').'<br>';
		}

		$arActFields['ACTIVE'] = (isset($_POST['ACTIVE']) && ($_POST['ACTIVE'] == "Y") ? "Y" : "N");
		$arActFields['SORT'] = (isset($_POST['SORT']) && ((int)$_POST['SORT'] > 0) ? (int)$_POST['SORT'] : 100);

		if (isset($_POST['DESCRIPTION']))
			$arActFields['DESCRIPTION'] = $_POST['DESCRIPTION'];

		if (strlen($errorMessage) <= 0)
		{
			if (isset($_POST["ACTION_FILE"]) && strlen(trim($_POST["ACTION_FILE"])) > 0)
				$actionFile = CCrmPaySystem::getActionPath($_POST["ACTION_FILE"]);
			else
				$errorMessage .= GetMessage("CRM_PS_EMPTY_SCRIP").".<br>";

			if (isset($actionFile))
			{
				$actionFile = str_replace("\\", "/", $actionFile);
				while (substr($actionFile, strlen($actionFile) - 1, 1) == "/")
					$actionFile = substr($actionFile, 0, strlen($actionFile) - 1);

				$pathToAction = $_SERVER["DOCUMENT_ROOT"].$actionFile;
				if (!file_exists($pathToAction))
					$errorMessage .= GetMessage("CRM_PS_NO_SCRIPT").".<br>";
			}

			if ($errorMessage === '')
			{
				$arActParams = array();

				if (isset($_POST['PS_ACTION_FIELDS_LIST']) && strlen($_POST['PS_ACTION_FIELDS_LIST']) > 0)
				{
					$filedList = explode(",", $_POST['PS_ACTION_FIELDS_LIST']);

					$arPsActFields = CCrmPaySystem::getPSCorrespondence($_POST["ACTION_FILE"] ?: 'bill');
					CCrmPaySystem::rewritePSCorrByRqSource($personTypeId, $arPsActFields, array('PSA_CODE' => $handler));

					foreach ($filedList as $val)
					{
						$val = trim($val);

						if (empty($arPsActFields[$val]))
							continue;

						$typeTmp = $_POST["TYPE_".$val];
						$valueTmp = $_POST["VALUE1_".$val];

						if (strlen($typeTmp) <= 0)
							$valueTmp = $_POST["VALUE2_".$val];

						if ($arPsActFields[$val]['TYPE'] == 'FILE' && $typeTmp != 'FILE')
							continue;

						if ($typeTmp == 'FILE')
						{
							$valueTmp = array();
							if (array_key_exists("VALUE1_".$val, $_FILES))
							{
								if ($_FILES["VALUE1_".$val]["error"] == 0)
								{
									$imageFileError = CFile::CheckImageFile($_FILES["VALUE1_".$val]);

									if (is_null($imageFileError))
										$valueTmp = $_FILES["VALUE1_".$val];
									else
										$errorMessage .= $imageFileError . ".<br>";
								}
							}

							if (trim($_POST[$val."_del"]) == 'Y')
							{
								if (intval($arPaySys['PARAMS'][$val]['VALUE']) == 0)
									continue;

								$valueTmp['old_file'] = $arPaySys['PARAMS'][$val]['VALUE'];
								$valueTmp['del'] = trim($_POST[$val."_del"]);
							}

							if (empty($valueTmp))
							{
								$typeTmp  = $arPaySys['PARAMS'][$val]['TYPE'];
								$valueTmp = $arPaySys['PARAMS'][$val]['VALUE'];
							}
						}

						$arActParams[$val] = array(
							"TYPE" => $typeTmp,
							"VALUE" => $valueTmp
						);

						if ($arActParams[$val]['TYPE'] == 'FILE' && is_array($arActParams[$val]['VALUE']))
						{
							$arActParams[$val]['VALUE']['MODULE_ID'] = 'sale';
							CFile::SaveForDB($arActParams[$val], 'VALUE', 'sale/paysystem/field');
						}
					}
				}

				$arActFields['PARAMS'] = serialize($arActParams);
				$path = \Bitrix\Sale\PaySystem\Manager::getPathToHandlerFolder($handler);
				if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$path.'/handler.php'))
				{
					require_once $documentRoot.$path.'/handler.php';

					$className = \Bitrix\Sale\PaySystem\Manager::getClassNameFromPath($path);
					$arActFields['HAVE_PAYMENT'] = 'Y';
					if (is_subclass_of($className, '\Bitrix\Sale\PaySystem\IPrePayable'))
						$arActFields['HAVE_PREPAY'] = 'Y';
					if (is_subclass_of($className, '\Bitrix\Sale\PaySystem\ServiceHandler'))
						$arActFields['HAVE_RESULT_RECEIVE'] = 'Y';
					if (is_subclass_of($className, '\Bitrix\Sale\PaySystem\IPayable'))
						$arActFields['HAVE_PRICE'] = 'Y';
				}
				else
				{
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$actionFile."/pre_payment.php"))
						$arActFields["HAVE_PREPAY"] = "Y";
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$actionFile."/result.php"))
						$arActFields["HAVE_RESULT"] = "Y";
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$actionFile."/action.php"))
						$arActFields["HAVE_ACTION"] = "Y";
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$actionFile."/payment.php"))
						$arActFields["HAVE_PAYMENT"] = "Y";
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$actionFile."/result_rec.php"))
						$arActFields["HAVE_RESULT_RECEIVE"] = "Y";
				}

				if ($psID === 0)
				{
					$image = '/bitrix/images/sale/sale_payments/'.$handler.'.png';
					if (\Bitrix\Main\IO\File::isFileExists($documentRoot.$image))
					{
						$arActFields['LOGOTIP'] = CFile::MakeFileArray($image);
						$arActFields['LOGOTIP']['MODULE_ID'] = "sale";
						CFile::SaveForDB($arActFields, 'LOGOTIP', 'sale/paysystem/logotip');
					}
				}

				if ($errorMessage === '')
				{
					if ($psID > 0)
					{
						$result = \Bitrix\Sale\Internals\PaySystemActionTable::update($psID, $arActFields);
					}
					else
					{
						$result = \Bitrix\Sale\Internals\PaySystemActionTable::add($arActFields);
						if ($result->isSuccess())
						{
							$psID = $result->getId();
							\Bitrix\Sale\Internals\PaySystemActionTable::update($psID, array('PAY_SYSTEM_ID' => $psID));
						}
					}

					if ($result->isSuccess())
					{
						if (array_key_exists('PARAMS', $arActFields))
						{
							$params = CSalePaySystemAction::prepareParamsForBusVal($psID, $arActFields);
							foreach ($params as $item)
								\Bitrix\Sale\BusinessValue::setMapping($item['CODE'], $item['CONSUMER'], $item['PERSON_TYPE_ID'], $item['MAP'], true);
						}

						if ($arActFields['PERSON_TYPE_ID'])
						{
							$params = array(
								'filter' => array(
									"SERVICE_ID" => $psID,
									"SERVICE_TYPE" => \Bitrix\Sale\Services\PaySystem\Restrictions\Manager::SERVICE_TYPE_PAYMENT,
									"=CLASS_NAME" => '\Bitrix\Sale\Services\PaySystem\Restrictions\PersonType'
								)
							);

							$dbRes = \Bitrix\Sale\Internals\ServiceRestrictionTable::getList($params);
							if ($data = $dbRes->fetch())
								$restrictionId = $data['ID'];
							else
								$restrictionId = 0;

							$fields = array(
								"SERVICE_ID" => $psID,
								"SERVICE_TYPE" => \Bitrix\Sale\Services\PaySystem\Restrictions\Manager::SERVICE_TYPE_PAYMENT,
								"SORT" => 100,
								"PARAMS" => array('PERSON_TYPE_ID' => array($arActFields['PERSON_TYPE_ID']))
							);

							\Bitrix\Sale\Services\PaySystem\Restrictions\PersonType::save($fields, $restrictionId);
						}
					}
					else
					{
						$errorMessage .= implode("<br>", $result->getErrorMessages());
					}
				}
			}
		}

		if (strlen($errorMessage) <= 0)
		{
			LocalRedirect(
				isset($_POST['apply'])
					? CComponentEngine::MakePathFromTemplate(
					$arParams['PATH_TO_PS_EDIT'],
					array('ps_id' => $psID)
				)
					: CComponentEngine::MakePathFromTemplate(
					$arParams['PATH_TO_PS_LIST'],
					array('ps_id' => $psID)
				)
			);
		}
		else
		{
			ShowError($errorMessage);
			$arPaySys = $arActFields;
		}
	}
	elseif ($_SERVER['REQUEST_METHOD'] == 'GET' &&  isset($_GET['delete']))
	{
		$psID = isset($arParams['PS_ID']) ? intval($arParams['PS_ID']) : 0;

		$result = \Bitrix\Sale\PaySystem\Manager::delete($psID);
		if (!$result->isSuccess())
			ShowError(implode('<br>', $result->getErrorMessages()));

		LocalRedirect(
			CComponentEngine::MakePathFromTemplate(
				$arParams['PATH_TO_PS_LIST'],
				array()
			)
		);
	}
}

$arResult['FIELDS'] = array();

if(strlen($arParams['PS_ID']) > 0)
{
	$arResult['FIELDS']['tab_props'][] = array(
		'id' => 'ID',
		'name' => 'ID',
		'value' => $psID,
		'type' =>  'label'
	);
}

$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'NAME',
	'name' =>  GetMessage('CRM_PS_FIELD_NAME'),
	'value' => htmlspecialcharsbx($arPaySys['NAME']),
	'type' =>  'text'
);

$io = CBXVirtualIo::GetInstance();
$arPaySys['ACTION_FILE'] = $io -> ExtractNameFromPath($arPaySys['ACTION_FILE'] ? $arPaySys['ACTION_FILE'] : 'bill');

$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'ACTION_FILE',
	'name' => GetMessage("CRM_PS_FIELD_ACTION_FILE"),
	'type' => 'list',
	'items' => CCrmPaySystem::getActionsList(),
	'value' => $arPaySys['ACTION_FILE']
);

$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'ACTIVE',
	'name' =>  GetMessage('CRM_PS_FIELD_ACTIVE'),
	'value' => ($arPaySys['ACTIVE']) ?: 'Y',
	'type' =>  'checkbox'
);

$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'SORT',
	'name' =>  GetMessage('CRM_PS_FIELD_SORT'),
	'value' => ($arPaySys['SORT']) ?: 100,
	'type' =>  'text'
);

$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'DESCRIPTION',
	'name' =>  GetMessage('CRM_PS_FIELD_DESCRIPTION'),
	'value' => $arPaySys['DESCRIPTION'],
	'type' =>  'textarea'
);

$ptList = CCrmPaySystem::getPersonTypesList();
$ptID = $arPaySys['PERSON_TYPE_ID'] ? $arPaySys['PERSON_TYPE_ID'] : key($ptList);
$arResult['FIELDS']['tab_props'][] = array(
	'id' => 'PERSON_TYPE_ID',
	'name' => GetMessage('CRM_PS_FIELD_PERSON_TYPE_ID'),
	'type' => 'list',
	'items' => $ptList,
	'value' => $ptID
);

$arResult['ACTION_FILE'] = $arPaySys['ACTION_FILE'];
$arResult['PS_ACT_FIELDS'] = CCrmPaySystem::getPSCorrespondence($arPaySys['ACTION_FILE']);
CCrmPaySystem::rewritePSCorrByRqSource($ptID, $arResult['PS_ACT_FIELDS'], array('PSA_CODE' => $arPaySys['ACTION_FILE']));
$arResult['ACTION_FIELDS_LIST'] =  implode(',', array_keys($arResult['PS_ACT_FIELDS']));
$arResult['SIMPLE_MODE'] = CCrmPaySystem::isFormSimple();

$arResult['USER_FIELDS'] = array();
$quoteUserFields = $GLOBALS['USER_FIELD_MANAGER']->GetUserFields(CCrmQuote::$sUFEntityID, 0, LANGUAGE_ID);
foreach($quoteUserFields as $quoteUserFieldName => &$quoteUserField)
{
	$arResult['USER_FIELDS']['quote'][$quoteUserFieldName] = $quoteUserField['EDIT_FORM_LABEL'];
}
unset($quoteUserField);

// gather requisite fields used in action settings
$requisiteFieldsUsed = array();
$fieldName = '';
foreach ($arResult['PS_ACT_FIELDS'] as $idCorr => $arCorr)
{
	if(isset($arPaySys['PARAMS'][$idCorr])
		&& isset($arPaySys['PARAMS'][$idCorr]['TYPE'])
		&& $arPaySys['PARAMS'][$idCorr]['TYPE'] === 'REQUISITE'
		&& isset($arPaySys['PARAMS'][$idCorr]['VALUE'])
	)
	{
		$fieldName = trim(strval($arPaySys['PARAMS'][$idCorr]['VALUE']));
		if (!empty($fieldName))
			$requisiteFieldsUsed[$fieldName] = true;
	}
}
unset($fieldName);

// requisite fields
$arResult['REQUISITE_FIELDS'] = CCrmPaySystem::getRequisiteFieldSelectItems(array_keys($requisiteFieldsUsed));

// bank detail fields
$arResult['BANK_DETAIL_FIELDS'] = CCrmPaySystem::getBankDetailFieldSelectItems();

// company fields
$arResult['CRM_COMPANY_FIELDS'] = array(
	'TITLE' => GetMessage('CRM_PS_COMPANY_FIELD_TITLE'),
	'PHONE' => GetMessage('CRM_PS_COMPANY_FIELD_PHONE'),
	'EMAIL' => GetMessage('CRM_PS_COMPANY_FIELD_EMAIL'),
);
$ar = CCrmFieldMulti::GetEntityTypeList('PHONE');
foreach($ar as $valueType => $value)
{
	if ($valueType === 'HOME')
		continue;

	$arResult['CRM_COMPANY_FIELDS']['PHONE_'.$valueType] = $value;
}
$ar = CCrmFieldMulti::GetEntityTypeList('EMAIL');
foreach($ar as $valueType => $value)
	$arResult['CRM_COMPANY_FIELDS']['EMAIL_'.$valueType] = $value;

// contact fields
$arResult['CRM_CONTACT_FIELDS'] = array(
	'FULL_NAME' => GetMessage('CRM_PS_CONTACT_FIELD_FULL_NAME'),
	'PHONE' => GetMessage('CRM_PS_CONTACT_FIELD_PHONE'),
	'EMAIL' => GetMessage('CRM_PS_CONTACT_FIELD_EMAIL')
);
$ar = CCrmFieldMulti::GetEntityTypeList('PHONE');
foreach($ar as $valueType => $value)
	$arResult['CRM_CONTACT_FIELDS']['PHONE_'.$valueType] = $value;
$ar = CCrmFieldMulti::GetEntityTypeList('EMAIL');
foreach($ar as $valueType => $value)
	$arResult['CRM_CONTACT_FIELDS']['EMAIL_'.$valueType] = $value;

$fieldsByGroups = array();
foreach ($arResult['PS_ACT_FIELDS'] as $idCorr => $arCorr)
{
	if (!array_key_exists($arCorr['GROUP'], $fieldsByGroups))
		$fieldsByGroups[$arCorr['GROUP']] = array();

	$fieldsByGroups[$arCorr['GROUP']][$idCorr] = $arCorr;
}
$arResult['PS_ACT_FIELDS_BY_GROUP'] = $fieldsByGroups;
$arResult['FIELD_LIST'] = $arResult['FIELDS'];
$fieldsGroups = \Bitrix\Sale\BusinessValue::getGroups();
foreach ($fieldsByGroups as $group => $fields)
{
	$arResult['FIELD_LIST']['tab_props'][] = array(
		'id' => $group,
		'name' => $fieldsGroups[$group]['NAME'],
		'type' => 'section'
	);

	foreach ($fields as $idCorr => $arCorr)
	{
		if ($arCorr['TYPE'] == 'SELECT')
			$arCorr['OPTIONS'] = $arCorr['VALUE'];

		if(is_array($arPaySys['PARAMS']) && array_key_exists($idCorr, $arPaySys['PARAMS'])
			&& is_array($arPaySys['PARAMS'][$idCorr])
			&& array_key_exists('TYPE', $arPaySys['PARAMS'][$idCorr])
			&& array_key_exists('VALUE', $arPaySys['PARAMS'][$idCorr])
			)
		{
			if ($arCorr['TYPE'] == 'FILE')
			{
				$arCorr['VALUE'] = CFile::ShowImage(
					$arPaySys['PARAMS'][$idCorr]['VALUE'],
					150, 150,
					'id="' . $idCorr . '_preview_img"'
				);
			}
			elseif ($arCorr['TYPE'] == 'SELECT' || $arCorr['TYPE'] == 'CHECKBOX')
			{
				$arCorr['VALUE'] = $arPaySys['PARAMS'][$idCorr]['VALUE'];
			}
			else
			{
				$arCorr['TYPE'] = $arPaySys['PARAMS'][$idCorr]['TYPE'];
				$arCorr['VALUE'] = $arPaySys['PARAMS'][$idCorr]['VALUE'];
			}
		}
		$res  = ' ' . CCrmPaySystem::getActionSelector($idCorr, $arCorr);
		$res .= ' '.CCrmPaySystem::getActionValueSelector(
				$idCorr, $arCorr, $ptID, $arResult['ACTION_FILE'], $arResult['USER_FIELDS'],
				$arResult['REQUISITE_FIELDS'], $arResult['BANK_DETAIL_FIELDS'],
				$arResult['CRM_COMPANY_FIELDS'], $arResult['CRM_CONTACT_FIELDS']
			);

		$arResult['FIELD_LIST']['tab_props'][] = $arResult['FIELDS']['tab_props'][] = array(
			'id' => $idCorr,
			'name' => $arCorr['NAME'],
			'title' => $arCorr['DESCR'],
			'type' => 'custom',
			'value' => $res
		);
	}
}
$arResult['FIELD_LIST']['tab_props'][] = $arResult['FIELDS']['tab_props'][] = array(
	'id' => 'SIMPLE_MODE',
	'colspan' => true,
	'type' => 'custom',
	'value' => "<a onclick='BX.crmPaySys.switchMode();'".
				" class='crm-ps-mode-switcher'".
				" href='javascript:void(0);'".
				" id='MODE_SWITCHER'>".
				($arResult['SIMPLE_MODE'] ? GetMessage("CRM_PS_SHOW_FIELDS") : GetMessage("CRM_PS_HIDE_FIELDS")).
				"</a>"
);
$this->IncludeComponentTemplate();
?>