<?
use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Loader;
use Bitrix\Crm\WebForm\Helper;
use Bitrix\Crm\WebForm\Form;
use Bitrix\Crm\WebForm\FieldSynchronizer;
use Bitrix\Crm\WebForm\Entity;
use Bitrix\Crm\WebForm\ResultEntity;
use Bitrix\Crm\WebForm\EntityFieldProvider;
use Bitrix\Crm\Category\DealCategory;

if(!defined('B_PROLOG_INCLUDED') || B_PROLOG_INCLUDED!==true)
	die();

Loc::loadMessages(__FILE__);


class CCrmWebFormEditComponent extends \CBitrixComponent
{
	protected $errors = array();

	/** @var  Form */
	protected $crmWebForm;

	protected function processPostFields($fields)
	{
		$fieldList = array();
		foreach($fields as $fieldId => $field)
		{
			$field['CODE'] = $fieldId;
			$fields[$fieldId] = $field;
		}
		$fields = EntityFieldProvider::getFieldsDescription($fields);

		foreach($fields as $fieldId => $field)
		{
			$fieldTmp = array(
				'CODE' => $fieldId,
				'TYPE' => $field['TYPE'],
				'CAPTION' => $field['CAPTION'],
				'SORT' => (int) $field['SORT'],
				'ITEMS' => array(),
				'VALUE_TYPE' => '',
			);

			$fieldTmp['REQUIRED'] = $field['REQUIRED'] == 'Y' ? 'Y' : 'N';
			$fieldTmp['MULTIPLE'] = $field['MULTIPLE'] == 'Y' ? 'Y' : 'N';

			if($field['TYPE'] == 'section')
			{
				$fieldTmp['REQUIRED'] = 'N';
				$fieldTmp['MULTIPLE'] = 'N';
			}
			elseif($field['TYPE'] == 'product')
			{
				//$fieldTmp['REQUIRED'] = 'Y';
				$fieldTmp['MULTIPLE'] = 'N';
			}
			else
			{
				$fieldTmp['PLACEHOLDER'] = $field['PLACEHOLDER'];
			}

			if(isset($field['VALUE_TYPE']) && isset($field['VALUE_TYPE_ORIGINAL']))
			{
				$isValueTypeExisted = false;
				foreach($field['VALUE_TYPE_ORIGINAL'] as $valueTypeItem)
				{
					if($valueTypeItem['ID'] == $field['VALUE_TYPE'])
					{
						$isValueTypeExisted = true;
						break;
					}
				}

				if($isValueTypeExisted)
				{
					$fieldTmp['VALUE_TYPE'] = $field['VALUE_TYPE'];
				}
			}

			if(is_array($field['ITEMS']))
			{
				foreach($field['ITEMS'] as $itemId => $item)
				{
					$unknownItemKeys = array_diff(
						array_keys($item),
						array('ID', 'VALUE', 'PRICE', 'NAME', 'SELECTED')
					);
					if(count($unknownItemKeys) == 0)
					{
						continue;
					}

					foreach($unknownItemKeys as $unknownItemKey)
					{
						unset($field['ITEMS'][$itemId][$unknownItemKey]);
					}
				}

				$fieldTmp['ITEMS'] = array_values($field['ITEMS']);
			}

			if($field['ID'] > 0)
			{
				$fieldTmp['ID'] = $field['ID'];
			}

			if($field['CAPTION'] == $field['ENTITY_FIELD_CAPTION'])
			{
				$fieldTmp['CAPTION'] = '';
			}

			$fieldList[] = $fieldTmp;
		}

		return $fieldList;
	}

	protected function processPostPresetFields($presetFields)
	{
		if(!is_array($presetFields) || count($presetFields) == 0)
		{
			return array();
		}

		$fields = array();
		$presetFieldCodeList = array_keys($presetFields);
		foreach($this->arResult['AVAILABLE_FIELDS'] as $field)
		{
			if(!in_array($field['name'], $presetFieldCodeList))
			{
				continue;
			}

			$fields[] = array(
				'ENTITY_NAME' => $field['entity_name'],
				'FIELD_NAME' => $field['entity_field_name'],
				'VALUE' => $presetFields[$field['name']]['VALUE'],
			);
		}

		return $fields;
	}

	protected function processPostDependencies($dependencies)
	{
		$dependencyList = array();
		foreach($dependencies as $dependencyId => $dependency)
		{
			$dependencyList[] = array(
				'IF_FIELD_CODE' => $dependency['IF_FIELD_CODE'],
				'IF_ACTION' => \Bitrix\Crm\WebForm\Internals\FieldDependenceTable::ACTION_ENUM_CHANGE,
				'IF_VALUE' => $dependency['IF_VALUE'],
				'DO_FIELD_CODE' => $dependency['DO_FIELD_CODE'],
				'DO_ACTION' => $dependency['DO_ACTION'],
			);
		}

		return $dependencyList;
	}

	public function processPostScripts($scripts)
	{
		return $scripts;
	}

	public function processPostBackgroundImage($fieldName)
	{
		$needDeleteImage = $this->request->get($fieldName . '_del') == 'Y';
		$needUpdateImage = true;

		$fileList = $this->request->getFile($fieldName);
		if(is_array($fileList['tmp_name']))
		{
			$fileListTmp = array();
			foreach($fileList as $fileKey => $fileValue)
			{
				foreach($fileValue as $valueIndex => $value)
				{
					$fileListTmp[$valueIndex][$fileKey] = $value;
				}
			}

			$fileList = $fileListTmp;
		}
		else
		{
			$fileList = array($fileList);
		}

		foreach($fileList as $file)
		{
			$fileId = CFile::SaveFile($file, 'crm/webform');
			if($fileId)
			{
				$isSuccessUpdate = Form::updateBackgroundImage($this->crmWebForm->getId(), $fileId);
				if($isSuccessUpdate)
				{
					$needDeleteImage = true;
					$needUpdateImage = false;
				}
			}
			break;
		}

		if($needDeleteImage)
		{
			$form = $this->crmWebForm->get();
			if($form[$fieldName])
			{

				CFile::Delete($form[$fieldName]);
			}

			if($needUpdateImage)
			{
				Form::updateBackgroundImage($this->crmWebForm->getId(), null);
			}
		}

		return $fileList;
	}

	public function processPostRemoveCopyRight($copyright)
	{
		return ($copyright == 'Y' && Form::canRemoveCopyright()) ? 'Y' : 'N';
	}

	public function processPostInvoiceSettings($requestInvoiceSettings)
	{
		$defaultPayer = ResultEntity::INVOICE_PAYER_CONTACT;
		$payerTypeList = array_keys($this->arResult['INVOICE_PAYER_TYPES']);
		$payerType = $requestInvoiceSettings['PAYER'];
		if(!$payerType || !in_array($payerType, $payerTypeList))
		{
			$payerType = $defaultPayer;
		}

		return array(
			'PAYER' => $payerType
		);
	}

	protected function processPostFormSettings($requestFormSettings)
	{
		if(!$requestFormSettings['DEAL_CATEGORY'] || !DealCategory::exists($requestFormSettings['DEAL_CATEGORY']))
		{
			$requestFormSettings['DEAL_CATEGORY'] = null;
		}

		return $requestFormSettings;
	}

	public function processPost()
	{
		global $USER;
		$request = \Bitrix\Main\Context::getCurrent()->getRequest();

		if($this->crmWebForm->isSystem())
		{
			$this->redirectToList();
		}

		$fields = is_array($request->get('FIELD')) ? $request->get('FIELD') : array();
		$presetFields = is_array($request->get('FIELD_PRESET')) ? $request->get('FIELD_PRESET') : array();
		$dependencies = is_array($request->get('DEPENDENCIES')) ? $request->get('DEPENDENCIES') : array();
		$scripts = is_array($request->get('SCRIPTS')) ? $request->get('SCRIPTS') : array();

		$entityScheme = (string) $request->get('ENTITY_SCHEME');

		$fieldSynchronizer = new FieldSynchronizer;
		$fieldSynchronizer->replacePostFields($entityScheme, $fields, $dependencies);

		$useLicence = $request->get('USE_LICENCE') == 'Y';
		$useCss = $request->get('USE_CSS_TEXT') == 'Y';

		$assignedById = (int) $request->get('ASSIGNED_BY_ID');
		if(!$assignedById)
		{
			$assignedById = $USER->GetID();
		}

		$canUseCallBack = Loader::includeModule('voximplant');

		$params = array(
			//'ACTIVE' => $request->get('ACTIVE') == 'Y' ? 'Y' : 'N',
			'ACTIVE_CHANGE_BY' => $USER->GetID(),
			'NAME' => $request->get('NAME'),
			'CAPTION' => $request->get('CAPTION'),
			'DESCRIPTION' => $request->get('DESCRIPTION'),
			'BUTTON_CAPTION' => $request->get('BUTTON_CAPTION'),
			'ENTITY_SCHEME' => $entityScheme,
			'FIELDS' => $this->processPostFields($fields),
			'SCRIPTS' => $this->processPostScripts($scripts),
			'DEPENDENCIES' => $this->processPostDependencies($dependencies),
			'PRESET_FIELDS' => $this->processPostPresetFields($presetFields),
			'TEMPLATE_ID' => $request->get('TEMPLATE_ID'),
			'IS_PAY' => $request->get('IS_PAY') ? 'Y' : 'N',
			'COPYRIGHT_REMOVED' => $this->processPostRemoveCopyRight($request->get('COPYRIGHT_REMOVED')),
			'DUPLICATE_MODE' => $request->get('DUPLICATE_MODE'),
			'SCRIPT_INCLUDE_SETTINGS' => array(),
			'INVOICE_SETTINGS' => $this->processPostInvoiceSettings($request->get('INVOICE_SETTINGS')),
			'FORM_SETTINGS' => $this->processPostFormSettings(array(
				'DEAL_CATEGORY' => $request->get('DEAL_CATEGORY')
			)),
			'ASSIGNED_BY_ID' => $assignedById,

			'YANDEX_METRIC_ID' => $request->get('YANDEX_METRIC_ID'),
			'GOOGLE_ANALYTICS_ID' => $request->get('GOOGLE_ANALYTICS_ID'),
			'GOOGLE_ANALYTICS_PAGE_VIEW' => $request->get('GOOGLE_ANALYTICS_PAGE_VIEW') == 'Y' ? 'Y' : 'N',

			'CSS_PATH' => $useCss ? $request->get('CSS_PATH') : '',
			'CSS_TEXT' => $useCss ? $request->get('CSS_TEXT') : '',
			'BUTTON_COLOR_FONT' => $request->get('BUTTON_COLOR_FONT'),
			'BUTTON_COLOR_BG' => $request->get('BUTTON_COLOR_BG'),

			'LICENCE_BUTTON_CAPTION' => $useLicence ? $request->get('LICENCE_BUTTON_CAPTION') : '',
			'LICENCE_TEXT' => $useLicence ? $request->get('LICENCE_TEXT') : '',

			'RESULT_SUCCESS_TEXT' => $request->get('USE_RESULT_SUCCESS_TEXT') == 'Y' ? $request->get('RESULT_SUCCESS_TEXT') : '',
			'RESULT_SUCCESS_URL' => $request->get('USE_RESULT_SUCCESS_URL') == 'Y' ? $request->get('RESULT_SUCCESS_URL') : '',
			'RESULT_FAILURE_TEXT' => $request->get('USE_RESULT_FAILURE_TEXT') == 'Y' ? $request->get('RESULT_FAILURE_TEXT') : '',
			'RESULT_FAILURE_URL' => $request->get('USE_RESULT_FAILURE_URL') == 'Y' ? $request->get('RESULT_FAILURE_URL') : '',

			'IS_CALLBACK_FORM' => ($canUseCallBack && $request->get('IS_CALLBACK_FORM') == 'Y') ? 'Y' : 'N',
			'CALL_TEXT' => ($canUseCallBack) ? $request->get('CALL_TEXT') : '',
			'CALL_FROM' => ($canUseCallBack) ? $request->get('CALL_FROM') : '',
		);


		$this->crmWebForm->merge($params);
		$this->crmWebForm->save();

		if(!$this->crmWebForm->hasErrors())
		{
			$this->processPostBackgroundImage('BACKGROUND_IMAGE');
			if($this->request->get('SCRIPTS_SEND_TO_EMAIL') == 'Y' && check_email($this->request->get('SCRIPTS_EMAIL_TO')))
			{
				$this->crmWebForm->sendScriptsEmail($this->request->get('SCRIPTS_EMAIL_TO'));
			}

			$this->redirectToList();
		}
		else
		{
			$this->errors = $this->crmWebForm->getErrors();
			$this->arResult['FORM'] = $this->crmWebForm->get();
		}
	}

	protected function redirectToList()
	{
		$replaceList = array('id' => $this->crmWebForm->getId(), 'form_id' => $this->crmWebForm->getId());
		$url = CComponentEngine::makePathFromTemplate($this->arParams['PATH_TO_WEB_FORM_LIST'], $replaceList);
		LocalRedirect($url);
	}

	protected function setResultCurrency()
	{
		/* Currency */
		$currencyFormatParams = \CCrmCurrency::GetCurrencyFormatParams($this->crmWebForm->getCurrencyId());
		if(!is_array($currencyFormatParams))
		{
			$this->arResult['CURRENCY'] = array(
				'SHORT_NAME' => $this->crmWebForm->getCurrencyId(),
				'FORMAT_STRING' => '# ' . $this->crmWebForm->getCurrencyId(),
				'DEC_POINT' => '.',
				'DECIMALS' => 2,
				'THOUSANDS_SEP' => ' ',
			);
		}
		else
		{
			$this->arResult['CURRENCY'] = array(
				'SHORT_NAME' => str_replace(array('# ', '#'), array('', ''), $currencyFormatParams['FORMAT_STRING']),
				'FORMAT_STRING' => $currencyFormatParams['FORMAT_STRING'],
				'DEC_POINT' => $currencyFormatParams['DEC_POINT'],
				'DECIMALS' => $currencyFormatParams['DECIMALS'],
				'THOUSANDS_SEP' => $currencyFormatParams['THOUSANDS_SEP'],
			);
		}
	}

	protected function setResultFields()
	{
		$this->arResult['FORM']['FIELDS'] = array();
		$fields = EntityFieldProvider::getFieldsDescription($this->crmWebForm->getFields());
		foreach($fields as $field)
		{
			if(!$field['CAPTION'])
			{
				$field['CAPTION'] = $field['ENTITY_FIELD_CAPTION'];
			}

			$this->arResult['FORM']['FIELDS'][] = $field;
		}
	}

	protected function setResultDependencyFields()
	{
		foreach($this->arResult['FORM']['DEPENDENCIES'] as $key => $dep)
		{
			if(!$dep['ID'])
			{
				$dep['ID'] = 'n' . mt_rand(11111, 99999);
			}

			$this->arResult['FORM']['DEPENDENCIES'][$key] = $dep;
		}
	}

	protected function setResultPresetFields()
	{
		$this->arResult['FORM']['PRESET_FIELDS'] = array();

		$presetFields = $this->crmWebForm->getPresetFields();
		foreach($this->arResult['AVAILABLE_FIELDS'] as $field)
		{
			foreach($presetFields as $presetFieldKey => $presetField)
			{
				if($presetField['ENTITY_NAME'] != $field['entity_name'])
				{
					continue;
				}

				if($presetField['FIELD_NAME'] != $field['entity_field_name'])
				{
					continue;
				}

				$presetField['CODE'] = $presetField['ENTITY_NAME'] . "_" . $presetField['FIELD_NAME'];
				$presetField['ENTITY_CAPTION'] = $field['entity_caption'];
				$presetField['ENTITY_FIELD_CAPTION'] = $field['caption'];
				$this->arResult['FORM']['PRESET_FIELDS'][] = $presetField;
				break;
			}
		}

		$this->arResult['FORM']['HAS_PRESET_FIELDS'] = count($this->arResult['FORM']['PRESET_FIELDS']) > 0;
	}

	protected function setResultTemplates()
	{
		$templates = Helper::getTemplateList();
		foreach($templates as $templateId => $templateName)
		{
			if(!$this->arResult['FORM']['TEMPLATE_ID'])
			{
				$this->arResult['FORM']['TEMPLATE_ID'] = $templateId;
			}

			$this->arResult['TEMPLATES'][] = array(
				'ID' => $templateId,
				'CAPTION' => $templateName,
				'SELECTED' => $this->arResult['FORM']['TEMPLATE_ID'] == $templateId,
			);
		}
	}

	protected function setResultDuplicateModes()
	{
		$duplicateModes = ResultEntity::getDuplicateModeList();
		foreach($duplicateModes as $duplicateModeId => $duplicateModeCaption)
		{
			if(!isset($this->arResult['FORM']['DUPLICATE_MODE']))
			{
				$this->arResult['FORM']['DUPLICATE_MODE'] = $duplicateModeId;
			}

			$this->arResult['DUPLICATE_MODES'][] = array(
				'ID' => $duplicateModeId,
				'CAPTION' => $duplicateModeCaption,
				'SELECTED' => $this->arResult['FORM']['DUPLICATE_MODE'] == $duplicateModeId,
			);
		}
	}

	protected function setResultDealCategory()
	{
		if(!is_array($this->arResult['FORM']['FORM_SETTINGS']))
		{
			$this->arResult['FORM']['FORM_SETTINGS'] = array();
		}

		$categoryId = null;
		if($this->arResult['FORM']['FORM_SETTINGS']['DEAL_CATEGORY'])
		{
			$categoryId = $this->arResult['FORM']['FORM_SETTINGS']['DEAL_CATEGORY'];
		}

		$this->arResult['DEAL_CATEGORY_LIST'] = array();
		$categoryList = DealCategory::getAll();
		foreach($categoryList as $category)
		{
			$this->arResult['DEAL_CATEGORY_LIST'][] = array(
				'ID' => $category['ID'],
				'NAME' => $category['NAME'],
				'SELECTED' => $categoryId == $category['ID'],
			);
		}
	}

	protected function setResultCallBackForm()
	{
		$this->arResult['CALL_BACK_FORM'] = array(
			'CAN_USE' => false,
			'CALL_FROM' => array()
		);
		if(!Loader::includeModule('voximplant'))
		{
			return;
		}

		$this->arResult['CALL_BACK_FORM']['CAN_USE'] = true;
		$callFromList = CVoxImplantConfig::GetPortalNumbers(true);
		foreach($callFromList as $phoneCode => $phoneCaption)
		{
			$this->arResult['CALL_BACK_FORM']['CALL_FROM'][] = array(
				'ID' => $phoneCode,
				'VALUE' => $phoneCaption,
				'SELECTED' => $phoneCode == $this->arResult['FORM']['CALL_FROM'],
			);
		}
	}

	protected function setResultEntitySchemes()
	{
		$allowedEntitySchemes = $this->crmWebForm->getAllowedEntitySchemes();
		$this->arResult['ENTITY_SCHEMES'] = Entity::getSchemesByInvoice(
			$this->arResult['FORM']['ENTITY_SCHEME']
			/*,$allowedEntitySchemes*/
		);
		/*
		$this->arResult['ENTITY_SCHEMES'] = Entity::getSchemes();

		$selectedScheme = null;
		foreach($this->arResult['ENTITY_SCHEMES'] as $entitySchemeNameCode => $entityScheme)
		{
			$entityScheme['DISABLED'] = !in_array($entitySchemeNameCode, $allowedEntitySchemes);
			if(!$entityScheme['DISABLED'])
			{
				$entityScheme['SELECTED'] = ($entitySchemeNameCode == $this->arResult['FORM']['ENTITY_SCHEME']);
			}

			$this->arResult['ENTITY_SCHEMES'][$entitySchemeNameCode] = $entityScheme;

			if(!$entityScheme['DISABLED'] && $entityScheme['SELECTED'])
			{
				$selectedScheme = $entityScheme;
			}
		}

		if(!$selectedScheme)
		{
			foreach($this->arResult['ENTITY_SCHEMES'] as $entitySchemeNameCode => $entityScheme)
			{
				if(!$entityScheme['DISABLED'])
				{
					$entityScheme['SELECTED'] = true;
					$this->arResult['ENTITY_SCHEMES'][$entitySchemeNameCode] = $entityScheme;

					$selectedScheme = $entityScheme;
					break;
				}
			}
		}

		$this->arResult['ENTITY_SCHEME_DESCRIPTION'] = $selectedScheme['DESCRIPTION'];
		*/
	}

	public function initInvoicePayerTypes()
	{
		$this->arResult['INVOICE_PAYER_TYPES'] = array(
			ResultEntity::INVOICE_PAYER_CONTACT => array(
				'ID' => ResultEntity::INVOICE_PAYER_CONTACT,
				'SELECTED' => true,
			),
			ResultEntity::INVOICE_PAYER_COMPANY => array(
				'ID' => ResultEntity::INVOICE_PAYER_COMPANY,
				'SELECTED' => false,
			),
		);

		if(!$this->arResult['FORM']['INVOICE_SETTINGS'])
		{
			return;
		}

		if(!$this->arResult['FORM']['INVOICE_SETTINGS']['PAYER'])
		{
			return;
		}

		if(!in_array($this->arResult['FORM']['INVOICE_SETTINGS']['PAYER'], array_keys($this->arResult['INVOICE_PAYER_TYPES'])))
		{
			return;
		}

		foreach($this->arResult['INVOICE_PAYER_TYPES'] as $payerTypeId => $payerType)
		{
			$isSelected = $payerTypeId == $this->arResult['FORM']['INVOICE_SETTINGS']['PAYER'];
			$this->arResult['INVOICE_PAYER_TYPES'][$payerTypeId]['SELECTED'] = $isSelected;
		}
	}

	public function setResultAssignedBy()
	{
		$userId = (int) $this->arResult['FORM']['ASSIGNED_BY_ID'];
		$this->arResult['FORM']['ASSIGNED_BY_ID'] = '';
		$this->arResult['FORM']['ASSIGNED_BY_NAME'] = '';

		if(!$userId)
		{
			/*@global $USER \CUser*/
			global $USER;
			$userId = $USER->GetID();
		}
		$userFields = \Bitrix\Main\UserTable::getRowById($userId);
		if(!$userFields)
		{
			return;
		}

		$this->arResult['FORM']['ASSIGNED_BY_ID'] = $userId;

		// format name
		$this->arResult['FORM']['ASSIGNED_BY_NAME'] = CUser::FormatName(
			$this->arParams['NAME_TEMPLATE'],
			array(
				'LOGIN' => $userFields['LOGIN'],
				'NAME' => $userFields['NAME'],
				'LAST_NAME' => $userFields['LAST_NAME'],
				'SECOND_NAME' => $userFields['SECOND_NAME']
			),
			true, false
		);
	}

	public function prepareResult()
	{
		/**@var $USER \CUser*/
		global $USER;
		$CrmPerms = new CCrmPerms($USER->GetID());

		if($CrmPerms->HavePerm('WEBFORM', BX_CRM_PERM_NONE))
		{
			ShowError(Loc::getMessage('CRM_PERMISSION_DENIED'));
			return;
		}
		$this->arResult['PERM_CAN_EDIT'] = !$CrmPerms->HavePerm('WEBFORM', BX_CRM_PERM_NONE, 'WRITE');

		$id = $this->arParams['ELEMENT_ID'];
		$this->arResult['AVAILABLE_FIELDS'] = EntityFieldProvider::getFields();
		$this->arResult['AVAILABLE_FIELDS_TREE'] = EntityFieldProvider::getFieldsTree();
		$this->arResult['PRESET_AVAILABLE_FIELDS_TREE'] = EntityFieldProvider::getPresetFieldsTree();
		$this->arResult['AVAILABLE_ENTITIES'] = Entity::getList();

		$this->crmWebForm = new Form($id);

		/* Set form data */
		$this->arResult['FORM'] = $this->crmWebForm->get();
		$this->arResult['FORM']['BACKGROUND_IMAGE_PATH'] = \CFile::GetPath($this->arResult['FORM']['BACKGROUND_IMAGE']);

		/* Set invoice payer types */
		$this->initInvoicePayerTypes();

		$request = \Bitrix\Main\Context::getCurrent()->getRequest();
		if($request->getRequestMethod() == "POST" && check_bitrix_sessid())
		{
			if(!$this->arResult['PERM_CAN_EDIT'])
			{
				ShowError(Loc::getMessage('CRM_PERMISSION_DENIED'));
				return;
			}
			else
			{
				$this->processPost();
				$this->showErrors();
			}

		}

		/* Set templates */
		$this->setResultTemplates();
		$this->setResultDuplicateModes();

		/* Set fields */
		$this->setResultFields();

		/* Set dependencies */
		$this->setResultDependencyFields();

		/* Set preset fields */
		$this->setResultPresetFields();

		/* Set allowed entity schemes */
		$this->setResultEntitySchemes();

		/* Set assigned user */
		$this->setResultAssignedBy();

		/* Set currency */
		$this->setResultCurrency();

		/* Set deal category */
		$this->setResultDealCategory();


		/* Set call back form */
		$this->setResultCallBackForm();

		/* Copyright */
		$this->arResult['CAN_REMOVE_COPYRIGHT'] = $this->crmWebForm->canRemoveCopyright();

		/* External analytics data */
		$this->arResult['EXTERNAL_ANALYTICS_DATA'] = $this->crmWebForm->getExternalAnalyticsData();

		$replaceList = array('id' => $id, 'form_id' => $id);
		$this->arResult['PATH_TO_WEB_FORM_LIST'] = CComponentEngine::makePathFromTemplate($this->arParams['PATH_TO_WEB_FORM_LIST'], $replaceList);
	}

	public function checkParams()
	{
		$this->arParams['NAME_TEMPLATE'] = empty($this->arParams['NAME_TEMPLATE']) ? CSite::GetNameFormat(false) : str_replace(array("#NOBR#","#/NOBR#"), array("",""), $this->arParams["NAME_TEMPLATE"]);

		return true;
	}

	public function executeComponent()
	{
		if (!$this->checkModules())
		{
			$this->showErrors();
			return;
		}

		if (!$this->checkParams())
		{
			$this->showErrors();
			return;
		}

		global $APPLICATION;
		$APPLICATION->SetTitle(Loc::getMessage('CRM_WEBFORM_EDIT_TITLE'));
		$this->prepareResult();

		$this->includeComponentTemplate();
	}

	protected function checkModules()
	{
		if(!Loader::includeModule('crm'))
		{
			$this->errors[] = Loc::getMessage('CRM_MODULE_NOT_INSTALLED');
			return false;
		}

		if(!CAllCrmInvoice::installExternalEntities())
		{
			return false;
		}

		if(!CCrmQuote::LocalComponentCausedUpdater())
		{
			return false;
		}

		if(!Loader::includeModule('currency'))
		{
			$this->errors[] = Loc::getMessage('CRM_MODULE_NOT_INSTALLED_CURRENCY');
			return false;
		}

		if(!Loader::includeModule('catalog'))
		{
			$this->errors[] = Loc::getMessage('CRM_MODULE_NOT_INSTALLED_CATALOG');
			return false;
		}

		if(!Loader::includeModule('sale'))
		{
			$this->errors[] = Loc::getMessage('CRM_MODULE_NOT_INSTALLED_CATALOG');
			return false;
		}

		return true;
	}

	protected function hasErrors()
	{
		return (count($this->errors) > 0);
	}

	protected function showErrors()
	{
		if(count($this->errors) <= 0)
		{
			return;
		}

		foreach($this->errors as $error)
		{
			ShowError($error);
		}
	}
}