var CrmFormEditor = function(params)
{

	this.init = function (params)
	{
		this.jsEventsManagerId = params.jsEventsManagerId;
		this.detailPageUrlTemplate = params.detailPageUrlTemplate;
		this.id = params.id;

		this.userBlockController = new CrmFormEditorUserBlockController(params.userBlocks);
		this.externalAnalytics = new CrmFormEditorExternalAnalytics();

		params.fields = params.fields || [];
		this.context = params.context;
		this.templates = params.templates;
		this.mess = params.mess;
		this.currency = params.currency;
		this.actionRequestUrl = params.actionRequestUrl;
		this.canRemoveCopyright = params.canRemoveCopyright;

		this.entityDictionary = params.entityDictionary;
		this.schemesDictionary = params.schemesDictionary;
		this.fieldsDictionary = params.fieldsDictionary;
		this.fields = [];

		/* init helper */
		this.helper = new CrmFormEditorHelper();

		/* init drag&drop fields */
		this.dragdrop = new CrmFormEditorDragDrop(params.dragdrop);
		BX.addCustomEvent(this.dragdrop, 'onSort', BX.delegate(this.onSort, this));

		/* init existed fields */
		this.initExistedFields(params);

		/* init filter of field selector */
		this.popupFieldSettings = new CrmWebFormEditPopupFieldSettings({
			caller: this,
			content: BX('CRM_WEB_FORM_POPUP_SETTINGS'),
			container: BX('CRM_WEB_FORM_POPUP_SETTINGS_CONTAINER')
		});


		this.responsible = new CrmFormEditorResponsible({
			caller: this,
			context: BX('CRM_WEBFORM_RESPONSIBLE')
		});
		/* init filter of field selector */
		this.fieldSelector = new CrmFormEditorFieldSelector({
			caller: this,
			context: BX('FIELD_SELECTOR')
		});

		/* init form submit button editing */
		this.formButton = new CrmWebFormEditFormButton({
			caller: this,
			context: BX('FORM_BUTTON_CONTAINER')
		});

		/* init entity scheme */
		this.entityScheme = new CrmWebFormEditEntityScheme({
			caller: this,
			context: BX('ENTITY_SCHEME_CONTAINER')
		});

		/* init dependencies */
		this.dependencies = new CrmFormEditorDependencies({
			caller: this,
			dependencies: params.dependencies
		});

		/* init preset fields */
		this.presetFields = new CrmFormEditorFieldPreset({
			caller: this,
			fields: params.presetFields
		});

		/* first fields sorting */
		this.sortFields();

		/* init animation */
		this.initAnimation();

		/* init interface  */
		this.initInterface();

		var backgroundImages = document.getElementsByName('BACKGROUND_IMAGE');
		BX.bind(backgroundImages[0], 'bxchange', function(){
			var parts = [];
			parts = this.value.replace(/\\/g, '/').split( '/' );
			BX('BACKGROUND_IMAGE_TEXT').innerText = parts[parts.length-1];
			BX('BACKGROUND_IMAGE_TEXT').style.display = '';
		});
	};

	this.initExistedFields = function(params)
	{
		if(!params.fields)
		{
			return;
		}

		params.fields.forEach(function(fieldParams)
		{
			var field = {
				node: BX(fieldParams.CODE),
				name: fieldParams.CODE,
				type: fieldParams.TYPE,
				items: fieldParams.ITEMS || [],
				entity: fieldParams.ENTITY_NAME || '',
				'getCaption': this.onGetFieldCaption
			};
			this.initField(field);
			this.initFieldSettings(field);

		}, this);
	};

	this.initInterface = function()
	{
		// init caption live-edititng
		this.bindEditInline(BX('CAPTION_CONTAINER'), 'dynamic-field');
		// init licence button text live-edititng
		this.bindEditInline(BX('LICENCE_CONTAINER'), 'dynamic-field');

		//checkboxes
		BX.bind(BX('USE_LICENCE'), 'click', function(){
			BX.toggleClass(BX('LICENCE_CONTAINER'), 'crm-webform-display-none');
		});
		BX.bind(BX('USE_CSS_TEXT'), 'click', function(){
			BX.toggleClass(BX('CSS_TEXT_CONTAINER'), 'crm-webform-display-none');
		});
		BX.bind(BX('USE_RESULT_SUCCESS_TEXT'), 'click', function(){
			BX.toggleClass(BX('RESULT_SUCCESS_TEXT_CONT'), 'crm-webform-display-none');
		});
		BX.bind(BX('USE_RESULT_SUCCESS_URL'), 'click', function(){
			BX.toggleClass(BX('RESULT_SUCCESS_URL_CONT'), 'crm-webform-display-none');
		});
		BX.bind(BX('USE_RESULT_FAILURE_TEXT'), 'click', function(){
			BX.toggleClass(BX('RESULT_FAILURE_TEXT_CONT'), 'crm-webform-display-none');
		});
		BX.bind(BX('USE_RESULT_FAILURE_URL'), 'click', function(){
			BX.toggleClass(BX('RESULT_FAILURE_URL_CONT'), 'crm-webform-display-none');
		});

		BX.bind(BX('IS_PAY'), 'click', function(){
			BX.toggleClass(BX('PAY_SYSTEM_CONT'), 'crm-webform-display-none');
		});

		BX.bind(BX('IS_CALLBACK_FORM'), 'click', function(){
			BX.toggleClass(BX('CALLBACK_CONTAINER'), 'crm-webform-display-none');
		});

		BX.bind(BX('DESCRIPTION_EDITOR_BUTTON'), 'click', function(){
			BX.toggleClass(BX('DESCRIPTION_EDITOR_BUTTON'), 'crm-webform-display-none');
			BX.toggleClass(BX('DESCRIPTION_EDITOR_CONTAINER'), 'crm-webform-edit-animate-show');
			var htmlEditor = BXHtmlEditor.Get('DESCRIPTION');
			if(htmlEditor)
			{
				htmlEditor.Focus();
			}
		});

		if(!this.canRemoveCopyright)
		{
			BX.bind(BX('COPYRIGHT_REMOVED_CONT'), 'click', BX.proxy(function(e){
				if(!B24 || !B24['licenseInfoPopup'])
				{
					return;
				}

				BX.PreventDefault(e);
				B24.licenseInfoPopup.show(
					'crm_webform_copyright',
					this.mess.dlgRemoveCopyrightTitle,
					'<span>' + this.mess.dlgRemoveCopyrightText + '</span>'
				);
			}, this));
		}

		var copyButton = BX('CRM_WEBFORM_COPY_BUTTON');
		if(copyButton)
		{
			BX.bind(copyButton, 'click', BX.proxy(function(e){

				BX.ajax({
					url: this.actionRequestUrl,
					method: 'POST',
					data: {
						'action': 'copy',
						'form_id': this.id,
						'sessid': BX.bitrix_sessid()
					},
					timeout: 30,
					dataType: 'json',
					processData: true,
					onsuccess: BX.proxy(function(data){
						data = data || {};
						if(data.error)
						{
							//error
						}
						else
						{
							window.location = this.detailPageUrlTemplate.replace('#id#', data.copiedId).replace('#form_id#', data.copiedId);
						}
					}, this),
					onfailure: BX.proxy(function(){
						//error
					}, this)
				});

			}, this));
		}
	};

	this.submitForm = function()
	{
		BX('crm_webform_edit_form').submit();
	};

	this.initAnimation = function()
	{
		var elementList = document.getElementsByClassName('.crm-webform-edit-animate');
		elementList = BX.convert.nodeListToArray(elementList);
		elementList.forEach(function(element){
			//get element height and set as max-height
		}, this);
	};

	this.bindEditInline = function(node, className)
	{
		var nameContainerNode = node.querySelector('[data-bx-web-form-lbl-cont]');
		var captionNode = node.querySelector('[data-bx-web-form-lbl-caption]');
		var inputNode = node.querySelector('[data-bx-web-form-btn-caption]');
		var buttonEdit = node.querySelector('[data-bx-web-form-lbl-btn-edit]');
		var buttonApply = node.querySelector('[data-bx-web-form-lbl-btn-apply]');
		BX.bind(buttonEdit, 'click', function(){
			BX.addClass(nameContainerNode, className);
		});
		BX.bind(buttonApply, 'click', function(){
			captionNode.innerText = inputNode.value;
			BX.removeClass(nameContainerNode, className);
		});
	};

	this.sendActionRequest = function (action, requestData, callbackSuccess, callbackFailure)
	{
		callbackSuccess = callbackSuccess || null;
		callbackFailure = callbackFailure || null;
		requestData = requestData || {};

		requestData.action = action;
		requestData.form_id = null;
		requestData.sessid = BX.bitrix_sessid();

		BX.ajax({
			url: this.actionRequestUrl,
			method: 'POST',
			data: requestData,
			timeout: 30,
			dataType: 'json',
			processData: true,
			onsuccess: BX.proxy(function(data){
				data = data || {};
				if(data.error)
				{
					callbackFailure.apply(this, [data]);
				}
				else if(callbackSuccess)
				{
					callbackSuccess.apply(this, [data]);
				}
			}, this),
			onfailure: BX.proxy(function(){
				var data = {'error': true, 'text': ''};
				callbackFailure.apply(this, [data]);
			}, this)
		});
	};

	this.onGetFieldCaption = function()
	{
		var inputs = document.getElementsByName('FIELD['+this.name+'][CAPTION]');
		return (inputs && inputs[0]) ? inputs[0].value : '';
	};

	this.getFieldNameList = function()
	{
		var fieldNameList = [];
		this.fields.forEach(function(field){
			fieldNameList.push(field.name);
		});

		return fieldNameList;
	};

	this.fireChangeFieldListEvent = function ()
	{
		BX.onCustomEvent(this, 'change-field-list', [this.fields]);
	};

	this.fireFieldChangeItemsEvent = function (field)
	{
		BX.onCustomEvent(field, 'change-field-items');
		BX.onCustomEvent(this, 'change-field-items', [this.fields, field]);
	};

	this.addFieldByCode = function (fieldCode)
	{
		var dataList = this.fieldsDictionary.filter(function(item){
				return item.name == fieldCode;
		});

		if(dataList[0])
		{
			this.addField(dataList[0]);
		}
	};


	this.addField = function (params, doNotSort)
	{
		if(!params.duplicated && this.findField(params.name))
		{
			return;
		}

		var templateId = this.templates.field.replace('%type%', params.type);
		if(!this.helper.getTemplate(templateId))
		{
			templateId = this.templates.field.replace('%type%', 'string');
		}

		var newFieldStringType = null;
		var newFieldPlaceholder = null;
		switch (params.entity_field_name)
		{
			case 'PHONE':
				newFieldStringType = 'phone';
				newFieldPlaceholder = '111-11-11';
				break;
			case 'WEB':
				newFieldPlaceholder = 'http://';
				break;
			case 'EMAIL':
				newFieldStringType = 'email';
				newFieldPlaceholder = 'my@example.com';
				break;
		}
		var fieldNode = this.helper.appendNodeByTemplate(
			BX('FIELD_CONTAINER'),
			templateId,
			{
				'id': params.id || '',
				'type': newFieldStringType || params.type,
				'name': params.name,
				'caption': params.caption,
				'placeholder': newFieldPlaceholder || '',
				'multiple': 'N',
				'required': 'N',
				'sort': 1000,
				'value_type': '',
				'items': '',
				'settings_items': '',
				'entity_field_caption': params.caption,
				'entity_name': params.entity_name,
				'entity_caption': params.entity_caption
			}
		);

		if(!fieldNode)
		{
			return;
		}

		var field = {
			type: params.type,
			node: fieldNode,
			name: params.name,
			items: params.items || [],
			entity: params.entity_name,
			'getCaption': this.onGetFieldCaption
		};

		this.initField(field);
		this.addFieldItems(field);
		this.addFieldSettingsItems(field);
		this.initFieldSettings(field);

		if(!doNotSort)
		{
			this.sortFields();
			this.fireChangeFieldListEvent();
		}

		return field;
	};

	this.addFieldItems = function(field, clean)
	{
		clean = clean || false;
		if(field.items.length == 0 || !field.itemsContainer)
		{
			return;
		}

		if(clean)
		{
			field.itemsContainer.innerHTML = '';
		}

		var templateId = this.templates.field.replace('%type%', field.type);
		var itemTemplateId = templateId + '_item';

		field.items.forEach(function(item){

			var fieldItemNode = this.helper.appendNodeByTemplate(
				field.itemsContainer,
				itemTemplateId,
				{
					'item_id': item.ID,
					'item_value': item.VALUE,
					'field_item_name': field.type + '_' + field.name + '_' + field.randomId + '[]',
					'field_item_id': field.type + '_' + item.ID + this.helper.generateId()
				}
			);

		}, this);
	};

	this.initField = function(field)
	{
		if(!field.node)
		{
			return;
		}

		var dictField = this.findDictionaryField(field.name);
		if(!dictField)
		{
			dictField = {
				code: field.name,
				type: field.type,
				required: false,
				multiple: false
			};
		}
		field.dict = dictField;

		field.randomId = this.helper.generateId();

		var nameInput = field.node.querySelector('[data-bx-web-form-btn-caption]');
		var editButton = field.node.querySelector('[data-bx-web-form-btn-edit]');
		var deleteButton = field.node.querySelector('[data-bx-web-form-btn-delete]');
		field.lblCaption = field.node.querySelector('[data-bx-web-form-lbl-caption]');

		field.itemsContainer = field.node.querySelector('[data-bx-web-form-field-display-cont]');
		field.settingsContainer = field.node.querySelector('[data-bx-web-form-field-settings-cont]');

		var _this = this;
		BX.bind(nameInput, 'change', function(){
			field.lblCaption.innerText = nameInput.value;
			_this.fireChangeFieldListEvent();
		});
		BX.bind(editButton, 'click', function(){
			_this.editField(field);
		});
		BX.bind(deleteButton, 'click', function(){
			_this.deleteField(field);
		});

		this.dragdrop.addItem(field.node);
		this.fields.push(field);

		BX.addCustomEvent(field, 'change-field-items', BX.proxy(function(){
			var firstChild = null;
			if(field.type == 'product')
			{
				firstChild = field.itemsContainer.children[0];
			}
			field.itemsContainer.innerHTML = '';
			if(firstChild)
			{
				field.itemsContainer.appendChild(firstChild);
			}
			this.addFieldItems(field);
		}, this));
	};

	this.initFieldSettings = function(field)
	{
		if(!field.node)
		{
			return;
		}

		var typeNode = field.node.querySelector('[data-bx-web-form-field-type]');
		var multipleValueNode = field.node.querySelector('[data-bx-web-form-btn-multiple-value]');
		var multipleCheckboxNode = field.node.querySelector('[data-bx-web-form-btn-multiple]');
		var multipleNode = field.node.querySelector('[data-bx-web-form-btn-multiple-cont]');
		var multipleAddNode = field.node.querySelector('[data-bx-web-form-btn-add]');
		var requiredValueNode = field.node.querySelector('[data-bx-web-form-btn-required-value]');
		var requiredCheckboxNode = field.node.querySelector('[data-bx-web-form-btn-required]');
		if(multipleNode && multipleValueNode && multipleCheckboxNode)
		{
			if(!field.dict.multiple)
			{
				this.helper.styleDisplay(multipleNode, false);
				multipleValueNode.value = 'N';
			}
			else
			{
				multipleCheckboxNode.checked = multipleValueNode.value == 'Y';

				if(field.dict.type == 'checkbox')
				{
					var clickHandler = function(){
						multipleValueNode.value = multipleCheckboxNode.checked ? 'Y' : 'N';
						typeNode.value = field.type = multipleCheckboxNode.checked ? 'checkbox' : 'radio';
						this.addFieldItems(field, true);
					};
					BX.bind(multipleCheckboxNode, 'click', BX.proxy(clickHandler, this));
					clickHandler.apply(this, []);
				}
				else
				{
					this.helper.styleDisplay(multipleAddNode, multipleCheckboxNode.checked, 'inline-block');
					BX.bind(multipleCheckboxNode, 'click', BX.proxy(function(){
						multipleValueNode.value = multipleCheckboxNode.checked ? 'Y' : 'N';
						this.helper.styleDisplay(multipleAddNode, multipleCheckboxNode.checked, 'inline-block');
					}, this));
				}
			}
		}
		if(requiredValueNode && requiredCheckboxNode)
		{
			if(requiredValueNode.value == 'Y')
			{
				requiredCheckboxNode.checked = true;
			}
			BX.bind(requiredCheckboxNode, 'change', function(){
				requiredValueNode.value = requiredCheckboxNode.checked ? 'Y' : 'N';
			});
		}

		var fieldType = field.dict['type'] ? field.dict.type : field.type;
		var fieldInitMethod = 'initFieldType' + fieldType.substring(0,1).toUpperCase() + fieldType.substring(1);
		if(this[fieldInitMethod])
		{
			this[fieldInitMethod](field);
		}
	};

	this.addFieldSettingsItems = function(field)
	{
		var settingsItemsContainer = field.node.querySelector('[data-bx-crm-webform-field-settings-items]');
		if(!settingsItemsContainer || field.items.length == 0 || !field.itemsContainer)
		{
			return;
		}


		 var templateId = this.templates.field.replace('%type%', field.type);
		 var itemSettingsTemplateId = templateId + '_settings_item';

		 field.items.forEach(function(item){

			 var fieldItemNode = this.helper.appendNodeByTemplate(
				 settingsItemsContainer,
				 itemSettingsTemplateId,
				 {
					 'name': field.name,
					 'item_id': item.ID,
					 'item_value': item.VALUE,
					 'field_item_name': field.type + '_' + field.name + '_' + field.randomId + '[]',
					 'field_item_id': field.type + '_' + item.ID + this.helper.generateId()
				 }
			 );

		 }, this);
	};

	this.initFieldTypeProduct = function(field)
	{
		field.productSelector = new CrmFormEditorProductSelector({
			jsEventsManagerId: this.jsEventsManagerId,
			context: field.node,
			caller: this,
			id: field.id,
			field: field
		});
	};

	this.initFieldTypeCheckbox = function(field)
	{
		this.initFieldTypeList(field);
	};

	this.initFieldTypeRadio = function(field)
	{
		this.initFieldTypeList(field);
	};

	this.initFieldTypeList = function(field)
	{
		field.fieldListManager = new CrmFormEditorFieldListSettings({
			caller: this,
			context: field.node,
			type: field.type,
			field: field
		});
	};

	this.initFieldTypeSection = function(field)
	{
		this.bindEditInline(field.node, 'dynamic-field');
	};

	this.initFieldTypeString = function(field)
	{
		var stringTypeNode = field.node.querySelector('[data-bx-web-form-field-string-type]');
		var typeNode = field.node.querySelector('[data-bx-web-form-field-type]');
		if(stringTypeNode)
		{
			stringTypeNode.value = typeNode.value;
			BX.bind(stringTypeNode, 'bxchange', function(){
				typeNode.value = stringTypeNode.value || 'string';
			});
		}
	};

	this.initFieldTypeTyped_string = function(field)
	{
		this.initFieldTypeString(field);

		if(!field.dict['value_type'])
		{
			return;
		}

		var valueTypesNode = field.node.querySelector('[data-bx-web-form-field-string-value-types]');
		var valueTypeNode = field.node.querySelector('[data-bx-web-form-field-string-value-type]');
		if(valueTypesNode)
		{
			var currentItemId = valueTypeNode.value;
			var items = [];
			field.dict.value_type.forEach(function(item){
				if(!currentItemId)
				{
					currentItemId = item.ID;
				}
				items.push({
					caption: item.VALUE,
					value: item.ID,
					selected: (item.ID == currentItemId)
				});
			}, this);
			this.helper.fillDropDownControl(valueTypesNode, items);
			valueTypeNode.value = currentItemId;

			BX.bind(valueTypesNode, 'bxchange', function(){
				valueTypeNode.value = valueTypesNode.value || 'OTHER';
			});
		}
	};

	this.findDictionaryField = function(fieldName)
	{
		var fieldsList = this.fieldsDictionary.filter(function(field){
			return (field.name == fieldName);
		});

		if(fieldsList && fieldsList.length > 0)
		{
			return fieldsList[0];
		}

		return null;
	};

	this.findField = function(fieldName)
	{
		var fieldsList = this.fields.filter(function(field){
			return (field.name == fieldName);
		});

		if(fieldsList && fieldsList.length > 0)
		{
			return fieldsList[0];
		}

		return null;
	};

	this.findFieldByNode = function(node)
	{
		var fieldsList = this.fields.filter(function(field){
			return (field.node == node);
		});

		if(fieldsList && fieldsList.length > 0)
		{
			return fieldsList[0];
		}

		return null;
	};

	this.editField = function(field, startEdit)
	{
		this.popupFieldSettings.show(field.settingsContainer);
	};

	this.deleteField = function(field)
	{
		BX.onCustomEvent(this, 'delete-field', [field]);

		this.dragdrop.removeItem(field.node);
		BX.remove(field.node);

		var itemIndex = BX.util.array_search(field, this.fields);
		if(itemIndex > -1)
		{
			delete this.fields[itemIndex];
		}

		this.sortFields();
		this.fireChangeFieldListEvent();
	};

	this.moveField = function(field, insertAfterField)
	{
		this.sortFields();
	};

	this.sortFields = function ()
	{
		this.fields.forEach(function(field){
			field.sortValue = BX.util.array_search(field.node, field.node.parentNode.children);
		});

		this.fields.sort(function(fieldA, fieldB){
			return fieldA.sortValue > fieldB.sortValue ? 1 : -1;
		});

		var weight = 10;
		this.fields.forEach(function(field){
			var sortInput = field.node.querySelector('[data-bx-web-form-field-sort]');
			if(sortInput)
			{
				sortInput.value = (field.sortValue + 1) * weight;
			}
		});
	};

	this.onSort = function(dragElement, catcherObj)
	{
		//catcherObj.appendChild(dragElement);
		this.sortFields();
	};

	this.init(params);
};


function CrmWebFormEditEntityScheme(params)
{
	this.caller = params.caller;
	this.context = params.context;
	this.submitButtonEnabled = true;

	this.init();
}
CrmWebFormEditEntityScheme.prototype =
{
	init: function ()
	{
		this.helper = BX.CrmFormEditorHelper;
		this.descriptionNode = this.context.querySelector('[data-bx-webform-edit-scheme-desc]');
		this.descriptionTopNode = BX('ENTITY_SCHEMES_TOP_DESCRIPTION');
		this.radioList = BX.convert.nodeListToArray(this.context.querySelectorAll('[data-bx-web-form-entity-scheme-value]'));
		this.invoiceCheckNode = this.context.querySelector('[data-bx-web-form-entity-scheme-invoice]');
		this.dealCategoryNode = this.context.querySelector('[data-bx-web-form-entity-scheme-deal-cat]');

		var valueNodes = document.getElementsByName('ENTITY_SCHEME');
		this.valueNode = valueNodes[0];

		BX.bind(this.invoiceCheckNode, 'change', BX.proxy(this.onChange, this));
		this.radioList.forEach(function(schemeNode){
			BX.bind(schemeNode, 'change', BX.proxy(this.onChange, this));
		}, this);

		this.submitButtonNode = BX('CRM_WEBFORM_SUBMIT_BUTTON');
		BX.bind(this.submitButtonNode, 'click', BX.proxy(this.checkCreateFields, this));
		this.initInvoiceBlock();
	},

	initInvoiceBlock: function ()
	{
		this.invoice = {
			node: this.context.querySelector('[data-bx-crm-webform-invoice]'),
			payerNode: this.context.querySelector('[data-bx-crm-webform-invoice-payer]'),
			payerTextNoNode: this.context.querySelector('[data-bx-crm-webform-invoice-payer-text-no]'),
			productNode: this.context.querySelector('[data-bx-crm-webform-invoice-product]'),
			productButton: this.context.querySelector('[data-bx-crm-webform-invoice-product-btn]'),
			productDraw: this.context.querySelector('[data-bx-crm-webform-invoice-product-draw]'),
			productField: null,
			payerTypeNodes: BX.convert.nodeListToArray(document.getElementsByName('INVOICE_SETTINGS[PAYER]')),
			hasFieldsContact: false,
			hasFieldsCompany: false
		};

		BX.bind(this.invoice.productButton, 'click', BX.proxy(function(){
			this.actualizeInvoiceProductField(true);
			if(this.invoice.productField)
			{
				this.caller.editField(this.invoice.productField);
			}
		}, this));

		BX.addCustomEvent(this.caller, 'change-field-list', BX.proxy(this.actualizeInvoiceSettings, this));
		this.invoice.payerTypeNodes.forEach(function(payerTypeNode){
			BX.bind(payerTypeNode, 'change', BX.proxy(this.onChange, this));
		}, this);


		this.actualizeInvoiceSettings();
	},

	isInvoiceChecked: function ()
	{
		return this.invoiceCheckNode.checked;
	},

	showPopupCreateProductField: function ()
	{
		if(!this.popupCreateProductField)
		{
			this.popupCreateProductField = BX.PopupWindowManager.create(
				'crm_webform_edit_create_prod_field',
				null,
				{
					content: this.caller.mess.dlgInvoiceEmptyProduct,
					titleBar: this.caller.mess.dlgInvoiceEmptyProductTitle,
					autoHide: false,
					lightShadow: true,
					closeByEsc: true,
					overlay: {backgroundColor: 'black', opacity: 500}
				}
			);
			this.popupCreateProductField.setButtons([
				new BX.PopupWindowButton({
					text: this.caller.mess.dlgContinue,
					className: 'webform-small-button-accept',
					events: {
						click: BX.proxy(function(){
							this.popupCreateProductField.close();
						}, this)
					}
				})
			]);
		}

		this.popupCreateProductField.show();
		this.popupCreateProductField.resizeOverlay();
	},

	showPopupCreateFields: function ()
	{
		if(!this.popupConfirmCreateFields)
		{
			this.popupConfirmCreateFields = BX.PopupWindowManager.create(
				'crm_webform_edit_create_fields',
				null,
				{
					content: BX('CRM_WEB_FORM_POPUP_CONFIRM_FIELD_CREATE'),
					autoHide: false,
					lightShadow: true,
					closeByEsc: true,
					overlay: {backgroundColor: 'black', opacity: 500},
					onPopupClose: BX.proxy(this.onClose)
				}
			);
			this.popupConfirmCreateFields.setButtons([
				new BX.PopupWindowButton({
					text: this.caller.mess.dlgContinue,
					className: 'webform-small-button-accept',
					events: {
						click: BX.proxy(function(){
							this.caller.submitForm();
						}, this)
					}
				}),
				new BX.PopupWindowButton({
					text: this.caller.mess.dlgCancel,
					className: 'webform-small-button-cancel',
					events: {
						click: BX.proxy(function(){
							this.submitButtonEnabled = true;
							BX.removeClass(this.submitButtonNode, 'crm-webform-submit-button-loader');
							this.popupConfirmCreateFields.close();
						}, this)
					}
				})
			]);
		}

		this.popupConfirmCreateFields.show();
	},

	checkCreateFields: function (e)
	{
		if(!this.submitButtonEnabled)
		{
			BX.PreventDefault(e);
			return false;
		}

		var fieldCaptionList = [];
		var currentScheme = this.getCurrent();

		var hasProductFieldNotEmpty = false;
		var fieldCodes = [];
		var fieldCodeToCaption = {};
		this.caller.fields.forEach(function(field){
			if(field.type == 'product' && field.items.length > 0)
			{
				hasProductFieldNotEmpty = true;
			}

			if(!field.dict['caption'])
			{
				return;
			}
			if(!field.dict['entity_name'])
			{
				return;
			}
			fieldCodes.push(field.name);
			fieldCodeToCaption[field.name] = field.dict.caption;
			if(BX.util.in_array(field.dict.entity_name, currentScheme.ENTITIES))
			{
				return;
			}
			fieldCaptionList.push(
				'<li class="crm-p-s-f-item">' + BX.util.htmlspecialchars(field.dict.caption) + '</li>'
			);
		}, this);


		if(this.isInvoiceChecked() && !hasProductFieldNotEmpty)
		{
			BX.PreventDefault(e);
			this.showPopupCreateProductField();
			return false;
		}


		if(fieldCaptionList.length == 0)
		{
			return true;
		}

		BX.PreventDefault(e);
		this.submitButtonEnabled = false;
		BX.addClass(this.submitButtonNode, 'crm-webform-submit-button-loader');

		var popupEntityCaptionCont = BX('CRM_WEB_FORM_POPUP_CONFIRM_FIELD_CREATE_ENTITY');
		if(currentScheme.ID == 2 || currentScheme.ID == 5)
		{
			if(this.invoice.hasFieldsContact)
			{
				popupEntityCaptionCont.innerText = this.caller.entityDictionary['CONTACT'];
			}
			else
			{
				popupEntityCaptionCont.innerText = this.caller.entityDictionary['COMPANY'];
			}
		}
		else
		{
			popupEntityCaptionCont.innerText = currentScheme.NAME;
		}

		var popupFieldListCont = BX('CRM_WEB_FORM_POPUP_CONFIRM_FIELD_CREATE_LIST');
		this.caller.sendActionRequest(
			'get_create_fields',
			{
				'schemeId': currentScheme.ID,
				'fieldCodes': fieldCodes
			},
			BX.proxy(function(data){
				data = data || {};

				var fieldSyncCaptionList = [];
				var syncFieldCodes = data['syncFieldCodes'] ? data.syncFieldCodes : [];

				if(syncFieldCodes.length == 0)
				{
					this.caller.submitForm();
					return;
				}

				syncFieldCodes.forEach(function(syncFieldCode){
					if(!fieldCodeToCaption[syncFieldCode])
					{
						return;
					}

					fieldSyncCaptionList.push(
						'<li class="crm-p-s-f-item">' + BX.util.htmlspecialchars(fieldCodeToCaption[syncFieldCode]) + '</li>'
					);
				});

				popupFieldListCont.innerHTML = fieldSyncCaptionList.join('');
				this.showPopupCreateFields();
			}, this),
			BX.proxy(function()
			{
				popupFieldListCont.innerHTML = fieldCaptionList.join('');
				this.showPopupCreateFields();
			}, this)
		);

		return false;
	},

	setCurrentId: function (value)
	{
		if(this.valueNode)
		{
			this.valueNode.value = value;
		}
	},


	getCurrentPayerEntityType: function ()
	{
		return this._currentPayerEntityType();
	},

	setCurrentPayerEntityType: function (value)
	{
		return this._currentPayerEntityType(value);
	},


	_currentPayerEntityType: function (value)
	{
		value = value || null;
		this.invoice.payerTypeNodes.forEach(function(node){
			if(value)
			{
				node.checked = value == node.value;
			}
			else if(node.checked)
			{
				value = node.value;
			}
		}, this);

		return value;
	},

	getCurrent: function ()
	{
		var section = this.invoiceCheckNode.checked ? 'BY_INVOICE' : 'BY_NON_INVOICE';
		var schemeId = null;
		this.radioList.forEach(function(radioNode){

			if(radioNode.checked)
			{
				schemeId = radioNode.value;
			}
		});
		return this.caller.schemesDictionary[section][schemeId];
	},

	onChange: function ()
	{
		var scheme = this.getCurrent();
		this.setCurrentId(scheme.ID);
		this.actualizeInvoiceSettings(scheme);
		this.actualizeDealCategory(scheme);

		BX.onCustomEvent(this.caller, 'change-entity-scheme', [scheme]);
	},

	getCurrentWillCreatedEntities: function()
	{
		var scheme = this.getCurrent();
		var entityTypes = [];
		var currentPayerEntityTypeName = this.getCurrentPayerEntityType();
		scheme.ENTITIES.forEach(function(entityTypeName){

			var isPayer = currentPayerEntityTypeName == entityTypeName;
			var isContact = entityTypeName == 'CONTACT';
			var isCompany = entityTypeName == 'COMPANY';

			if(isContact && !this.invoice.hasFieldsContact && !isPayer)
			{
				return;
			}
			if(isCompany && !this.invoice.hasFieldsCompany && !isPayer)
			{
				return;
			}

			if(!this.caller.entityDictionary[entityTypeName])
			{
				return;
			}

			entityTypes.push(entityTypeName);
		}, this);

		return entityTypes;
	},

	actualizeDealCategory: function()
	{
		var scheme = this.getCurrent();
		var isAdd = BX.util.in_array('DEAL', scheme.ENTITIES);
		this.helper.changeClass(this.dealCategoryNode, 'crm-webform-edit-animate-show-120', isAdd);
		//this.helper.styleDisplay(this.dealCategoryNode, BX.util.in_array('DEAL', scheme.ENTITIES));
	},

	actualizeTextWillCreatedEntities: function()
	{
		var entityTypeCaptions = [];
		this.getCurrentWillCreatedEntities().forEach(function(entityTypeName){
			entityTypeCaptions.push(this.caller.entityDictionary[entityTypeName].toLowerCase());
		}, this);

		var scheme = this.getCurrent();
		var description = entityTypeCaptions.length > 0 ? entityTypeCaptions.join(', ') : scheme['DESCRIPTION'];
		if(this.descriptionNode)
		{
			this.descriptionNode.innerText = description;
		}
		if(this.descriptionTopNode)
		{
			this.descriptionTopNode.innerText = description;
		}
	},

	actualizeInvoiceProductField: function(isInvoiceChecked)
	{
		var productFields = this.caller.fields.filter(function(field){
			return field.type == 'product';
		}, this);

		if(productFields.length > 0 && !this.invoice.productField)
		{
			this.invoice.productField = productFields[0];
			BX.addCustomEvent(this.invoice.productField, 'change-field-items', BX.proxy(this.actualizeInvoiceSettings, this));
		}
		else if(productFields.length == 0)
		{
			if(this.invoice.productField)
			{
				this.invoice.productField = null;
			}
			else if(isInvoiceChecked)
			{
				this.invoice.productField = this.caller.fieldSelector.addSpecialFormField('product');
				BX.addCustomEvent(this.invoice.productField, 'change-field-items', BX.proxy(this.actualizeInvoiceSettings, this));
				this.invoice.productField.productSelector.addFieldItem({
					'id': 'n' + this.helper.generateId(),
					'name': this.caller.mess.defaultProductName,
					'price': 1
				});
			}
		}

		if(!isInvoiceChecked && this.invoice.productField)
		{
			//this.caller.deleteField
		}

		if(this.invoice.productDraw)
		{
			this.invoice.productDraw.innerHTML = '';

			if(this.invoice.productField)
			{
				this.invoice.productField.items.forEach(function(item){
					this.helper.appendNodeByTemplate(
						this.invoice.productDraw,
						'tmpl_field_product_items_draw',
						{
							'name': item.VALUE || '',
							'price': item.PRICE || ''
						}
					);
				}, this);
			}
		}

		this.helper.styleDisplay(this.invoice.productNode, isInvoiceChecked);
	},

	actualizePayer: function()
	{
		// actualize payer
		var hasFieldsContact = false, hasFieldsCompany = false;
		var clientFields = this.caller.fields.filter(function(field){
			if(!field.dict['entity_name']) return false;
			if(field.dict.entity_name == 'CONTACT') hasFieldsContact = true;
			if(field.dict.entity_name == 'COMPANY') hasFieldsCompany = true;

			return (BX.util.in_array(field.dict.entity_name, ['CONTACT', 'COMPANY']));
		}, this);

		this.invoice.hasFieldsCompany = hasFieldsCompany;
		this.invoice.hasFieldsContact = hasFieldsContact;

		var hasContactAndCompany = (hasFieldsContact && hasFieldsCompany);
		if(!hasContactAndCompany)
		{
			if(hasFieldsContact)
			{
				this.setCurrentPayerEntityType('CONTACT');
			}
			else if(hasFieldsCompany)
			{
				this.setCurrentPayerEntityType('COMPANY');
			}
		}

		var showBlock = (clientFields.length == 0 || hasContactAndCompany);
		this.helper.styleDisplay(this.invoice.payerNode, showBlock);
		this.helper.styleDisplay(this.invoice.payerTextNoNode, !hasContactAndCompany);
	},

	actualizeInvoiceSettings: function()
	{
		var scheme = this.getCurrent();
		var isInvoiceChecked = this.isInvoiceChecked();

		// show block
		this.helper.changeClass(this.invoice.node, 'crm-webform-display-none', !isInvoiceChecked);

		// actualize products
		this.actualizeInvoiceProductField(isInvoiceChecked);

		// actualize payer
		this.actualizePayer();

		// actualize description text of entities that will created
		this.actualizeTextWillCreatedEntities();
	}
};


function CrmWebFormEditFormButton(params)
{
	this.caller = params.caller;
	this.context = params.context;
	this.helper = BX.CrmFormEditorHelper;

	this.buttonNode = BX('FORM_BUTTON');
	this.buttonInputNode = BX('FORM_BUTTON_INPUT');
	this.buttonTextErrorNode = BX('FORM_BUTTON_INPUT_ERROR');
	this.buttonColorBgNode = BX('BUTTON_COLOR_BG');
	this.buttonColorFontNode = BX('BUTTON_COLOR_FONT');
	this.popupButtonNameNode = BX('CRM_WEB_FORM_POPUP_BUTTON_NAME');

	this.init();
}
CrmWebFormEditFormButton.prototype =
{
	init: function()
	{
		BX.bind(this.buttonNode.parentNode, 'click', BX.proxy(this.editButton, this));
		BX.bind(this.buttonInputNode, 'bxchange', BX.proxy(this.onButtonCaptionChange, this));
		this.initColorPicker();
	},

	editButton: function()
	{
		this.caller.popupFieldSettings.show(this.popupButtonNameNode, BX.proxy(this.onEditButtonClosePopup, this));
	},

	onEditButtonClosePopup: function()
	{
		var isFilled = !!BX.util.trim(this.buttonInputNode.value);
		this.helper.styleDisplay(this.buttonTextErrorNode, !isFilled, 'block');

		return isFilled;
	},

	onButtonCaptionChange: function()
	{
		this.buttonNode.innerText = this.buttonInputNode.value;
	},

	updateButtonColors: function()
	{
		this.buttonNode.style.background = this.buttonColorBgNode.value;
		this.buttonNode.style.color = this.buttonColorFontNode.value;
	},

	initColorPicker: function()
	{
		this.picker = new window.BXColorPicker({'id': "picker", 'name': 'picker'});
		this.picker.Create();

		var _this = this;

		var clickHandler = function ()
		{
			var element = this;
			element.parentNode.appendChild(_this.picker.pCont);
			_this.picker.oPar.OnSelect = BX.proxy(function (color)
			{
				if(!color)
					color = '';

				element.value = color;
				var colorBox = BX.nextSibling(element);
				if(colorBox)
				{
					colorBox.style.background = color;
				}
				BX.fireEvent(element, 'change');
			}, _this);

			_this.picker.pCont.style.display = '';
			_this.picker.Close();
			_this.picker.Open(element);
			_this.picker.pCont.style.display = 'none';

		};

		var changeHandler = function()
		{
			var colorBox = BX.nextSibling(this);
			if (colorBox)
			{
				colorBox.style.background = this.value;
				_this.updateButtonColors();
			}
		};


		var inputList = this.context.querySelectorAll('[data-web-form-color-picker]');
		inputList = BX.convert.nodeListToArray(inputList);
		for(var i in inputList)
		{
			var inputCtrl = inputList[i];
			var colorBox = BX.nextSibling(inputCtrl);

			BX.bind(colorBox, 'click', BX.proxy(clickHandler, inputCtrl));
			BX.bind(inputCtrl, 'click', clickHandler);
			BX.bind(inputCtrl, "focus", clickHandler);

			BX.bind(inputCtrl, "bxchange", BX.delegate(changeHandler, inputCtrl));
			BX.fireEvent(inputCtrl, 'change');
		}
	}

};


function CrmWebFormEditPopupFieldSettings(params)
{
	this.caller = params.caller;
	this.popup = null;
	this.popupContent = params.content;
	this.popupContainer = params.container;
	this.popupCloseHandler = null;

	this.fieldContainer = null;
}
CrmWebFormEditPopupFieldSettings.prototype =
{
	setAutoHide: function (doAutoHide)
	{
		if(this.popup)
		{
			this.popup.params.autoHide = doAutoHide;
		}
	},

	show: function (fieldContainer, popupCloseHandler)
	{
		this.popupCloseHandler = popupCloseHandler || null;

		this.initPopup();
		this.moveSettingsBack();

		this.fieldContainer = fieldContainer;
		this.moveSettingsOn();
		this.popup.show();
	},

	moveSettingsOn: function ()
	{
		if(!this.fieldContainer)
		{
			return;
		}

		this.replaceSettingsNode(this.fieldContainer, this.popupContainer);
	},

	moveSettingsBack: function ()
	{
		if(!this.fieldContainer)
		{
			return;
		}

		this.replaceSettingsNode(this.popupContainer, this.fieldContainer);
		this.fieldContainer = null;
	},

	_popupClose: function (event)
	{
		if(this.popupCloseHandler && !this.popupCloseHandler.apply(this, []))
		{
			return;
		}

		this.onClose();
		BX.PopupWindow.prototype.close.apply(this.popup, [event]);
	},

	onClose: function ()
	{
		/*
		if(this.popup.isShown())
		{
			this.popup.close();
		}
		*/

		this.moveSettingsBack();
		this.popupCloseHandler = null;
	},

	replaceSettingsNode: function (source, destination)
	{
		if(!source || !source.firstElementChild)
		{
			return;
		}

		destination.appendChild(source.firstElementChild);
	},

	initPopup: function ()
	{
		if(this.popup)
		{
			return;
		}

		var popup = BX.PopupWindowManager.create(
			'crm_webform_edit_field_settings',
			null,
			{
				content: this.popupContent,
				autoHide: false,
				lightShadow: true,
				closeByEsc: true,
				overlay: {backgroundColor: 'black', opacity: 500},
				zIndex: -400,
				titleBar: this.caller.mess.dlgTitle,
				closeIcon: true,
				//onPopupClose: BX.proxy(this.onClose)
			}
		);
		popup.setButtons([
			new BX.PopupWindowButton({
				text: this.caller.mess.dlgClose,
				className: 'webform-small-button-accept crm-webform-edit-popup-button',
				events: {
					click: BX.proxy(function(){
						this.popup.close();
					}, this)
				}
			})
		]);

		popup.close = BX.proxy(this._popupClose, this);
		//BX.addCustomEvent(popup, 'onPopupClose', BX.proxy(this.onClose, this));
		this.popup = popup;
	}
};

function CrmFormEditorResponsible(params)
{
	this.caller = params.caller;
	this.context = params.context;

	this.attributeContainer = 'data-bx-webform-responsible-cont';
	this.container = this.context.querySelector(this.attributeContainer);

	this.helper = BX.CrmFormEditorHelper;
	this.init(params);
}
CrmFormEditorResponsible.prototype =
{
	init: function ()
	{
		this.initControl();
	},

	initControl: function ()
	{
		var destinationParams = {
			'name' : null, //
			'searchInput' : null, //
			'extranetUser' :  false,
			'bindMainPopup' : { node : this.container, 'offsetTop' : '5px', 'offsetLeft': '15px'}, //
			'bindSearchPopup' : { node : this.container, 'offsetTop' : '5px', 'offsetLeft': '15px'}, //
			departmentSelectDisable : true,
			'callback' : {
				'select' : BX.delegate(this.select, this),
				'unSelect' : BX.delegate(this.unSelect, this),
				'openDialog' : BX.delegate(this.openDialog, this),
				'closeDialog' : BX.delegate(this.closeDialog, this),
				'openSearch' : BX.delegate(this.openDialog, this),
				'closeSearch' : BX.delegate(this.closeSearch, this)
			},
			items : {
				users : {},
				groups : {},
				sonetgroups : {},
				department : {},
				departmentRelation : {},
				contacts : {},
				companies : {},
				leads : {},
				deals : {}
			},
			itemsLast : {
				users : {},
				sonetgroups : {},
				department : {},
				groups : {},
				contacts : {},
				companies : {},
				leads : {},
				deals : {},
				crm : []
			},
			itemsSelected : {},
			isCrmFeed : false,
			destSort : {}
		};

		BX.SocNetLogDestination.init(destinationParams);
	}
};

function CrmFormEditorFieldSelector(params)
{
	this.caller = params.caller;
	this.context = params.context;

	this.attributeSearch = 'data-bx-crm-wf-selector-search';
	this.attributeSearchButton = 'data-bx-crm-wf-selector-search-btn';
	this.attributeGroup = 'data-bx-crm-wf-selector-field-group';
	this.attributeField = 'data-bx-crm-wf-selector-field-name';
	this.attributeAddButton = 'data-bx-crm-wf-selector-btn-add';

	this.classFieldSelected = 'crm-webform-edit-right-inner-list-active';
	this.classSearchActive = 'crm-webform-edit-constructor-right-search-active';
	this.classGroupShow = 'crm-webform-edit-open';

	this.helper = BX.CrmFormEditorHelper;
	this.init(params);
}
CrmFormEditorFieldSelector.prototype =
{
	getFieldNode: function (fieldName)
	{
		return this.context.querySelector('[' + this.attributeField + '="' + fieldName + '"]');
	},

	init: function (params)
	{
		// init groups
		this.groupNodeList = this.context.querySelectorAll('[' + this.attributeGroup + ']');
		this.groupNodeList = BX.convert.nodeListToArray(this.groupNodeList);
		this.groupNodeList.forEach(this.initGroup, this);

		// init fields
		this.fieldNodeList = this.context.querySelectorAll('[' + this.attributeField + ']');
		this.fieldNodeList = BX.convert.nodeListToArray(this.fieldNodeList);
		this.fieldNodeList.forEach(this.initField, this);

		// init special adding
		var specialAddNodeList = this.context.querySelectorAll('[' + this.attributeAddButton + ']');
		specialAddNodeList = BX.convert.nodeListToArray(specialAddNodeList);
		specialAddNodeList.forEach(function(specialAddNode){
			BX.bind(specialAddNode, 'click', BX.proxy(function() {
				this.addSpecialFormField(specialAddNode.getAttribute(this.attributeAddButton));
			}, this));
		}, this);

		// init delete callback
		BX.addCustomEvent(this.caller, 'delete-field', BX.proxy(this.onCallerFieldDelete, this));

		// init search button
		this.searchButton = this.context.querySelector('[' + this.attributeSearchButton + ']');
		BX.bind(this.searchButton, 'click', BX.proxy(this.onSearchButtonClick, this));

		// init search input
		this.searchInput = this.context.querySelector('[' + this.attributeSearch + ']');
		BX.bind(this.searchInput, 'bxchange', BX.proxy(this.onSearchChange, this));
	},

	showSearchResult: function (q)
	{
		q = q || '';
		q = q.toLowerCase();
		this.fieldNodeList.forEach(function(fieldNode){
			var hasSubstring = !q ? true : fieldNode.innerText.toLowerCase().indexOf(q) > -1;
			this.helper.styleDisplay(fieldNode, hasSubstring, 'list-item');
		}, this);
	},

	onSearchChange: function ()
	{
		this.showSearchResult(this.searchInput.value);
	},

	onSearchButtonClick: function ()
	{
		var isActive = BX.hasClass(this.searchButton, this.classSearchActive);
		if(isActive)
		{
			this.showSearchResult();
		}
		else
		{
			this.groupNodeList.forEach(function(groupNode){
				BX.addClass(groupNode, this.classGroupShow);
			}, this);

			this.showSearchResult(this.searchInput.value);
		}


		BX.toggleClass(this.searchButton, this.classSearchActive);
		this.searchInput.focus();
	},

	initGroup: function (groupNode)
	{
		BX.bind(groupNode.children[0], 'click', BX.proxy(function() {
			BX.toggleClass(groupNode, this.classGroupShow);
		}, this));
	},

	initField: function (fieldNode)
	{
		var fieldName = fieldNode.getAttribute(this.attributeField);
		var filteredFields = this.caller.fields.filter(function(field){
			return field.name == fieldName;
		});
		if(filteredFields.length > 0)
		{
			BX.addClass(fieldNode, this.classFieldSelected);
		}

		BX.bind(fieldNode, 'click', BX.proxy(function(e) {

			BX.addClass(fieldNode, this.classFieldSelected);
			var fieldName = fieldNode.getAttribute(this.attributeField);
			this.caller.addFieldByCode(fieldName);
			BX.PreventDefault(e);
			return true;

		}, this));
	},

	addSpecialFormField: function (type)
	{
		var fieldParams = {
			type: type,
			name: type + '_' + this.helper.generateId()
		};
		switch(type)
		{
			case 'product':
				fieldParams.caption = this.caller.mess.newFieldProductsCaption;
				fieldParams.id = 'n' + this.helper.generateId();
				break;
			case 'section':
				fieldParams.caption = this.caller.mess.newFieldSectionCaption;
				break;
			case 'hr':
				break;
			case 'br':
				break;
		}

		return this.caller.addField(fieldParams);
	},

	onCallerFieldDelete: function (field)
	{
		var selectorFieldNode = this.getFieldNode(field.name);
		if(selectorFieldNode)
		{
			BX.removeClass(selectorFieldNode, this.classFieldSelected);
		}
	}
};

function CrmFormEditorDragDrop(params)
{
	this.init(params);
}
CrmFormEditorDragDrop.prototype =
{
	addItem: function(node)
	{
		this.dragdrop.addDragItem([node]);
		this.dragdrop.addSortableItem(node);
	},

	removeItem: function(node)
	{
		this.dragdrop.removeSortableItem(node);
	},

	init: function(params)
	{
		this.dragdrop = BX.DragDrop.create({

			sortable: {
				rootElem: BX('FIELD_CONTAINER'),
				gagClass: 'gag-class'
			},
			dragActiveClass: 'crm-webform-field-drag',
			dragItemClassName: 'field-selected',
			dragDrop: BX.delegate(function(catcherObj, dragElement, event)
			{
				BX.onCustomEvent(this, 'onSort', [dragElement, catcherObj]);
			}, this),
			dragStart: BX.delegate(function(eventObj, dragElement, event){
				if(!dragElement)
				{
					dragElement = eventObj.dragElement;
				}
				if(!dragElement)
				{
					return;
				}

				if(eventObj.event.dataTransfer && eventObj.event.dataTransfer.setDragImage)
				{

					var dragIcon = document.createElement('img');
					eventObj.event.dataTransfer.setDragImage(dragIcon, -10, -10);
				}

			}, this),
			dragOver: BX.delegate(function(catcherObj, dragElement, event){}, this),
			dragEnter: BX.delegate(function(catcherObj, dragElement, event){}, this),
			dragLeave: BX.delegate(function(catcherObj, dragElement, event){}, this),
			dragEnd: BX.delegate(function(catcherObj, dragElement, event){

				BX.onCustomEvent(this, 'onSort', [dragElement, catcherObj]);
			}, this)
		});
	}
};




function CrmFormEditorDependencies(params)
{
	params = params || {};
	params.dependencies = params.dependencies || [];
	this.helper = BX.CrmFormEditorHelper;

	this.deps = [];
	this.container = BX('DEPENDENCY_CONTAINER');
	this.buttonAdd = BX('DEPENDENCY_BUTTON_ADD');
	this.init(params);
}
CrmFormEditorDependencies.prototype =
{
	add: function (params)
	{
		var id = 'n' + this.helper.generateId();
		var data = {
			'ID': id,
			'IF_FIELD_CODE': '',
			'IF_VALUE': '',
			'DO_FIELD_CODE': '',
			'DO_ACTION': ''
		};
		var depNode = this.helper.appendNodeByTemplate(this.container, this.caller.templates.dependency, data);

		this.bind(data);

		return depNode;
	},
	bind: function (dep)
	{
		dep.node = BX('DEPENDENCIES_' + dep.ID);

		dep.ifValueNodeCtrlS = BX('DEPENDENCIES_' + dep.ID + '_IF_VALUE_CTRL_S');
		dep.ifValueNodeCtrlI = BX('DEPENDENCIES_' + dep.ID + '_IF_VALUE_CTRL_I');
		dep.ifFieldNodeCtrl = BX('DEPENDENCIES_' + dep.ID + '_IF_FIELD_CODE_CTRL');
		dep.doFieldNodeCtrl = BX('DEPENDENCIES_' + dep.ID + '_DO_FIELD_CODE_CTRL');

		dep.ifValueNode = BX('DEPENDENCIES_' + dep.ID + '_IF_VALUE');
		dep.ifFieldNode = BX('DEPENDENCIES_' + dep.ID + '_IF_FIELD_CODE');
		dep.doFieldNode = BX('DEPENDENCIES_' + dep.ID + '_DO_FIELD_CODE');

		dep.actionNodeCtrl = BX('DEPENDENCIES_' + dep.ID + '_DO_ACTION');
		dep.elseHideTextNode = BX('DEPENDENCIES_' + dep.ID + '_ELSE_HIDE');
		dep.elseShowTextNode = BX('DEPENDENCIES_' + dep.ID + '_ELSE_SHOW');

		var _this = this;
		BX.bind(BX('DEPENDENCIES_' + dep.ID + '_BTN_REMOVE'), 'click', function(){
			_this.remove(dep);
		});
		BX.bind(dep.ifFieldNodeCtrl, 'change', function(){
			if(_this.canChangeFields)
			{
				dep.ifFieldNode.value = this.value;
				_this.actualizeFieldValues(dep);
			}
		});
		BX.bind(dep.doFieldNodeCtrl, 'change', function(){
			if(_this.canChangeFields)
			{
				dep.doFieldNode.value = this.value;
			}

			var caption = '';
			if(this.selectedOptions && this.selectedOptions.length > 0)
			{
				caption = this.selectedOptions[0].innerText;
				caption = '"' + caption + '"';
			}
			dep.elseShowTextNode.children[0].innerText = caption;
			dep.elseHideTextNode.children[0].innerText = caption;
		});
		BX.bind(dep.ifValueNodeCtrlS, 'change', function(){
			if(_this.canChangeFields)
			{
				dep.ifValueNode.value = this.value;
			}
		});
		BX.bind(dep.ifValueNodeCtrlI, 'change', function(){
			if(_this.canChangeFields)
			{
				dep.ifValueNode.value = this.value;
			}
		});
		BX.bind(dep.actionNodeCtrl, 'change', BX.proxy(function(){
			var isActionHide = dep.actionNodeCtrl.value == 'hide';
			this.helper.styleDisplay(dep.elseHideTextNode, !isActionHide);
			this.helper.styleDisplay(dep.elseShowTextNode, isActionHide);
		}, this));

		this.actualize(dep);

		this.deps.push(dep);
	},
	remove: function (dep)
	{
		BX.remove(dep.node);

		var itemIndex = BX.util.array_search(dep, this.deps);
		if(itemIndex > -1)
		{
			delete this.deps[itemIndex];
		}
	},
	actualizeFieldList: function(node, values, isDo)
	{
		this.canChangeFields = false;

		var exceptFieldTypes = ['hr', 'br'];
		var fields;
		var defaultOptionText = '';

		isDo = isDo || false;
		if (isDo)
		{
			defaultOptionText = this.caller.mess.selectFieldOrSection;
			fields = this.caller.fields.filter(function(field){
				return !BX.util.in_array(field.type, exceptFieldTypes);
			}, this);
		}
		else
		{
			defaultOptionText = this.caller.mess.selectField;
			fields = this.caller.fields.filter(function(field){
				return (field.type != 'section' && !BX.util.in_array(field.type, exceptFieldTypes));
			}, this);
		}

		fields = BX.util.array_merge(
			[{caption: defaultOptionText, value: '', selected: false}],
			fields.map(function(field){
				return {
					value: field.name || '',
					caption: field.getCaption(),
					selected: BX.util.in_array(field.name, values)
				};
			})
		);

		this.helper.fillDropDownControl(node, fields);


		this.canChangeFields = true;
	},
	actualizeFieldValues: function (dep)
	{
		this.canChangeFields = false;

		var listTypes = ['product', 'list', 'checkbox', 'radio'];

		var fieldName = dep.ifFieldNode.value;
		if(!fieldName) return;

		var field = this.caller.findField(fieldName);
		if(!field) return;

		var isList = BX.util.in_array(field.type, listTypes);
		dep.ifValueNodeCtrlI.style.display = isList ? 'none' : '';
		dep.ifValueNodeCtrlS.style.display = !isList ? 'none' : '';

		var defaultOptionText = this.caller.mess.selectValue;

		var values = [dep.ifValueNode.value];
		if(isList)
		{
			var items;
			if(field.items && field.items.length > 0)
			{
				items = field.items.map(function(item){
					return {
						value: item.ID,
						caption: item.VALUE,
						selected: BX.util.in_array(item.ID, values)
					};
				})
			}
			else
			{
				items = [];
			}

			this.helper.fillDropDownControl(
				dep.ifValueNodeCtrlS,
				BX.util.array_merge(
					[{caption: defaultOptionText, value: '', selected: false}],
					items
				)
			);
		}
		else if(!isList)
		{
			dep.ifValueNodeCtrlI.value = values[0];
		}

		this.canChangeFields = true;
	},
	actualizeFieldListConditional: function (node, values)
	{
		this.actualizeFieldList(node, values, false);
	},
	actualizeFieldListOperational: function (node, values)
	{
		this.actualizeFieldList(node, values, true);
	},
	actualize: function (dep)
	{
		// get selected dependence field names
		var valueIf = dep.ifFieldNode.value;
		var valueDo = dep.doFieldNode.value;

		// get added field names
		var fieldNameList = this.caller.getFieldNameList();

		if(
			(valueIf && !BX.util.in_array(valueIf, fieldNameList))
			||
			(valueDo && !BX.util.in_array(valueDo, fieldNameList))
		)
		{
			// if field deleted, delete existed dependence
			this.remove(dep);
		}
		else
		{
			// actualize field list in dependency selectors
			this.actualizeFieldListConditional(dep.ifFieldNodeCtrl, [valueIf]);
			this.actualizeFieldValues(dep);
			this.actualizeFieldListOperational(dep.doFieldNodeCtrl, [valueDo]);
		}
	},
	onChangeFormFields: function()
	{
		this.deps.forEach(this.actualize, this);
	},
	init: function (params)
	{
		this.caller = params.caller;
		this.helper = new CrmFormEditorHelper();

		this.canChangeFields = true;
		// init add button
		BX.bind(this.buttonAdd, 'click', BX.proxy(this.add, this));

		// init existed deps
		params.dependencies.forEach(this.bind, this);

		// listen events of changing form fields
		BX.addCustomEvent(this.caller, 'change-field-list', BX.proxy(this.onChangeFormFields, this));
		BX.addCustomEvent(this.caller, 'change-field-items', BX.proxy(this.onChangeFormFields, this));
	}
};



function CrmFormEditorFieldPreset(params)
{
	params = params || {};
	params.fields = params.fields || [];

	this.fields = [];
	this.container = BX('PRESET_FIELD_CONTAINER');
	this.buttonAdd = BX('PRESET_FIELD_SELECTOR_BTN');
	this.init(params);
}
CrmFormEditorFieldPreset.prototype =
{
	isExists: function (code)
	{
		var isExists = false;
		this.fields.forEach(function(field){
			if(field.CODE == code)
			{
				isExists = true;
			}
		});

		return isExists;
	},
	add: function (params)
	{
		if(this.isExists(params.CODE))
		{
			return null;
		}

		var fieldData = this.caller.findDictionaryField(params.CODE);
		if(!fieldData)
		{
			return null;
		}

		var data = {
			'CODE': fieldData.name,
			'ENTITY_CAPTION': fieldData.entity_caption,
			'ENTITY_FIELD_CAPTION': fieldData.caption,
			'ENTITY_NAME': fieldData.entity_name,
			'ENTITY_FIELD_NAME': fieldData.entity_field_name,
			'VALUE': ''
		};
		var node = this.helper.appendNodeByTemplate(this.container, this.caller.templates.presetField, data);
		data.ITEMS = fieldData.items || null;
		this.bind(data);

		return node;
	},
	bind: function (field)
	{
		field.node = BX('FIELD_PRESET_' + field.CODE);

		field.valueNodeCtrl = BX('FIELD_PRESET_' + field.CODE + '_VALUE');
		field.valueNodeCtrlS = BX('FIELD_PRESET_' + field.CODE + '_VALUE_CTRL_S');
		field.valueNodeCtrlI = BX('FIELD_PRESET_' + field.CODE + '_VALUE_CTRL_I');

		var _this = this;
		BX.bind(BX('FIELD_PRESET_' + field.CODE + '_BTN_REMOVE'), 'click', function(){
			_this.remove(field);
		});

		BX.bind(field.valueNodeCtrlS, 'change', function(){
			if(_this.canChangeFields)
			{
				field.valueNodeCtrl.value = this.value;
			}
		});

		BX.bind(field.valueNodeCtrlI, 'change', function(){
			if(_this.canChangeFields)
			{
				field.valueNodeCtrl.value = this.value;
			}
		});

		this.actualize(field);

		this.fields.push(field);
	},
	remove: function (field)
	{
		BX.remove(field.node);

		var itemIndex = BX.util.array_search(field, this.fields);
		if(itemIndex > -1)
		{
			delete this.fields[itemIndex];
		}
	},
	actualize: function (field)
	{
		this.canChangeFields = false;

		var listTypes = ['list', 'checkbox', 'radio'];

		var fieldName = field.CODE;
		if(!fieldName) return;

		var fieldData = this.caller.findDictionaryField(fieldName);
		if(!fieldData) return;

		var isList = BX.util.in_array(fieldData.type, listTypes);
		field.valueNodeCtrlI.style.display = isList ? 'none' : '';
		field.valueNodeCtrlS.style.display = !isList ? 'none' : '';

		var values = [field.valueNodeCtrl.value];
		if(isList && fieldData.items && fieldData.items.length > 0)
		{
			this.helper.fillDropDownControl(
				field.valueNodeCtrlS,
				fieldData.items.map(function(item){
					return {
						value: item.ID,
						caption: item.VALUE,
						selected: BX.util.in_array(item.ID, values)
					};
				})
			);
		}
		else if(!isList)
		{
			field.valueNodeCtrlI.value = values[0];
		}

		this.canChangeFields = true;
	},

	onChangeEntityScheme: function(schemeData)
	{
		var entityTypes = this.caller.entityScheme.getCurrentWillCreatedEntities();
		var fieldsForDelete = this.fields.filter(function(field){
			return !BX.util.in_array(field.ENTITY_NAME, entityTypes);
		}, this);

		if(fieldsForDelete)
		{
			fieldsForDelete.reverse();
			fieldsForDelete.forEach(function(field){this.remove(field); }, this);
		}

		var firstVisibleOption;
		var optGroupList = BX.convert.nodeListToArray(BX('PRESET_FIELD_SELECTOR').querySelectorAll('optgroup'));
		optGroupList.forEach(function(optGroup){
			var isVisible = BX.util.in_array(optGroup.getAttribute('data-bx-crm-wf-entity'), entityTypes);
			optGroup.style.display = isVisible ? 'block' : 'none';
			if(!firstVisibleOption && isVisible) firstVisibleOption = optGroup.querySelector('option');
		}, this);

		if(firstVisibleOption)
		{
			firstVisibleOption.selected = true;
		}
	},
	onButtonAddClick: function()
	{
		this.add({CODE: BX('PRESET_FIELD_SELECTOR').value});
	},
	init: function (params)
	{
		this.caller = params.caller;
		this.helper = new CrmFormEditorHelper();

		this.canChangeFields = true;
		// init add button
		BX.bind(this.buttonAdd, 'click', BX.proxy(this.onButtonAddClick, this));

		// init existed fields
		if(params.fields)
		{
			params.fields.forEach(this.bind, this);
		}

		// listen events of changing form fields
		BX.addCustomEvent(this.caller, 'change-entity-scheme', BX.proxy(this.onChangeEntityScheme, this));
		this.onChangeEntityScheme(this.caller.entityScheme.getCurrent());
	}
};

function CrmFormEditorExternalAnalytics(params)
{
	this.context = BX('CRM_WEBFORM_EXTERNAL_ANALYTICS');

	this.classNameExisted = 'crm-webform-edit-task-options-metric-exist';
	this.attributeItem = 'data-bx-crm-webform-ext-an';
	this.attributeClose = 'data-bx-crm-webform-ext-an-close';
	this.attributeAdd = 'data-bx-crm-webform-ext-an-add';
	this.attributeValue = 'data-bx-crm-webform-ext-an-val';

	var itemNodeList = this.context.querySelectorAll('[' + this.attributeItem + ']');
	itemNodeList = BX.convert.nodeListToArray(itemNodeList);
	itemNodeList.forEach(this.bind, this);
}
CrmFormEditorExternalAnalytics.prototype =
{
	bind: function(itemNode)
	{
		var closeBtnNode = itemNode.querySelector('[' + this.attributeClose + ']');
		var addBtnNode = itemNode.querySelector('[' + this.attributeAdd + ']');
		var valueNode = itemNode.querySelector('[' + this.attributeValue + ']');

		BX.bind(closeBtnNode, 'click', BX.proxy(function(){
			BX.removeClass(itemNode, this.classNameExisted);
			valueNode.value = '';
		}, this));
		BX.bind(addBtnNode, 'click',BX.proxy(function(){
			BX.addClass(itemNode, this.classNameExisted);
			valueNode.focus();
		}, this));
	}
};

function CrmFormEditorUserBlockController(params)
{
	this.context = BX('CRM_WEBFORM_ADDITIONAL_OPTIONS');
	this.additionalOptionContainer = BX('ADDITIONAL_OPTION_CONTAINER');

	this.attributeOption = 'data-bx-crm-webform-edit-option';
	this.attributeNav = 'data-bx-crm-webform-edit-option-nav';
	this.attributePin = 'data-bx-crm-webform-edit-option-pin';

	this.userOptionPin = params.optionPinName;
	this.blocks = [];

	this.helper = BX.CrmFormEditorHelper;

	var blockNodeList = this.context.querySelectorAll('[' + this.attributeOption + ']');
	blockNodeList = BX.convert.nodeListToArray(blockNodeList);
	blockNodeList.forEach(this.initBlock, this);

	BX.bind(BX('ADDITIONAL_OPTION_BUTTON'), 'click', BX.proxy(function(){
		BX.toggleClass(this.additionalOptionContainer, 'crm-webform-edit-open');
	}, this));
}
CrmFormEditorUserBlockController.prototype =
{
	initBlock: function (blockNode)
	{
		var id = blockNode.getAttribute(this.attributeOption);
		var pinNode = blockNode.querySelector('[' + this.attributePin + ']');
		var block = {
			'id': id,
			'node': blockNode,
			'navNode': this.context.querySelector('[' + this.attributeNav + '="' + id + '"]'),
			'pinNode': pinNode,
			'isFixed': this.isBlockPinFixed(pinNode)
		};
		this.blocks.push(block);


		var _this = this;

		//bind block pin
		BX.bind(block.pinNode, 'click', function(){
			_this.onClickBlockPin(block);
		});

		//bind nav button
		BX.bind(block.navNode, 'click', function(e){
			BX.PreventDefault(e);
			_this.highLightBlock(block);
			return true;
		});

		if(block.id == 'ENTITY_SCHEME')
		{
			//bind top nav button to document
			BX.bind(BX('CRM_WEBFORM_STICKER_ENTITY_SCHEME_NAV'), 'click', function(e){
				BX.PreventDefault(e);
				_this.highLightBlock(block);
				return true;
			});
		}
	},
	isBlockPinFixed: function (pinNode)
	{
		return BX.hasClass(pinNode, 'task-option-fixed-state');
	},
	onClickBlockPin: function (block)
	{
		// set pin state of block
		block.isFixed = !this.isBlockPinFixed(block.pinNode);

		// save pin state of block
		this.saveBlockPinState(block.id, block.isFixed);

		//change icon state indicator
		this.helper.changeClass(block.pinNode, 'task-option-fixed-state', block.isFixed);

		// move block
		this.moveBlock(block);

		// change visibility of nav button
		this.helper.changeClass(block.navNode, 'crm-webform-display-none', block.isFixed);
	},
	show: function ()
	{
		BX.addClass(this.additionalOptionContainer, 'crm-webform-edit-open');
	},
	hide: function ()
	{
		BX.removeClass(this.additionalOptionContainer, 'crm-webform-edit-open');
	},
	highLightBlock: function (block)
	{
		this.show();
		var highlightClassName = 'crm-webform-edit-highlight';
		BX.addClass(block.node, highlightClassName);


		setTimeout(function(){
			block.node.scrollIntoView();
		}, 600);

		setTimeout(function(){
			BX.removeClass(block.node, highlightClassName);
		}, 3000);
	},
	moveBlock: function (block)
	{
		this.show();
		//hide block
		BX.addClass(block.node, 'crm-webform-display-none');

		//append block
		var target = block.isFixed ? BX('FIXED_OPTION_PLACE') : BX('ADDITIONAL_OPTION_PLACE_' + block.id);
		target.appendChild(block.node);

		//show block
		BX.removeClass(block.node, 'crm-webform-display-none');
	},
	saveBlockPinState: function (blockId, isPinned)
	{
		BX.userOptions.save('crm', this.userOptionPin, blockId, isPinned ? 'Y' : 'N');
	}
};



function CrmFormEditorFieldListSettings(params)
{
	this.type = params.type;
	this.context = params.context;
	this.caller = params.caller;
	this.field = params.field;

	this.helper = BX.CrmFormEditorHelper;
	this.nodeItemsContainer = this.context.querySelector('[data-bx-crm-webform-field-settings-items]');

	this.attributeItem = 'data-bx-crm-webform-field-settings-item';
	this.attributeItemCheck = 'data-bx-crm-webform-field-settings-item-check';
	this.attributeItemRadio = 'data-bx-crm-webform-field-settings-item-radio';
	this.attributeItemClear = 'data-bx-crm-webform-field-settings-item-clear';
	this.attributeItemInput = 'data-bx-crm-webform-field-settings-item-input';

	this.init();
}
CrmFormEditorFieldListSettings.prototype =
{
	init: function()
	{
		if(!this.nodeItemsContainer)
		{
			return;
		}

		var items = this.nodeItemsContainer.querySelectorAll('[' + this.attributeItem + ']');
		items = BX.convert.nodeListToArray(items);
		items.forEach(function(item){
			var clearButton = item.querySelector('[' + this.attributeItemClear + ']');
			var input = item.querySelector('[' + this.attributeItemInput + ']');
			var itemId = item.getAttribute(this.attributeItem);
			BX.bind(clearButton, 'click', function(){
				input.value = '';
				BX.fireEvent(input, 'change');
			});
			BX.bind(input, 'change', BX.proxy(function(){
				this.field.items.forEach(function(fieldItem){
					if(fieldItem.ID == itemId)
					{
						fieldItem.VALUE = input.value;
						this.caller.fireFieldChangeItemsEvent(this.field);
					}
				}, this);

			}, this));

		}, this);


		if(this.type == 'checkbox')
		{
			this.showControls(this.attributeItemCheck);
		}
		else if(this.type == 'radio')
		{
			this.showControls(this.attributeItemRadio);
		}

	},

	showControls: function(attribute)
	{
		var controlList = this.nodeItemsContainer.querySelectorAll('[' + attribute + ']')
		controlList = BX.convert.nodeListToArray(controlList);
		controlList.forEach(function(control){
			control.disabled = false;
			this.helper.styleDisplay(control, true);
		}, this);
	}
};




function CrmFormEditorProductSelector(params)
{
	this.helper = BX.CrmFormEditorHelper;

	this.caller = params.caller;
	this.id = params.id;
	this.field = params.field;
	this.context = params.context;
	this.node = this.context.querySelector('[data-bx-crm-webform-product]');
	if(!this.node)
	{
		return;
	}

	this.nodeItems = this.node.querySelector('[data-bx-crm-webform-product-items]');
	this.nodeSelect = this.node.querySelector('[data-bx-crm-webform-product-select]');
	this.nodeAddRow = this.node.querySelector('[data-bx-crm-webform-product-add-row]');

	this.attributeItem = 'data-bx-crm-webform-product-item';
	this.attributeItemDelete = 'data-bx-crm-webform-product-item-del';
	this.attributeItemInput = 'data-bx-crm-webform-product-item-input';

	this.random = Math.random();
	this.jsEventsManagerId = params.jsEventsManagerId;
	this.caller = params.caller;
	BX.bind(this.nodeSelect, 'click', BX.proxy(this.onClick, this));
	BX.bind(this.nodeAddRow, 'click', BX.proxy(this.onClickAddRow, this));
	this._choiceBtnEnabled = true;

	BX.addCustomEvent('CrmProductSearchDialog_SelectProduct', BX.proxy(this.onProductClick, this));

	this.isCallSearchDialog = false;
	this.initItems();
}
CrmFormEditorProductSelector.prototype =
{
	onShow: function(e)
	{
		/*
		var choiceBtn = document.getElementById(this.getSetting('choiceBtnID', ''));
		if(choiceBtn)
			BX.removeClass(choiceBtn, "webform-small-button-wait");
		*/
		this.isCallSearchDialog = true;
		this.helper.overlay.removeOverlay();
		this.caller.popupFieldSettings.setAutoHide(false);
	},
	onClose: function(e)
	{
		this.isCallSearchDialog = false;
		this._choiceBtnEnabled = true;
		this.caller.popupFieldSettings.setAutoHide(false);
	},
	onClick: function(e)
	{
		if (!this._choiceBtnEnabled)
			return;

		this._choiceBtnEnabled = false;

		this.helper.overlay.createOverlay(2000);

		/*
		var choiceBtn = document.getElementById(this.getSetting('choiceBtnID', ''));
		if(choiceBtn)
			BX.addClass(choiceBtn, "webform-small-button-wait");
		*/

		var caller = 'crm_productrow_list';
		var dlg = BX.CrmProductSearchDialogWindow.create({
			content_url: "/bitrix/components/bitrix/crm.webform.edit/product_choice_dialog.php" +
				"?caller=" + caller + "&JS_EVENTS_MANAGER_ID=" + BX.util.urlencode(this.jsEventsManagerId) +
				"&sessid=" + BX.bitrix_sessid(),
			closeWindowHandler: BX.delegate(this.onClose, this),
			showWindowHandler: BX.delegate(this.onShow, this),
			jsEventsManagerId: this.jsEventsManagerId,
			height: Math.max(500, window.innerHeight - 400),
			width: Math.max(800, window.innerWidth - 400),
			minHeight: 500,
			minWidth: 800,
			draggable: true,
			resizable: true
		});
		dlg.show();
	},

	onClickAddRow: function()
	{
		this.addFieldItem({
			'id': 'n' + this.helper.generateId(),
			'name': '',
			'price': ''
		});
	},

	initItems: function()
	{
		this.field.items.forEach(function(fieldItem){
			var itemNode = this.nodeItems.querySelector('[' + this.attributeItem + '="' + fieldItem.ID + '"' + ']');
			if(!itemNode)
			{
				return;
			}

			this.initProductItem(itemNode, fieldItem);

		}, this);
	},


	handleProductChoice: function(data, skipFocus)
	{
		skipFocus = !!skipFocus;
		var item = typeof(data['product']) != 'undefined' && typeof(data['product'][0]) != 'undefined' ? data['product'][0] : null;
		if(!item)
		{
			return;
		}

		var customData = typeof(item['customData']) !== 'undefined' ? item['customData'] : {};
		var measure = typeof(customData['measure']) !== 'undefined' ? customData['measure'] : {};
		var itemData =
		{
			id: item['id'],
			name: item['title'],
			quantity: 1.0,
			price: typeof(customData['price']) != 'undefined' ? parseFloat(customData['price']) : 0.0,
			customized: false,
			measureCode: typeof(measure['code']) !== 'undefined' ? parseInt(measure['code']) : 0,
			measureName: typeof(measure['name']) !== 'undefined' ? measure['name'] : '',
			tax: typeof(customData['tax']) !== 'undefined' ? customData['tax'] : {}
		};

		this.addFieldItem(itemData);
		/*
		 if (this._viewMode)
		 this.toggleMode();
		 this._addItem(itemData, true);
		 if (!skipFocus)
		 this.focusLastRow();
		 */
	},

	addFieldItem: function(itemData)
	{
		var itemNode = this.helper.getNodeByTemplate('tmpl_field_product_settings_item', {
			'id': this.id,
			'name': this.field.name,
			'item_id': itemData.id,
			'item_value': itemData.name,
			'item_price': itemData.price,
			'currency_short_name': this.caller.currency.SHORT_NAME
		});

		var fieldItem = {
			'ID': itemData.id,
			'PRICE': itemData.price,
			'VALUE': itemData.name
		};

		this.initProductItem(itemNode, fieldItem);
		this.field.items.push(fieldItem);

		this.nodeItems.appendChild(itemNode);
		this.fireFieldItemsChange();
	},

	fireFieldItemsChange: function()
	{
		this.caller.fireFieldChangeItemsEvent(this.field);
	},

	initProductItem: function(itemNode, fieldItem)
	{
		var itemId = itemNode.getAttribute(this.attributeItem);
		var itemDeleteNode = itemNode.querySelector('[' + this.attributeItemDelete + ']');
		var itemInputNode = itemNode.querySelector('[' + this.attributeItemInput + ']');
		BX.bind(itemDeleteNode, 'click', BX.proxy(function(){
			this.removeProductItem(itemNode, fieldItem);
		}, this));

		BX.bind(itemInputNode, 'change', BX.proxy(function(){
			this.field.items.forEach(function(fieldItem){
				if(fieldItem.ID == itemId)
				{
					fieldItem.VALUE = itemInputNode.value;
					this.caller.fireFieldChangeItemsEvent(this.field);
				}
			}, this);

		}, this));
	},

	removeProductItem: function(itemNode, fieldItem)
	{
		var index =BX.util.array_search(fieldItem, this.field.items);
		if(index > -1)
		{
			this.field.items = BX.util.deleteFromArray(this.field.items, index);
		}
		BX.remove(itemNode);

		this.fireFieldItemsChange();
	},

	onProductClick: function(productId)
	{
		if(!this.isCallSearchDialog)
		{
			return;
		}

		productId = parseInt(productId);
		if (productId <= 0)
		{
			return;
		}

		var currencyID = '';//this.getCurrencyId();
		BX.ajax({
			'url': '/bitrix/components/bitrix/crm.product.list/list.ajax.php',
			'method': 'POST',
			'dataType': 'json',
			'data':
			{
				"sessid": BX.bitrix_sessid(),
				"MODE": "SEARCH",
				"RESULT_WITH_VALUE" : "Y",
				"CURRENCY_ID": currencyID,
				"ENABLE_RAW_PRICES": "Y",
				"ENABLE_SEARCH_BY_ID": "N",
				"MULTI": "N",
				"VALUE": "[" + productId + "]",
				"LIMIT": 1
			},
			onsuccess: BX.delegate(this.onProductChoiceByIdSuccess, this),
			onfailure: BX.delegate(this.onProductChoiceByIdFailure, this)
		});
	},
	onProductChoiceByIdSuccess: function(response)
	{
		var data;
		if (response && response["data"])
		{
			data = response["data"];
			if (data[0])
			{
				data = {"product": [data[0]]};
				this.handleProductChoice(data, true);
			}
		}
	},
	onProductChoiceByIdFailure: function(data)
	{
	}
};

function CrmFormEditorHelper(){}
CrmFormEditorHelper.prototype =
{
	generateId: function (min, max)
	{
		min = min || 1000000;
		max = max || 9999999;
		return Math.floor(Math.random() * (max - min)) + min;
	},

	fillDropDownControl: function(node, items)
	{
		items = items || [];
		node.innerHTML = '';

		items.forEach(function(item){
			if(!item || !item.caption)
			{
				return;
			}

			var option = document.createElement('option');
			option.value = item.value;
			option.selected = !!item.selected;
			option.innerText = item.caption;
			node.appendChild(option);
		});
	},

	appendNodeByTemplate: function(container, templateId, replaceData)
	{
		var node = this.getNodeByTemplate(templateId, replaceData);
		if(node)
		{
			container.appendChild(node);
		}

		return node;
	},

	getNodeByTemplate: function(id, replaceData)
	{
		var tmpl = this.getTemplate(id, replaceData);
		if(!tmpl)
		{
			return null;
		}

		var div = BX.create('div', {html: tmpl});
		return div.firstElementChild;
	},

	getTemplate: function(id, replaceData)
	{
		var tmplNode = BX(id);
		if(!tmplNode)
		{
			return null;
		}

		var tmpl = tmplNode.innerHTML;
		if(replaceData)
		{
			for(var i in replaceData)
			{
				if(replaceData[i] === undefined)
				{
					continue;
				}

				var replaceFrom = i;
				var replaceTo = BX.util.htmlspecialchars(replaceData[i]);
				tmpl = tmpl.replace(new RegExp('%' + replaceFrom + '%','g'), replaceTo);
			}
		}

		return tmpl;
	},

	changeClass: function (node, className, isAdd)
	{
		isAdd = isAdd || false;
		if(!node)
		{
			return;
		}

		if(isAdd)
		{
			BX.addClass(node, className);
		}
		else
		{
			BX.removeClass(node, className);
		}
	},

	styleDisplay: function (node, isShow, displayValue)
	{
		isShow = isShow || false;
		displayValue = displayValue || '';
		if(!node)
		{
			return;
		}

		node.style.display = isShow ? displayValue : 'none';
	},

	overlay: {

		createOverlay: function(zIndex)
		{
			zIndex = parseInt(zIndex);
			if (!this._overlay)
			{
				var windowSize = BX.GetWindowScrollSize();
				this._overlay = document.body.appendChild(BX.create("DIV", {
					style: {
						position: 'absolute',
						top: '0px',
						left: '0px',
						zIndex: zIndex || (parseInt(this.DIV.style.zIndex)-2),
						width: windowSize.scrollWidth + "px",
						height: windowSize.scrollHeight + "px"
					}
				}));
				BX.unbind(window, 'resize', BX.proxy(this._resizeOverlay, this));
				BX.bind(window, 'resize', BX.proxy(this._resizeOverlay, this));
			}
		},
		removeOverlay: function()
		{
			if (this._overlay && this._overlay.parentNode)
			{
				this._overlay.parentNode.removeChild(this._overlay);
				BX.unbind(window, 'resize', BX.proxy(this._resizeOverlay, this));
				this._overlay = null;
			}
		},
		_resizeOverlay: function()
		{
			var windowSize = BX.GetWindowScrollSize();
			this._overlay.style.width = windowSize.scrollWidth + "px";
		}
	}
};
BX.CrmFormEditorHelper = new CrmFormEditorHelper();


if (typeof(BX.CrmProductSearchDialogWindow) === "undefined")
{
	BX.CrmProductSearchDialogWindow = function()
	{
		this._settings = {};
		this.popup = null;
		this.random = "";
		this.contentContainer = null;
		this.zIndex = 100;
		this.jsEventsManager = null;
		this.pos = null;
		this.height = 0;
		this.width = 0;
		this.resizeCorner = null;
	};

	BX.CrmProductSearchDialogWindow.prototype = {
		initialize: function (settings)
		{
			this.random = Math.random().toString().substring(2);

			this._settings = settings ? settings : {};

			var size = BX.CrmProductSearchDialogWindow.size;

			this._settings.width = size.width || this._settings.width || 1100;
			this._settings.height = size.height || this._settings.height || 530;
			this._settings.minWidth = this._settings.minWidth || 500;
			this._settings.minHeight = this._settings.minHeight || 800;
			this._settings.draggable = !!this._settings.draggable || true;
			this._settings.resizable = !!this._settings.resizable || true;
			if (typeof(this._settings.closeWindowHandler) !== "function")
				this._settings.closeWindowHandler = null;
			if (typeof(this._settings.showWindowHandler) !== "function")
				this._settings.showWindowHandler = null;

			this.jsEventsManager = BX.Crm[this._settings.jsEventsManagerId] || null;

			this.contentContainer = BX.create(
				"DIV",
				{
					attrs: {
						className: "crm-catalog",
						style: "display: block; background-color: #f3f6f7; height: " + this._settings.height +
						"px; overflow: hidden; width: " + this._settings.width + "px;"
					}
				}
			);
		},
		_handleCloseDialog: function(popup)
		{
			if(popup)
				popup.destroy();
			this.popup = null;
			if (this.jsEventsManager)
			{
				this.jsEventsManager.unregisterEventHandlers("CrmProduct_SelectSection");
			}
			if (typeof(this._settings.closeWindowHandler) === "function")
				this._settings.closeWindowHandler();
		},
		_handleAfterShowDialog: function(popup)
		{
			popup.popupContainer.style.position = "fixed";
			popup.popupContainer.style.top =
				(parseInt(popup.popupContainer.style.top) - BX.GetWindowScrollPos().scrollTop) + 'px';
			if (typeof(this._settings.showWindowHandler) === "function")
				this._settings.showWindowHandler();
		},
		setContent: function (htmlData)
		{
			if (BX.type.isString(htmlData) && BX.type.isDomNode(this.contentContainer))
				this.contentContainer.innerHTML = htmlData;
		},
		show: function ()
		{
			BX.ajax({
				method: "GET",
				dataType: 'html',
				url: this._settings.content_url,
				data: {},
				skipAuthCheck: true,
				onsuccess: BX.delegate(function(data) {
					this.setContent(data || "&nbsp;");
					this.showWindow();
				}, this),
				onfailure: BX.delegate(function() {
					if (typeof(this._settings.showWindowHandler) === "function")
						this._settings.showWindowHandler();
				}, this)
			});
		},
		showWindow: function ()
		{
			this.popup = new BX.PopupWindow(
				"CrmProductSearchDialogWindow_" + this.random,
				null,
				{
					overlay: {opacity: 82},
					autoHide: false,
					draggable: this._settings.draggable,
					offsetLeft: 0,
					offsetTop: 0,
					bindOptions: { forceBindPosition: false },
					bindOnResize: false,
					zIndex: this.zIndex - 300,
					closeByEsc: true,
					closeIcon: { top: '10px', right: '15px' },
					"titleBar":
					{
						"content": BX.create("SPAN", { "attrs":
								{ "className": "popup-window-titlebar-text" },
								"text": BX.message('CRM_WEBFORM_EDIT_JS_PRODUCT_CHOICE')
							})
					},
					events:
					{
						onPopupClose: BX.delegate(this._handleCloseDialog, this),
						onAfterPopupShow: BX.delegate(this._handleAfterShowDialog, this)
					},
					"content": this.contentContainer
				}
			);
			if (this.popup.popupContainer)
			{
				this.resizeCorner = BX.create(
					'SPAN',
					{
						attrs: {className: "bx-crm-dialog-resize"},
						events: {mousedown : BX.delegate(this.resizeWindowStart, this)}
					}
				);
				this.popup.popupContainer.appendChild(this.resizeCorner);
				if (!this._settings.resizable)
					this.resizeCorner.style.display = "none";
			}
			this.popup.show();
		},
		setResizable: function(resizable)
		{
			resizable = !!resizable;
			if (this._settings.resizable !== resizable)
			{
				this._settings.resizable = resizable;
				if (this.resizeCorner)
				{
					if (resizable)
						this.resizeCorner.style.display = "inline-block";
					else
						this.resizeCorner.style.display = "none";
				}
			}
		},
		resizeWindowStart: function(e)
		{
			if (!this._settings.resizable)
				return;

			e =  e || window.event;
			BX.PreventDefault(e);

			this.pos = BX.pos(this.contentContainer);

			BX.bind(document, "mousemove", BX.proxy(this.resizeWindowMove, this));
			BX.bind(document, "mouseup", BX.proxy(this.resizeWindowStop, this));

			if (document.body.setCapture)
				document.body.setCapture();

			try { document.onmousedown = false; } catch(e) {}
			try { document.body.onselectstart = false; } catch(e) {}
			try { document.body.ondragstart = false; } catch(e) {}
			try { document.body.style.MozUserSelect = "none"; } catch(e) {}
			try { document.body.style.cursor = "nwse-resize"; } catch(e) {}
		},
		resizeWindowMove: function(e)
		{
			var windowScroll = BX.GetWindowScrollPos();
			var x = e.clientX + windowScroll.scrollLeft;
			var y = e.clientY + windowScroll.scrollTop;

			BX.CrmProductSearchDialogWindow.size.height = this.height = Math.max(y-this.pos.top, this._settings.minHeight);
			BX.CrmProductSearchDialogWindow.size.width = this.width = Math.max(x-this.pos.left, this._settings.minWidth);

			this.contentContainer.style.height = this.height+'px';
			this.contentContainer.style.width = this.width+'px';
		},
		resizeWindowStop: function(e)
		{
			if(document.body.releaseCapture)
				document.body.releaseCapture();

			BX.unbind(document, "mousemove", BX.proxy(this.resizeWindowMove, this));
			BX.unbind(document, "mouseup", BX.proxy(this.resizeWindowStop, this));

			try { document.onmousedown = null; } catch(e) {}
			try { document.body.onselectstart = null; } catch(e) {}
			try { document.body.ondragstart = null; } catch(e) {}
			try { document.body.style.MozUserSelect = ""; } catch(e) {}
			try { document.body.style.cursor = "auto"; } catch(e) {}
		}
	};

	BX.CrmProductSearchDialogWindow.create = function(settings)
	{
		var self = new BX.CrmProductSearchDialogWindow();
		self.initialize(settings);
		return self;
	};
	BX.CrmProductSearchDialogWindow.loadCSS = function(settings)
	{
		BX.ajax({
			method: "GET",
			dataType: 'html',
			url: settings.content_url,
			data: {},
			skipAuthCheck: true
		});
	};


	BX.CrmProductSearchDialogWindow.size = {width: 0, height: 0};
}

BX.namespace("BX.Crm");
if (typeof(BX.Crm.PageEventsManagerClass) === "undefined")
{
	BX.Crm.PageEventsManagerClass = function()
	{
		this._settings = {};
	};

	BX.Crm.PageEventsManagerClass.prototype = {
		initialize: function (settings)
		{
			this._settings = settings ? settings : {};
			this.eventHandlers = {};
		},
		registerEventHandler: function(eventName, eventHandler)
		{
			if (!this.eventHandlers[eventName])
				this.eventHandlers[eventName] = [];
			this.eventHandlers[eventName].push(eventHandler);
			BX.addCustomEvent(this, eventName, eventHandler);
		},
		fireEvent: function(eventName, eventParams)
		{
			BX.onCustomEvent(this, eventName, eventParams);
		},
		unregisterEventHandlers: function(eventName)
		{
			if (this.eventHandlers[eventName])
			{
				for (var i = 0; i < this.eventHandlers[eventName].length; i++)
				{
					BX.removeCustomEvent(this, eventName, this.eventHandlers[eventName][i]);
					delete this.eventHandlers[eventName][i];
				}
			}
		}
	};

	BX.Crm.PageEventsManagerClass.create = function(settings)
	{
		var self = new BX.Crm.PageEventsManagerClass();
		self.initialize(settings);
		return self;
	};
}