CREATE TABLE B_CRM_LEAD
(
	ID int IDENTITY(1,1) NOT NULL,
	DATE_CREATE datetime NULL,
	DATE_MODIFY datetime NULL,
	CREATED_BY_ID int NOT NULL,
	MODIFY_BY_ID int NULL,
	ASSIGNED_BY_ID int NULL,
	OPENED char(1) NULL,
	CONTACT_ID int NULL,	
	STATUS_ID varchar(50) NULL,
	PRODUCT_ID varchar(50) NULL,
	STATUS_DESCRIPTION text NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	OPPORTUNITY decimal(18, 2) NULL,
	CURRENCY_ID varchar(50) NULL,
	OPPORTUNITY_ACCOUNT decimal(18, 2) NULL,
	ACCOUNT_CURRENCY_ID varchar(50) NULL,
	COMPANY_ID int NULL,
	SOURCE_ID varchar(50) NULL,
	SOURCE_DESCRIPTION text NULL,
	TITLE varchar(255) NULL,
	COMPANY_TITLE varchar(255) NULL,
	NAME varchar(50) NULL,
	LAST_NAME varchar(50) NULL,
	SECOND_NAME varchar(50) NULL,
	FULL_NAME varchar(100) NULL,
	POST varchar(255) NULL,
	ADDRESS text NULL,
	COMMENTS text NULL,
	EXCH_RATE decimal(18,2) DEFAULT 1,
	ORIGINATOR_ID varchar(255) NULL,
	ORIGIN_ID varchar(255) NULL,
	DATE_CLOSED datetime NULL,
	BIRTHDATE datetime NULL,
	BIRTHDAY_SORT int NOT NULL DEFAULT 1024,
	HONORIFIC VARCHAR(128) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL
) 
GO
ALTER TABLE B_CRM_LEAD ADD CONSTRAINT PK_B_CRM_LEAD PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CRM_CT_FULL_NAME ON B_CRM_LEAD (FULL_NAME)
GO
CREATE INDEX IX_LEAD_1 ON B_CRM_LEAD (STATUS_ID, ASSIGNED_BY_ID)
GO
CREATE INDEX IX_LEAD_2 ON B_CRM_LEAD (DATE_MODIFY)
GO
CREATE INDEX IX_LEAD_3 ON B_CRM_LEAD (CONTACT_ID)
GO
CREATE INDEX IX_LEAD_4 ON B_CRM_LEAD (COMPANY_ID)
GO
CREATE INDEX IX_LEAD_5 ON B_CRM_LEAD (BIRTHDAY_SORT, ASSIGNED_BY_ID, ID)
GO
CREATE INDEX IX_LEAD_6 ON B_CRM_LEAD (BIRTHDATE, ID)
GO
CREATE INDEX IX_LEAD_7 ON B_CRM_LEAD (HAS_PHONE, ID)
GO
CREATE INDEX IX_LEAD_8 ON B_CRM_LEAD (HAS_EMAIL, ID)
GO
CREATE INDEX IX_LEAD_9 ON B_CRM_LEAD (HAS_PHONE, HAS_EMAIL, ID)
GO
CREATE INDEX IX_LEAD_10 ON B_CRM_LEAD (STATUS_SEMANTIC_ID, ID)
GO

ALTER TABLE B_CRM_LEAD ADD CONSTRAINT DF_B_CRM_LEAD_DATE_CREATE DEFAULT GETDATE() FOR DATE_CREATE
GO
ALTER TABLE B_CRM_LEAD ADD CONSTRAINT DF_B_CRM_LEAD_DATE_MODIFY DEFAULT GETDATE() FOR DATE_MODIFY
GO
ALTER TABLE B_CRM_LEAD ADD CONSTRAINT DF_B_CRM_LEAD_OPENED DEFAULT 'N' FOR OPENED
GO
CREATE TABLE B_CRM_DEAL
(
	ID int IDENTITY(1,1) NOT NULL,
	DATE_CREATE datetime NULL,
	DATE_MODIFY datetime NULL,
	CREATED_BY_ID int NOT NULL,
	MODIFY_BY_ID int NULL,
	ASSIGNED_BY_ID int NULL,
	OPENED char(1) NULL,
	LEAD_ID int NULL,
	COMPANY_ID int NULL,
	CONTACT_ID int NULL,
	QUOTE_ID int NULL,
	TITLE varchar(255) NULL,
	PRODUCT_ID varchar(50) NULL,
	CATEGORY_ID int NULL,
	STAGE_ID varchar(50) NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	IS_NEW CHAR(1) NULL,
	CLOSED char(1) NULL,
	TYPE_ID varchar(50) NULL,
	OPPORTUNITY decimal(18, 2) NULL,
	TAX_VALUE DECIMAL(18, 2) NULL,
	CURRENCY_ID varchar(50) NULL,
	OPPORTUNITY_ACCOUNT decimal(18, 2) NULL,
	TAX_VALUE_ACCOUNT DECIMAL(18, 2) NULL,
	ACCOUNT_CURRENCY_ID varchar(50) NULL,
	PROBABILITY smallint NULL,
	COMMENTS text NULL,
	BEGINDATE datetime NULL,
	CLOSEDATE datetime NULL,
	EVENT_DATE datetime NULL,
	EVENT_ID varchar(50) NULL,
	EVENT_DESCRIPTION text NULL,
	EXCH_RATE decimal(18,2) DEFAULT 1,
	LOCATION_ID varchar(100) NULL,
	ORIGINATOR_ID varchar(255) NULL,
	ORIGIN_ID varchar(255) NULL,
	ADDITIONAL_INFO text NULL
)
GO
ALTER TABLE B_CRM_DEAL ADD CONSTRAINT PK_B_CRM_DEAL PRIMARY KEY (ID)
GO
CREATE INDEX IX_DEAL_1 ON B_CRM_DEAL (STAGE_ID, ASSIGNED_BY_ID)
GO
CREATE INDEX IX_DEAL_2 ON B_CRM_DEAL (COMPANY_ID)
GO
CREATE INDEX IX_DEAL_3 ON B_CRM_DEAL (CONTACT_ID)
GO
CREATE INDEX IX_DEAL_4 ON B_CRM_DEAL (CATEGORY_ID, ID)
GO
CREATE INDEX IX_DEAL_5 ON B_CRM_DEAL (STAGE_SEMANTIC_ID, ID)
GO
CREATE INDEX IX_DEAL_6 ON B_CRM_DEAL (IS_NEW, ID)
GO
ALTER TABLE B_CRM_DEAL ADD CONSTRAINT DF_B_CRM_DEAL_DATE_CREATE DEFAULT GETDATE() FOR DATE_CREATE
GO
ALTER TABLE B_CRM_DEAL ADD CONSTRAINT DF_B_CRM_DEAL_DATE_MODIFY DEFAULT GETDATE() FOR DATE_MODIFY
GO
ALTER TABLE B_CRM_DEAL ADD CONSTRAINT DF_B_CRM_DEAL_CLOSED DEFAULT 'N' FOR CLOSED
GO
ALTER TABLE B_CRM_DEAL ADD CONSTRAINT DF_B_CRM_DEAL_OPENED DEFAULT 'N' FOR OPENED
GO
CREATE TABLE B_CRM_CONTACT
(
	ID int IDENTITY(1,1) NOT NULL,
	LEAD_ID int NULL,
	DATE_CREATE datetime NULL,
	DATE_MODIFY datetime NULL,
	CREATED_BY_ID int NOT NULL,
	MODIFY_BY_ID int NULL,
	ASSIGNED_BY_ID int NULL,
	OPENED char(1) NULL,
	COMPANY_ID int NULL,
	SOURCE_ID varchar(50) NULL,
	SOURCE_DESCRIPTION text NULL,
	FULL_NAME varchar(100) NULL,
	NAME varchar(50) NULL,
	LAST_NAME varchar(50) NULL,
	SECOND_NAME varchar(50) NULL,
	PHOTO varchar(10) NULL,
	POST varchar(255) NULL,
	ADDRESS text NULL,
	COMMENTS text NULL,
	TYPE_ID varchar(50) NULL,	
	EXPORT char(1) NULL,
	ORIGINATOR_ID varchar(255) NULL,
	ORIGIN_ID varchar(255) NULL,
	ORIGIN_VERSION varchar(255) NULL,
	BIRTHDATE datetime NULL,
	BIRTHDAY_SORT int NOT NULL DEFAULT 1024,
	HONORIFIC VARCHAR(128) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL
) 
GO
ALTER TABLE B_CRM_CONTACT ADD CONSTRAINT PK_B_CRM_CONTACT PRIMARY KEY (ID)
GO
ALTER TABLE B_CRM_CONTACT ADD CONSTRAINT DF_B_CRM_CONTACT_EXPORT DEFAULT 'N' FOR EXPORT
GO
ALTER TABLE B_CRM_CONTACT ADD CONSTRAINT DF_B_CRM_CONTACT_DATE_CREATE DEFAULT GETDATE() FOR DATE_CREATE
GO
ALTER TABLE B_CRM_CONTACT ADD CONSTRAINT DF_B_CRM_CONTACT_DATE_MODIFY DEFAULT GETDATE() FOR DATE_MODIFY
GO
ALTER TABLE B_CRM_CONTACT ADD CONSTRAINT DF_B_CRM_CONTACT_OPENED DEFAULT 'N' FOR OPENED
GO
CREATE INDEX IX_B_CRM_CT_FULL_NAME ON B_CRM_CONTACT (FULL_NAME)
GO
CREATE INDEX IX_B_CRM_CT_LAST_NAME ON B_CRM_CONTACT (LAST_NAME)
GO
CREATE INDEX IX_B_CRM_CT_NAME ON B_CRM_CONTACT (NAME)
GO
CREATE INDEX IX_B_CRM_CT_BDAY ON B_CRM_CONTACT (BIRTHDAY_SORT, ASSIGNED_BY_ID, ID)
GO
CREATE INDEX IX_B_CRM_CT_BDATE ON B_CRM_CONTACT (BIRTHDATE, ID)
GO
CREATE INDEX IX_B_CRM_CT_COMPANY ON B_CRM_CONTACT (COMPANY_ID, ID)
GO
CREATE INDEX IX_B_CRM_CT_PHONE ON B_CRM_CONTACT (HAS_PHONE, ID)
GO
CREATE INDEX IX_B_CRM_CT_EMAIL ON B_CRM_CONTACT (HAS_EMAIL, ID)
GO
CREATE INDEX IX_B_CRM_CT_COMM ON B_CRM_CONTACT (HAS_PHONE, HAS_EMAIL, ID)
GO

CREATE TABLE B_CRM_COMPANY
(
	ID int IDENTITY(1,1) NOT NULL,
	LEAD_ID int NULL,
	DATE_CREATE datetime NULL,
	DATE_MODIFY datetime NULL,
	CREATED_BY_ID int NOT NULL,
	MODIFY_BY_ID int NULL,
	ASSIGNED_BY_ID int NULL,
	OPENED char(1) NULL,
	TITLE varchar(255) NULL,
	LOGO varchar(10) NULL,
	ADDRESS text NULL,
	ADDRESS_LEGAL text NULL,
	BANKING_DETAILS text NULL,
	COMMENTS text NULL,	
	COMPANY_TYPE varchar(50) NULL,
	INDUSTRY varchar(50) NULL,
	REVENUE varchar(255) NULL,
	EMPLOYEES varchar(50) NULL,
	CURRENCY_ID varchar(50) NULL,
	ORIGINATOR_ID varchar(255) NULL,
	ORIGIN_ID varchar(255) NULL,
	ORIGIN_VERSION varchar(255) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL,
	IS_MY_COMPANY CHAR(1) NOT NULL DEFAULT 'N'
) 
GO
ALTER TABLE B_CRM_COMPANY ADD CONSTRAINT PK_B_CRM_COMPANY PRIMARY KEY (ID)
GO
ALTER TABLE B_CRM_COMPANY ADD CONSTRAINT DF_B_CRM_COMPANY_DATE_CREATE DEFAULT GETDATE() FOR DATE_CREATE
GO
ALTER TABLE B_CRM_COMPANY ADD CONSTRAINT DF_B_CRM_COMPANY_DATE_MODIFY DEFAULT GETDATE() FOR DATE_MODIFY
GO
ALTER TABLE B_CRM_COMPANY ADD CONSTRAINT DF_B_CRM_COMPANY_OPENED DEFAULT 'N' FOR OPENED
GO
CREATE INDEX IX_B_CRM_CY_TITLE ON B_CRM_COMPANY (TITLE)
GO
CREATE INDEX IX_B_CRM_CY_PHONE ON B_CRM_COMPANY (HAS_PHONE, ID)
GO
CREATE INDEX IX_B_CRM_CY_EMAIL ON B_CRM_COMPANY (HAS_EMAIL, ID)
GO
CREATE INDEX IX_B_CRM_CY_COMM ON B_CRM_COMPANY (HAS_PHONE, HAS_EMAIL, ID)
GO
CREATE INDEX IX_B_CRM_CY_ISMY ON B_CRM_COMPANY (IS_MY_COMPANY)
GO
CREATE TABLE B_CRM_STATUS
(
	ID int IDENTITY(1,1) NOT NULL,
	ENTITY_ID varchar(50) NOT NULL,
	STATUS_ID varchar(50) NOT NULL,
	NAME varchar(100) NOT NULL,
	NAME_INIT varchar(100) NULL,
	SORT int NOT NULL,
	SYSTEM char NOT NULL
)
GO
ALTER TABLE B_CRM_STATUS ADD CONSTRAINT PK_B_CRM_STATUS PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CRM_STATUS ON B_CRM_STATUS (STATUS_ID, ENTITY_ID)
GO

CREATE TABLE B_CRM_FIELD_MULTI
(
	ID int IDENTITY(1,1) NOT NULL,
	ENTITY_ID varchar(16) NOT NULL,
	ELEMENT_ID int NOT NULL,
	TYPE_ID varchar(16) NOT NULL,
	VALUE_TYPE varchar(50) NOT NULL,
	COMPLEX_ID varchar(100) NOT NULL,
	VALUE varchar(250) NOT NULL
)
GO
ALTER TABLE B_CRM_FIELD_MULTI ADD CONSTRAINT PK_B_CRM_FIELD_MULTI PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CRM_FM_EE ON B_CRM_FIELD_MULTI (ENTITY_ID, ELEMENT_ID)
GO
CREATE INDEX IX_B_CRM_FM_2 ON B_CRM_FIELD_MULTI (ENTITY_ID, TYPE_ID, ELEMENT_ID, ID)
GO
CREATE INDEX IX_B_CRM_FM_3 ON B_CRM_FIELD_MULTI (TYPE_ID, VALUE, ENTITY_ID, ELEMENT_ID)
GO

CREATE TABLE B_CRM_EVENT
(
	ID int IDENTITY(1,1) NOT NULL,
	DATE_CREATE datetime NULL,
	CREATED_BY_ID int NOT NULL,
	EVENT_ID varchar(50) NULL,
	EVENT_NAME varchar(255) NOT NULL,
	EVENT_TEXT_1 text NULL,
	EVENT_TEXT_2 text NULL,
	EVENT_TYPE int NULL,
	FILES text NULL
)
GO
ALTER TABLE B_CRM_EVENT ADD CONSTRAINT PK_B_CRM_EVENT PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CRM_EVENT_1 ON B_CRM_EVENT (DATE_CREATE)
GO
CREATE INDEX IX_B_CRM_EVENT_2 ON B_CRM_EVENT (EVENT_TYPE, EVENT_ID, ID)
GO
CREATE INDEX IX_B_CRM_EVENT_3 ON B_CRM_EVENT (CREATED_BY_ID, ID)
GO
CREATE INDEX IX_B_CRM_EVENT_4 ON B_CRM_EVENT (EVENT_TYPE, DATE_CREATE)
GO

CREATE TABLE B_CRM_EVENT_RELATIONS
(
	ID int IDENTITY(1,1) NOT NULL,
	ASSIGNED_BY_ID int NOT NULL,
	ENTITY_TYPE varchar(50) NULL,
	ENTITY_ID int NOT NULL,
	ENTITY_FIELD varchar(255) NULL,
	EVENT_ID int NOT NULL
)
GO
ALTER TABLE B_CRM_EVENT_RELATIONS ADD CONSTRAINT PK_B_CRM_EVENT_RELATIONS PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_CRM_EVENT_REL ON B_CRM_EVENT_RELATIONS (ENTITY_TYPE, ENTITY_ID, ENTITY_FIELD)
GO
CREATE INDEX IX_EVENT_REL_1 ON B_CRM_EVENT_RELATIONS (EVENT_ID)
GO
CREATE INDEX IX_EVENT_REL_2 ON B_CRM_EVENT_RELATIONS (ENTITY_TYPE, ENTITY_ID, EVENT_ID)
GO

CREATE TABLE B_CRM_ENTITY_LOCK
(
	ENTITY_ID int NOT NULL,
	ENTITY_TYPE VARCHAR (10) NOT NULL,
	DATE_LOCK datetime NULL,
	LOCKED_BY VARCHAR (32) NULL
)
GO
ALTER TABLE B_CRM_ENTITY_LOCK ADD CONSTRAINT PK_B_CRM_ENTITY_LOCK PRIMARY KEY (ENTITY_ID)
GO
CREATE TABLE B_CRM_ENTITY_PERMS
(
	ID INT NOT NULL IDENTITY(1,1),
	ENTITY VARCHAR(20) NOT NULL,
	ENTITY_ID INT NOT NULL,
	ATTR VARCHAR(30) NOT NULL
)
GO
ALTER TABLE B_CRM_ENTITY_PERMS ADD CONSTRAINT PK_B_CRM_ENTITY_PERMS PRIMARY KEY (ID)
GO
CREATE INDEX IX_ENTITY_ATTR ON B_CRM_ENTITY_PERMS (ENTITY, ENTITY_ID, ATTR)
GO
CREATE INDEX IX_ENTITY_ATTR_1 ON B_CRM_ENTITY_PERMS (ENTITY, ATTR, ENTITY_ID)
GO

CREATE TABLE B_CRM_ROLE
(
	ID INT NOT NULL IDENTITY(1,1),
	NAME VARCHAR(255) NOT NULL
)
GO
ALTER TABLE B_CRM_ROLE ADD CONSTRAINT PK_B_CRM_ROLE PRIMARY KEY (ID)
GO
CREATE TABLE B_CRM_ROLE_PERMS
(
	ID INT NOT NULL IDENTITY(1,1),
	ROLE_ID INT NOT NULL,
	ENTITY VARCHAR(20) NOT NULL,
	FIELD VARCHAR(30),
	FIELD_VALUE VARCHAR(255),
	PERM_TYPE VARCHAR(20) NOT NULL,
	ATTR CHAR(1)
)
GO
ALTER TABLE B_CRM_ROLE_PERMS ADD CONSTRAINT PK_B_CRM_ROLE_PERMS PRIMARY KEY (ID)
GO
CREATE INDEX IX_ROLE_ID ON B_CRM_ROLE_PERMS (ROLE_ID)
GO
ALTER TABLE B_CRM_ROLE_PERMS ADD CONSTRAINT DF_B_CRM_ROLE_PERMS_FIELD DEFAULT '-' FOR FIELD
GO
ALTER TABLE B_CRM_ROLE_PERMS ADD CONSTRAINT DF_B_CRM_ROLE_PERMS_FIELD_VALUE DEFAULT NULL FOR FIELD_VALUE
GO
ALTER TABLE B_CRM_ROLE_PERMS ADD CONSTRAINT DF_B_CRM_ROLE_PERMS_ATTR DEFAULT '' FOR ATTR
GO
CREATE TABLE B_CRM_ROLE_RELATION
(
	ID INT NOT NULL IDENTITY(1,1),
	ROLE_ID INT NOT NULL,
	RELATION VARCHAR(100) NOT NULL
)
GO
ALTER TABLE B_CRM_ROLE_RELATION ADD CONSTRAINT PK_B_CRM_ROLE_RELATION PRIMARY KEY (ID)
GO
CREATE INDEX IX_ROLE_RELATION ON B_CRM_ROLE_RELATION (ROLE_ID, RELATION)
GO

IF OBJECT_ID(N'B_CRM_EXTERNAL_SALE', N'U') IS NULL
	CREATE TABLE B_CRM_EXTERNAL_SALE
	(
		ID INT NOT NULL IDENTITY(1,1),
		ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
		DATE_CREATE DATETIME NOT NULL,
		DATE_UPDATE DATETIME NOT NULL,
		NAME VARCHAR(128) NULL,
		SCHEME VARCHAR(5) NOT NULL DEFAULT 'http',
		SERVER VARCHAR(192) NOT NULL,
		PORT INT NOT NULL DEFAULT 80,
		LOGIN VARCHAR(64) NOT NULL,
		PASSWORD VARCHAR(128) NOT NULL,
		MODIFICATION_LABEL INT NULL,
		IMPORT_SIZE INT NULL,
		IMPORT_PERIOD INT NULL,
		IMPORT_PROBABILITY INT NULL,
		IMPORT_RESPONSIBLE INT NULL,
		IMPORT_PUBLIC CHAR(1) NULL,
		IMPORT_PREFIX VARCHAR(128) NULL,
		IMPORT_ERRORS INT NULL,
		IMPORT_GROUP_ID INT NULL,
		COOKIE TEXT NULL,
		LAST_STATUS TEXT NULL,
		LAST_STATUS_DATE DATETIME NULL,
		SYNC_DATA TEXT NULL,
		CONSTRAINT PK_B_CRM_EXTERNAL_SALE PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_CATALOG', N'U') IS NULL
	CREATE TABLE B_CRM_CATALOG
	(
	ID INT NOT NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	CONSTRAINT PK_B_CRM_CATALOG PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_PRODUCT', N'U') IS NULL
	CREATE TABLE B_CRM_PRODUCT
	(
	ID INT NOT NULL,
	CATALOG_ID INT NOT NULL,
	CURRENCY_ID VARCHAR(50) NOT NULL,
	PRICE DECIMAL(18,2) NOT NULL DEFAULT 0,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	CONSTRAINT PK_B_CRM_PRODUCT PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_PRODUCT_ROW', N'U') IS NULL
	CREATE TABLE B_CRM_PRODUCT_ROW
	(
	ID INT IDENTITY(1,1) NOT NULL,
	OWNER_ID INT NOT NULL,
	OWNER_TYPE CHAR(3) NOT NULL,
	PRODUCT_ID INT NOT NULL,
	PRODUCT_NAME VARCHAR(256) NULL,
	PRICE DECIMAL(18,2) NOT NULL,
	PRICE_ACCOUNT DECIMAL(18,2) NOT NULL DEFAULT 0,
	PRICE_EXCLUSIVE DECIMAL(18,2) NULL,
	PRICE_NETTO DECIMAL(18,2) NULL,
	PRICE_BRUTTO DECIMAL(18,2) NULL,
	QUANTITY DECIMAL(18,4) NOT NULL,
	DISCOUNT_TYPE_ID SMALLINT NULL,
	DISCOUNT_RATE DECIMAL(18,2) NULL,
	DISCOUNT_SUM DECIMAL(18,2) NULL,
	TAX_RATE DECIMAL(18,2) NULL,
	TAX_INCLUDED CHAR(1) NULL,
	CUSTOMIZED CHAR(1) NULL,
	MEASURE_CODE INT NULL,
	MEASURE_NAME VARCHAR(50) NULL,
	SORT INT NULL,
	CONSTRAINT PK_B_CRM_PRODUCT_ROW PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_PRODUCT_ROW_CFG', N'U') IS NULL
	CREATE TABLE B_CRM_PRODUCT_ROW_CFG
	(
	OWNER_ID INT NOT NULL,
	OWNER_TYPE CHAR(3) NOT NULL,
	SETTINGS TEXT,
	CONSTRAINT PK_B_CRM_PRODUCT_ROW_CFG PRIMARY KEY CLUSTERED(OWNER_ID ASC, OWNER_TYPE ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_PROD_ROW_1')
	CREATE INDEX IX_B_CRM_PROD_ROW_1 ON B_CRM_PRODUCT_ROW (OWNER_ID ASC, OWNER_TYPE ASC, SORT ASC, ID ASC)
GO

IF OBJECT_ID(N'B_CRM_ACT', N'U') IS NULL
	CREATE TABLE B_CRM_ACT
	(
		ID INT IDENTITY(1,1) NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		PROVIDER_ID VARCHAR(100) NULL,
		PROVIDER_TYPE_ID VARCHAR(100) NULL,
		OWNER_ID INT NOT NULL,
		OWNER_TYPE_ID SMALLINT NOT NULL,
		ASSOCIATED_ENTITY_ID INT,
		CALENDAR_EVENT_ID INT NOT NULL DEFAULT 0,
		SUBJECT VARCHAR(512) NOT NULL,
		COMPLETED CHAR(1) NOT NULL DEFAULT 'N',
		RESPONSIBLE_ID INT NOT NULL,
		PRIORITY INT NOT NULL,
		NOTIFY_TYPE INT NOT NULL,
		NOTIFY_VALUE INT,
		DESCRIPTION TEXT NULL,
		DESCRIPTION_TYPE SMALLINT NULL,
		DIRECTION SMALLINT NOT NULL,
		LOCATION VARCHAR(256),
		CREATED DATETIME NOT NULL,
		LAST_UPDATED DATETIME NOT NULL,
		START_TIME DATETIME,
		END_TIME DATETIME,
		DEADLINE DATETIME,
		STORAGE_TYPE_ID SMALLINT NULL,
		STORAGE_ELEMENT_IDS TEXT NULL,
		PARENT_ID INT NOT NULL DEFAULT 0,
		URN VARCHAR(64) NULL,
		SETTINGS TEXT NULL,
		ORIGINATOR_ID varchar(255) NULL,
		ORIGIN_ID VARCHAR(255) NULL,
		AUTHOR_ID INT NULL,
		EDITOR_ID INT NULL,
		PROVIDER_PARAMS TEXT NULL,
		PROVIDER_DATA TEXT NULL,
		RESULT_STATUS INT NOT NULL DEFAULT 0,
		RESULT_STREAM INT NOT NULL DEFAULT 0,
		RESULT_SOURCE_ID VARCHAR(255) NULL,
		RESULT_MARK INT NOT NULL DEFAULT 0,
		RESULT_VALUE decimal(18, 4) NULL,
		RESULT_SUM decimal(18, 4) NULL,
		RESULT_CURRENCY_ID CHAR(3) NULL,
		CONSTRAINT PK_B_CRM_ACT PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT')
	CREATE INDEX IX_B_CRM_ACT ON B_CRM_ACT (ID ASC, PARENT_ID ASC, OWNER_ID ASC, OWNER_TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_1')
	CREATE INDEX IX_B_CRM_ACT_1 ON B_CRM_ACT (RESPONSIBLE_ID ASC, COMPLETED ASC, DEADLINE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_2')
	CREATE INDEX IX_B_CRM_ACT_2 ON B_CRM_ACT (ASSOCIATED_ENTITY_ID ASC, TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_4')
	CREATE INDEX IX_B_CRM_ACT_4 ON B_CRM_ACT (DEADLINE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_5')
	CREATE INDEX IX_B_CRM_ACT_5 ON B_CRM_ACT (ORIGIN_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_6')
	CREATE INDEX IX_B_CRM_ACT_6 ON B_CRM_ACT (TYPE_ID ASC, COMPLETED ASC, DEADLINE ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_7')
	CREATE INDEX IX_B_CRM_ACT_7 ON B_CRM_ACT (OWNER_ID ASC, OWNER_TYPE_ID ASC, TYPE_ID ASC, COMPLETED ASC, DEADLINE ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_8')
	CREATE INDEX IX_B_CRM_ACT_8 ON B_CRM_ACT (PROVIDER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_9')
	CREATE INDEX IX_B_CRM_ACT_9 ON B_CRM_ACT (CALENDAR_EVENT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_10')
	CREATE INDEX IX_B_CRM_ACT_10 ON B_CRM_ACT (URN ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_11')
	CREATE INDEX IX_B_CRM_ACT_11 ON B_CRM_ACT (PARENT_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_ACT_BIND', N'U') IS NULL
	CREATE TABLE B_CRM_ACT_BIND
	(
		ID INT IDENTITY(1,1) NOT NULL,
		ACTIVITY_ID INT NOT NULL,
		OWNER_ID INT NOT NULL,
		OWNER_TYPE_ID SMALLINT NOT NULL,
		CONSTRAINT PK_B_CRM_ACT_BIND PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_BIND_1')
	CREATE INDEX IX_B_CRM_ACT_BIND_1 ON B_CRM_ACT_BIND (ACTIVITY_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_BIND_2')
	CREATE INDEX IX_B_CRM_ACT_BIND_2 ON B_CRM_ACT_BIND (OWNER_ID ASC, OWNER_TYPE_ID ASC, ACTIVITY_ID ASC, ID ASC)
GO

IF OBJECT_ID(N'B_CRM_ACT_COMM', N'U') IS NULL
	CREATE TABLE B_CRM_ACT_COMM
	(
		ID INT IDENTITY(1,1) NOT NULL,
		ACTIVITY_ID INT NOT NULL,
		OWNER_ID INT NOT NULL,
		OWNER_TYPE_ID SMALLINT NOT NULL,
		TYPE VARCHAR(64),
		VALUE VARCHAR(256),
		ENTITY_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_SETTINGS TEXT,
		CONSTRAINT PK_B_CRM_ACT_COMM PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_COMM')
	CREATE INDEX IX_B_CRM_ACT_COMM ON B_CRM_ACT_COMM (ACTIVITY_ID ASC, OWNER_ID ASC, OWNER_TYPE_ID ASC, ENTITY_ID ASC, ENTITY_TYPE_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_COMM_2')
	CREATE INDEX IX_B_CRM_ACT_COMM_2 ON B_CRM_ACT_COMM (ENTITY_ID ASC, ENTITY_TYPE_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_COMM_3')
	CREATE INDEX IX_B_CRM_ACT_COMM_3 ON B_CRM_ACT_COMM (OWNER_ID ASC, OWNER_TYPE_ID ASC, TYPE ASC)
GO

IF OBJECT_ID(N'B_CRM_ACT_ELEM', N'U') IS NULL
	CREATE TABLE B_CRM_ACT_ELEM
	(
		ACTIVITY_ID INT NOT NULL,
		STORAGE_TYPE_ID SMALLINT NOT NULL,
		ELEMENT_ID INT NOT NULL,
		CONSTRAINT PK_B_CRM_ACT_ELEM PRIMARY KEY CLUSTERED(ACTIVITY_ID ASC, STORAGE_TYPE_ID ASC, ELEMENT_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_USR_ACT', N'U') IS NULL
	CREATE TABLE B_CRM_USR_ACT
	(
		USER_ID INT NOT NULL,
		OWNER_ID INT NOT NULL,
		OWNER_TYPE_ID SMALLINT NOT NULL,
		ACTIVITY_TIME DATETIME NOT NULL,
		ACTIVITY_ID INT NOT NULL,
		DEPARTMENT_ID INT NOT NULL,
		SORT VARCHAR(16) NULL,
		CONSTRAINT PK_B_CRM_USR_ACT PRIMARY KEY CLUSTERED(USER_ID ASC, OWNER_ID ASC, OWNER_TYPE_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_USR_ACT')
	CREATE INDEX IX_B_CRM_USR_ACT ON B_CRM_USR_ACT (USER_ID ASC, OWNER_ID ASC, OWNER_TYPE_ID ASC, ACTIVITY_TIME ASC, ACTIVITY_ID ASC, DEPARTMENT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_USR_ACT_2')
	CREATE INDEX IX_B_CRM_USR_ACT_2 ON B_CRM_USR_ACT (OWNER_ID ASC, OWNER_TYPE_ID ASC, USER_ID DESC, SORT DESC)
GO

IF OBJECT_ID(N'B_CRM_USR_MT', N'U') IS NULL
	CREATE TABLE B_CRM_USR_MT
	(
		ID INT IDENTITY(1,1) NOT NULL,
		OWNER_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		SCOPE SMALLINT NOT NULL,
		IS_ACTIVE CHAR(1) NOT NULL DEFAULT 'N',
		TITLE VARCHAR(128),
		EMAIL_FROM VARCHAR(255),
		SUBJECT VARCHAR(255),
		BODY TEXT,
		SING_REQUIRED CHAR(1) NOT NULL DEFAULT 'N',
		SORT INT NOT NULL DEFAULT 100,
		CREATED DATETIME NOT NULL,
		LAST_UPDATED DATETIME NOT NULL,
		AUTHOR_ID INT NOT NULL,
		EDITOR_ID INT NOT NULL,
		CONSTRAINT PK_B_CRM_USR_MT PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_USR_MT')
	CREATE INDEX IX_B_CRM_USR_MT ON B_CRM_USR_MT (OWNER_ID ASC, ENTITY_TYPE_ID ASC, SCOPE ASC, IS_ACTIVE ASC)
GO

IF OBJECT_ID(N'B_CRM_SL_REL', N'U') IS NULL
	CREATE TABLE B_CRM_SL_REL
	(
		SL_ID INT NOT NULL,
		SL_EVENT_ID VARCHAR(50) NOT NULL,
		SL_ENTITY_TYPE VARCHAR(50) NOT NULL,
		SL_LAST_UPDATED DATETIME,
		ENTITY_ID INT NOT NULL,
		SL_PARENT_ENTITY_TYPE VARCHAR(50) NOT NULL,
		PARENT_ENTITY_ID INT NOT NULL,
		LVL SMALLINT NOT NULL DEFAULT 1,
		TYPE_ID SMALLINT NOT NULL DEFAULT 1
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_SL_REL_1')
	CREATE INDEX IX_B_CRM_SL_REL_1 ON B_CRM_SL_REL (SL_PARENT_ENTITY_TYPE, PARENT_ENTITY_ID, LVL, SL_ENTITY_TYPE, SL_EVENT_ID, SL_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_SL_REL_2')
	CREATE INDEX IX_B_CRM_SL_REL_2 ON B_CRM_SL_REL (SL_ENTITY_TYPE, ENTITY_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_SL_REL_3')
	CREATE INDEX IX_B_CRM_SL_REL_3 ON B_CRM_SL_REL (SL_ID, TYPE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_SL_REL_4')
	CREATE INDEX IX_B_CRM_SL_REL_4 ON B_CRM_SL_REL (SL_LAST_UPDATED, SL_PARENT_ENTITY_TYPE, PARENT_ENTITY_ID, LVL, SL_ENTITY_TYPE, SL_EVENT_ID, SL_ID)
GO

IF OBJECT_ID(N'B_CRM_SL_SUBSCR', N'U') IS NULL
	CREATE TABLE B_CRM_SL_SUBSCR
	(
		USER_ID INT NOT NULL,
		SL_ENTITY_TYPE VARCHAR(50) NOT NULL,
		ENTITY_ID INT NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		CONSTRAINT PK_B_CRM_SL_SUBSCR PRIMARY KEY CLUSTERED(USER_ID ASC, SL_ENTITY_TYPE ASC, ENTITY_ID ASC, TYPE_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_SL_SUBSCR_1')
	CREATE INDEX IX_B_CRM_SL_SUBSCR_1 ON B_CRM_SL_SUBSCR (SL_ENTITY_TYPE, ENTITY_ID, TYPE_ID)
GO

IF OBJECT_ID(N'B_CRM_BIZ_TYPE', N'U') IS NULL
	CREATE TABLE B_CRM_BIZ_TYPE(
		CODE VARCHAR(32) NOT NULL,
		NAME VARCHAR(256) NOT NULL,
		LANG VARCHAR(2),
		CONSTRAINT PK_B_CRM_BIZ_TYPE PRIMARY KEY CLUSTERED(CODE ASC)
	);
GO

IF OBJECT_ID(N'B_CRM_DP_ORG_MCD', N'U') IS NULL
	CREATE TABLE B_CRM_DP_ORG_MCD(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		TITLE VARCHAR(256) NOT NULL,
		CONSTRAINT PK_B_CRM_DP_ORG_MCD PRIMARY KEY CLUSTERED(ID ASC)
	);
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_ORG_MCD_1')
	CREATE INDEX IX_B_CRM_DP_ORG_MCD_1 ON B_CRM_DP_ORG_MCD (TITLE, ENTITY_TYPE_ID, ENTITY_ID, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_ORG_MCD_2')
	CREATE INDEX IX_B_CRM_DP_ORG_MCD_2 ON B_CRM_DP_ORG_MCD (ENTITY_TYPE_ID, ENTITY_ID, ID)
GO

IF OBJECT_ID(N'B_CRM_DP_COMM_MCD', N'U') IS NULL
	CREATE TABLE B_CRM_DP_COMM_MCD(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		TYPE VARCHAR(32) NOT NULL,
		VALUE VARCHAR(256) NOT NULL,
		CONSTRAINT PK_B_CRM_DP_COMM_MCD PRIMARY KEY CLUSTERED(ID ASC)
	);
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_COMM_MCD_1')
	CREATE INDEX IX_B_CRM_DP_COMM_MCD_1 ON B_CRM_DP_COMM_MCD (TYPE, VALUE, ENTITY_TYPE_ID, ENTITY_ID, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_COMM_MCD_2')
	CREATE INDEX IX_B_CRM_DP_COMM_MCD_2 ON B_CRM_DP_COMM_MCD (ENTITY_TYPE_ID, ENTITY_ID, TYPE, ID)
GO

IF OBJECT_ID(N'B_CRM_DP_PRSN_MCD', N'U') IS NULL
	CREATE TABLE B_CRM_DP_PRSN_MCD(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		LAST_NAME VARCHAR (64) NOT NULL,
		NAME VARCHAR (64),
		SECOND_NAME VARCHAR (64),
		CONSTRAINT PK_B_CRM_DP_PRSN_MCD PRIMARY KEY CLUSTERED(ID ASC)
	);
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_PRSN_MCD_1')
	CREATE INDEX IX_B_CRM_DP_PRSN_MCD_1 ON B_CRM_DP_PRSN_MCD (LAST_NAME, NAME, SECOND_NAME, ENTITY_TYPE_ID, ENTITY_ID, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_PRSN_MCD_2')
	CREATE INDEX IX_B_CRM_DP_PRSN_MCD_2 ON B_CRM_DP_PRSN_MCD (ENTITY_TYPE_ID, ENTITY_ID, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_PRSN_MCD_3')
	CREATE INDEX IX_B_CRM_DP_PRSN_MCD_3 ON B_CRM_DP_PRSN_MCD (LAST_NAME, ENTITY_TYPE_ID, ENTITY_ID, NAME, SECOND_NAME, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_PRSN_MCD_4')
	CREATE INDEX IX_B_CRM_DP_PRSN_MCD_4 ON B_CRM_DP_PRSN_MCD (ENTITY_TYPE_ID, ENTITY_ID, LAST_NAME, NAME, SECOND_NAME, ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_PRSN_MCD_5')
	CREATE INDEX IX_B_CRM_DP_PRSN_MCD_5 ON B_CRM_DP_PRSN_MCD (ENTITY_TYPE_ID, LAST_NAME, NAME, SECOND_NAME)
GO

IF OBJECT_ID(N'B_CRM_QUOTE', N'U') IS NULL
	CREATE TABLE B_CRM_QUOTE
	(
		ID INT IDENTITY(1,1) NOT NULL,
		DATE_CREATE DATETIME NOT NULL,
		DATE_MODIFY DATETIME NOT NULL,
		CREATED_BY_ID INT NOT NULL,
		MODIFY_BY_ID INT NULL,
		ASSIGNED_BY_ID INT NULL,
		OPENED CHAR(1) NULL,
		LEAD_ID INT NULL,
		DEAL_ID INT NULL,
		COMPANY_ID INT NULL,
		CONTACT_ID INT NULL,
		PERSON_TYPE_ID INT NOT NULL,
		MYCOMPANY_ID INT NULL,
		TITLE VARCHAR(255) NULL,
		STATUS_ID VARCHAR(50) NULL,
		CLOSED CHAR(1) NULL,
		OPPORTUNITY DECIMAL(18, 2) NULL,
		TAX_VALUE DECIMAL(18, 2) NULL,
		CURRENCY_ID VARCHAR(50) NULL,
		OPPORTUNITY_ACCOUNT DECIMAL(18, 2) NULL,
		TAX_VALUE_ACCOUNT DECIMAL(18, 2) NULL,
		ACCOUNT_CURRENCY_ID VARCHAR(50) NULL,
		COMMENTS TEXT NULL,
		COMMENTS_TYPE SMALLINT NULL,
		BEGINDATE DATETIME,
		CLOSEDATE DATETIME,
		EXCH_RATE DECIMAL(18,2) DEFAULT 1,
		QUOTE_NUMBER VARCHAR(100) NULL,
		CONTENT TEXT NULL,
		CONTENT_TYPE SMALLINT NULL,
		TERMS TEXT NULL,
		TERMS_TYPE SMALLINT NULL,
		STORAGE_TYPE_ID SMALLINT NULL,
		STORAGE_ELEMENT_IDS TEXT NULL,
		LOCATION_ID varchar(100) NULL,
		CLIENT_TITLE VARCHAR(255) NULL,
		CLIENT_ADDR VARCHAR(255) NULL,
		CLIENT_CONTACT VARCHAR(255) NULL,
		CLIENT_EMAIL VARCHAR(255) NULL,
		CLIENT_PHONE VARCHAR(255) NULL,
		CLIENT_TP_ID VARCHAR(255) NULL,
		CLIENT_TPA_ID VARCHAR(255) NULL,
		CONSTRAINT PK_B_CRM_QUOTE PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_QUOTE_1')
	CREATE INDEX IX_QUOTE_1 ON B_CRM_QUOTE (STATUS_ID ASC, ASSIGNED_BY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_QUOTE_2')
	CREATE INDEX IX_QUOTE_2 ON B_CRM_QUOTE (COMPANY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_QUOTE_3')
	CREATE INDEX IX_QUOTE_3 ON B_CRM_QUOTE (CONTACT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_QUOTE_4')
	CREATE INDEX IX_QUOTE_4 ON B_CRM_QUOTE (DEAL_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_QUOTE_5')
	CREATE INDEX IX_QUOTE_5 ON B_CRM_QUOTE (LEAD_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IXS_QUOTE_NUMBER')
	CREATE UNIQUE INDEX IXS_QUOTE_NUMBER ON B_CRM_QUOTE(QUOTE_NUMBER)
GO

IF OBJECT_ID(N'B_CRM_QUOTE_ELEM', N'U') IS NULL
	CREATE TABLE B_CRM_QUOTE_ELEM
	(
		QUOTE_ID INT NOT NULL,
		STORAGE_TYPE_ID SMALLINT NOT NULL,
		ELEMENT_ID INT NOT NULL,
		CONSTRAINT PK_B_CRM_QUOTE_ELEM PRIMARY KEY CLUSTERED(QUOTE_ID ASC, STORAGE_TYPE_ID ASC, ELEMENT_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_DP_INDEX', N'U') IS NULL
	CREATE TABLE B_CRM_DP_INDEX(
		USER_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		MATCH_HASH VARCHAR(32) NOT NULL,
		MATCHES TEXT NULL,
		QUANTITY INT NOT NULL,
		ROOT_ENTITY_ID INT NOT NULL,
		ROOT_ENTITY_NAME_FLAG CHAR(1) NOT NULL,
		ROOT_ENTITY_NAME VARCHAR(256) NOT NULL,
		ROOT_ENTITY_TITLE_FLAG CHAR(1) NOT NULL,
		ROOT_ENTITY_TITLE VARCHAR(256) NOT NULL,
		ROOT_ENTITY_PHONE_FLAG CHAR(1) NOT NULL,
		ROOT_ENTITY_PHONE VARCHAR(256) NOT NULL,
		ROOT_ENTITY_EMAIL_FLAG CHAR(1) NOT NULL,
		ROOT_ENTITY_EMAIL VARCHAR(256) NOT NULL,
		IS_JUNK CHAR(1) NULL,
		CONSTRAINT PK_B_CRM_DP_IND PRIMARY KEY CLUSTERED(USER_ID ASC, ENTITY_TYPE_ID ASC, TYPE_ID ASC, MATCH_HASH ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_1')
	CREATE INDEX IX_B_CRM_DP_IND_1 ON B_CRM_DP_INDEX (USER_ID ASC, ENTITY_TYPE_ID ASC, ROOT_ENTITY_NAME_FLAG ASC, ROOT_ENTITY_NAME ASC, TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_2')
	CREATE INDEX IX_B_CRM_DP_IND_2 ON B_CRM_DP_INDEX (USER_ID ASC, ENTITY_TYPE_ID ASC, ROOT_ENTITY_TITLE_FLAG ASC, ROOT_ENTITY_TITLE ASC, TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_3')
	CREATE INDEX IX_B_CRM_DP_IND_3 ON B_CRM_DP_INDEX (USER_ID ASC, ENTITY_TYPE_ID ASC, ROOT_ENTITY_PHONE_FLAG ASC, ROOT_ENTITY_PHONE ASC, TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_4')
	CREATE INDEX IX_B_CRM_DP_IND_4 ON B_CRM_DP_INDEX (USER_ID ASC, ENTITY_TYPE_ID ASC, ROOT_ENTITY_EMAIL_FLAG ASC, ROOT_ENTITY_EMAIL ASC, TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_5')
	CREATE INDEX IX_B_CRM_DP_IND_5 ON B_CRM_DP_INDEX (MATCH_HASH ASC, TYPE_ID ASC, ENTITY_TYPE_ID ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_6')
	CREATE INDEX IX_B_CRM_DP_IND_6 ON B_CRM_DP_INDEX (ROOT_ENTITY_ID ASC, ENTITY_TYPE_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_DP_INDEX_MISMATCH', N'U') IS NULL
	CREATE TABLE B_CRM_DP_INDEX_MISMATCH(
		USER_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		MATCH_HASH VARCHAR(32) NOT NULL,
		L_ENTITY_ID INT NOT NULL,
		R_ENTITY_ID INT NOT NULL,
		CONSTRAINT PK_B_CRM_DP_IND_MISM PRIMARY KEY CLUSTERED(USER_ID ASC, ENTITY_TYPE_ID ASC, TYPE_ID ASC, MATCH_HASH ASC, L_ENTITY_ID ASC, R_ENTITY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_MISM_1')
	CREATE INDEX IX_B_CRM_DP_IND_MISM_1 ON B_CRM_DP_INDEX_MISMATCH (L_ENTITY_ID ASC, ENTITY_TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_IND_MISM_2')
	CREATE INDEX IX_B_CRM_DP_IND_MISM_2 ON B_CRM_DP_INDEX_MISMATCH (R_ENTITY_ID ASC, ENTITY_TYPE_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_DP_ENTITY_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_DP_ENTITY_STAT(
		ENTITY_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		RANKING_DATA VARCHAR(512) NULL,
		CONSTRAINT PK_B_CRM_DP_ENTITY_STAT PRIMARY KEY CLUSTERED(ENTITY_ID ASC, ENTITY_TYPE_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_DP_ENTITY_HASH', N'U') IS NULL
	CREATE TABLE B_CRM_DP_ENTITY_HASH(
		ENTITY_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		MATCH_HASH VARCHAR(32) NOT NULL,
		IS_PRIMARY CHAR(1) NOT NULL,
		CONSTRAINT PK_B_CRM_DP_ENTITY_HASH PRIMARY KEY CLUSTERED(ENTITY_ID ASC, ENTITY_TYPE_ID ASC, TYPE_ID ASC, MATCH_HASH ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DP_ENT_HASH_1')
	CREATE INDEX IX_B_CRM_DP_ENT_HASH_1 ON B_CRM_DP_ENTITY_HASH (ENTITY_TYPE_ID ASC, TYPE_ID ASC, MATCH_HASH ASC, ENTITY_ID ASC, IS_PRIMARY ASC)
GO

IF OBJECT_ID(N'B_CRM_ADDR', N'U') IS NULL
	CREATE TABLE B_CRM_ADDR(
		TYPE_ID SMALLINT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		ANCHOR_TYPE_ID SMALLINT,
		ANCHOR_ID INT,
		ADDRESS_1 VARCHAR(256),
		ADDRESS_2 VARCHAR(128),
		CITY VARCHAR(128),
		POSTAL_CODE VARCHAR(16),
		REGION VARCHAR(128),
		PROVINCE VARCHAR(128),
		COUNTRY VARCHAR(128),
		COUNTRY_CODE VARCHAR(100),
		CONSTRAINT PK_B_CRM_ADDR PRIMARY KEY CLUSTERED(TYPE_ID ASC, ENTITY_TYPE_ID ASC, ENTITY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_1')
	CREATE INDEX IX_B_CRM_ADDR_1 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, ADDRESS_1 ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_2')
	CREATE INDEX IX_B_CRM_ADDR_2 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, ADDRESS_2 ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_3')
	CREATE INDEX IX_B_CRM_ADDR_3 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, CITY ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_4')
	CREATE INDEX IX_B_CRM_ADDR_4 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, POSTAL_CODE ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_5')
	CREATE INDEX IX_B_CRM_ADDR_5 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, REGION ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_6')
	CREATE INDEX IX_B_CRM_ADDR_6 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, PROVINCE ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_7')
	CREATE INDEX IX_B_CRM_ADDR_7 ON B_CRM_ADDR (ENTITY_TYPE_ID ASC, TYPE_ID ASC, COUNTRY ASC, ENTITY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_8')
	CREATE INDEX IX_B_CRM_ADDR_8 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, ADDRESS_1 ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_9')
	CREATE INDEX IX_B_CRM_ADDR_9 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, ADDRESS_2 ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_10')
	CREATE INDEX IX_B_CRM_ADDR_10 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, CITY ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_11')
	CREATE INDEX IX_B_CRM_ADDR_11 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, POSTAL_CODE ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_12')
	CREATE INDEX IX_B_CRM_ADDR_12 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, REGION ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_13')
	CREATE INDEX IX_B_CRM_ADDR_13 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, PROVINCE ASC, ANCHOR_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ADDR_14')
	CREATE INDEX IX_B_CRM_ADDR_14 ON B_CRM_ADDR (ANCHOR_TYPE_ID ASC, TYPE_ID ASC, COUNTRY ASC, ANCHOR_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_DEAL_STAGE_HISTORY', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_STAGE_HISTORY(
		ID INT IDENTITY(1,1) NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		OWNER_ID INT NOT NULL,
		CREATED_TIME DATETIME NOT NULL,
		CREATED_DATE DATETIME NULL,
		START_DATE DATETIME NOT NULL,
		END_DATE DATETIME NOT NULL,
		PERIOD_YEAR INT NOT NULL,
		PERIOD_QUARTER INT NOT NULL,
		PERIOD_MONTH INT NOT NULL,
		START_PERIOD_YEAR INT NOT NULL,
		START_PERIOD_QUARTER INT NOT NULL,
		START_PERIOD_MONTH INT NOT NULL,
		END_PERIOD_YEAR INT NOT NULL,
		END_PERIOD_QUARTER INT NOT NULL,
		END_PERIOD_MONTH INT NOT NULL,
		RESPONSIBLE_ID INT NOT NULL,
		CATEGORY_ID INT NULL,
		STAGE_SEMANTIC_ID VARCHAR(3) NULL,
		STAGE_ID VARCHAR(50) NULL,
		IS_LOST CHAR(1) NULL,
		CONSTRAINT PK_CRM_DEAL_SHX PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_1')
	CREATE INDEX IX_B_CRM_DEAL_SHX_1 ON B_CRM_DEAL_STAGE_HISTORY(OWNER_ID ASC, TYPE_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_2')
	CREATE INDEX IX_B_CRM_DEAL_SHX_2 ON B_CRM_DEAL_STAGE_HISTORY(TYPE_ID ASC, STAGE_ID ASC, START_DATE ASC, RESPONSIBLE_ID ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_3')
	CREATE INDEX IX_B_CRM_DEAL_SHX_3 ON B_CRM_DEAL_STAGE_HISTORY(TYPE_ID ASC, STAGE_ID ASC, END_DATE ASC, RESPONSIBLE_ID ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_4')
	CREATE INDEX IX_B_CRM_DEAL_SHX_4 ON B_CRM_DEAL_STAGE_HISTORY(TYPE_ID ASC, STAGE_ID ASC, CREATED_TIME ASC, RESPONSIBLE_ID ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_5')
	CREATE INDEX IX_B_CRM_DEAL_SHX_5 ON B_CRM_DEAL_STAGE_HISTORY(PERIOD_YEAR ASC, PERIOD_MONTH ASC, RESPONSIBLE_ID ASC, OWNER_ID ASC, START_DATE ASC, END_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_6')
	CREATE INDEX IX_B_CRM_DEAL_SHX_6 ON B_CRM_DEAL_STAGE_HISTORY(PERIOD_YEAR ASC, PERIOD_MONTH ASC, STAGE_SEMANTIC_ID ASC, RESPONSIBLE_ID ASC, OWNER_ID ASC, START_DATE ASC, END_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_7')
	CREATE INDEX IX_B_CRM_DEAL_SHX_7 ON B_CRM_DEAL_STAGE_HISTORY(PERIOD_YEAR ASC, PERIOD_MONTH ASC, OWNER_ID ASC, START_DATE ASC, END_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_8')
	CREATE INDEX IX_B_CRM_DEAL_SHX_8 ON B_CRM_DEAL_STAGE_HISTORY(TYPE_ID ASC, OWNER_ID ASC, END_DATE ASC, START_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_9')
	CREATE INDEX IX_B_CRM_DEAL_SHX_9 ON B_CRM_DEAL_STAGE_HISTORY(TYPE_ID, STAGE_ID, CREATED_DATE, RESPONSIBLE_ID, OWNER_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_10')
	CREATE INDEX IX_B_CRM_DEAL_SHX_10 ON B_CRM_DEAL_STAGE_HISTORY(CATEGORY_ID, TYPE_ID, STAGE_ID, START_DATE, RESPONSIBLE_ID, OWNER_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_11')
	CREATE INDEX IX_B_CRM_DEAL_SHX_11 ON B_CRM_DEAL_STAGE_HISTORY(CATEGORY_ID, TYPE_ID, STAGE_ID, END_DATE, RESPONSIBLE_ID, OWNER_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_12')
	CREATE INDEX IX_B_CRM_DEAL_SHX_12 ON B_CRM_DEAL_STAGE_HISTORY(CATEGORY_ID, TYPE_ID, STAGE_ID, CREATED_DATE, RESPONSIBLE_ID, OWNER_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_13')
	CREATE INDEX IX_B_CRM_DEAL_SHX_13 ON B_CRM_DEAL_STAGE_HISTORY(IS_LOST, CREATED_DATE, OWNER_ID, RESPONSIBLE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SHX_14')
	CREATE INDEX IX_B_CRM_DEAL_SHX_14 ON B_CRM_DEAL_STAGE_HISTORY(CATEGORY_ID, IS_LOST, CREATED_DATE, OWNER_ID, RESPONSIBLE_ID)
GO

IF OBJECT_ID(N'B_CRM_DEAL_SUM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_SUM_STAT(
	OWNER_ID INT NOT NULL,
	CREATED_DATE DATETIME NOT NULL,
	START_DATE DATETIME NOT NULL,
	END_DATE DATETIME NOT NULL,
	PERIOD_YEAR INT NOT NULL,
	PERIOD_QUARTER INT NOT NULL,
	PERIOD_MONTH INT NOT NULL,
	PERIOD_DAY INT NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CATEGORY_ID INT NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,2) NULL,
	UF_SUM_1 DECIMAL(18,2) NULL,
	UF_SUM_2 DECIMAL(18,2) NULL,
	UF_SUM_3 DECIMAL(18,2) NULL,
	UF_SUM_4 DECIMAL(18,2) NULL,
	UF_SUM_5 DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	CONSTRAINT PK_CRM_DEAL_SUM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SSTAT_1')
	CREATE INDEX IX_B_CRM_DEAL_SSTAT_1 ON B_CRM_DEAL_SUM_STAT(STAGE_SEMANTIC_ID ASC, OWNER_ID ASC, END_DATE ASC, START_DATE ASC, CREATED_DATE ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SSTAT_2')
	CREATE INDEX IX_B_CRM_DEAL_SSTAT_2 ON B_CRM_DEAL_SUM_STAT(IS_LOST ASC, OWNER_ID ASC, END_DATE ASC, START_DATE ASC, CREATED_DATE ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SSTAT_3')
	CREATE INDEX IX_B_CRM_DEAL_SSTAT_3 ON B_CRM_DEAL_SUM_STAT(END_DATE, START_DATE, OWNER_ID, CREATED_DATE, RESPONSIBLE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_SSTAT_4')
	CREATE INDEX IX_B_CRM_DEAL_SSTAT_4 ON B_CRM_DEAL_SUM_STAT(CATEGORY_ID, END_DATE, START_DATE, OWNER_ID, CREATED_DATE, RESPONSIBLE_ID)
GO

IF OBJECT_ID(N'B_CRM_DEAL_INV_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_INV_STAT(
	OWNER_ID INT NOT NULL,
	CREATED_DATE DATETIME NOT NULL,
	START_DATE DATETIME NOT NULL,
	END_DATE DATETIME NOT NULL,
	PERIOD_YEAR INT NOT NULL,
	PERIOD_QUARTER INT NOT NULL,
	PERIOD_MONTH INT NOT NULL,
	PERIOD_DAY INT NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CATEGORY_ID INT NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	INVOICE_SUM DECIMAL(18,2) NULL,
	INVOICE_QTY INT NULL,
	TOTAL_INVOICE_SUM DECIMAL(18,2) NULL,
	TOTAL_INVOICE_QTY INT NULL,
	TOTAL_SUM DECIMAL(18,2) NULL,
	TOTAL_OWED DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	CONSTRAINT PK_CRM_DEAL_INV_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ISTAT_1')
	CREATE INDEX IX_B_CRM_DEAL_ISTAT_1 ON B_CRM_DEAL_INV_STAT(STAGE_SEMANTIC_ID ASC, OWNER_ID ASC, END_DATE ASC, START_DATE ASC, CREATED_DATE ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ISTAT_2')
	CREATE INDEX IX_B_CRM_DEAL_ISTAT_2 ON B_CRM_DEAL_INV_STAT(IS_LOST ASC, OWNER_ID ASC, END_DATE ASC, START_DATE ASC, CREATED_DATE ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ISTAT_3')
	CREATE INDEX IX_B_CRM_DEAL_ISTAT_3 ON B_CRM_DEAL_INV_STAT(CATEGORY_ID, IS_LOST, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ISTAT_4')
	CREATE INDEX IX_B_CRM_DEAL_ISTAT_4 ON B_CRM_DEAL_INV_STAT(CATEGORY_ID, STAGE_SEMANTIC_ID, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID)
GO

IF OBJECT_ID(N'B_CRM_DEAL_ACT_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_ACT_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	DEADLINE_YEAR INT NOT NULL,
	DEADLINE_QUARTER INT NOT NULL,
	DEADLINE_MONTH INT NOT NULL,
	DEADLINE_DAY INT NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CATEGORY_ID INT NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CALL_QTY INT NULL,
	MEETING_QTY INT NULL,
	EMAIL_QTY INT NULL,
	UF_ATTR_1 INT NULL,
	CONSTRAINT PK_CRM_DEAL_ACT_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ASTAT_1')
	CREATE INDEX IX_B_CRM_DEAL_ASTAT_1 ON B_CRM_DEAL_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, CALL_QTY ASC, MEETING_QTY ASC, EMAIL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ASTAT_2')
	CREATE INDEX IX_B_CRM_DEAL_ASTAT_2 ON B_CRM_DEAL_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, CALL_QTY ASC, MEETING_QTY ASC, EMAIL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ASTAT_3')
	CREATE INDEX IX_B_CRM_DEAL_ASTAT_3 ON B_CRM_DEAL_ACT_STAT(IS_LOST, DEADLINE_DATE, OWNER_ID, RESPONSIBLE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ASTAT_4')
	CREATE INDEX IX_B_CRM_DEAL_ASTAT_4 ON B_CRM_DEAL_ACT_STAT(CATEGORY_ID, IS_LOST, DEADLINE_DATE, OWNER_ID, RESPONSIBLE_ID)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_ASTAT_5')
	CREATE INDEX IX_B_CRM_DEAL_ASTAT_5 ON B_CRM_DEAL_ACT_STAT(CATEGORY_ID, RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY)
GO

IF OBJECT_ID(N'B_CRM_CONV_MAP', N'U') IS NULL
	CREATE TABLE B_CRM_CONV_MAP(
		SRC_TYPE_ID SMALLINT NOT NULL,
		DST_TYPE_ID SMALLINT NOT NULL,
		LAST_UPDATED DATETIME NOT NULL,
		DATA TEXT NULL,
		CONSTRAINT PK_CRM_CONV_MAP PRIMARY KEY CLUSTERED(DST_TYPE_ID ASC, SRC_TYPE_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_LEAD_STATUS_HISTORY', N'U') IS NULL
	CREATE TABLE B_CRM_LEAD_STATUS_HISTORY(
		ID INT IDENTITY(1,1) NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		OWNER_ID INT NOT NULL,
		CREATED_TIME DATETIME NOT NULL,
		CREATED_DATE DATETIME NULL,
		PERIOD_YEAR INT NOT NULL,
		PERIOD_QUARTER INT NOT NULL,
		PERIOD_MONTH INT NOT NULL,
		RESPONSIBLE_ID INT NOT NULL,
		STATUS_SEMANTIC_ID VARCHAR(3) NULL,
		STATUS_ID VARCHAR(50) NULL,
		IS_IN_WORK CHAR(1) NULL,
		IS_JUNK CHAR(1) NULL,
		CONSTRAINT PK_CRM_LEAD_SHX PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_1')
	CREATE INDEX IX_B_CRM_LEAD_SHX_1 ON B_CRM_LEAD_STATUS_HISTORY(OWNER_ID ASC, TYPE_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_2')
	CREATE INDEX IX_B_CRM_LEAD_SHX_2 ON B_CRM_LEAD_STATUS_HISTORY(TYPE_ID ASC, CREATED_DATE ASC, STATUS_ID ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_3')
	CREATE INDEX IX_B_CRM_LEAD_SHX_3 ON B_CRM_LEAD_STATUS_HISTORY(TYPE_ID ASC, IS_JUNK ASC, CREATED_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_4')
	CREATE INDEX IX_B_CRM_LEAD_SHX_4 ON B_CRM_LEAD_STATUS_HISTORY(TYPE_ID ASC, IS_JUNK ASC, RESPONSIBLE_ID ASC, CREATED_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_5')
	CREATE INDEX IX_B_CRM_LEAD_SHX_5 ON B_CRM_LEAD_STATUS_HISTORY(IS_IN_WORK ASC, OWNER_ID ASC, CREATED_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SHX_6')
	CREATE INDEX IX_B_CRM_LEAD_SHX_6 ON B_CRM_LEAD_STATUS_HISTORY(IS_IN_WORK ASC, OWNER_ID ASC, RESPONSIBLE_ID ASC, CREATED_DATE ASC)
GO

IF OBJECT_ID(N'B_CRM_LEAD_SUM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_LEAD_SUM_STAT(
		OWNER_ID INT NOT NULL,
		CREATED_DATE DATETIME NOT NULL,
		PERIOD_YEAR INT NOT NULL,
		PERIOD_QUARTER INT NOT NULL,
		PERIOD_MONTH INT NOT NULL,
		PERIOD_DAY INT NOT NULL,
		RESPONSIBLE_ID INT NOT NULL,
		STATUS_SEMANTIC_ID VARCHAR(3) NULL,
		STATUS_ID VARCHAR(50) NULL,
		SOURCE_ID VARCHAR(50) NULL,
		IS_JUNK CHAR(1) NULL,
		CURRENCY_ID CHAR(3) NULL,
		SUM_TOTAL DECIMAL(18,2) NULL,
		UF_SUM_1 DECIMAL(18,2) NULL,
		UF_SUM_2 DECIMAL(18,2) NULL,
		UF_SUM_3 DECIMAL(18,2) NULL,
		UF_SUM_4 DECIMAL(18,2) NULL,
		UF_SUM_5 DECIMAL(18,2) NULL,
		UF_ATTR_1 INT NULL,
		CONSTRAINT PK_CRM_LEAD_SUM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SSTAT_1')
	CREATE INDEX IX_B_CRM_LEAD_SSTAT_1 ON B_CRM_LEAD_SUM_STAT(CREATED_DATE ASC, OWNER_ID ASC, SOURCE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_SSTAT_2')
	CREATE INDEX IX_B_CRM_LEAD_SSTAT_2 ON B_CRM_LEAD_SUM_STAT(SOURCE_ID ASC, OWNER_ID ASC, CREATED_DATE ASC)
GO

IF OBJECT_ID(N'B_CRM_LEAD_ACT_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_LEAD_ACT_STAT(
		OWNER_ID INT NOT NULL,
		DEADLINE_DATE DATETIME NOT NULL,
		DEADLINE_YEAR INT NOT NULL,
		DEADLINE_QUARTER INT NOT NULL,
		DEADLINE_MONTH INT NOT NULL,
		DEADLINE_DAY INT NOT NULL,
		CREATED_DATE DATETIME NULL,
		RESPONSIBLE_ID INT NOT NULL,
		STATUS_SEMANTIC_ID VARCHAR(3) NULL,
		STATUS_ID VARCHAR(50) NULL,
		IS_JUNK CHAR(1) NULL,
		CALL_QTY INT NULL,
		MEETING_QTY INT NULL,
		EMAIL_QTY INT NULL,
		UF_ATTR_1 INT NULL,
		CONSTRAINT PK_CRM_LEAD_ACT_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_ASTAT_1')
	CREATE INDEX IX_B_CRM_LEAD_ASTAT_1 ON B_CRM_LEAD_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, CALL_QTY ASC, MEETING_QTY ASC, EMAIL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_ASTAT_2')
	CREATE INDEX IX_B_CRM_LEAD_ASTAT_2 ON B_CRM_LEAD_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, CALL_QTY ASC, MEETING_QTY ASC, EMAIL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_ASTAT_3')
	CREATE INDEX IX_B_CRM_LEAD_ASTAT_3 ON B_CRM_LEAD_ACT_STAT(IS_JUNK ASC, OWNER_ID ASC, DEADLINE_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_ASTAT_4')
	CREATE INDEX IX_B_CRM_LEAD_ASTAT_4 ON B_CRM_LEAD_ACT_STAT(IS_JUNK ASC, OWNER_ID ASC, RESPONSIBLE_ID ASC, DEADLINE_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_ASTAT_5')
	CREATE INDEX IX_B_CRM_LEAD_ASTAT_5 ON B_CRM_LEAD_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, CALL_QTY ASC, MEETING_QTY ASC, EMAIL_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_LEAD_CONV_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_LEAD_CONV_STAT(
		OWNER_ID INT NOT NULL,
		ENTRY_DATE DATETIME NOT NULL,
		CREATED_DATE DATETIME NULL,
		RESPONSIBLE_ID INT NOT NULL,
		DEAL_QTY INT NULL,
		CONTACT_QTY INT NULL,
		COMPANY_QTY INT NULL,
		TOTALS_DATE DATETIME NULL,
		CONSTRAINT PK_CRM_LEAD_CSTAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, ENTRY_DATE ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_CSTAT_1')
	CREATE INDEX IX_B_CRM_LEAD_CSTAT_1 ON B_CRM_LEAD_CONV_STAT(RESPONSIBLE_ID ASC, ENTRY_DATE ASC, OWNER_ID ASC, DEAL_QTY ASC, CONTACT_QTY ASC, COMPANY_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_LEAD_CSTAT_2')
	CREATE INDEX IX_B_CRM_LEAD_CSTAT_2 ON B_CRM_LEAD_CONV_STAT(CREATED_DATE ASC, OWNER_ID ASC, DEAL_QTY ASC, CONTACT_QTY ASC, COMPANY_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_INV_STATUS_HISTORY', N'U') IS NULL
	CREATE TABLE B_CRM_INV_STATUS_HISTORY(
		ID INT IDENTITY(1,1) NOT NULL,
		TYPE_ID SMALLINT NOT NULL,
		OWNER_ID INT NOT NULL,
		CREATED_TIME DATETIME NOT NULL,
		CREATED_DATE DATETIME NULL,
		BILL_DATE DATETIME NOT NULL,
		PAY_BEFORE_DATE DATETIME NOT NULL,
		ACTIVITY_DATE DATETIME NOT NULL,
		RESPONSIBLE_ID INT NOT NULL,
		STATUS_SEMANTIC_ID VARCHAR(3) NULL,
		STATUS_ID VARCHAR(50) NULL,
		IS_NEW CHAR(1) NULL,
		IS_JUNK CHAR(1) NULL,
		CONSTRAINT PK_CRM_INV_SHX PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SHX_1')
	CREATE INDEX IX_B_CRM_INV_SHX_1 ON B_CRM_INV_STATUS_HISTORY(ACTIVITY_DATE ASC, OWNER_ID ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SHX_2')
	CREATE INDEX IX_B_CRM_INV_SHX_2 ON B_CRM_INV_STATUS_HISTORY(TYPE_ID ASC, CREATED_DATE ASC, STATUS_ID ASC, OWNER_ID ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SHX_3')
	CREATE INDEX IX_B_CRM_INV_SHX_3 ON B_CRM_INV_STATUS_HISTORY(STATUS_SEMANTIC_ID ASC, ACTIVITY_DATE ASC, OWNER_ID ASC, ID ASC)
GO

IF OBJECT_ID(N'B_CRM_INVOICE_SUM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_INVOICE_SUM_STAT(
		OWNER_ID INT NOT NULL,
		CREATED_DATE DATETIME NOT NULL,
		BILL_DATE DATETIME NOT NULL,
		PAY_BEFORE_DATE DATETIME NOT NULL,
		PAID_DATE DATETIME NULL,
		IS_PAID_INTIME CHAR(1) NULL,
		CLOSED_DATE DATETIME NULL,
		RESPONSIBLE_ID INT NOT NULL,
		COMPANY_ID INT NULL,
		CONTACT_ID INT NULL,
		STATUS_SEMANTIC_ID VARCHAR(3) NULL,
		STATUS_ID VARCHAR(50) NULL,
		IS_JUNK CHAR(1) NULL,
		CURRENCY_ID CHAR(3) NULL,
		SUM_TOTAL DECIMAL(18,2) NULL,
		UF_SUM_1 DECIMAL(18,2) NULL,
		UF_SUM_2 DECIMAL(18,2) NULL,
		UF_SUM_3 DECIMAL(18,2) NULL,
		UF_SUM_4 DECIMAL(18,2) NULL,
		UF_SUM_5 DECIMAL(18,2) NULL,
		UF_ATTR_1 INT NULL,
		CONSTRAINT PK_CRM_INV_SUM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SSTAT_1')
	CREATE INDEX IX_B_CRM_INV_SSTAT_1 ON B_CRM_INVOICE_SUM_STAT(CREATED_DATE ASC, OWNER_ID ASC, RESPONSIBLE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SSTAT_2')
	CREATE INDEX IX_B_CRM_INV_SSTAT_2 ON B_CRM_INVOICE_SUM_STAT(STATUS_SEMANTIC_ID ASC, CREATED_DATE ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SSTAT_3')
	CREATE INDEX IX_B_CRM_INV_SSTAT_3 ON B_CRM_INVOICE_SUM_STAT(STATUS_SEMANTIC_ID ASC, PAY_BEFORE_DATE ASC, OWNER_ID ASC, CREATED_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SSTAT_4')
	CREATE INDEX IX_B_CRM_INV_SSTAT_4 ON B_CRM_INVOICE_SUM_STAT(COMPANY_ID ASC, STATUS_SEMANTIC_ID ASC, PAY_BEFORE_DATE ASC, OWNER_ID ASC, CREATED_DATE ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_INV_SSTAT_5')
	CREATE INDEX IX_B_CRM_INV_SSTAT_5 ON B_CRM_INVOICE_SUM_STAT(CONTACT_ID ASC, STATUS_SEMANTIC_ID ASC, PAY_BEFORE_DATE ASC, OWNER_ID ASC, CREATED_DATE ASC)
GO

IF OBJECT_ID(N'B_CRM_REQUISITE', N'U') IS NULL
	CREATE TABLE B_CRM_REQUISITE(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		PRESET_ID INT NOT NULL DEFAULT 0,
		DATE_CREATE DATETIME NOT NULL,
		DATE_MODIFY DATETIME NULL,
		CREATED_BY_ID INT NOT NULL,
		MODIFY_BY_ID INT NULL,
		NAME VARCHAR(255) NOT NULL,
		CODE VARCHAR(45) NULL,
		XML_ID VARCHAR(45) NULL,
		ORIGINATOR_ID varchar(255) NULL,
		ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
		SORT INT NOT NULL DEFAULT 500,
		RQ_NAME VARCHAR(150) NULL,
		RQ_FIRST_NAME VARCHAR(50) NULL,
		RQ_LAST_NAME VARCHAR(50) NULL,
		RQ_SECOND_NAME VARCHAR(50) NULL,
		RQ_COMPANY_NAME VARCHAR(255) NULL,
		RQ_COMPANY_FULL_NAME VARCHAR(300) NULL,
		RQ_COMPANY_REG_DATE VARCHAR(30) NULL,
		RQ_DIRECTOR VARCHAR(150) NULL,
		RQ_ACCOUNTANT VARCHAR(150) NULL,
		RQ_CEO_NAME VARCHAR(150) NULL,
		RQ_CEO_WORK_POS VARCHAR(150) NULL,
		RQ_CONTACT VARCHAR(150) NULL,
		RQ_EMAIL VARCHAR(255) NULL,
		RQ_PHONE VARCHAR(30) NULL,
		RQ_FAX VARCHAR(30) NULL,
		RQ_IDENT_DOC VARCHAR(255) NULL,
		RQ_IDENT_DOC_SER VARCHAR(25) NULL,
		RQ_IDENT_DOC_NUM VARCHAR(25) NULL,
		RQ_IDENT_DOC_DATE VARCHAR(30) NULL,
		RQ_IDENT_DOC_ISSUED_BY VARCHAR(255) NULL,
		RQ_IDENT_DOC_DEP_CODE VARCHAR(25) NULL,
		RQ_INN VARCHAR(15) NULL,
		RQ_KPP VARCHAR(9) NULL,
		RQ_USRLE VARCHAR(20) NULL,
		RQ_IFNS VARCHAR(255) NULL,
		RQ_OGRN VARCHAR(13) NULL,
		RQ_OGRNIP VARCHAR(15) NULL,
		RQ_OKPO VARCHAR(10) NULL,
		RQ_OKTMO VARCHAR(11) NULL,
		RQ_OKVED VARCHAR(255) NULL,
		RQ_EDRPOU VARCHAR(8) NULL,
		RQ_DRFO VARCHAR(10) NULL,
		RQ_KBE VARCHAR(2) NULL,
		RQ_IIN VARCHAR(12) NULL,
		RQ_BIN VARCHAR(12) NULL,
		RQ_VAT_PAYER CHAR(1) NOT NULL DEFAULT 'N',
		RQ_VAT_ID VARCHAR(20) NULL,
		RQ_VAT_CERT_SER VARCHAR(10) NULL,
		RQ_VAT_CERT_NUM VARCHAR(15) NULL,
		RQ_VAT_CERT_DATE VARCHAR(30) NULL,
		RQ_RESIDENCE_COUNTRY VARCHAR(128) NULL,
		CONSTRAINT PK_B_CRM_REQUISITE PRIMARY KEY CLUSTERED (ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_1')
	CREATE INDEX IX_B_CRM_REQUISITE_1 ON B_CRM_REQUISITE (ENTITY_ID ASC, ENTITY_TYPE_ID ASC, SORT ASC, ID ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_2')
	CREATE INDEX IX_B_CRM_REQUISITE_2 ON B_CRM_REQUISITE (RQ_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_3')
	CREATE INDEX IX_B_CRM_REQUISITE_3 ON B_CRM_REQUISITE (RQ_LAST_NAME ASC, RQ_FIRST_NAME ASC, RQ_SECOND_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_4')
	CREATE INDEX IX_B_CRM_REQUISITE_4 ON B_CRM_REQUISITE (RQ_COMPANY_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_5')
	CREATE INDEX IX_B_CRM_REQUISITE_5 ON B_CRM_REQUISITE (RQ_DIRECTOR ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_6')
	CREATE INDEX IX_B_CRM_REQUISITE_6 ON B_CRM_REQUISITE (RQ_ACCOUNTANT ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_7')
	CREATE INDEX IX_B_CRM_REQUISITE_7 ON B_CRM_REQUISITE (RQ_CEO_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_8')
	CREATE INDEX IX_B_CRM_REQUISITE_8 ON B_CRM_REQUISITE (RQ_CONTACT ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_9')
	CREATE INDEX IX_B_CRM_REQUISITE_9 ON B_CRM_REQUISITE (RQ_EMAIL ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_10')
	CREATE INDEX IX_B_CRM_REQUISITE_10 ON B_CRM_REQUISITE (RQ_PHONE ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_11')
	CREATE INDEX IX_B_CRM_REQUISITE_11 ON B_CRM_REQUISITE (RQ_FAX ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_12')
	CREATE INDEX IX_B_CRM_REQUISITE_12 ON B_CRM_REQUISITE (RQ_IDENT_DOC_NUM ASC, RQ_IDENT_DOC_SER ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_13')
	CREATE INDEX IX_B_CRM_REQUISITE_13 ON B_CRM_REQUISITE (RQ_IDENT_DOC_DEP_CODE ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_14')
	CREATE INDEX IX_B_CRM_REQUISITE_14 ON B_CRM_REQUISITE (RQ_INN ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_15')
	CREATE INDEX IX_B_CRM_REQUISITE_15 ON B_CRM_REQUISITE (RQ_KPP ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_16')
	CREATE INDEX IX_B_CRM_REQUISITE_16 ON B_CRM_REQUISITE (RQ_USRLE ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_17')
	CREATE INDEX IX_B_CRM_REQUISITE_17 ON B_CRM_REQUISITE (RQ_IFNS ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_18')
	CREATE INDEX IX_B_CRM_REQUISITE_18 ON B_CRM_REQUISITE (RQ_OGRN ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_19')
	CREATE INDEX IX_B_CRM_REQUISITE_19 ON B_CRM_REQUISITE (RQ_OGRNIP ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_20')
	CREATE INDEX IX_B_CRM_REQUISITE_20 ON B_CRM_REQUISITE (RQ_OKPO ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_21')
	CREATE INDEX IX_B_CRM_REQUISITE_21 ON B_CRM_REQUISITE (RQ_OKTMO ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_22')
	CREATE INDEX IX_B_CRM_REQUISITE_22 ON B_CRM_REQUISITE (RQ_OKVED ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_23')
	CREATE INDEX IX_B_CRM_REQUISITE_23 ON B_CRM_REQUISITE (RQ_EDRPOU ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_24')
	CREATE INDEX IX_B_CRM_REQUISITE_24 ON B_CRM_REQUISITE (RQ_DRFO ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_25')
	CREATE INDEX IX_B_CRM_REQUISITE_25 ON B_CRM_REQUISITE (RQ_KBE ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_26')
	CREATE INDEX IX_B_CRM_REQUISITE_26 ON B_CRM_REQUISITE (RQ_IIN ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_27')
	CREATE INDEX IX_B_CRM_REQUISITE_27 ON B_CRM_REQUISITE (RQ_BIN ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_28')
	CREATE INDEX IX_B_CRM_REQUISITE_28 ON B_CRM_REQUISITE (RQ_VAT_PAYER ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_29')
	CREATE INDEX IX_B_CRM_REQUISITE_29 ON B_CRM_REQUISITE (RQ_VAT_ID ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_30')
	CREATE INDEX IX_B_CRM_REQUISITE_30 ON B_CRM_REQUISITE (RQ_VAT_CERT_NUM ASC, RQ_VAT_CERT_SER ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_REQUISITE_31')
	CREATE INDEX IX_B_CRM_REQUISITE_31 ON B_CRM_REQUISITE (RQ_RESIDENCE_COUNTRY ASC)
GO

IF OBJECT_ID(N'B_CRM_PRESET', N'U') IS NULL
	CREATE TABLE B_CRM_PRESET(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		COUNTRY_ID INT NOT NULL DEFAULT 0,
		DATE_CREATE DATETIME NOT NULL,
		DATE_MODIFY DATETIME NULL,
		CREATED_BY_ID INT NOT NULL,
		MODIFY_BY_ID INT NULL,
		NAME VARCHAR(255) NOT NULL,
		XML_ID VARCHAR(45) NULL,
		ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
		SORT INT NOT NULL DEFAULT 500,
		SETTINGS TEXT NULL,
		CONSTRAINT PK_B_CRM_PRESET PRIMARY KEY CLUSTERED (ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_PRESET_1')
	CREATE INDEX IX_B_CRM_PRESET_1 ON B_CRM_PRESET (ID ASC, ENTITY_TYPE_ID ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_PRESET_2')
	CREATE INDEX IX_B_CRM_PRESET_2 ON B_CRM_PRESET (COUNTRY_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_REQUISITE_CFG', N'U') IS NULL
	CREATE TABLE B_CRM_REQUISITE_CFG
	(
		ENTITY_ID INT NOT NULL,
		ENTITY_TYPE_ID INT NOT NULL,
		SETTINGS TEXT,
		CONSTRAINT PK_B_CRM_REQUISITE_CFG PRIMARY KEY CLUSTERED(ENTITY_ID ASC, ENTITY_TYPE_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_REQUISITE_LINK', N'U') IS NULL
	CREATE TABLE B_CRM_REQUISITE_LINK
	(
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID      INT      NOT NULL,
		REQUISITE_ID      INT      NOT NULL DEFAULT 0,
		BANK_DETAIL_ID INT      NOT NULL DEFAULT 0,
		MC_REQUISITE_ID   INT      NOT NULL DEFAULT 0,
		MC_BANK_DETAIL_ID INT      NOT NULL DEFAULT 0,
		CONSTRAINT PK_B_CRM_REQUISITE_LINK PRIMARY KEY CLUSTERED (ENTITY_TYPE_ID ASC, ENTITY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_CRM_REQUISITE_LINK_1')
	CREATE INDEX IX_CRM_REQUISITE_LINK_1 ON B_CRM_REQUISITE_LINK (REQUISITE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_CRM_REQUISITE_LINK_2')
	CREATE INDEX IX_CRM_REQUISITE_LINK_2 ON B_CRM_REQUISITE_LINK (MC_REQUISITE_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_BANK_DETAIL', N'U') IS NULL
	CREATE TABLE B_CRM_BANK_DETAIL(
		ID INT IDENTITY(1,1) NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		ENTITY_ID INT NOT NULL,
		COUNTRY_ID INT NOT NULL DEFAULT 0,
		DATE_CREATE DATETIME NOT NULL,
		DATE_MODIFY DATETIME NULL,
		CREATED_BY_ID INT NOT NULL,
		MODIFY_BY_ID INT NULL,
		NAME VARCHAR(255) NOT NULL,
		CODE VARCHAR(45) NULL,
		XML_ID VARCHAR(45) NULL,
		ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
		SORT INT NOT NULL DEFAULT 500,
		RQ_BANK_NAME VARCHAR(255) NULL,
		RQ_BANK_ADDR VARCHAR(255) NULL,
		RQ_BANK_ROUTE_NUM VARCHAR(9) NULL,
		RQ_BIK VARCHAR(9) NULL,
		RQ_MFO VARCHAR(6) NULL,
		RQ_ACC_NAME VARCHAR(150) NULL,
		RQ_ACC_NUM VARCHAR(34) NULL,
		RQ_IIK VARCHAR(20) NULL,
		RQ_ACC_CURRENCY VARCHAR(100) NULL,
		RQ_COR_ACC_NUM VARCHAR(34) NULL,
		RQ_IBAN VARCHAR(34) NULL,
		RQ_SWIFT VARCHAR(11) NULL,
		RQ_BIC VARCHAR(11) NULL,
		COMMENTS VARCHAR(500) NULL,
		CONSTRAINT PK_B_CRM_BANK_DETAIL PRIMARY KEY CLUSTERED (ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_1')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_1 ON B_CRM_BANK_DETAIL (ENTITY_ID ASC, ENTITY_TYPE_ID ASC, SORT ASC, ID ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_2')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_2 ON B_CRM_BANK_DETAIL (RQ_BANK_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_3')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_3 ON B_CRM_BANK_DETAIL (RQ_BANK_ADDR ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_4')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_4 ON B_CRM_BANK_DETAIL (RQ_BANK_ROUTE_NUM ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_5')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_5 ON B_CRM_BANK_DETAIL (RQ_BIK ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_6')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_6 ON B_CRM_BANK_DETAIL (RQ_MFO ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_7')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_7 ON B_CRM_BANK_DETAIL (RQ_ACC_NAME ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_8')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_8 ON B_CRM_BANK_DETAIL (RQ_ACC_NUM ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_9')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_9 ON B_CRM_BANK_DETAIL (RQ_IIK ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_10')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_10 ON B_CRM_BANK_DETAIL (RQ_ACC_CURRENCY ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_11')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_11 ON B_CRM_BANK_DETAIL (RQ_COR_ACC_NUM ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_12')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_12 ON B_CRM_BANK_DETAIL (RQ_IBAN ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_13')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_13 ON B_CRM_BANK_DETAIL (RQ_SWIFT ASC)
GO
IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_BANK_DETAIL_14')
	CREATE INDEX IX_B_CRM_BANK_DETAIL_14 ON B_CRM_BANK_DETAIL (RQ_BIC ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_COMPANY', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_COMPANY(
		CONTACT_ID INT NOT NULL,
		COMPANY_ID INT NOT NULL,
		SORT INT NOT NULL,
		ROLE_ID SMALLINT NOT NULL,
		IS_PRIMARY CHAR(1) NOT NULL,
		CONSTRAINT PK_B_CRM_CONTACT_COMPANY PRIMARY KEY CLUSTERED(CONTACT_ID ASC, COMPANY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_COMPANY_1')
	CREATE INDEX IX_B_CRM_CONTACT_COMPANY_1 ON B_CRM_CONTACT_COMPANY(COMPANY_ID ASC, CONTACT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_COMPANY_2')
	CREATE INDEX IX_B_CRM_CONTACT_COMPANY_2 ON B_CRM_CONTACT_COMPANY(CONTACT_ID ASC, IS_PRIMARY ASC, COMPANY_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_COMPANY_3')
	CREATE INDEX IX_B_CRM_CONTACT_COMPANY_3 ON B_CRM_CONTACT_COMPANY(CONTACT_ID ASC, SORT ASC, COMPANY_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_DEAL_CONTACT', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_CONTACT(
		DEAL_ID INT NOT NULL,
		CONTACT_ID INT NOT NULL,
		SORT INT NOT NULL,
		ROLE_ID SMALLINT NOT NULL,
		IS_PRIMARY CHAR(1) NOT NULL,
		CONSTRAINT PK_B_CRM_DEAL_CONTACT PRIMARY KEY CLUSTERED(DEAL_ID ASC, CONTACT_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_CONTACT_1')
	CREATE INDEX IX_B_CRM_DEAL_CONTACT_1 ON B_CRM_DEAL_CONTACT(CONTACT_ID ASC, DEAL_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_CONTACT_2')
	CREATE INDEX IX_B_CRM_DEAL_CONTACT_2 ON B_CRM_DEAL_CONTACT(DEAL_ID ASC, IS_PRIMARY ASC, CONTACT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_CONTACT_3')
	CREATE INDEX IX_B_CRM_DEAL_CONTACT_3 ON B_CRM_DEAL_CONTACT(DEAL_ID ASC, SORT ASC, CONTACT_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_QUOTE_CONTACT', N'U') IS NULL
	CREATE TABLE B_CRM_QUOTE_CONTACT(
		QUOTE_ID INT NOT NULL,
		CONTACT_ID INT NOT NULL,
		SORT INT NOT NULL,
		ROLE_ID SMALLINT NOT NULL,
		IS_PRIMARY CHAR(1) NOT NULL,
		CONSTRAINT PK_B_CRM_QUOTE_CONTACT PRIMARY KEY CLUSTERED(QUOTE_ID ASC, CONTACT_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_QUOTE_CONTACT_1')
	CREATE INDEX IX_B_CRM_QUOTE_CONTACT_1 ON B_CRM_QUOTE_CONTACT(CONTACT_ID ASC, QUOTE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_QUOTE_CONTACT_2')
	CREATE INDEX IX_B_CRM_QUOTE_CONTACT_2 ON B_CRM_QUOTE_CONTACT(QUOTE_ID ASC, IS_PRIMARY ASC, CONTACT_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_QUOTE_CONTACT_3')
	CREATE INDEX IX_B_CRM_QUOTE_CONTACT_3 ON B_CRM_QUOTE_CONTACT(QUOTE_ID ASC, SORT ASC, CONTACT_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_ENTITY_CFG', N'U') IS NULL
	CREATE TABLE B_CRM_ENTITY_CFG(
		ENTITY_ID INT NOT NULL,
		ENTITY_TYPE_ID SMALLINT NOT NULL,
		USER_ID INT NOT NULL,
		SETTINGS TEXT NULL,
		CONSTRAINT PK_B_CRM_ENTITY_CFG PRIMARY KEY CLUSTERED(ENTITY_ID ASC, ENTITY_TYPE_ID ASC, USER_ID ASC)
	)
GO

IF OBJECT_ID(N'B_CRM_PS_RQ_CONV_RELATION', N'U') IS NULL
	CREATE TABLE B_CRM_PS_RQ_CONV_RELATION
	(
		ENTITY_ID       INT  NOT NULL,
		COMPANY_ID      INT  NOT NULL DEFAULT 0,
		REQUISITE_ID    INT  NOT NULL DEFAULT 0,
		BANK_DETAIL_ID  INT  NOT NULL DEFAULT 0,
		CONSTRAINT PK_B_CRM_PS_RQ_CONV_RELATION PRIMARY KEY CLUSTERED (ENTITY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_CRM_PS_RQ_CONV_RELATION_1')
	CREATE INDEX IX_CRM_PS_RQ_CONV_RELATION_1 ON B_CRM_PS_RQ_CONV_RELATION (REQUISITE_ID ASC)
GO
IF OBJECT_ID(N'B_CRM_WEBFORM', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM (
    ID int IDENTITY(1,1) NOT NULL,
    NAME varchar(255) NOT NULL,
    DESCRIPTION text DEFAULT NULL,
    BUTTON_CAPTION varchar(255) DEFAULT NULL,
    TEMPLATE_ID varchar(255) DEFAULT NULL,
    IS_PAY char(1) NOT NULL,
    GOOGLE_ANALYTICS_ID varchar(50) DEFAULT NULL,
    YANDEX_METRIC_ID varchar(50) DEFAULT NULL,
    SECURITY_CODE varchar(32) DEFAULT NULL,
    BUTTON_COLOR_FONT varchar(10) DEFAULT NULL,
    BUTTON_COLOR_BG varchar(10) DEFAULT NULL,
    CSS_PATH varchar(255) DEFAULT NULL,
    DATE_CREATE datetime NOT NULL,
    ENTITY_SCHEME int NOT NULL,
    GOOGLE_ANALYTICS_PAGE_VIEW char(1) NOT NULL DEFAULT 'N',
    CAPTION varchar(255) DEFAULT NULL,
    RESULT_SUCCESS_TEXT varchar(255) DEFAULT NULL,
    RESULT_SUCCESS_URL varchar(500) DEFAULT NULL,
    RESULT_FAILURE_TEXT varchar(255) DEFAULT NULL,
    RESULT_FAILURE_URL varchar(500) DEFAULT NULL,
    ACTIVE char(1) NOT NULL DEFAULT 'N',
    CSS_TEXT text,
    ACTIVE_CHANGE_BY int DEFAULT NULL,
    ACTIVE_CHANGE_DATE datetime NULL,
    IS_SYSTEM char(1) NOT NULL DEFAULT 'N',
    COPYRIGHT_REMOVED char(1) NOT NULL DEFAULT 'N',
    BACKGROUND_IMAGE int DEFAULT NULL,
    CODE varchar(255) DEFAULT NULL,
    OPENLINE_ID int DEFAULT NULL,
    DUPLICATE_MODE varchar(20) DEFAULT NULL,
    XML_ID varchar(50) DEFAULT NULL,
    LICENCE_TEXT text,
    LICENCE_BUTTON_CAPTION varchar(255) DEFAULT NULL,
    SCRIPT_INCLUDE_SETTINGS text,
    INVOICE_SETTINGS text,
    FORM_SETTINGS text,
    IS_CALLBACK_FORM char(1) NOT NULL DEFAULT 'N',
    CALL_FROM varchar(50) DEFAULT NULL,
    CALL_TEXT text,
    ASSIGNED_BY_ID int DEFAULT NULL,
    PRIMARY KEY (ID)
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM')
	CREATE INDEX IX_B_CRM_WEBFORM ON B_CRM_WEBFORM (CODE ASC)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_COUNTER', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_COUNTER (
    ID int IDENTITY(1,1) NOT NULL,
    FORM_ID int NOT NULL,
    DATE_CREATE datetime NULL,
    VIEWS int DEFAULT 0,
    START_FILL int DEFAULT 0,
    END_FILL int DEFAULT 0,
    MONEY int DEFAULT 0,
    ENTITY_CONTACT int DEFAULT 0,
    ENTITY_COMPANY int DEFAULT 0,
    ENTITY_DEAL int DEFAULT 0,
    ENTITY_LEAD int DEFAULT 0,
    ENTITY_QUOTE int DEFAULT 0,
    ENTITY_INVOICE int DEFAULT 0,
    PRIMARY KEY (ID)
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_COUNTER')
	CREATE INDEX IX_B_CRM_WEBFORM_COUNTER ON B_CRM_WEBFORM_COUNTER(FORM_ID)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_VIEW', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_VIEW (
    FORM_ID int NOT NULL,
    DATE_CREATE datetime NULL
  )
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_START_EDIT', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_START_EDIT (
    FORM_ID int NOT NULL,
    DATE_CREATE datetime NULL
  )
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_FIELD', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_FIELD (
    ID int IDENTITY(1,1) NOT NULL,
    TYPE varchar(50) DEFAULT NULL,
    CODE varchar(50) DEFAULT NULL,
    CAPTION varchar(255) DEFAULT NULL,
    FORM_ID int NOT NULL,
    GROUP_ID int NULL,
    SORT int NOT NULL DEFAULT '100',
    REQUIRED char(1) DEFAULT 'N',
    PLACEHOLDER varchar(255) DEFAULT NULL,
    MULTIPLE char(1) NOT NULL DEFAULT 'N',
    ITEMS text NOT NULL,
    VALUE_TYPE varchar(50) DEFAULT NULL,
    PRIMARY KEY (ID)
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_FIELD')
	CREATE INDEX IX_B_CRM_WEBFORM_FIELD ON B_CRM_WEBFORM_FIELD(FORM_ID)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_FIELD_DEP', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_FIELD_DEP (
    ID int IDENTITY(1,1) NOT NULL,
    FORM_ID int NOT NULL,
    IF_FIELD_CODE varchar(25) DEFAULT NULL,
    IF_ACTION varchar(20) NOT NULL,
    IF_VALUE varchar(255) DEFAULT NULL,
    IF_VALUE_OPERATION varchar(10) DEFAULT NULL,
    DO_FIELD_CODE varchar(25) DEFAULT NULL,
    DO_ACTION varchar(20) NOT NULL,
    DO_VALUE varchar(255) DEFAULT NULL,
    PRIMARY KEY (ID)
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_FIELD_DEP')
	CREATE INDEX IX_B_CRM_WEBFORM_FIELD_DEP ON B_CRM_WEBFORM_FIELD_DEP(FORM_ID)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_FIELD_PRESET', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_FIELD_PRESET (
    FORM_ID int NOT NULL,
    ENTITY_NAME varchar(25) NOT NULL,
    FIELD_NAME varchar(50) NOT NULL,
    VALUE varchar(255) DEFAULT NULL
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_FIELD_PRST')
	CREATE INDEX IX_B_CRM_WEBFORM_FIELD_PRST ON B_CRM_WEBFORM_FIELD_PRESET(FORM_ID)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_RESULT', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_RESULT (
    ID int IDENTITY(1,1) NOT NULL,
    DATE_INSERT datetime NOT NULL,
    FORM_ID int NOT NULL,
    PRIMARY KEY (ID)
  )
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_RESULT')
	CREATE INDEX IX_B_CRM_WEBFORM_RESULT ON B_CRM_WEBFORM_RESULT(FORM_ID)
GO

IF OBJECT_ID(N'B_CRM_WEBFORM_RESULT_ENTITY', N'U') IS NULL
  CREATE TABLE B_CRM_WEBFORM_RESULT_ENTITY (
    FORM_ID int NOT NULL,
    RESULT_ID int NOT NULL,
    ENTITY_NAME varchar(20) NOT NULL,
    ITEM_ID int NOT NULL
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_WEBFORM_RESULT_ENT')
	CREATE INDEX IX_B_CRM_WEBFORM_RESULT_ENT ON B_CRM_WEBFORM_RESULT_ENTITY(RESULT_ID)
GO

IF OBJECT_ID(N'B_CRM_ACT_MAIL_META', N'U') IS NULL
	CREATE TABLE B_CRM_ACT_MAIL_META
	(
		ACTIVITY_ID INT NOT NULL,
		MSG_ID_HASH VARCHAR(40) NULL,
		MSG_HEADER_HASH VARCHAR(40) NULL,
		CONSTRAINT PK_B_CRM_ACT_MAIL_META PRIMARY KEY CLUSTERED (ACTIVITY_ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_MAIL_META_1')
	CREATE INDEX IX_B_CRM_ACT_MAIL_META_1 ON B_CRM_ACT_MAIL_META (MSG_ID_HASH ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_MAIL_META_2')
	CREATE INDEX IX_B_CRM_ACT_MAIL_META_2 ON B_CRM_ACT_MAIL_META (MSG_HEADER_HASH ASC)
GO

IF OBJECT_ID(N'B_CRM_DEAL_CATEGORY', N'U') IS NULL
	CREATE TABLE B_CRM_DEAL_CATEGORY(
		ID INT IDENTITY(1,1) NOT NULL,
		CREATED_DATE DATETIME NOT NULL,
		NAME VARCHAR(255) NOT NULL,
		IS_LOCKED CHAR(1) NOT NULL DEFAULT 'N',
		SORT INT NOT NULL,
		CONSTRAINT PK_B_CRM_DEAL_CATEGORY PRIMARY KEY CLUSTERED(ID ASC)
	)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_CAT_1')
	CREATE INDEX IX_B_CRM_DEAL_CAT_1 ON B_CRM_DEAL_CATEGORY(SORT ASC, ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_DEAL_CAT_2')
	CREATE INDEX IX_B_CRM_DEAL_CAT_2 ON B_CRM_DEAL_CATEGORY(IS_LOCKED ASC, SORT ASC, ID ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_ACT_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_ACT_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	TOTAL_QTY INT NULL,
	CONSTRAINT PK_B_CRM_COMPANY_ACT_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTAT_1')
	CREATE INDEX IX_B_CRM_COMPANY_ASTAT_1 ON B_CRM_COMPANY_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, TOTAL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTAT_2')
	CREATE INDEX IX_B_CRM_COMPANY_ASTAT_2 ON B_CRM_COMPANY_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, TOTAL_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_ACT_STS_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_ACT_STS_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	ANSWERED_QTY INT NULL,
	UNANSWERED_QTY INT NULL,
	CONSTRAINT PK_B_CRM_COMPANY_ACT_STS_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTS_1')
	CREATE INDEX IX_B_CRM_COMPANY_ASTS_1 ON B_CRM_COMPANY_ACT_STS_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, ANSWERED_QTY ASC, UNANSWERED_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTS_2')
	CREATE INDEX IX_B_CRM_COMPANY_ASTS_2 ON B_CRM_COMPANY_ACT_STS_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, ANSWERED_QTY ASC, UNANSWERED_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_ACT_MARK_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_ACT_MARK_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(50) NOT NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	NONE_QTY INT NULL,
	POSITIVE_QTY INT NULL,
	NEGATIVE_QTY INT NULL,
	CONSTRAINT PK_B_CRM_COMPANY_ACT_MARK_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_AMRS_1')
	CREATE INDEX IX_B_CRM_COMPANY_AMRS_1 ON B_CRM_COMPANY_ACT_MARK_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC, NONE_QTY ASC, POSITIVE_QTY ASC, NEGATIVE_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_AMRS_2')
	CREATE INDEX IX_B_CRM_COMPANY_AMRS_2 ON B_CRM_COMPANY_ACT_MARK_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC, NONE_QTY ASC, POSITIVE_QTY ASC, NEGATIVE_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_ACT_STM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_ACT_STM_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	INCOMING_QTY INT NULL,
	OUTGOING_QTY INT NULL,
	REVERSING_QTY INT NULL,
	MISSING_QTY INT NULL,
	CONSTRAINT PK_B_CRM_COMPANY_ACT_STM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTM_1')
	CREATE INDEX IX_B_CRM_COMPANY_ASTM_1 ON B_CRM_COMPANY_ACT_STM_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, INCOMING_QTY ASC, OUTGOING_QTY ASC, REVERSING_QTY ASC, MISSING_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASTM_2')
	CREATE INDEX IX_B_CRM_COMPANY_ASTM_2 ON B_CRM_COMPANY_ACT_STM_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, INCOMING_QTY ASC, OUTGOING_QTY ASC, REVERSING_QTY ASC, MISSING_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_ACT_SUM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_ACT_SUM_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL decimal(18, 4) NULL,
	CONSTRAINT PK_B_CRM_COMPANY_ACT_SUM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASUM_1')
	CREATE INDEX IX_B_CRM_COMPANY_ASUM_1 ON B_CRM_COMPANY_ACT_SUM_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_ASUM_2')
	CREATE INDEX IX_B_CRM_COMPANY_ASUM_2 ON B_CRM_COMPANY_ACT_SUM_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_COMPANY_GROWTH_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_COMPANY_GROWTH_STAT(
	OWNER_ID INT NOT NULL,
	CREATED_DATE DATETIME NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CONSTRAINT PK_B_CRM_COMPANY_GROWTH_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_GRTH_1')
	CREATE INDEX IX_B_CRM_COMPANY_GRTH_1 ON B_CRM_COMPANY_GROWTH_STAT(CREATED_DATE ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_COMPANY_GRTH_2')
	CREATE INDEX IX_B_CRM_COMPANY_GRTH_2 ON B_CRM_COMPANY_GROWTH_STAT(RESPONSIBLE_ID ASC, CREATED_DATE ASC, OWNER_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_ACT_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_ACT_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	TOTAL_QTY INT NULL,
	CONSTRAINT PK_B_CRM_CONTACT_ACT_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTAT_1')
	CREATE INDEX IX_B_CRM_CONTACT_ASTAT_1 ON B_CRM_CONTACT_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, TOTAL_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTAT_2')
	CREATE INDEX IX_B_CRM_CONTACT_ASTAT_2 ON B_CRM_CONTACT_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, TOTAL_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_ACT_STS_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_ACT_STS_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	ANSWERED_QTY INT NULL,
	UNANSWERED_QTY INT NULL,
	CONSTRAINT PK_B_CRM_CONTACT_ACT_STS_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTS_1')
	CREATE INDEX IX_B_CRM_CONTACT_ASTS_1 ON B_CRM_CONTACT_ACT_STS_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, ANSWERED_QTY ASC, UNANSWERED_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTS_2')
	CREATE INDEX IX_B_CRM_CONTACT_ASTS_2 ON B_CRM_CONTACT_ACT_STS_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, ANSWERED_QTY ASC, UNANSWERED_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_ACT_MARK_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_ACT_MARK_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(50) NOT NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	NONE_QTY INT NULL,
	POSITIVE_QTY INT NULL,
	NEGATIVE_QTY INT NULL,
	CONSTRAINT PK_B_CRM_CONTACT_ACT_MARK_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_AMRS_1')
	CREATE INDEX IX_B_CRM_CONTACT_AMRS_1 ON B_CRM_CONTACT_ACT_MARK_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC, NONE_QTY ASC, POSITIVE_QTY ASC, NEGATIVE_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_AMRS_2')
	CREATE INDEX IX_B_CRM_CONTACT_AMRS_2 ON B_CRM_CONTACT_ACT_MARK_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, SOURCE_ID ASC, NONE_QTY ASC, POSITIVE_QTY ASC, NEGATIVE_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_ACT_STM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_ACT_STM_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	INCOMING_QTY INT NULL,
	OUTGOING_QTY INT NULL,
	REVERSING_QTY INT NULL,
	MISSING_QTY INT NULL,
	CONSTRAINT PK_B_CRM_CONTACT_ACT_STM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTM_1')
	CREATE INDEX IX_B_CRM_CONTACT_ASTM_1 ON B_CRM_CONTACT_ACT_STM_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, INCOMING_QTY ASC, OUTGOING_QTY ASC, REVERSING_QTY ASC, MISSING_QTY ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASTM_2')
	CREATE INDEX IX_B_CRM_CONTACT_ASTM_2 ON B_CRM_CONTACT_ACT_STM_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, INCOMING_QTY ASC, OUTGOING_QTY ASC, REVERSING_QTY ASC, MISSING_QTY ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_ACT_SUM_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_ACT_SUM_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(100) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL decimal(18, 4) NULL,
	CONSTRAINT PK_B_CRM_CONTACT_ACT_SUM_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASUM_1')
	CREATE INDEX IX_B_CRM_CONTACT_ASUM_1 ON B_CRM_CONTACT_ACT_SUM_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_ASUM_2')
	CREATE INDEX IX_B_CRM_CONTACT_ASUM_2 ON B_CRM_CONTACT_ACT_SUM_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_CONTACT_GROWTH_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_CONTACT_GROWTH_STAT(
	OWNER_ID INT NOT NULL,
	CREATED_DATE DATETIME NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	CONSTRAINT PK_B_CRM_CONTACT_GROWTH_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, CREATED_DATE)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_GRTH_1')
	CREATE INDEX IX_B_CRM_CONTACT_GRTH_1 ON B_CRM_CONTACT_GROWTH_STAT(CREATED_DATE ASC, OWNER_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_CONTACT_GRTH_2')
	CREATE INDEX IX_B_CRM_CONTACT_GRTH_2 ON B_CRM_CONTACT_GROWTH_STAT(RESPONSIBLE_ID ASC, CREATED_DATE ASC, OWNER_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_ACT_STAT', N'U') IS NULL
	CREATE TABLE B_CRM_ACT_STAT(
	OWNER_ID INT NOT NULL,
	DEADLINE_DATE DATETIME NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(50) NOT NULL,
	RESPONSIBLE_ID INT NOT NULL,
	COMPLETED CHAR(1) NULL,
	STATUS_ID INT NULL,
	MARK_ID INT NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	STREAM_ID INT NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL decimal(18, 4) NULL,
	CONSTRAINT PK_B_CRM_ACT_STAT PRIMARY KEY CLUSTERED(OWNER_ID ASC, DEADLINE_DATE ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC)
)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_1')
	CREATE INDEX IX_B_CRM_ACT_STAT_1 ON B_CRM_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, COMPLETED ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_2')
	CREATE INDEX IX_B_CRM_ACT_STAT_2 ON B_CRM_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, COMPLETED ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_5')
	CREATE INDEX IX_B_CRM_ACT_STAT_5 ON B_CRM_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, STATUS_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_6')
	CREATE INDEX IX_B_CRM_ACT_STAT_6 ON B_CRM_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, STATUS_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_7')
	CREATE INDEX IX_B_CRM_ACT_STAT_7 ON B_CRM_ACT_STAT(RESPONSIBLE_ID ASC, DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, STREAM_ID ASC)
GO

IF NOT EXISTS (SELECT name FROM sys.indexes WHERE name = N'IX_B_CRM_ACT_STAT_8')
	CREATE INDEX IX_B_CRM_ACT_STAT_8 ON B_CRM_ACT_STAT(DEADLINE_DATE ASC, OWNER_ID ASC, PROVIDER_ID ASC, PROVIDER_TYPE_ID ASC, STREAM_ID ASC)
GO

IF OBJECT_ID(N'B_CRM_BUTTON', N'U') IS NULL
  CREATE TABLE B_CRM_BUTTON (
  ID INT IDENTITY(1,1) NOT NULL,
  DATE_CREATE DATETIME NULL,
  CREATED_BY INT DEFAULT NULL,
  ACTIVE CHAR(1) NOT NULL DEFAULT 'N',
  ACTIVE_CHANGE_BY INT DEFAULT NULL,
  ACTIVE_CHANGE_DATE DATETIME NULL,
  NAME VARCHAR(255) NOT NULL,
  BACKGROUND_COLOR VARCHAR(10) DEFAULT NULL,
  ICON_COLOR VARCHAR(10) DEFAULT NULL,
  LOCATION INT NOT NULL DEFAULT 1,
  DELAY INT NOT NULL DEFAULT 0,
  ITEMS TEXT,
  SECURITY_CODE varchar(32) DEFAULT NULL,
  IS_SYSTEM char(1) NOT NULL DEFAULT 'N',
  XML_ID varchar(50) DEFAULT NULL,
  PRIMARY KEY (ID)
)
GO