CREATE TABLE b_crm_lead
(
	ID INT (18) UNSIGNED NOT NULL AUTO_INCREMENT,
	DATE_CREATE DATETIME DEFAULT '0000-00-00 00:00:00',
	DATE_MODIFY DATETIME DEFAULT '0000-00-00 00:00:00',
	CREATED_BY_ID INT (18) UNSIGNED NOT NULL,
	MODIFY_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	ASSIGNED_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	OPENED CHAR(1) DEFAULT 'N',
	COMPANY_ID INT (18) UNSIGNED DEFAULT NULL,
	CONTACT_ID INT (18) UNSIGNED DEFAULT NULL,
	STATUS_ID VARCHAR (50) DEFAULT NULL,
	STATUS_DESCRIPTION TEXT DEFAULT NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	PRODUCT_ID VARCHAR (50) DEFAULT NULL,
	OPPORTUNITY DECIMAL(18,2) DEFAULT NULL,
	CURRENCY_ID VARCHAR (50) DEFAULT NULL,
	OPPORTUNITY_ACCOUNT DECIMAL(18,2) DEFAULT NULL,
	ACCOUNT_CURRENCY_ID VARCHAR (50) DEFAULT NULL,
	SOURCE_ID VARCHAR (50) DEFAULT NULL,
	SOURCE_DESCRIPTION TEXT DEFAULT NULL,
	TITLE VARCHAR (255) DEFAULT NULL,
	FULL_NAME VARCHAR (100) DEFAULT NULL,
	NAME VARCHAR (50) DEFAULT NULL,
	LAST_NAME VARCHAR (50) DEFAULT NULL,
	SECOND_NAME VARCHAR (50) DEFAULT NULL,
	COMPANY_TITLE VARCHAR (255) DEFAULT NULL,
	POST VARCHAR (255) DEFAULT NULL,
	ADDRESS TEXT DEFAULT NULL,
	COMMENTS TEXT DEFAULT NULL,
	EXCH_RATE DECIMAL(18,2) DEFAULT 1,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	DATE_CLOSED DATETIME NULL,
	BIRTHDATE DATE NULL,
	BIRTHDAY_SORT INT(1) NOT NULL DEFAULT 1024,
	HONORIFIC VARCHAR(128) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL,
	PRIMARY KEY (ID),
	INDEX IX_LEAD_FULL_NAME (FULL_NAME),
	INDEX IX_LEAD_1 (STATUS_ID, ASSIGNED_BY_ID),
	INDEX IX_LEAD_2 (DATE_MODIFY),
	INDEX IX_LEAD_3 (CONTACT_ID),
	INDEX IX_LEAD_4 (COMPANY_ID),
	INDEX IX_LEAD_5 (BIRTHDAY_SORT, ASSIGNED_BY_ID, ID),
	INDEX IX_LEAD_6 (BIRTHDATE, ID),
	INDEX IX_LEAD_7 (HAS_PHONE, ID),
	INDEX IX_LEAD_8 (HAS_EMAIL, ID),
	INDEX IX_LEAD_9 (HAS_PHONE, HAS_EMAIL, ID),
	INDEX IX_LEAD_10 (STATUS_SEMANTIC_ID, ID)
);

CREATE TABLE b_crm_deal
(
	ID INT (18) UNSIGNED NOT NULL AUTO_INCREMENT,
	DATE_CREATE DATETIME DEFAULT '0000-00-00 00:00:00',
	DATE_MODIFY DATETIME DEFAULT '0000-00-00 00:00:00',
	CREATED_BY_ID INT (18) UNSIGNED NOT NULL,
	MODIFY_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	ASSIGNED_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	OPENED CHAR(1) DEFAULT 'N',
	LEAD_ID INT (18) DEFAULT NULL,
	COMPANY_ID INT (18) UNSIGNED DEFAULT NULL,
	CONTACT_ID INT (18) UNSIGNED DEFAULT NULL,
	QUOTE_ID INT(1) UNSIGNED DEFAULT NULL,
	TITLE VARCHAR (255) DEFAULT NULL,
	PRODUCT_ID VARCHAR (50) DEFAULT NULL,
	CATEGORY_ID INT (18) UNSIGNED NULL,
	STAGE_ID VARCHAR (50) DEFAULT NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	IS_NEW CHAR(1) NULL,
	CLOSED CHAR (1) DEFAULT 'N',
	TYPE_ID VARCHAR (50) DEFAULT NULL,
	OPPORTUNITY DECIMAL(18,2) DEFAULT NULL,
	TAX_VALUE DECIMAL(18,2) DEFAULT NULL,
	CURRENCY_ID VARCHAR (50) DEFAULT NULL,
	OPPORTUNITY_ACCOUNT DECIMAL(18,2) DEFAULT NULL,
	TAX_VALUE_ACCOUNT DECIMAL(18,2) DEFAULT NULL,
	ACCOUNT_CURRENCY_ID VARCHAR (50) DEFAULT NULL,
	PROBABILITY TINYINT (3) DEFAULT NULL,
	COMMENTS TEXT DEFAULT NULL,
	BEGINDATE DATETIME DEFAULT NULL,
	CLOSEDATE DATETIME DEFAULT NULL,
	EVENT_DATE DATETIME NULL,
	EVENT_ID VARCHAR(50) NULL,
	EVENT_DESCRIPTION text NULL,
	EXCH_RATE DECIMAL(18,2) DEFAULT 1,
	LOCATION_ID VARCHAR(100) NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	ADDITIONAL_INFO TEXT NULL,
	PRIMARY KEY (ID),
	INDEX IX_DEAL_1 (STAGE_ID, ASSIGNED_BY_ID),
	INDEX IX_DEAL_2 (COMPANY_ID),
	INDEX IX_DEAL_3 (CONTACT_ID),
	INDEX IX_DEAL_4 (CATEGORY_ID, ID),
	INDEX IX_DEAL_5 (STAGE_SEMANTIC_ID, ID),
	INDEX IX_DEAL_6 (IS_NEW, ID)
);

CREATE TABLE b_crm_contact
(
	ID INT (18) UNSIGNED NOT NULL AUTO_INCREMENT,
	DATE_CREATE DATETIME DEFAULT '0000-00-00 00:00:00',
	DATE_MODIFY DATETIME DEFAULT '0000-00-00 00:00:00',
	CREATED_BY_ID INT (18) UNSIGNED NOT NULL,
	MODIFY_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	ASSIGNED_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	OPENED CHAR(1) DEFAULT 'N',
	COMPANY_ID INT (18) UNSIGNED DEFAULT NULL,
	SOURCE_ID VARCHAR (50) DEFAULT NULL,
	SOURCE_DESCRIPTION TEXT DEFAULT NULL,
	FULL_NAME VARCHAR (100) DEFAULT NULL,
	NAME VARCHAR (50) DEFAULT NULL,
	LAST_NAME VARCHAR (50) DEFAULT NULL,
	SECOND_NAME VARCHAR (50) DEFAULT NULL,
	PHOTO VARCHAR (10) DEFAULT NULL,
	POST VARCHAR (255) DEFAULT NULL,
	ADDRESS TEXT DEFAULT NULL,
	COMMENTS TEXT DEFAULT NULL,
	LEAD_ID INT (18) DEFAULT NULL,
	EXPORT CHAR (1) DEFAULT 'N',
	TYPE_ID VARCHAR (50) DEFAULT NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	ORIGIN_VERSION VARCHAR(255) NULL,
	BIRTHDATE DATE DEFAULT NULL,
	BIRTHDAY_SORT INT(1) NOT NULL DEFAULT 1024,
	HONORIFIC VARCHAR(128) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL,
	PRIMARY KEY (ID),
	INDEX IX_CONTACT_LAST_NAME (NAME),
	INDEX IX_CONTACT_NAME (LAST_NAME),
	INDEX IX_CONTACT_FULL_NAME (FULL_NAME),
	INDEX IX_CONTACT_BDAY (BIRTHDAY_SORT, ASSIGNED_BY_ID, ID),
	INDEX IX_CONTACT_BDATE (BIRTHDATE, ID),
	INDEX IX_CONTACT_COMPANY (COMPANY_ID, ID),
	INDEX IX_CONTACT_PHONE (HAS_PHONE, ID),
	INDEX IX_CONTACT_EMAIL (HAS_EMAIL, ID),
	INDEX IX_CONTACT_COMM (HAS_PHONE, HAS_EMAIL, ID)
);

CREATE TABLE b_crm_company
(
	ID INT (18) UNSIGNED NOT NULL AUTO_INCREMENT,
	DATE_CREATE DATETIME DEFAULT '0000-00-00 00:00:00',
	DATE_MODIFY DATETIME DEFAULT '0000-00-00 00:00:00',
	CREATED_BY_ID INT (18) UNSIGNED NOT NULL,
	MODIFY_BY_ID INT (18) UNSIGNED DEFAULT NULL,
	ASSIGNED_BY_ID INT (1) UNSIGNED DEFAULT NULL,
	OPENED CHAR(1) DEFAULT 'N',
	TITLE varchar(255) DEFAULT NULL,
	LOGO VARCHAR (10) DEFAULT NULL,
	ADDRESS text DEFAULT NULL,
	ADDRESS_LEGAL text DEFAULT NULL,
	BANKING_DETAILS text DEFAULT NULL,
	COMMENTS text DEFAULT NULL,
	COMPANY_TYPE varchar(50) DEFAULT NULL,
	INDUSTRY varchar(50) DEFAULT NULL,
	REVENUE varchar(255) DEFAULT NULL,
	CURRENCY_ID varchar(50) DEFAULT NULL,
	EMPLOYEES varchar(50) DEFAULT NULL,
	LEAD_ID int(18) DEFAULT NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	ORIGIN_VERSION VARCHAR(255) NULL,
	HAS_PHONE CHAR(1) NULL,
	HAS_EMAIL CHAR(1) NULL,
	IS_MY_COMPANY CHAR(1) NOT NULL DEFAULT 'N',
	PRIMARY KEY  (ID),
	KEY IX_COMPANY_TITLE (TITLE),
	INDEX IX_COMPANY_PHONE (HAS_PHONE, ID),
	INDEX IX_COMPANY_EMAIL (HAS_EMAIL, ID),
	INDEX IX_COMPANY_COMM (HAS_PHONE, HAS_EMAIL, ID),
	INDEX IX_COMPANY_ISMY (IS_MY_COMPANY)
);

CREATE TABLE b_crm_status
(
	ID INT (18) NOT NULL AUTO_INCREMENT,
	ENTITY_ID VARCHAR (50) NOT NULL,
	STATUS_ID VARCHAR (50) NOT NULL,
	NAME VARCHAR (100) NOT NULL,
	NAME_INIT VARCHAR (100) NULL,
	SORT INT (18) NOT NULL,
	SYSTEM CHAR (1) NOT NULL,
	PRIMARY KEY (ID),
	INDEX IX_B_CRM_STATUS (STATUS_ID, ENTITY_ID)
);

CREATE TABLE b_crm_field_multi
(
	ID int(18) NOT NULL auto_increment,
	ENTITY_ID VARCHAR (16) NOT NULL,
	ELEMENT_ID int(18) NOT NULL,
	TYPE_ID VARCHAR (16) NOT NULL,
	VALUE_TYPE VARCHAR (50) NOT NULL,
	COMPLEX_ID  VARCHAR(100) NOT NULL,
	VALUE VARCHAR (250) NOT NULL,
	PRIMARY KEY  (ID),
	KEY IX_FM_ENTITY_ELEMENT (ENTITY_ID, ELEMENT_ID),
	KEY IX_B_CRM_FM_2 (ENTITY_ID, TYPE_ID, ELEMENT_ID, ID),
	KEY IX_B_CRM_FM_3 (TYPE_ID, VALUE, ENTITY_ID, ELEMENT_ID)
);

CREATE TABLE b_crm_event
(
	ID int(18) NOT NULL auto_increment,
	DATE_CREATE datetime NOT NULL,
	CREATED_BY_ID int(18) NOT NULL,
	EVENT_ID varchar(50) NULL,
	EVENT_NAME varchar(255) NOT NULL,
	EVENT_TEXT_1 text NULL,
	EVENT_TEXT_2 text NULL,
	EVENT_TYPE int(11) NULL,
	FILES text NULL,
	PRIMARY KEY  (ID),
	INDEX IX_EVENT_1 (DATE_CREATE),
	INDEX IX_B_CRM_EVENT_2 (EVENT_TYPE, EVENT_ID, ID),
	INDEX IX_B_CRM_EVENT_3 (CREATED_BY_ID, ID),
	INDEX IX_B_CRM_EVENT_4 (EVENT_TYPE, DATE_CREATE)
);

CREATE TABLE b_crm_event_relations
(
	ID int(18) NOT NULL auto_increment,
	ASSIGNED_BY_ID int(18) NOT NULL,
	ENTITY_TYPE varchar(50) NULL,
	ENTITY_ID int(11) NOT NULL,
	ENTITY_FIELD varchar(255) NULL,
	EVENT_ID int(18) NOT NULL,
	PRIMARY KEY  (ID),
	KEY IX_EVENT_REL_ENTITY (ENTITY_TYPE, ENTITY_ID, ENTITY_FIELD),
	INDEX IX_EVENT_REL_1 (EVENT_ID),
	INDEX IX_EVENT_REL_2 (ENTITY_TYPE, ENTITY_ID, EVENT_ID)
);

CREATE TABLE b_crm_entity_lock
(
	ENTITY_ID INT (11) NOT NULL,
	ENTITY_TYPE VARCHAR (10) NOT NULL,
	DATE_LOCK DATETIME DEFAULT NULL,
	LOCKED_BY VARCHAR (32) DEFAULT NULL,
	PRIMARY KEY (ENTITY_ID)
);

CREATE TABLE b_crm_entity_perms
(
	ID INT(18) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY VARCHAR(20) NOT NULL,
	ENTITY_ID INT(18) NOT NULL,
	ATTR VARCHAR(30) NOT NULL,
	PRIMARY KEY (ID),
	INDEX IX_ENTITY_ATTR (ENTITY, ENTITY_ID, ATTR),
	INDEX IX_ENTITY_ATTR_1(ENTITY, ATTR, ENTITY_ID)
);

CREATE TABLE b_crm_role
(
	ID INT(18) UNSIGNED NOT NULL AUTO_INCREMENT,
	NAME VARCHAR(255) NOT NULL,
	PRIMARY KEY (ID)
);

CREATE TABLE b_crm_role_perms
(
	ID INT(18) UNSIGNED NOT NULL AUTO_INCREMENT,
	ROLE_ID INT(18) UNSIGNED NOT NULL,
	ENTITY VARCHAR(20) NOT NULL,
	FIELD VARCHAR(30) DEFAULT '-',
	FIELD_VALUE VARCHAR(255) DEFAULT NULL,
	PERM_TYPE VARCHAR(20) NOT NULL,
	ATTR CHAR(1) DEFAULT '',
	PRIMARY KEY (ID),
	INDEX IX_ROLE_ID (ROLE_ID)
);

CREATE TABLE b_crm_role_relation
(
	ID INT(18) NOT NULL AUTO_INCREMENT,
	ROLE_ID INT(18) UNSIGNED NOT NULL,
	RELATION VARCHAR(100) NOT NULL,
	PRIMARY KEY (ID),
	INDEX IX_ROLE_RELATION (ROLE_ID, RELATION)
);

CREATE TABLE IF NOT EXISTS b_crm_external_sale
(
	ID INT(18) NOT NULL AUTO_INCREMENT,
	ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
	DATE_CREATE datetime NOT NULL,
	DATE_UPDATE datetime NOT NULL,
	NAME VARCHAR(128) NULL,
	SCHEME VARCHAR(5) NOT NULL DEFAULT 'http',
	SERVER VARCHAR(192) NOT NULL,
	PORT INT(18) NOT NULL DEFAULT 80,
	LOGIN VARCHAR(64) NOT NULL,
	PASSWORD VARCHAR(128) NOT NULL,
	MODIFICATION_LABEL INT(18) NULL,
	IMPORT_SIZE INT(18) NULL,
	IMPORT_PERIOD INT(18) NULL,
	IMPORT_PROBABILITY INT(18) NULL,
	IMPORT_RESPONSIBLE INT(18) NULL,
	IMPORT_PUBLIC char(1) NULL,
	IMPORT_PREFIX VARCHAR(128) NULL,
	IMPORT_ERRORS INT(18) NULL,
	IMPORT_GROUP_ID INT(18) NULL,
	COOKIE TEXT NULL,
	LAST_STATUS TEXT NULL,
	LAST_STATUS_DATE datetime NULL,
	SYNC_DATA TEXT NULL,
	PRIMARY KEY (ID)
);

CREATE TABLE IF NOT EXISTS b_crm_catalog
(
	ID INT(1) NOT NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	PRIMARY KEY(ID)
);

CREATE TABLE IF NOT EXISTS b_crm_product
(
	ID INT(1) NOT NULL,
	CATALOG_ID INT(1) NOT NULL,
	CURRENCY_ID VARCHAR(50) NOT NULL,
	PRICE DECIMAL(18,2) NOT NULL DEFAULT 0,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	PRIMARY KEY(ID)
);

CREATE TABLE IF NOT EXISTS b_crm_product_row
(
	ID INT(18) UNSIGNED NOT NULL AUTO_INCREMENT,
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE CHAR(3) NOT NULL,
	PRODUCT_ID INT(1) NOT NULL,
	PRODUCT_NAME VARCHAR(256) NULL,
	PRICE DECIMAL(18,2) NOT NULL,
	PRICE_ACCOUNT DECIMAL(18,2) NOT NULL DEFAULT 0,
	PRICE_EXCLUSIVE DECIMAL(18,2) NULL,
	PRICE_NETTO DECIMAL(18,2) NULL,
	PRICE_BRUTTO DECIMAL(18,2) NULL,
	QUANTITY DECIMAL(18,4) NOT NULL,
	DISCOUNT_TYPE_ID TINYINT(1) UNSIGNED NULL,
	DISCOUNT_RATE DECIMAL(18,2) NULL,
	DISCOUNT_SUM DECIMAL(18,2) NULL,
	TAX_RATE DECIMAL(18,2) NULL,
	TAX_INCLUDED CHAR(1) NULL,
	CUSTOMIZED CHAR(1) NULL,
	MEASURE_CODE INT(1) UNSIGNED NULL,
	MEASURE_NAME VARCHAR(50) NULL,
	SORT INT(1) NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_PROD_ROW_1(OWNER_ID, OWNER_TYPE, SORT, ID)
);

CREATE TABLE IF NOT EXISTS b_crm_product_row_cfg(
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE CHAR(3) NOT NULL,
	SETTINGS TEXT,
	PRIMARY KEY(OWNER_ID, OWNER_TYPE)
);

CREATE TABLE IF NOT EXISTS b_crm_act(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	PROVIDER_ID varchar(100) NULL,
	PROVIDER_TYPE_ID varchar(100) NULL,
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ASSOCIATED_ENTITY_ID INT(1),
	CALENDAR_EVENT_ID INT(1) UNSIGNED NOT NULL DEFAULT 0,
	SUBJECT VARCHAR(512) NOT NULL,
	COMPLETED CHAR(1) NOT NULL DEFAULT 'N',
	RESPONSIBLE_ID INT(1) NOT NULL,
	PRIORITY INT(1) NOT NULL,
	NOTIFY_TYPE INT(1) NOT NULL,
	NOTIFY_VALUE INT(1) UNSIGNED,
	DESCRIPTION LONGTEXT NULL,
	DESCRIPTION_TYPE TINYINT(1) UNSIGNED NULL,
	DIRECTION TINYINT(1) UNSIGNED NOT NULL,
	LOCATION VARCHAR(256),
	CREATED DATETIME NOT NULL,
	LAST_UPDATED DATETIME NOT NULL,
	START_TIME DATETIME,
	END_TIME DATETIME,
	DEADLINE DATETIME,
	STORAGE_TYPE_ID TINYINT(1) UNSIGNED NULL,
	STORAGE_ELEMENT_IDS TEXT NULL,
	PARENT_ID INT(1) UNSIGNED NOT NULL DEFAULT 0,
	URN VARCHAR(64) NULL,
	SETTINGS TEXT NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ORIGIN_ID VARCHAR(255) NULL,
	AUTHOR_ID INT(1) UNSIGNED NULL,
	EDITOR_ID INT(1) UNSIGNED NULL,
	PROVIDER_PARAMS LONGTEXT NULL,
	PROVIDER_DATA LONGTEXT NULL,
	RESULT_STATUS INT(1) NOT NULL DEFAULT 0,
	RESULT_STREAM INT(1) NOT NULL DEFAULT 0,
	RESULT_SOURCE_ID VARCHAR(255) NULL,
	RESULT_MARK INT(1) NOT NULL DEFAULT 0,
	RESULT_VALUE DECIMAL(18,4) DEFAULT NULL,
	RESULT_SUM DECIMAL(18,4) DEFAULT NULL,
	RESULT_CURRENCY_ID CHAR(3) NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_ACT(ID, PARENT_ID, OWNER_ID, OWNER_TYPE_ID),
	INDEX IX_B_CRM_ACT_1(RESPONSIBLE_ID, COMPLETED, DEADLINE),
	INDEX IX_B_CRM_ACT_2 (ASSOCIATED_ENTITY_ID, TYPE_ID),
	INDEX IX_B_CRM_ACT_4 (DEADLINE),
	INDEX IX_B_CRM_ACT_5 (ORIGIN_ID),
	INDEX IX_B_CRM_ACT_6 (TYPE_ID, COMPLETED, DEADLINE, ID),
	INDEX IX_B_CRM_ACT_7 (OWNER_ID, OWNER_TYPE_ID, TYPE_ID, COMPLETED, DEADLINE, ID),
	INDEX IX_B_CRM_ACT_8 (PROVIDER_ID),
	INDEX IX_B_CRM_ACT_9 (CALENDAR_EVENT_ID),
	INDEX IX_B_CRM_ACT_10 (URN),
	INDEX IX_B_CRM_ACT_11 (PARENT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_act_bind(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ACTIVITY_ID INT(1) NOT NULL,
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_ACT_BIND_1(ACTIVITY_ID),
	INDEX IX_B_CRM_ACT_BIND_2(OWNER_ID, OWNER_TYPE_ID, ACTIVITY_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_act_comm(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ACTIVITY_ID INT(1) UNSIGNED NOT NULL,
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	TYPE VARCHAR(64),
	VALUE VARCHAR(256),
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_SETTINGS TEXT,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_ACT_COMM(ACTIVITY_ID, OWNER_ID, OWNER_TYPE_ID, ENTITY_ID, ENTITY_TYPE_ID),
	INDEX IX_B_CRM_ACT_COMM_2(ENTITY_ID, ENTITY_TYPE_ID, ID),
	INDEX IX_B_CRM_ACT_COMM_3(OWNER_ID, OWNER_TYPE_ID, TYPE)
);

CREATE TABLE IF NOT EXISTS b_crm_act_elem(
	ACTIVITY_ID INT(1) UNSIGNED NOT NULL,
	STORAGE_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ELEMENT_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(ACTIVITY_ID, STORAGE_TYPE_ID, ELEMENT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_usr_act(
	USER_ID INT(1) UNSIGNED NOT NULL,
	OWNER_ID INT(1) NOT NULL,
	OWNER_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ACTIVITY_TIME DATETIME NOT NULL,
	ACTIVITY_ID INT(1) UNSIGNED NOT NULL,
	DEPARTMENT_ID INT(1) UNSIGNED NOT NULL,
	SORT VARCHAR(16) NULL,
	PRIMARY KEY(USER_ID, OWNER_ID, OWNER_TYPE_ID),
	INDEX IX_B_CRM_USR_ACT(USER_ID, OWNER_ID, OWNER_TYPE_ID, ACTIVITY_TIME, ACTIVITY_ID, DEPARTMENT_ID),
	INDEX IX_B_CRM_USR_ACT_2 (OWNER_ID ASC, OWNER_TYPE_ID ASC, USER_ID DESC, SORT DESC)
);

CREATE TABLE IF NOT EXISTS b_crm_usr_mt(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	OWNER_ID INT(1) NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	SCOPE TINYINT(1) UNSIGNED NOT NULL,
	IS_ACTIVE CHAR(1) NOT NULL DEFAULT 'N',
	TITLE VARCHAR(128),
	EMAIL_FROM VARCHAR(255),
	SUBJECT VARCHAR(255),
	BODY TEXT,
	SING_REQUIRED CHAR(1) NOT NULL DEFAULT 'N',
	SORT INT(1) NOT NULL DEFAULT 100,
	CREATED DATETIME NOT NULL,
	LAST_UPDATED DATETIME NOT NULL,
	AUTHOR_ID INT(1) UNSIGNED NOT NULL,
	EDITOR_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_USR_MT(OWNER_ID, ENTITY_TYPE_ID, SCOPE, IS_ACTIVE)
);

CREATE TABLE IF NOT EXISTS b_crm_sl_rel(
	SL_ID INT NOT NULL,
	SL_EVENT_ID VARCHAR(50) NOT NULL,
	SL_ENTITY_TYPE VARCHAR(50) NOT NULL,
	SL_LAST_UPDATED DATETIME,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	SL_PARENT_ENTITY_TYPE VARCHAR(50) NOT NULL,
	PARENT_ENTITY_ID INT(1) UNSIGNED NOT NULL,
	LVL TINYINT(1) UNSIGNED NOT NULL DEFAULT 1,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL DEFAULT 1,
	INDEX IX_B_CRM_SL_REL_1(SL_PARENT_ENTITY_TYPE, PARENT_ENTITY_ID, LVL, SL_ENTITY_TYPE, SL_EVENT_ID, SL_ID),
	INDEX IX_B_CRM_SL_REL_2(SL_ENTITY_TYPE, ENTITY_ID),
	INDEX IX_B_CRM_SL_REL_3(SL_ID, TYPE_ID),
	INDEX IX_B_CRM_SL_REL_4(SL_LAST_UPDATED, SL_PARENT_ENTITY_TYPE, PARENT_ENTITY_ID, LVL, SL_ENTITY_TYPE, SL_EVENT_ID, SL_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_sl_subscr(
	USER_ID INT(1) UNSIGNED NOT NULL,
	SL_ENTITY_TYPE VARCHAR(50) NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(USER_ID, SL_ENTITY_TYPE, ENTITY_ID, TYPE_ID),
	INDEX IX_B_CRM_SL_SUBSCR_1(SL_ENTITY_TYPE, ENTITY_ID, TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_biz_type(
	CODE VARCHAR(32) NOT NULL,
	NAME VARCHAR(256) NOT NULL,
	LANG VARCHAR(2),
	PRIMARY KEY(CODE)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_org_mcd(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	TITLE VARCHAR(256) NOT NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_DP_ORG_MCD_1(TITLE(255), ENTITY_TYPE_ID, ENTITY_ID, ID),
	INDEX IX_B_CRM_DP_ORG_MCD_2(ENTITY_TYPE_ID, ENTITY_ID, ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_comm_mcd(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	TYPE VARCHAR(32) NOT NULL,
	VALUE VARCHAR(256) NOT NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_DP_COMM_MCD_1(TYPE, VALUE(255), ENTITY_TYPE_ID, ENTITY_ID, ID),
	INDEX IX_B_CRM_DP_COMM_MCD_2(ENTITY_TYPE_ID, ENTITY_ID, TYPE, ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_prsn_mcd(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	LAST_NAME VARCHAR (64) NOT NULL,
	NAME VARCHAR (64),
	SECOND_NAME VARCHAR (64),
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_DP_PRSN_MCD_1(LAST_NAME, NAME, SECOND_NAME, ENTITY_TYPE_ID, ENTITY_ID, ID),
	INDEX IX_B_CRM_DP_PRSN_MCD_2(ENTITY_TYPE_ID, ENTITY_ID, ID),
	INDEX IX_B_CRM_DP_PRSN_MCD_3(LAST_NAME, ENTITY_TYPE_ID, ENTITY_ID, NAME, SECOND_NAME, ID),
	INDEX IX_B_CRM_DP_PRSN_MCD_4(ENTITY_TYPE_ID, ENTITY_ID, LAST_NAME, NAME, SECOND_NAME, ID),
	INDEX IX_B_CRM_DP_PRSN_MCD_5(ENTITY_TYPE_ID, LAST_NAME, NAME, SECOND_NAME)
);

CREATE TABLE IF NOT EXISTS b_crm_quote
(
	ID INT(18) UNSIGNED NOT NULL AUTO_INCREMENT,
	DATE_CREATE DATETIME NOT NULL,
	DATE_MODIFY DATETIME NOT NULL,
	CREATED_BY_ID INT(18) UNSIGNED NOT NULL,
	MODIFY_BY_ID INT(18) UNSIGNED DEFAULT NULL,
	ASSIGNED_BY_ID INT(18) UNSIGNED DEFAULT NULL,
	OPENED CHAR(1) DEFAULT 'N',
	LEAD_ID INT(18) UNSIGNED DEFAULT NULL,
	DEAL_ID INT(18) UNSIGNED DEFAULT NULL,
	COMPANY_ID INT(18) UNSIGNED DEFAULT NULL,
	CONTACT_ID INT(18) UNSIGNED DEFAULT NULL,
	PERSON_TYPE_ID INT NOT NULL,
	MYCOMPANY_ID INT(18) UNSIGNED DEFAULT NULL,
	TITLE VARCHAR(255) DEFAULT NULL,
	STATUS_ID VARCHAR(50) DEFAULT NULL,
	CLOSED CHAR(1) DEFAULT 'N',
	OPPORTUNITY DECIMAL(18,2) DEFAULT NULL,
	TAX_VALUE DECIMAL(18,2) DEFAULT NULL,
	CURRENCY_ID VARCHAR(50) DEFAULT NULL,
	OPPORTUNITY_ACCOUNT DECIMAL(18,2) DEFAULT NULL,
	TAX_VALUE_ACCOUNT DECIMAL(18,2) DEFAULT NULL,
	ACCOUNT_CURRENCY_ID VARCHAR(50) DEFAULT NULL,
	COMMENTS TEXT DEFAULT NULL,
	COMMENTS_TYPE TINYINT(1) UNSIGNED NULL,
	BEGINDATE DATETIME,
	CLOSEDATE DATETIME,
	EXCH_RATE DECIMAL(18,2) DEFAULT 1,
	QUOTE_NUMBER VARCHAR(100) null,
	CONTENT TEXT DEFAULT NULL,
	CONTENT_TYPE TINYINT(1) UNSIGNED NULL,
	TERMS TEXT DEFAULT NULL,
	TERMS_TYPE TINYINT(1) UNSIGNED NULL,
	STORAGE_TYPE_ID TINYINT(1) UNSIGNED NULL,
	STORAGE_ELEMENT_IDS TEXT NULL,
	LOCATION_ID varchar(100) NULL,
	CLIENT_TITLE VARCHAR(255) NULL,
	CLIENT_ADDR VARCHAR(255) NULL,
	CLIENT_CONTACT VARCHAR(255) NULL,
	CLIENT_EMAIL VARCHAR(255) NULL,
	CLIENT_PHONE VARCHAR(255) NULL,
	CLIENT_TP_ID VARCHAR(255) NULL,
	CLIENT_TPA_ID VARCHAR(255) NULL,
	PRIMARY KEY (ID),
	INDEX IX_QUOTE_1 (STATUS_ID, ASSIGNED_BY_ID),
	INDEX IX_QUOTE_2 (COMPANY_ID),
	INDEX IX_QUOTE_3 (CONTACT_ID),
	INDEX IX_QUOTE_4 (DEAL_ID),
	INDEX IX_QUOTE_5 (LEAD_ID),
	UNIQUE IXS_QUOTE_NUMBER(QUOTE_NUMBER)
);

CREATE TABLE IF NOT EXISTS b_crm_quote_elem(
	QUOTE_ID INT(1) UNSIGNED NOT NULL,
	STORAGE_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ELEMENT_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(QUOTE_ID, STORAGE_TYPE_ID, ELEMENT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_index(
	USER_ID INT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	MATCH_HASH VARCHAR(32) NOT NULL,
	MATCHES TEXT NULL,
	QUANTITY INT(1) UNSIGNED NOT NULL,
	ROOT_ENTITY_ID INT(1) UNSIGNED NOT NULL,
	ROOT_ENTITY_NAME_FLAG CHAR(1) NOT NULL,
	ROOT_ENTITY_NAME VARCHAR(256) NOT NULL,
	ROOT_ENTITY_TITLE_FLAG CHAR(1) NOT NULL,
	ROOT_ENTITY_TITLE VARCHAR(256) NOT NULL,
	ROOT_ENTITY_PHONE_FLAG CHAR(1) NOT NULL,
	ROOT_ENTITY_PHONE VARCHAR(256) NOT NULL,
	ROOT_ENTITY_EMAIL_FLAG CHAR(1) NOT NULL,
	ROOT_ENTITY_EMAIL VARCHAR(256) NOT NULL,
	IS_JUNK CHAR(1) NULL,
	PRIMARY KEY(USER_ID, ENTITY_TYPE_ID, TYPE_ID, MATCH_HASH),
	INDEX IX_B_CRM_DP_IND_1(USER_ID, ENTITY_TYPE_ID, ROOT_ENTITY_NAME_FLAG, ROOT_ENTITY_NAME(255), TYPE_ID),
	INDEX IX_B_CRM_DP_IND_2(USER_ID, ENTITY_TYPE_ID, ROOT_ENTITY_TITLE_FLAG, ROOT_ENTITY_TITLE(255), TYPE_ID),
	INDEX IX_B_CRM_DP_IND_3(USER_ID, ENTITY_TYPE_ID, ROOT_ENTITY_PHONE_FLAG, ROOT_ENTITY_PHONE(255), TYPE_ID),
	INDEX IX_B_CRM_DP_IND_4(USER_ID, ENTITY_TYPE_ID, ROOT_ENTITY_EMAIL_FLAG, ROOT_ENTITY_EMAIL(255), TYPE_ID),
	INDEX IX_B_CRM_DP_IND_5(MATCH_HASH, TYPE_ID, ENTITY_TYPE_ID),
	INDEX IX_B_CRM_DP_IND_6(ROOT_ENTITY_ID, ENTITY_TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_index_mismatch(
	USER_ID INT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	MATCH_HASH VARCHAR(32) NOT NULL,
	L_ENTITY_ID INT(1) UNSIGNED NOT NULL,
	R_ENTITY_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(USER_ID, ENTITY_TYPE_ID, TYPE_ID, MATCH_HASH, L_ENTITY_ID, R_ENTITY_ID),
	INDEX IX_B_CRM_DP_IND_MISM_1(L_ENTITY_ID, ENTITY_TYPE_ID),
	INDEX IX_B_CRM_DP_IND_MISM_2(R_ENTITY_ID, ENTITY_TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_entity_stat(
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	RANKING_DATA VARCHAR(512) NULL,
	PRIMARY KEY(ENTITY_ID, ENTITY_TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_dp_entity_hash(
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	MATCH_HASH VARCHAR(32) NOT NULL,
	IS_PRIMARY CHAR(1) NOT NULL,
	PRIMARY KEY(ENTITY_ID, ENTITY_TYPE_ID, TYPE_ID, MATCH_HASH),
	INDEX IX_B_CRM_DP_ENT_HASH_1(ENTITY_TYPE_ID, TYPE_ID, MATCH_HASH, ENTITY_ID, IS_PRIMARY)
);

CREATE TABLE IF NOT EXISTS b_crm_addr(
	TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	ANCHOR_TYPE_ID TINYINT(1) UNSIGNED,
	ANCHOR_ID INT(1) UNSIGNED,
	ADDRESS_1 VARCHAR(256),
	ADDRESS_2 VARCHAR(128),
	CITY VARCHAR(128),
	POSTAL_CODE VARCHAR(16),
	REGION VARCHAR(128),
	PROVINCE VARCHAR(128),
	COUNTRY VARCHAR(128),
	COUNTRY_CODE VARCHAR(100),
	PRIMARY KEY(TYPE_ID, ENTITY_TYPE_ID, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_1(ENTITY_TYPE_ID, TYPE_ID, ADDRESS_1(255), ENTITY_ID),
	INDEX IX_B_CRM_ADDR_2(ENTITY_TYPE_ID, TYPE_ID, ADDRESS_2, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_3(ENTITY_TYPE_ID, TYPE_ID, CITY, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_4(ENTITY_TYPE_ID, TYPE_ID, POSTAL_CODE, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_5(ENTITY_TYPE_ID, TYPE_ID, REGION, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_6(ENTITY_TYPE_ID, TYPE_ID, PROVINCE, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_7(ENTITY_TYPE_ID, TYPE_ID, COUNTRY, ENTITY_ID),
	INDEX IX_B_CRM_ADDR_8(ANCHOR_TYPE_ID, TYPE_ID, ADDRESS_1(255), ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_9(ANCHOR_TYPE_ID, TYPE_ID, ADDRESS_2, ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_10(ANCHOR_TYPE_ID, TYPE_ID, CITY, ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_11(ANCHOR_TYPE_ID, TYPE_ID, POSTAL_CODE, ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_12(ANCHOR_TYPE_ID, TYPE_ID, REGION, ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_13(ANCHOR_TYPE_ID, TYPE_ID, PROVINCE, ANCHOR_ID),
	INDEX IX_B_CRM_ADDR_14(ANCHOR_TYPE_ID, TYPE_ID, COUNTRY, ANCHOR_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_stage_history(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	TYPE_ID INT(1) UNSIGNED NOT NULL,
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_TIME DATETIME NOT NULL,
	CREATED_DATE DATE NULL,
	START_DATE DATE NOT NULL,
	END_DATE DATE NOT NULL,
	PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	START_PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	START_PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	START_PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	END_PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	END_PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	END_PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CATEGORY_ID INT (18) UNSIGNED NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_DEAL_SHX_1(OWNER_ID, TYPE_ID, ID),
	INDEX IX_B_CRM_DEAL_SHX_2(TYPE_ID, STAGE_ID, START_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_3(TYPE_ID, STAGE_ID, END_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_4(TYPE_ID, STAGE_ID, CREATED_TIME, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_5(PERIOD_YEAR, PERIOD_MONTH, RESPONSIBLE_ID, OWNER_ID, START_DATE, END_DATE),
	INDEX IX_B_CRM_DEAL_SHX_6(PERIOD_YEAR, PERIOD_MONTH, STAGE_SEMANTIC_ID, RESPONSIBLE_ID, OWNER_ID, START_DATE, END_DATE),
	INDEX IX_B_CRM_DEAL_SHX_7(PERIOD_YEAR, PERIOD_MONTH, OWNER_ID, START_DATE, END_DATE),
	INDEX IX_B_CRM_DEAL_SHX_8(TYPE_ID, OWNER_ID, END_DATE, START_DATE),
	INDEX IX_B_CRM_DEAL_SHX_9(TYPE_ID, STAGE_ID, CREATED_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_10(CATEGORY_ID, TYPE_ID, STAGE_ID, START_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_11(CATEGORY_ID, TYPE_ID, STAGE_ID, END_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_12(CATEGORY_ID, TYPE_ID, STAGE_ID, CREATED_DATE, RESPONSIBLE_ID, OWNER_ID),
	INDEX IX_B_CRM_DEAL_SHX_13(IS_LOST, CREATED_DATE, OWNER_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_SHX_14(CATEGORY_ID, IS_LOST, CREATED_DATE, OWNER_ID, RESPONSIBLE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_sum_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	START_DATE DATE NOT NULL,
	END_DATE DATE NOT NULL,
	PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	PERIOD_DAY INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CATEGORY_ID INT(1) UNSIGNED NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,2) NULL,
	UF_SUM_1 DECIMAL(18,2) NULL,
	UF_SUM_2 DECIMAL(18,2) NULL,
	UF_SUM_3 DECIMAL(18,2) NULL,
	UF_SUM_4 DECIMAL(18,2) NULL,
	UF_SUM_5 DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_DEAL_SSTAT_1(STAGE_SEMANTIC_ID, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_SSTAT_2(IS_LOST, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_SSTAT_3(END_DATE, START_DATE, OWNER_ID, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_SSTAT_4(CATEGORY_ID, END_DATE, START_DATE, OWNER_ID, CREATED_DATE, RESPONSIBLE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_inv_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	START_DATE DATE NOT NULL,
	END_DATE DATE NOT NULL,
	PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	PERIOD_DAY INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CATEGORY_ID INT(1) UNSIGNED NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	INVOICE_SUM DECIMAL(18,2) NULL,
	INVOICE_QTY INT(1) UNSIGNED NULL,
	TOTAL_INVOICE_SUM DECIMAL(18,2) NULL,
	TOTAL_INVOICE_QTY INT(1) UNSIGNED NULL,
	TOTAL_SUM DECIMAL(18,2) NULL,
	TOTAL_OWED DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_DEAL_ISTAT_1(STAGE_SEMANTIC_ID, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_ISTAT_2(IS_LOST, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_ISTAT_3(CATEGORY_ID, IS_LOST, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_ISTAT_4(CATEGORY_ID, STAGE_SEMANTIC_ID, OWNER_ID, END_DATE, START_DATE, CREATED_DATE, RESPONSIBLE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_act_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	DEADLINE_YEAR INT(1) UNSIGNED NOT NULL,
	DEADLINE_QUARTER INT(1) UNSIGNED NOT NULL,
	DEADLINE_MONTH INT(1) UNSIGNED NOT NULL,
	DEADLINE_DAY INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CATEGORY_ID INT(1) UNSIGNED NULL,
	STAGE_SEMANTIC_ID VARCHAR(3) NULL,
	STAGE_ID VARCHAR(50) NULL,
	IS_LOST CHAR(1) NULL,
	CALL_QTY INT(1) UNSIGNED NULL,
	MEETING_QTY INT(1) UNSIGNED NULL,
	EMAIL_QTY INT(1) UNSIGNED NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE),
	INDEX IX_B_CRM_DEAL_ASTAT_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY),
	INDEX IX_B_CRM_DEAL_ASTAT_2(DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY),
	INDEX IX_B_CRM_DEAL_ASTAT_3(IS_LOST, DEADLINE_DATE, OWNER_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_ASTAT_4(CATEGORY_ID, IS_LOST, DEADLINE_DATE, OWNER_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_DEAL_ASTAT_5(CATEGORY_ID, RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY)
);

CREATE TABLE IF NOT EXISTS b_crm_conv_map(
	SRC_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	DST_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	LAST_UPDATED DATETIME NOT NULL,
	DATA LONGTEXT NULL,
	PRIMARY KEY(DST_TYPE_ID, SRC_TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_lead_status_history(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	TYPE_ID INT(1) UNSIGNED NOT NULL,
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_TIME DATETIME NOT NULL,
	CREATED_DATE DATE NULL,
	PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	STATUS_ID VARCHAR(50) NULL,
	IS_IN_WORK CHAR(1) NULL,
	IS_JUNK CHAR(1) NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_LEAD_SHX_1(OWNER_ID, TYPE_ID, ID),
	INDEX IX_B_CRM_LEAD_SHX_2(TYPE_ID, CREATED_DATE, STATUS_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_LEAD_SHX_3(TYPE_ID, IS_JUNK, CREATED_DATE),
	INDEX IX_B_CRM_LEAD_SHX_4(TYPE_ID, IS_JUNK, RESPONSIBLE_ID, CREATED_DATE),
	INDEX IX_B_CRM_LEAD_SHX_5(IS_IN_WORK, OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_LEAD_SHX_6(IS_IN_WORK, OWNER_ID, RESPONSIBLE_ID, CREATED_DATE)
);

CREATE TABLE IF NOT EXISTS b_crm_lead_sum_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	PERIOD_YEAR INT(1) UNSIGNED NOT NULL,
	PERIOD_QUARTER INT(1) UNSIGNED NOT NULL,
	PERIOD_MONTH INT(1) UNSIGNED NOT NULL,
	PERIOD_DAY INT(1) UNSIGNED NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	STATUS_ID VARCHAR(50) NULL,
	SOURCE_ID VARCHAR(50) NULL,
	IS_JUNK CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,2) NULL,
	UF_SUM_1 DECIMAL(18,2) NULL,
	UF_SUM_2 DECIMAL(18,2) NULL,
	UF_SUM_3 DECIMAL(18,2) NULL,
	UF_SUM_4 DECIMAL(18,2) NULL,
	UF_SUM_5 DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_LEAD_SSTAT_1(CREATED_DATE, OWNER_ID, SOURCE_ID),
	INDEX IX_B_CRM_LEAD_SSTAT_2(SOURCE_ID, OWNER_ID, CREATED_DATE)
);

CREATE TABLE IF NOT EXISTS b_crm_lead_act_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	DEADLINE_YEAR INT(1) UNSIGNED NOT NULL,
	DEADLINE_QUARTER INT(1) UNSIGNED NOT NULL,
	DEADLINE_MONTH INT(1) UNSIGNED NOT NULL,
	DEADLINE_DAY INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	STATUS_ID VARCHAR(50) NULL,
	IS_JUNK CHAR(1) NULL,
	CALL_QTY INT(1) UNSIGNED NULL,
	MEETING_QTY INT(1) UNSIGNED NULL,
	EMAIL_QTY INT(1) UNSIGNED NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE),
	INDEX IX_B_CRM_LEAD_ASTAT_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY),
	INDEX IX_B_CRM_LEAD_ASTAT_2(DEADLINE_DATE, OWNER_ID, CALL_QTY, MEETING_QTY, EMAIL_QTY),
	INDEX IX_B_CRM_LEAD_ASTAT_3(IS_JUNK, OWNER_ID, DEADLINE_DATE),
	INDEX IX_B_CRM_LEAD_ASTAT_4(IS_JUNK, OWNER_ID, RESPONSIBLE_ID, DEADLINE_DATE),
	INDEX IX_B_CRM_LEAD_ASTAT_5(RESPONSIBLE_ID, DEADLINE_DATE, CALL_QTY, MEETING_QTY, EMAIL_QTY)
);

CREATE TABLE IF NOT EXISTS b_crm_lead_conv_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	ENTRY_DATE DATE NOT NULL,
	CREATED_DATE DATE NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	DEAL_QTY INT(1) UNSIGNED NULL,
	CONTACT_QTY INT(1) UNSIGNED NULL,
	COMPANY_QTY INT(1) UNSIGNED NULL,
	TOTALS_DATE DATE NULL,
	PRIMARY KEY(OWNER_ID, ENTRY_DATE),
	INDEX IX_B_CRM_LEAD_CSTAT_1(RESPONSIBLE_ID, ENTRY_DATE, OWNER_ID, DEAL_QTY, CONTACT_QTY, COMPANY_QTY),
	INDEX IX_B_CRM_LEAD_CSTAT_2(CREATED_DATE, OWNER_ID, DEAL_QTY, CONTACT_QTY, COMPANY_QTY)
);

CREATE TABLE IF NOT EXISTS b_crm_inv_status_history(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	TYPE_ID INT(1) UNSIGNED NOT NULL,
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_TIME DATETIME NOT NULL,
	CREATED_DATE DATE NULL,
	BILL_DATE DATE NOT NULL,
	PAY_BEFORE_DATE DATE NOT NULL,
	ACTIVITY_DATE DATE NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	STATUS_ID VARCHAR(50) NULL,
	IS_NEW CHAR(1) NULL,
	IS_JUNK CHAR(1) NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_INV_SHX_1(ACTIVITY_DATE, OWNER_ID, ID),
	INDEX IX_B_CRM_INV_SHX_2(TYPE_ID, CREATED_DATE, STATUS_ID, OWNER_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_INV_SHX_3(STATUS_SEMANTIC_ID, ACTIVITY_DATE, OWNER_ID, ID)
);

CREATE TABLE IF NOT EXISTS b_crm_invoice_sum_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	BILL_DATE DATE NOT NULL,
	PAY_BEFORE_DATE DATE NOT NULL,
	PAID_DATE DATE NULL,
	IS_PAID_INTIME CHAR(1) NULL,
	CLOSED_DATE DATE NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	COMPANY_ID INT(1) UNSIGNED NULL,
	CONTACT_ID INT(1) UNSIGNED NULL,
	STATUS_SEMANTIC_ID VARCHAR(3) NULL,
	STATUS_ID VARCHAR(50) NULL,
	IS_JUNK CHAR(1) NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,2) NULL,
	UF_SUM_1 DECIMAL(18,2) NULL,
	UF_SUM_2 DECIMAL(18,2) NULL,
	UF_SUM_3 DECIMAL(18,2) NULL,
	UF_SUM_4 DECIMAL(18,2) NULL,
	UF_SUM_5 DECIMAL(18,2) NULL,
	UF_ATTR_1 INT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_INV_SSTAT_1(CREATED_DATE, OWNER_ID, RESPONSIBLE_ID),
	INDEX IX_B_CRM_INV_SSTAT_2(STATUS_SEMANTIC_ID, CREATED_DATE, OWNER_ID),
	INDEX IX_B_CRM_INV_SSTAT_3(STATUS_SEMANTIC_ID, PAY_BEFORE_DATE, OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_INV_SSTAT_4(COMPANY_ID, STATUS_SEMANTIC_ID, IS_PAID_INTIME, OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_INV_SSTAT_5(CONTACT_ID, STATUS_SEMANTIC_ID, IS_PAID_INTIME, OWNER_ID, CREATED_DATE)
);

CREATE TABLE IF NOT EXISTS b_crm_requisite(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	PRESET_ID INT(1) UNSIGNED NOT NULL DEFAULT 0,
	DATE_CREATE DATETIME NOT NULL,
	DATE_MODIFY DATETIME NULL,
	CREATED_BY_ID INT UNSIGNED NOT NULL,
	MODIFY_BY_ID INT UNSIGNED DEFAULT NULL,
	NAME VARCHAR(255) NOT NULL,
	CODE VARCHAR(45) NULL,
	XML_ID VARCHAR(45) NULL,
	ORIGINATOR_ID VARCHAR(255) NULL,
	ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
	SORT INT NOT NULL DEFAULT 500,
	RQ_NAME VARCHAR(150) NULL,
	RQ_FIRST_NAME VARCHAR(50) NULL,
	RQ_LAST_NAME VARCHAR(50) NULL,
	RQ_SECOND_NAME VARCHAR(50) NULL,
	RQ_COMPANY_NAME VARCHAR(255) NULL,
	RQ_COMPANY_FULL_NAME VARCHAR(300) NULL,
	RQ_COMPANY_REG_DATE VARCHAR(30) NULL,
	RQ_DIRECTOR VARCHAR(150) NULL,
	RQ_ACCOUNTANT VARCHAR(150) NULL,
	RQ_CEO_NAME VARCHAR(150) NULL,
	RQ_CEO_WORK_POS VARCHAR(150) NULL,
	RQ_CONTACT VARCHAR(150) NULL,
	RQ_EMAIL VARCHAR(255) NULL,
	RQ_PHONE VARCHAR(30) NULL,
	RQ_FAX VARCHAR(30) NULL,
	RQ_IDENT_DOC VARCHAR(255) NULL,
	RQ_IDENT_DOC_SER VARCHAR(25) NULL,
	RQ_IDENT_DOC_NUM VARCHAR(25) NULL,
	RQ_IDENT_DOC_DATE VARCHAR(30) NULL,
	RQ_IDENT_DOC_ISSUED_BY VARCHAR(255) NULL,
	RQ_IDENT_DOC_DEP_CODE VARCHAR(25) NULL,
	RQ_INN VARCHAR(15) NULL,
	RQ_KPP VARCHAR(9) NULL,
	RQ_USRLE VARCHAR(20) NULL,
	RQ_IFNS VARCHAR(255) NULL,
	RQ_OGRN VARCHAR(13) NULL,
	RQ_OGRNIP VARCHAR(15) NULL,
	RQ_OKPO VARCHAR(10) NULL,
	RQ_OKTMO VARCHAR(11) NULL,
	RQ_OKVED VARCHAR(255) NULL,
	RQ_EDRPOU VARCHAR(8) NULL,
	RQ_DRFO VARCHAR(10) NULL,
	RQ_KBE VARCHAR(2) NULL,
	RQ_IIN VARCHAR(12) NULL,
	RQ_BIN VARCHAR(12) NULL,
	RQ_VAT_PAYER CHAR(1) NOT NULL DEFAULT 'N',
	RQ_VAT_ID VARCHAR(20) NULL,
	RQ_VAT_CERT_SER VARCHAR(10) NULL,
	RQ_VAT_CERT_NUM VARCHAR(15) NULL,
	RQ_VAT_CERT_DATE VARCHAR(30) NULL,
	RQ_RESIDENCE_COUNTRY VARCHAR(128) NULL,
	PRIMARY KEY (ID),
	INDEX IX_B_CRM_REQUISITE_1 (ENTITY_ID, ENTITY_TYPE_ID, SORT, ID),
	INDEX IX_B_CRM_REQUISITE_2 (RQ_NAME),
	INDEX IX_B_CRM_REQUISITE_3 (RQ_LAST_NAME,RQ_FIRST_NAME,RQ_SECOND_NAME),
	INDEX IX_B_CRM_REQUISITE_4 (RQ_COMPANY_NAME),
	INDEX IX_B_CRM_REQUISITE_5 (RQ_DIRECTOR),
	INDEX IX_B_CRM_REQUISITE_6 (RQ_ACCOUNTANT),
	INDEX IX_B_CRM_REQUISITE_7 (RQ_CEO_NAME),
	INDEX IX_B_CRM_REQUISITE_8 (RQ_CONTACT),
	INDEX IX_B_CRM_REQUISITE_9 (RQ_EMAIL),
	INDEX IX_B_CRM_REQUISITE_10 (RQ_PHONE),
	INDEX IX_B_CRM_REQUISITE_11 (RQ_FAX),
	INDEX IX_B_CRM_REQUISITE_12 (RQ_IDENT_DOC_NUM,RQ_IDENT_DOC_SER),
	INDEX IX_B_CRM_REQUISITE_13 (RQ_IDENT_DOC_DEP_CODE),
	INDEX IX_B_CRM_REQUISITE_14 (RQ_INN),
	INDEX IX_B_CRM_REQUISITE_15 (RQ_KPP),
	INDEX IX_B_CRM_REQUISITE_16 (RQ_USRLE),
	INDEX IX_B_CRM_REQUISITE_17 (RQ_IFNS),
	INDEX IX_B_CRM_REQUISITE_18 (RQ_OGRN),
	INDEX IX_B_CRM_REQUISITE_19 (RQ_OGRNIP),
	INDEX IX_B_CRM_REQUISITE_20 (RQ_OKPO),
	INDEX IX_B_CRM_REQUISITE_21 (RQ_OKTMO),
	INDEX IX_B_CRM_REQUISITE_22 (RQ_OKVED),
	INDEX IX_B_CRM_REQUISITE_23 (RQ_EDRPOU),
	INDEX IX_B_CRM_REQUISITE_24 (RQ_DRFO),
	INDEX IX_B_CRM_REQUISITE_25 (RQ_KBE),
	INDEX IX_B_CRM_REQUISITE_26 (RQ_IIN),
	INDEX IX_B_CRM_REQUISITE_27 (RQ_BIN),
	INDEX IX_B_CRM_REQUISITE_28 (RQ_VAT_PAYER),
	INDEX IX_B_CRM_REQUISITE_29 (RQ_VAT_ID),
	INDEX IX_B_CRM_REQUISITE_30 (RQ_VAT_CERT_NUM,RQ_VAT_CERT_SER),
	INDEX IX_B_CRM_REQUISITE_31 (RQ_RESIDENCE_COUNTRY)
);
CREATE TABLE IF NOT EXISTS b_crm_preset(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	COUNTRY_ID INT NOT NULL DEFAULT 0,
	DATE_CREATE DATETIME NOT NULL,
	DATE_MODIFY DATETIME NULL,
	CREATED_BY_ID INT UNSIGNED NOT NULL,
	MODIFY_BY_ID INT UNSIGNED DEFAULT NULL,
	NAME VARCHAR(255) NOT NULL,
	XML_ID VARCHAR(45) NULL,
	ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
	SORT INT NOT NULL DEFAULT 500,
	SETTINGS TEXT DEFAULT NULL,
	PRIMARY KEY (ID),
	INDEX IX_B_CRM_PRESET_1 (ID, ENTITY_TYPE_ID),
	INDEX IX_B_CRM_PRESET_2 (COUNTRY_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_requisite_cfg(
	ENTITY_ID INT(1) NOT NULL,
	ENTITY_TYPE_ID INT(1) NOT NULL,
	SETTINGS TEXT,
	PRIMARY KEY(ENTITY_ID, ENTITY_TYPE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_requisite_link(
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID      INT(1) UNSIGNED     NOT NULL,
	REQUISITE_ID      INT(1)     UNSIGNED NOT NULL DEFAULT 0,
	BANK_DETAIL_ID INT(1) UNSIGNED     NOT NULL DEFAULT 0,
	MC_REQUISITE_ID   INT(1)     UNSIGNED NOT NULL DEFAULT 0,
	MC_BANK_DETAIL_ID INT(1)     UNSIGNED NOT NULL DEFAULT 0,
	PRIMARY KEY (ENTITY_TYPE_ID, ENTITY_ID),
	INDEX IX_CRM_REQUISITE_LINK_1 (REQUISITE_ID),
	INDEX IX_CRM_REQUISITE_LINK_2 (MC_REQUISITE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_bank_detail(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	ENTITY_TYPE_ID TINYINT(1) UNSIGNED NOT NULL,
	ENTITY_ID INT(1) UNSIGNED NOT NULL,
	COUNTRY_ID INT NOT NULL DEFAULT 0,
	DATE_CREATE DATETIME NOT NULL,
	DATE_MODIFY DATETIME NULL,
	CREATED_BY_ID INT UNSIGNED NOT NULL,
	MODIFY_BY_ID INT UNSIGNED DEFAULT NULL,
	NAME VARCHAR(255) NOT NULL,
	CODE VARCHAR(45) NULL,
	XML_ID VARCHAR(45) NULL,
	ACTIVE CHAR(1) NOT NULL DEFAULT 'Y',
	SORT INT NOT NULL DEFAULT 500,
	RQ_BANK_NAME VARCHAR(255) NULL,
	RQ_BANK_ADDR VARCHAR(255) NULL,
	RQ_BANK_ROUTE_NUM VARCHAR(9) NULL,
	RQ_BIK VARCHAR(9) NULL,
	RQ_MFO VARCHAR(6) NULL,
	RQ_ACC_NAME VARCHAR(150) NULL,
	RQ_ACC_NUM VARCHAR(34) NULL,
	RQ_IIK VARCHAR(20) NULL,
	RQ_ACC_CURRENCY VARCHAR(100) NULL,
	RQ_COR_ACC_NUM VARCHAR(34) NULL,
	RQ_IBAN VARCHAR(34) NULL,
	RQ_SWIFT VARCHAR(11) NULL,
	RQ_BIC VARCHAR(11) NULL,
	COMMENTS VARCHAR(500) NULL,
	PRIMARY KEY (ID),
	INDEX IX_B_CRM_BANK_DETAIL_1 (ENTITY_ID, ENTITY_TYPE_ID, SORT, ID),
	INDEX IX_B_CRM_BANK_DETAIL_2 (RQ_BANK_NAME),
	INDEX IX_B_CRM_BANK_DETAIL_3 (RQ_BANK_ADDR),
	INDEX IX_B_CRM_BANK_DETAIL_4 (RQ_BANK_ROUTE_NUM),
	INDEX IX_B_CRM_BANK_DETAIL_5 (RQ_BIK),
	INDEX IX_B_CRM_BANK_DETAIL_6 (RQ_MFO),
	INDEX IX_B_CRM_BANK_DETAIL_7 (RQ_ACC_NAME),
	INDEX IX_B_CRM_BANK_DETAIL_8 (RQ_ACC_NUM),
	INDEX IX_B_CRM_BANK_DETAIL_9 (RQ_IIK),
	INDEX IX_B_CRM_BANK_DETAIL_10 (RQ_ACC_CURRENCY),
	INDEX IX_B_CRM_BANK_DETAIL_11 (RQ_COR_ACC_NUM),
	INDEX IX_B_CRM_BANK_DETAIL_12 (RQ_IBAN),
	INDEX IX_B_CRM_BANK_DETAIL_13 (RQ_SWIFT),
	INDEX IX_B_CRM_BANK_DETAIL_14 (RQ_BIC)
);

CREATE TABLE IF NOT EXISTS b_crm_contact_company(
	CONTACT_ID INT(1) UNSIGNED NOT NULL,
	COMPANY_ID INT(1) UNSIGNED NOT NULL,
	SORT INT NOT NULL,
	ROLE_ID TINYINT(1) NOT NULL,
	IS_PRIMARY CHAR(1) NOT NULL,
	PRIMARY KEY (CONTACT_ID, COMPANY_ID),
	INDEX IX_B_CRM_CONTACT_COMPANY_1(COMPANY_ID, CONTACT_ID),
	INDEX IX_B_CRM_CONTACT_COMPANY_2(CONTACT_ID, IS_PRIMARY, COMPANY_ID),
	INDEX IX_B_CRM_CONTACT_COMPANY_3(CONTACT_ID, SORT, COMPANY_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_contact(
	DEAL_ID INT(1) UNSIGNED NOT NULL,
	CONTACT_ID INT(1) UNSIGNED NOT NULL,
	SORT INT NOT NULL,
	ROLE_ID TINYINT(1) NOT NULL,
	IS_PRIMARY CHAR(1) NOT NULL,
	PRIMARY KEY (DEAL_ID, CONTACT_ID),
	INDEX IX_B_CRM_DEAL_CONTACT_1(CONTACT_ID, DEAL_ID),
	INDEX IX_B_CRM_DEAL_CONTACT_2(DEAL_ID, IS_PRIMARY, CONTACT_ID),
	INDEX IX_B_CRM_DEAL_CONTACT_3(DEAL_ID, SORT, CONTACT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_quote_contact(
	QUOTE_ID INT(1) UNSIGNED NOT NULL,
	CONTACT_ID INT(1) UNSIGNED NOT NULL,
	SORT INT NOT NULL,
	ROLE_ID TINYINT(1) NOT NULL,
	IS_PRIMARY CHAR(1) NOT NULL,
	PRIMARY KEY (QUOTE_ID, CONTACT_ID),
	INDEX IX_B_CRM_QUOTE_CONTACT_1(CONTACT_ID, QUOTE_ID),
	INDEX IX_B_CRM_QUOTE_CONTACT_2(QUOTE_ID, IS_PRIMARY, CONTACT_ID),
	INDEX IX_B_CRM_QUOTE_CONTACT_3(QUOTE_ID, SORT, CONTACT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_entity_cfg(
	ENTITY_ID INT(1) NOT NULL,
	ENTITY_TYPE_ID INT(1) NOT NULL,
	USER_ID INT(1) NOT NULL,
	SETTINGS TEXT NULL,
	PRIMARY KEY(ENTITY_ID, ENTITY_TYPE_ID, USER_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform (
  ID int(18) NOT NULL AUTO_INCREMENT,
  NAME varchar(255) NOT NULL,
  DESCRIPTION text DEFAULT NULL,
  BUTTON_CAPTION varchar(255) DEFAULT NULL,
  TEMPLATE_ID varchar(255) DEFAULT NULL,
  IS_PAY char(1) NOT NULL,
  GOOGLE_ANALYTICS_ID varchar(50) DEFAULT NULL,
  YANDEX_METRIC_ID varchar(50) DEFAULT NULL,
  SECURITY_CODE varchar(32) DEFAULT NULL,
  BUTTON_COLOR_FONT varchar(10) DEFAULT NULL,
  BUTTON_COLOR_BG varchar(10) DEFAULT NULL,
  CSS_PATH varchar(255) DEFAULT NULL,
  DATE_CREATE datetime NOT NULL,
  ENTITY_SCHEME int(18) NOT NULL,
  GOOGLE_ANALYTICS_PAGE_VIEW char(1) NOT NULL DEFAULT 'N',
  CAPTION varchar(255) DEFAULT NULL,
  RESULT_SUCCESS_TEXT varchar(255) DEFAULT NULL,
  RESULT_SUCCESS_URL varchar(500) DEFAULT NULL,
  RESULT_FAILURE_TEXT varchar(255) DEFAULT NULL,
  RESULT_FAILURE_URL varchar(500) DEFAULT NULL,
  ACTIVE char(1) NOT NULL DEFAULT 'N',
  CSS_TEXT longtext,
  ACTIVE_CHANGE_BY int(18) DEFAULT NULL,
  ACTIVE_CHANGE_DATE datetime NULL,
  IS_SYSTEM char(1) NOT NULL DEFAULT 'N',
  COPYRIGHT_REMOVED char(1) NOT NULL DEFAULT 'N',
  BACKGROUND_IMAGE int(18) DEFAULT NULL,
  CODE varchar(255) DEFAULT NULL,
  OPENLINE_ID int(18) DEFAULT NULL,
  DUPLICATE_MODE varchar(20) DEFAULT NULL,
  XML_ID varchar(50) DEFAULT NULL,
  LICENCE_TEXT longtext,
  LICENCE_BUTTON_CAPTION varchar(255) DEFAULT NULL,
  SCRIPT_INCLUDE_SETTINGS text,
  INVOICE_SETTINGS text,
  FORM_SETTINGS text,
  IS_CALLBACK_FORM char(1) NOT NULL DEFAULT 'N',
  CALL_FROM varchar(50) DEFAULT NULL,
  CALL_TEXT text,
  ASSIGNED_BY_ID int(18) DEFAULT NULL,
  PRIMARY KEY (ID),
  INDEX IX_B_CRM_WEBFORM (CODE)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_counter (
  ID int(18) NOT NULL AUTO_INCREMENT,
  FORM_ID int(18) NOT NULL,
  DATE_CREATE datetime NULL,
  VIEWS int(18) DEFAULT 0,
  START_FILL int(18) DEFAULT 0,
  END_FILL int(18) DEFAULT 0,
  MONEY int(18) DEFAULT 0,
  ENTITY_CONTACT int(18) DEFAULT 0,
  ENTITY_COMPANY int(18) DEFAULT 0,
  ENTITY_DEAL int(18) DEFAULT 0,
  ENTITY_LEAD int(18) DEFAULT 0,
  ENTITY_QUOTE int(18) DEFAULT 0,
  ENTITY_INVOICE int(18) DEFAULT 0,
  PRIMARY KEY (ID),
  INDEX IX_B_CRM_WEBFORM_COUNTER (FORM_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_view (
  FORM_ID int(18) NOT NULL,
  DATE_CREATE datetime NULL
);

CREATE TABLE b_crm_webform_start_edit (
  FORM_ID int(18) NOT NULL,
  DATE_CREATE datetime NULL
);

CREATE TABLE IF NOT EXISTS b_crm_webform_field (
  ID int(18) NOT NULL AUTO_INCREMENT,
  TYPE varchar(50) DEFAULT NULL,
  CODE varchar(50) DEFAULT NULL,
  CAPTION varchar(255) DEFAULT NULL,
  FORM_ID int(18) NOT NULL,
  GROUP_ID int(18) NULL,
  SORT int(18) NOT NULL DEFAULT '100',
  REQUIRED char(1) DEFAULT 'N',
  PLACEHOLDER varchar(255) DEFAULT NULL,
  MULTIPLE char(1) NOT NULL DEFAULT 'N',
  ITEMS longtext NOT NULL,
  VALUE_TYPE varchar(50) DEFAULT NULL,
  PRIMARY KEY (ID),
	INDEX IX_B_CRM_WEBFORM_FIELD (FORM_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_field_dep (
  ID int(18) NOT NULL AUTO_INCREMENT,
  FORM_ID int(18) NOT NULL,
  IF_FIELD_CODE varchar(25) DEFAULT NULL,
  IF_ACTION varchar(20) NOT NULL,
  IF_VALUE varchar(255) DEFAULT NULL,
  IF_VALUE_OPERATION varchar(10) DEFAULT NULL,
  DO_FIELD_CODE varchar(25) DEFAULT NULL,
  DO_ACTION varchar(20) NOT NULL,
  DO_VALUE varchar(255) DEFAULT NULL,
  PRIMARY KEY (ID),
	INDEX IX_B_CRM_WEBFORM_FIELD_DEP (FORM_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_field_preset (
  FORM_ID int(18) NOT NULL,
  ENTITY_NAME varchar(25) NOT NULL,
  FIELD_NAME varchar(50) NOT NULL,
  VALUE varchar(255) DEFAULT NULL,
	INDEX IX_B_CRM_WEBFORM_FIELD_PRST (FORM_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_result (
  ID int(18) NOT NULL AUTO_INCREMENT,
  DATE_INSERT datetime NOT NULL,
  FORM_ID int(18) NOT NULL,
  PRIMARY KEY (ID),
	INDEX IX_B_CRM_WEBFORM_RESULT (FORM_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_webform_result_entity (
  FORM_ID int(18) NOT NULL,
  RESULT_ID int(18) NOT NULL,
  ENTITY_NAME varchar(20) NOT NULL,
  ITEM_ID int(18) NOT NULL,
	INDEX IX_B_CRM_WEBFORM_RESULT_ENT (RESULT_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_ps_rq_conv_relation(
	ENTITY_ID       INT(1)  UNSIGNED NOT NULL,
	COMPANY_ID      INT(1)  UNSIGNED NOT NULL DEFAULT 0,
	REQUISITE_ID    INT(1)  UNSIGNED NOT NULL DEFAULT 0,
	BANK_DETAIL_ID  INT(1)  UNSIGNED NOT NULL DEFAULT 0,
	PRIMARY KEY (ENTITY_ID),
	INDEX IX_CRM_PS_RQ_CONV_RELATION_1 (REQUISITE_ID)
);

CREATE TABLE IF NOT EXISTS b_crm_act_mail_meta(
	ACTIVITY_ID INT UNSIGNED NOT NULL,
	MSG_ID_HASH VARCHAR(40) NULL,
	MSG_HEADER_HASH VARCHAR(40) NULL,
	PRIMARY KEY (ACTIVITY_ID),
	INDEX IX_B_CRM_ACT_MAIL_META_1 (MSG_ID_HASH),
	INDEX IX_B_CRM_ACT_MAIL_META_2 (MSG_HEADER_HASH)
);

CREATE TABLE IF NOT EXISTS b_crm_deal_category(
	ID INT(1) UNSIGNED NOT NULL AUTO_INCREMENT,
	CREATED_DATE DATE NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	IS_LOCKED CHAR(1) NOT NULL DEFAULT 'N',
	SORT INT NOT NULL,
	PRIMARY KEY(ID),
	INDEX IX_B_CRM_DEAL_CAT_1(SORT, ID),
	INDEX IX_B_CRM_DEAL_CAT_2(IS_LOCKED, SORT, ID)
);

CREATE TABLE IF NOT EXISTS b_crm_company_act_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	TOTAL_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_COMPANY_ASTAT_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, TOTAL_QTY),
	INDEX IX_B_CRM_COMPANY_ASTAT_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, TOTAL_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_company_act_sts_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	ANSWERED_QTY INT(1) UNSIGNED NULL,
	UNANSWERED_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_COMPANY_ASTS_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, ANSWERED_QTY, UNANSWERED_QTY),
	INDEX IX_B_CRM_COMPANY_ASTS_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, ANSWERED_QTY, UNANSWERED_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_company_act_mark_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(50) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	NONE_QTY INT(1) UNSIGNED NULL,
	POSITIVE_QTY INT(1) UNSIGNED NULL,
	NEGATIVE_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID),
	INDEX IX_B_CRM_COMPANY_AMRS_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID, NONE_QTY, POSITIVE_QTY, NEGATIVE_QTY),
	INDEX IX_B_CRM_COMPANY_AMRS_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID, NONE_QTY, POSITIVE_QTY, NEGATIVE_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_company_act_stm_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	INCOMING_QTY INT(1) UNSIGNED NULL,
	OUTGOING_QTY INT(1) UNSIGNED NULL,
	REVERSING_QTY INT(1) UNSIGNED NULL,
	MISSING_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_COMPANY_ASTM_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, INCOMING_QTY, OUTGOING_QTY, REVERSING_QTY, MISSING_QTY),
	INDEX IX_B_CRM_COMPANY_ASTM_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, INCOMING_QTY, OUTGOING_QTY, REVERSING_QTY, MISSING_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_company_act_sum_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,4) NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_COMPANY_ASUM_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_COMPANY_ASUM_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID)
);
CREATE TABLE IF NOT EXISTS b_crm_company_growth_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_COMPANY_GRTH_1(CREATED_DATE, OWNER_ID),
	INDEX IX_B_CRM_COMPANY_GRTH_2(RESPONSIBLE_ID, CREATED_DATE, OWNER_ID)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_act_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	TOTAL_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_CONTACT_ASTAT_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, TOTAL_QTY),
	INDEX IX_B_CRM_CONTACT_ASTAT_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, TOTAL_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_act_sts_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	ANSWERED_QTY INT(1) UNSIGNED NULL,
	UNANSWERED_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_CONTACT_ASTS_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, ANSWERED_QTY, UNANSWERED_QTY),
	INDEX IX_B_CRM_CONTACT_ASTS_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, ANSWERED_QTY, UNANSWERED_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_act_mark_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(50) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	NONE_QTY INT(1) UNSIGNED NULL,
	POSITIVE_QTY INT(1) UNSIGNED NULL,
	NEGATIVE_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID),
	INDEX IX_B_CRM_CONTACT_AMRS_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID, NONE_QTY, POSITIVE_QTY, NEGATIVE_QTY),
	INDEX IX_B_CRM_CONTACT_AMRS_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, SOURCE_ID, NONE_QTY, POSITIVE_QTY, NEGATIVE_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_act_stm_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	INCOMING_QTY INT(1) UNSIGNED NULL,
	OUTGOING_QTY INT(1) UNSIGNED NULL,
	REVERSING_QTY INT(1) UNSIGNED NULL,
	MISSING_QTY INT(1) UNSIGNED NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_CONTACT_ASTM_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, INCOMING_QTY, OUTGOING_QTY, REVERSING_QTY, MISSING_QTY),
	INDEX IX_B_CRM_CONTACT_ASTM_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, INCOMING_QTY, OUTGOING_QTY, REVERSING_QTY, MISSING_QTY)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_act_sum_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID varchar(100) NOT NULL,
	PROVIDER_TYPE_ID varchar(100) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,4) NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_CONTACT_ASUM_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_CONTACT_ASUM_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID)
);
CREATE TABLE IF NOT EXISTS b_crm_contact_growth_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	CREATED_DATE DATE NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	PRIMARY KEY(OWNER_ID, CREATED_DATE),
	INDEX IX_B_CRM_CONTACT_GRTH_1(CREATED_DATE, OWNER_ID),
	INDEX IX_B_CRM_CONTACT_GRTH_2(RESPONSIBLE_ID, CREATED_DATE, OWNER_ID)
);
CREATE TABLE IF NOT EXISTS b_crm_act_stat(
	OWNER_ID INT(1) UNSIGNED NOT NULL,
	DEADLINE_DATE DATE NOT NULL,
	PROVIDER_ID VARCHAR(100) NOT NULL,
	PROVIDER_TYPE_ID VARCHAR(50) NOT NULL,
	RESPONSIBLE_ID INT(1) UNSIGNED NOT NULL,
	COMPLETED CHAR(1) NULL,
	STATUS_ID INT(1) UNSIGNED NULL,
	MARK_ID INT(1) UNSIGNED NULL,
	SOURCE_ID VARCHAR(100) NOT NULL,
	STREAM_ID INT(1) UNSIGNED NULL,
	CURRENCY_ID CHAR(3) NULL,
	SUM_TOTAL DECIMAL(18,4) NULL,
	PRIMARY KEY(OWNER_ID, DEADLINE_DATE, PROVIDER_ID, PROVIDER_TYPE_ID),
	INDEX IX_B_CRM_ACT_STAT_1(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, COMPLETED),
	INDEX IX_B_CRM_ACT_STAT_2(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, COMPLETED),
	INDEX IX_B_CRM_ACT_STAT_5(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, STATUS_ID),
	INDEX IX_B_CRM_ACT_STAT_6(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, STATUS_ID),
	INDEX IX_B_CRM_ACT_STAT_7(RESPONSIBLE_ID, DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, STREAM_ID),
	INDEX IX_B_CRM_ACT_STAT_8(DEADLINE_DATE, OWNER_ID, PROVIDER_ID, PROVIDER_TYPE_ID, STREAM_ID)
);
CREATE TABLE IF NOT EXISTS b_crm_button (
	ID int(18) NOT NULL AUTO_INCREMENT,
	DATE_CREATE datetime NULL,
	CREATED_BY int(18) DEFAULT NULL,
	ACTIVE char(1) NOT NULL DEFAULT 'N',
	ACTIVE_CHANGE_BY int(18) DEFAULT NULL,
	ACTIVE_CHANGE_DATE datetime NULL,
	NAME varchar(255) NOT NULL,
	BACKGROUND_COLOR varchar(10) DEFAULT NULL,
	ICON_COLOR varchar(10) DEFAULT NULL,
	LOCATION int(3) NOT NULL DEFAULT 1,
	DELAY int(3) NOT NULL DEFAULT 0,
	ITEMS text,
	SECURITY_CODE varchar(32) DEFAULT NULL,
	IS_SYSTEM char(1) NOT NULL DEFAULT 'N',
	XML_ID varchar(50) DEFAULT NULL,
	PRIMARY KEY (ID)
);