;(function (window) {
	if (!window.BX)
	{
		window.BX = {};
	}
	else if (window.BX.SiteButton)
	{
		return;
	}

	var BX = window.BX;

	BX.SiteButton = {

		init: function(config)
		{
			this.loadBxAnalytics();

			this.config = config;

			if(!this.check())
			{
				return;
			}

			this.isButtonsShown = false;
			this.loadedButtonCount = 0;
			this.load();

			if(this.config.delay)
			{
				var _this = this;
				window.setTimeout(
					function(){
						_this.show();
					},
					1000 * this.config.delay
				);
			}
			else
			{
				this.show();
			}
		},
		check: function()
		{
			if(this.config.widgets.length == 0)
			{
				return false;
			}

			if (!this.config.widgets.some(this.checkWidgetPages, this))
			{
				return false;
			}

			return true;
		},
		loadBxAnalytics: function()
		{
			if(typeof window._ba != "undefined")
			{
				return;
			}

			var targetHost = document.location.hostname;

			window._ba = window._ba || [];
			window._ba.push(["aid", "ext:" + targetHost]);
			window._ba.push(["host", targetHost]);
			(function() {
				var ba = document.createElement("script"); ba.type = "text/javascript"; ba.async = true;
				ba.src = (document.location.protocol == "https:" ? "https://" : "http://") + "bitrix.info/ba.js";
				var s = document.getElementsByTagName("script")[0];
				s.parentNode.insertBefore(ba, s);
			})();
		},
		load: function()
		{
			// set common classes
			if(this.isIOS()) this.addClass(document.documentElement, 'bx-ios');
			if(this.isMobile()) this.addClass(document.documentElement, 'bx-touch');

			// load resources
			this.config.resources.forEach(function(href){
				//var cssNode = document.createElement('LINK');
				//cssNode.type = 'text/css'; cssNode.rel = 'stylesheet'; cssNode.href = href;
				var cssNode = document.createElement('STYLE');
				cssNode.setAttribute("type", "text/css");
				if(cssNode.styleSheet)
				{
					cssNode.styleSheet.cssText = href;
				}
				else
				{
					cssNode.appendChild(document.createTextNode(href))
				}
				document.head.appendChild(cssNode);
			}, this);

			// insert layout
			this.container = this.getNodeFromText(this.config.layout);
			if (!this.container)
			{
				return;
			}
			document.body.appendChild(this.container);

			// init nodes
			var _this = this;
			this.blockNode = this.container.querySelector('[data-b24-crm-button-block]');
			this.blockInnerNode = this.container.querySelector('[data-b24-crm-button-block-inner]');

			if(this.isMobile())
			{
				this.addEventListener(this.blockNode, 'click', function (e) {
					_this.showButtons();
				});
			}
			else
			{
				this.addEventListener(this.blockNode, 'mouseover', function (e) {
					_this.showButtons();
				});
				this.addEventListener(this.blockNode, 'mouseout', function (e) {
					_this.hideButtons();
				});
			}

			// load panels
			this.config.widgets.forEach(this.loadWidget, this);

			// start animation
			this.addClass(this.blockInnerNode, 'b24-crm-button-animate-' + this.loadedButtonCount);
		},
		getNodeFromText: function(text)
		{
			var node = document.createElement('div');
			node.innerHTML = text;
			return node.children[0];
		},
		checkWidgetPages: function(widget)
		{
			var filtered = widget.pages.list.filter(function (page) {
				var pattern = this.prepareUrl(page).split('*').map(function(chunk){
					return chunk.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
				}).join('.*');
				return (new RegExp(pattern)).test(this.prepareUrl(window.location.href));
			}, this);

			var isPageFound = filtered.length > 0;
			if(widget.pages.mode == 'EXCLUDE')
			{
				return !isPageFound;
			}
			else
			{
				return isPageFound;
			}
		},
		prepareUrl: function(url)
		{
			if (url.substring(0, 5) == 'http:')
			{
				result = url.substring(7);
			}
			else if (url.substring(0, 6) == 'https:')
			{
				result = url.substring(8);
			}
			else
			{
				result = url;
			}

			return result;
		},
		loadWidget: function(widget)
		{
			widget.isLoaded = false;
			widget.buttonNode = this.container.querySelector('[data-b24-crm-button-widget="' + widget.id + '"]');
			if(!widget.buttonNode)
			{
				return;
			}

			if(!this.checkWidgetPages(widget))
			{
				widget.buttonNode.style.display = 'none';
				return;
			}

			var scriptText = '';
			var isAddInHead = false;
			var parsedScript = widget.script.match(/<script\b[^>]*>(.*?)<\/script>/i);
			if(parsedScript && parsedScript[1])
			{
				scriptText = parsedScript[1];
				isAddInHead = true;
			}
			else
			{
				widget.node = this.getNodeFromText(widget.script);
				if(!widget.node)
				{
					return;
				}
				isAddInHead = false;
			}

			if (isAddInHead)
			{
				widget.node = document.createElement("script");
				try {
					widget.node.appendChild(document.createTextNode(scriptText));
				} catch(e) {
					widget.node.text = scriptText;
				}
				document.head.appendChild(widget.node);
			}
			else
			{
				document.body.insertBefore(widget.node, document.body.firstChild);
			}

			var _this = this;
			this.addEventListener(widget.buttonNode, 'click', function () {
				if(!_this.isMobile())
				{
					_this.showWidget(widget);
				}
				else if(_this.isButtonsShown || _this.loadedButtonCount == 1)
				{
					_this.showWidget(widget);
				}
			});

			widget.isLoaded = true;
			this.loadedButtonCount++;
		},
		show: function()
		{
			this.removeClass(this.container, 'b24-crm-button-disable');
			this.addClass(this.container, 'b24-crm-button-visible');
		},
		showButtons: function()
		{
			if(this.isMobile() && this.isButtonsShown)
			{
				return;
			}

			if(this.isIOS()) this.addClass(document.documentElement, 'bx-ios-fix-frame-focus');

			this.isButtonsShown = true;
			this.addClass(this.blockNode, 'b24-crm-button-block-active');
			this.addClass(this.blockNode, 'b24-crm-button-block-active-' + this.loadedButtonCount);
			this.removeClass(this.blockInnerNode, 'b24-crm-button-animate-' + this.loadedButtonCount);
		},
		hideButtons: function()
		{
			if(this.isIOS()) this.removeClass(document.documentElement, 'bx-ios-fix-frame-focus');

			this.isButtonsShown = false;
			this.removeClass(this.blockNode, 'b24-crm-button-block-active');
			this.removeClass(this.blockNode, 'b24-crm-button-block-active-' + this.loadedButtonCount);
			this.addClass(this.blockInnerNode, 'b24-crm-button-animate-' + this.loadedButtonCount);
		},
		hide: function()
		{
			this.addClass(this.container, 'b24-crm-button-disable');
		},
		onWidgetClose: function()
		{
			this.show();
		},
		addClass: function(element, className)
		{
			if (element && element.className && element.className.indexOf(className) === -1)
			{
				element.className += " " + className;
				element.className = element.className.replace('  ', ' ');
			}
		},
		removeClass: function(element, className)
		{
			if (!element || !element.className)
			{
				return;
			}

			element.className = element.className.replace(className, '').replace('  ', ' ');
		},
		isIOS: function()
		{
			return (/(iPad;)|(iPhone;)/i.test(navigator.userAgent));
		},

		isMobile: function()
		{
			return (/(ipad|iphone|android|mobile|touch)/i.test(navigator.userAgent));
		},
		showWidget: function(widget)
		{
			if(widget.show)
			{
				eval(widget.show);
			}
			this.hide();
		},
		addEventListener: function(el, eventName, handler)
		{
			el = el || window;
			if (window.addEventListener)
			{
				el.addEventListener(eventName, handler, false);
			}
			else
			{
				el.attachEvent('on' + eventName, handler);
			}
		}
	};


})(window);