<?php
namespace Bitrix\Crm\Activity\Provider;

use Bitrix\Main\Localization\Loc;
use Bitrix\Crm\Activity;
use Bitrix\Crm\Activity\CommunicationStatistics;

Loc::loadMessages(__FILE__);

class Email extends Activity\Provider\Base
{
	public static function getId()
	{
		return 'CRM_EMAIL';
	}

	public static function getTypeId(array $activity)
	{
		return 'EMAIL';
	}

	public static function getTypes()
	{
		return array(
			array(
				'NAME' => 'E-mail',
				'PROVIDER_ID' => static::getId(),
				'PROVIDER_TYPE_ID' => 'EMAIL'
			)
		);
	}

	public static function getName()
	{
		return 'E-mail';
	}

	public static function getCommunicationType($providerTypeId = null)
	{
		return static::COMMUNICATION_TYPE_EMAIL;
	}

	/**
	 * @param null|string $providerTypeId Provider type id.
	 * @return bool
	 */
	public static function canUseLiveFeedEvents($providerTypeId = null)
	{
		return true;
	}

	/**
	 * @param null|string $providerTypeId Provider type id.
	 * @param int $direction Activity direction.
	 * @return bool
	 */
	public static function isTypeEditable($providerTypeId = null, $direction = \CCrmActivityDirection::Undefined)
	{
		return false;
	}

	public static function getSupportedCommunicationStatistics()
	{
		return array(
			CommunicationStatistics::STATISTICS_QUANTITY
		);
	}

	public static function renderView(array $activity)
	{
		$html = '<div class="crm-task-list-mail">
			<div class="crm-task-list-mail-header">
				<span class="crm-task-list-mail-header-title">%s</span>
				<!-- total -->
			</div>
			<div class="crm-task-list-mail-item-inner">
				<div class="crm-task-list-mail-item-inner-header">
					<span class="crm-task-list-mail-item-inner-user" %s></span>
					<span class="crm-task-list-mail-item-inner-user-container">
						<span class="crm-task-list-mail-item-inner-user-info">
							%s
							<div class="crm-task-list-mail-item-inner-send">
								<span class="crm-task-list-mail-item-inner-send-item">'
									. Loc::getMessage('EMAIL_ACTIVITY_PROVIDER_RCPT')
									. ':</span>%s
							</div>
						</span>
					</span>
				</div>
				<div class="crm-task-list-mail-item-inner-body">%s</div>
			</div>
		</div>';

		$author = \Bitrix\Main\UserTable::getList(array(
			'select' => array('NAME', 'LAST_NAME', 'SECOND_NAME', 'LOGIN', 'PERSONAL_PHOTO'),
			'filter' => array('=ID' => $activity['AUTHOR_ID']),
		))->fetch();

		if (empty($author))
		{
			$author = array(
				'NAME'           => $activity['RESPONSIBLE_NAME'],
				'LAST_NAME'      => $activity['RESPONSIBLE_LAST_NAME'],
				'SECOND_NAME'    => $activity['RESPONSIBLE_SECOND_NAME'],
				'LOGIN'          => $activity['RESPONSIBLE_LOGIN'],
				'PERSONAL_PHOTO' => $activity['RESPONSIBLE_PERSONAL_PHOTO'],
			);
		}

		$authorName = \CUser::formatName(\CSite::getNameFormat(false), array(
			'NAME'        => $author['NAME'],
			'LAST_NAME'   => $author['LAST_NAME'],
			'SECOND_NAME' => $author['SECOND_NAME'],
			'LOGIN'       => $author['LOGIN']
		), !isModuleInstalled('bitrix24'));

		$authorPhoto = \CFile::resizeImageGet(
			$author['PERSONAL_PHOTO'], array('width' => 38, 'height' => 38),
			BX_RESIZE_IMAGE_EXACT, false
		);

		if (\CCrmActivityDirection::Incoming == $activity['DIRECTION'])
		{
			$item = reset($activity['COMMUNICATIONS']);

			$itemPhoto = array('src' => $item['IMAGE_URL']);

			$from = sprintf(
				'<span class="crm-task-list-mail-item-inner-user-title">%s</span><span class="crm-task-list-mail-item-inner-user-mail">%s</span>',
				$item['TITLE'], $item['VALUE']
			);

			$rcpt = sprintf(
				'<span class="crm-task-list-mail-item-inner-send-user" %s></span><span class="crm-task-list-mail-item-inner-send-mail">%s</span>',
				empty($authorPhoto['src']) ? '' : 'style="background: url(\''.$authorPhoto['src'].'\'); background-size: 23px 23px; "', $authorName
			);
		}
		else
		{
			$itemPhoto = $authorPhoto;

			$from = sprintf('<span class="crm-task-list-mail-item-inner-user-title">%s</span>', $authorName);

			$rcpt = '';
			foreach ($activity['COMMUNICATIONS'] as $item)
			{
				$rcpt .= sprintf(
					'<span class="crm-task-list-mail-item-inner-send-user" %s></span><span class="crm-task-list-mail-item-inner-send-mail">%s</span>',
					empty($item['IMAGE_URL']) ? '' : 'style="background: url(\''.$item['IMAGE_URL'].'\'); background-size: 23px 23px; "', $item['VALUE']
				);
			}
		}

		return sprintf(
			$html,
			$activity['SUBJECT'],
			empty($itemPhoto['src']) ? '' : 'style="background: url(\''.$itemPhoto['src'].'\'); background-size: 40px 40px; "',
			$from, $rcpt,
			$activity['DESCRIPTION_HTML']
		);
	}

}
