<?php

namespace Bitrix\Disk\Uf;

use Bitrix\Disk\Ui;
use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

final class CalendarEventConnector extends StubConnector
{
	private $canRead = null;

	public function canRead($userId)
	{
		if($this->canRead !== null)
		{
			return $this->canRead;
		}
		$this->canRead = \CCalendarEvent::canView($this->entityId, $userId);

		return $this->canRead;
	}

	public function canUpdate($userId)
	{
		return $this->canRead($userId);
	}

	public function canConfidenceReadInOperableEntity()
	{
		return true;
	}

	public function canConfidenceUpdateInOperableEntity()
	{
		return true;
	}

	public function getDataToShow()
	{
		$event = \CCalendarEvent::getById($this->entityId);
		if(empty($event))
		{
			return array();
		}
		$members = array();
		if ($event['IS_MEETING'])
		{
			if(is_array($event['~ATTENDEES']))
			{
				foreach($event['~ATTENDEES'] as $user)
				{
					$members[] = array(
						"NAME" => $user['DISPLAY_NAME'],
						"LINK" => \CComponentEngine::makePathFromTemplate($this->getPathToUser(), array("user_id" => $user['USER_ID'])),
						'AVATAR_SRC' => $user['AVATAR'],
						"IS_EXTRANET" => "N",
					);
				}
			}
		}
		else
		{
			$by = "id";
			$order = "asc";
			$userRow = \CUser::getList(
				$by,
				$order,
				array("ID_EQUAL_EXACT" => $event['CREATED_BY'], "ACTIVE" => "Y"),
				array("SELECT" => array(
					'ID', 'NAME', 'LAST_NAME', 'LOGIN', 'PERSONAL_PHOTO',
				))
			)->fetch();
			if($userRow)
			{
				$name = trim($userRow['NAME'].' '.$userRow['LAST_NAME']);
				if ($name == '')
				{
					$name = trim($userRow['LOGIN']);
				}
				$members[] = array(
					"NAME" => $name,
					"LINK" => \CComponentEngine::makePathFromTemplate($this->getPathToUser(), array("user_id" => $event['CREATED_BY'])),
					'AVATAR_SRC' => Ui\Avatar::getPerson($userRow['PERSONAL_PHOTO']),
					"IS_EXTRANET" => "N",
				);
			}
		}

		return array(
			'TITLE' => Loc::getMessage('DISK_UF_CAL_EVENT_CONNECTOR_TITLE').": ".$event['NAME'],
			'DETAIL_URL' => null,
			'DESCRIPTION' => Ui\Text::killTags($event['DESCRIPTION']),
			'MEMBERS' => $members
		);
	}
}
