CREATE TABLE B_IM_STATUS
(
	USER_ID int not null,
	COLOR varchar(255) NULL,
	STATUS varchar(50),
	STATUS_TEXT varchar(255) null,
	IDLE datetime null,
	DESKTOP_LAST_DATE datetime null,
	MOBILE_LAST_DATE datetime null,
	EVENT_ID int null,
	EVENT_UNTIL_DATE datetime null
)
GO
ALTER TABLE B_IM_STATUS ADD CONSTRAINT PK_B_IM_STATUS PRIMARY KEY (USER_ID)
GO
CREATE INDEX IX_IM_STATUS_EUD ON B_IM_STATUS (EVENT_UNTIL_DATE)
GO
ALTER TABLE B_IM_STATUS ADD CONSTRAINT DF_B_IM_STATUS_ST DEFAULT 'online' FOR STATUS
GO


CREATE TABLE B_IM_CHAT
(
	ID int NOT NULL IDENTITY (1, 1),
	TITLE varchar(255) NULL,
	DESCRIPTION text NULL,
	COLOR varchar(255) NULL,
	TYPE char(1) NULL,
	EXTRANET char(1) NULL,
	AUTHOR_ID int NOT NULL,
	AVATAR int NULL,
	CALL_TYPE tinyint,
	CALL_NUMBER varchar(20) NULL,
	ENTITY_TYPE varchar(50) NULL,
	ENTITY_ID varchar(255) NULL,
	ENTITY_DATA_1 varchar(255) NULL,
	ENTITY_DATA_2 varchar(255) NULL,
	ENTITY_DATA_3 varchar(255) NULL,
	DISK_FOLDER_ID int null,
	LAST_MESSAGE_ID int null
)
ALTER TABLE B_IM_CHAT ADD CONSTRAINT PK_B_IMC PRIMARY KEY (ID)
GO
CREATE INDEX IX_IM_CHAT_1 ON B_IM_CHAT(AUTHOR_ID)
GO
CREATE INDEX IX_IM_CHAT_2 ON B_IM_CHAT(ENTITY_TYPE, ENTITY_ID, AUTHOR_ID)
GO
CREATE INDEX IX_IM_CHAT_3 ON B_IM_CHAT(CALL_NUMBER, AUTHOR_ID)
GO
CREATE INDEX IX_IM_CHAT_4 ON B_IM_CHAT(TYPE)
GO
ALTER TABLE B_IM_CHAT ADD CONSTRAINT DF_B_IM_CHAT_CT DEFAULT '0' FOR CALL_TYPE
GO

CREATE TABLE B_IM_MESSAGE
(
	ID int NOT NULL IDENTITY (1, 1),
	CHAT_ID int NOT NULL,
	AUTHOR_ID int NOT NULL,
	MESSAGE text NULL,
	MESSAGE_OUT text NULL,
	DATE_CREATE datetime NOT NULL,
	EMAIL_TEMPLATE varchar(255) NULL,
	NOTIFY_TYPE tinyint,
	NOTIFY_MODULE varchar(255) NULL,
	NOTIFY_EVENT varchar(255) NULL,
	NOTIFY_TAG varchar(255) NULL,
	NOTIFY_SUB_TAG varchar(255) NULL,
	NOTIFY_TITLE varchar(255) NULL,
	NOTIFY_BUTTONS text NULL,
	NOTIFY_READ char(1),
	IMPORT_ID int NULL
)
ALTER TABLE B_IM_MESSAGE ADD CONSTRAINT PK_B_IMM PRIMARY KEY (ID)
GO
ALTER TABLE B_IM_MESSAGE ADD CONSTRAINT DF_B_IMM_NT DEFAULT '0' FOR NOTIFY_TYPE
GO
ALTER TABLE B_IM_MESSAGE ADD CONSTRAINT DF_B_IMM_NR DEFAULT 'N' FOR NOTIFY_READ
GO
CREATE INDEX IX_IM_MESS_2 ON B_IM_MESSAGE(NOTIFY_TAG, AUTHOR_ID)
GO
CREATE INDEX IX_IM_MESS_3 ON B_IM_MESSAGE(NOTIFY_SUB_TAG, AUTHOR_ID)
GO
CREATE INDEX IX_IM_MESS_4 ON B_IM_MESSAGE(CHAT_ID, NOTIFY_READ)
GO
CREATE INDEX IX_IM_MESS_5 ON B_IM_MESSAGE(CHAT_ID, DATE_CREATE)
GO
CREATE INDEX IX_IM_MESS_6 ON B_IM_MESSAGE(AUTHOR_ID)
GO
CREATE INDEX IX_IM_MESS_7 ON B_IM_MESSAGE(CHAT_ID, ID)
GO
CREATE INDEX IX_IM_MESS_8 ON B_IM_MESSAGE(NOTIFY_TYPE, DATE_CREATE)
GO

CREATE TABLE B_IM_MESSAGE_PARAM
(
	ID int NOT NULL IDENTITY (1, 1),
	MESSAGE_ID INT NOT NULL,
	PARAM_NAME VARCHAR(100) NOT NULL,
	PARAM_VALUE VARCHAR(100) NULL,
	PARAM_JSON text NULL
)
GO
ALTER TABLE B_IM_MESSAGE_PARAM ADD CONSTRAINT PK_B_IM_MESSAGE_PARAM PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_IM_MESSAGE_PARAM_1 ON B_IM_MESSAGE_PARAM(MESSAGE_ID, PARAM_NAME)
GO
CREATE INDEX IX_B_IM_MESSAGE_PARAM_2 ON B_IM_MESSAGE_PARAM(PARAM_NAME, PARAM_VALUE, MESSAGE_ID);
GO

CREATE TABLE B_IM_RELATION
(
	ID int NOT NULL IDENTITY (1, 1),
	CHAT_ID int NOT NULL,
	MESSAGE_TYPE char(1),
	USER_ID int NOT NULL,
	START_ID int,
	LAST_ID int,
	LAST_SEND_ID int,
	LAST_FILE_ID int,
	LAST_READ datetime null,
	STATUS tinyint,
	CALL_STATUS tinyint,
	NOTIFY_BLOCK char(1)
)
ALTER TABLE B_IM_RELATION ADD CONSTRAINT PK_B_IMR PRIMARY KEY (ID)
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_MT DEFAULT 'P' FOR MESSAGE_TYPE
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_NB DEFAULT 'N' FOR NOTIFY_BLOCK
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_SI DEFAULT '0' FOR START_ID
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_LI DEFAULT '0' FOR LAST_ID
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_LSI DEFAULT '0' FOR LAST_SEND_ID
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_LFI DEFAULT '0' FOR LAST_FILE_ID
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IM_REL_CS DEFAULT '0' FOR CALL_STATUS
GO
ALTER TABLE B_IM_RELATION ADD CONSTRAINT DF_B_IMR_S DEFAULT '0' FOR STATUS
GO
CREATE INDEX IX_IM_REL_1 ON B_IM_RELATION(CHAT_ID)
GO
CREATE INDEX IX_IM_REL_2 ON B_IM_RELATION(USER_ID, MESSAGE_TYPE, STATUS)
GO
CREATE INDEX IX_IM_REL_3 ON B_IM_RELATION(USER_ID, MESSAGE_TYPE, CHAT_ID)
GO
CREATE INDEX IX_IM_REL_4 ON B_IM_RELATION(USER_ID, STATUS)
GO
CREATE INDEX IX_IM_REL_5 ON B_IM_RELATION(MESSAGE_TYPE, STATUS)
GO
CREATE INDEX IX_IM_REL_6 ON B_IM_RELATION(CHAT_ID, USER_ID)
GO

CREATE TABLE B_IM_RECENT
(
	USER_ID int not null,
	ITEM_TYPE char(1) not null,
	ITEM_ID int not null,
	ITEM_MID int not null
)
GO
ALTER TABLE B_IM_RECENT ADD CONSTRAINT PK_B_IM_RECENT PRIMARY KEY (USER_ID, ITEM_TYPE, ITEM_ID)
GO
ALTER TABLE B_IM_RECENT ADD CONSTRAINT DF_B_IMREC_IT DEFAULT 'P' FOR ITEM_TYPE
GO
CREATE INDEX IX_IM_REC_1 ON B_IM_RECENT(ITEM_TYPE, ITEM_ID)
GO

CREATE TABLE B_IM_BOT
(
	BOT_ID int not null,
	MODULE_ID varchar(50) not null,
	TYPE char(1) not null,
	CODE varchar(50) not null,
	CLASS varchar(255) NULL,
	LANG varchar(50) null,
	METHOD_BOT_DELETE varchar(255) NULL,
	METHOD_MESSAGE_ADD varchar(255) NULL,
	METHOD_WELCOME_MESSAGE varchar(255) NULL,
	TEXT_PRIVATE_WELCOME_MESSAGE text NULL,
	TEXT_CHAT_WELCOME_MESSAGE text NULL,
	COUNT_COMMAND int,
	COUNT_MESSAGE int,
	COUNT_CHAT int,
	COUNT_USER int,
	VERIFIED char(1) null,
	OPENLINE char(1) null,
	APP_ID varchar(128) NULL
)
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT PK_B_IM_BOT PRIMARY KEY (BOT_ID)
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_TYPE DEFAULT 'B' FOR TYPE
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_V DEFAULT 'N' FOR VERIFIED
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_OL DEFAULT 'N' FOR OPENLINE
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_CS DEFAULT '0' FOR COUNT_COMMAND
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_CM DEFAULT '0' FOR COUNT_MESSAGE
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_CC DEFAULT '0' FOR COUNT_CHAT
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_CU DEFAULT '0' FOR COUNT_USER
GO
ALTER TABLE B_IM_BOT ADD CONSTRAINT DF_B_IMBOT_L DEFAULT '' FOR LANG
GO

CREATE TABLE B_IM_BOT_CHAT
(
	ID int NOT NULL IDENTITY (1, 1),
	BOT_ID int not null,
	CHAT_ID int not null
)
GO
ALTER TABLE B_IM_BOT_CHAT ADD CONSTRAINT PK_B_IM_BOT_CHAT PRIMARY KEY (ID)
GO
CREATE INDEX IX_IM_BC_1 ON B_IM_BOT_CHAT(BOT_ID, CHAT_ID)
GO

CREATE TABLE B_IM_BOT_TOKEN
(
	ID int NOT NULL IDENTITY (1, 1),
	TOKEN varchar(32) null,
	DATE_CREATE datetime NOT NULL,
	DATE_EXPIRE datetime NULL,
	BOT_ID int null,
	DIALOG_ID varchar(255) not null,
)
GO
ALTER TABLE B_IM_BOT_TOKEN ADD CONSTRAINT PK_B_IM_BOT_TOKEN PRIMARY KEY (ID)
GO
ALTER TABLE B_IM_BOT_TOKEN ADD CONSTRAINT DF_B_IM_BOT_TOKEN_BI DEFAULT '0' FOR BOT_ID
GO
CREATE INDEX IX_IM_BOT_TOKEN_1 ON B_IM_BOT_TOKEN(DATE_EXPIRE, BOT_ID)
GO
CREATE INDEX IX_IM_BOT_TOKEN_2 ON B_IM_BOT_TOKEN(TOKEN)
GO

CREATE TABLE B_IM_COMMAND
(
	ID int NOT NULL IDENTITY (1, 1),
	MODULE_ID varchar(50) not null,
	BOT_ID int not null,
	APP_ID varchar(128) NULL,
	COMMAND varchar(255) not null,
	COMMON char(1) NULL,
	HIDDEN char(1) NULL,
	EXTRANET_SUPPORT char(1) NULL,
	SONET_SUPPORT char(1) NULL,
	CLASS varchar(255) NULL,
	METHOD_COMMAND_ADD varchar(255) NULL,
	METHOD_LANG_GET varchar(255) NULL,
)
ALTER TABLE B_IM_COMMAND ADD CONSTRAINT PK_B_IM_COMMAND PRIMARY KEY (ID)
GO
ALTER TABLE B_IM_COMMAND ADD CONSTRAINT DF_B_IM_COMMAND_C DEFAULT 'N' FOR COMMON
GO
ALTER TABLE B_IM_COMMAND ADD CONSTRAINT DF_B_IM_COMMAND_H DEFAULT 'N' FOR HIDDEN
GO
ALTER TABLE B_IM_COMMAND ADD CONSTRAINT DF_B_IM_COMMAND_E DEFAULT 'N' FOR EXTRANET_SUPPORT
GO
ALTER TABLE B_IM_COMMAND ADD CONSTRAINT DF_B_IM_COMMAND_SS DEFAULT 'N' FOR SONET_SUPPORT
GO
CREATE INDEX IX_IM_COMMAND_1 ON B_IM_COMMAND(BOT_ID)
GO

CREATE TABLE B_IM_COMMAND_LANG
(
	ID int NOT NULL IDENTITY (1, 1),
	COMMAND_ID int not null,
	LANGUAGE_ID char(2) NOT NULL,
	TITLE varchar(255) NULL,
	PARAMS varchar(255) NULL
)
GO
ALTER TABLE B_IM_COMMAND_LANG ADD CONSTRAINT PK_B_IM_COMMAND_LANG PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_IM_COMMAND_LANG ON B_IM_COMMAND_LANG (COMMAND_ID, LANGUAGE_ID)
GO

CREATE TABLE B_IM_ALIAS
(
	ID int NOT NULL IDENTITY (1, 1),
	ALIAS varchar(255) NOT NULL,
	ENTITY_TYPE varchar(255) NOT NULL,
	ENTITY_ID varchar(255) NOT NULL
)
GO
ALTER TABLE B_IM_ALIAS ADD CONSTRAINT PK_B_IM_ALIAS PRIMARY KEY (ID)
GO
CREATE UNIQUE INDEX UX_B_IM_ALIAS ON B_IM_ALIAS (ALIAS)
GO

CREATE TABLE B_IM_EXTERNAL_AVATAR
(
	ID int NOT NULL IDENTITY (1, 1),
	LINK_MD5 varchar(32) NOT NULL,
	AVATAR_ID int NOT NULL
)
GO
ALTER TABLE B_IM_EXTERNAL_AVATAR ADD CONSTRAINT PK_B_IM_EXTERNAL_AVATAR PRIMARY KEY (ID)
GO
CREATE INDEX IX_IM_NA_1 ON B_IM_EXTERNAL_AVATAR(LINK_MD5)
GO