CREATE TABLE B_INTRANET_SHAREPOINT
(
	IBLOCK_ID int not null,
	SP_LIST_ID varchar(32) not null,
	SP_URL varchar(255) not null,
	SP_AUTH_USER varchar(50) null,
	SP_AUTH_PASS varchar(50) null,
	SYNC_DATE datetime null,
	SYNC_PERIOD int null,
	SYNC_ERRORS int null,
	SYNC_LAST_TOKEN varchar(100) null,
	SYNC_PAGING varchar(100) null,
	HANDLER_MODULE varchar(50) null,
	HANDLER_CLASS varchar(100) null,
	PRIORITY varchar(1) null,
	CONSTRAINT PK_B_INTRANET_SHAREPOINT PRIMARY KEY (IBLOCK_ID)
)
GO
ALTER TABLE B_INTRANET_SHAREPOINT ADD CONSTRAINT DF_B_INTRANET_SHAREPOINT_SYNC_DATE DEFAULT GETDATE() FOR SYNC_DATE
GO
ALTER TABLE B_INTRANET_SHAREPOINT ADD CONSTRAINT DF_B_INTRANET_SHAREPOINT_SYNC_PERIOD DEFAULT 86400 FOR SYNC_PERIOD
GO
ALTER TABLE B_INTRANET_SHAREPOINT ADD CONSTRAINT DF_B_INTRANET_SHAREPOINT_PRIORITY DEFAULT 'B' FOR PRIORITY
GO

CREATE TABLE B_INTRANET_SHAREPOINT_FIELD
(
	IBLOCK_ID int not null,
	FIELD_ID varchar(50) not null,
	SP_FIELD varchar(50) not null,
	SP_FIELD_TYPE varchar(50) not null,
	SETTINGS text,
	CONSTRAINT PK_B_INTRANET_SHAREPOINT_FIELD PRIMARY KEY (IBLOCK_ID, FIELD_ID)
)
GO

CREATE TABLE B_INTRANET_SHAREPOINT_QUEUE
(
	ID int NOT NULL IDENTITY (1, 1),
	IBLOCK_ID int NOT NULL,
	SP_METHOD varchar(100) NOT NULL,
	SP_METHOD_PARAMS text NULL,
	CALLBACK text NULL,
	CONSTRAINT PK_B_INTRANET_SHAREPOINT_QUEUE PRIMARY KEY (ID)
)
GO
CREATE INDEX IX_B_INTRANET_SHAREPOINT_QUEUE_1 ON B_INTRANET_SHAREPOINT_QUEUE (IBLOCK_ID)
GO

CREATE TABLE B_INTRANET_SHAREPOINT_LOG
(
	ID int NOT NULL IDENTITY (1, 1),
	IBLOCK_ID int not null,
	ELEMENT_ID int not null,
	VERSION int null,
	CONSTRAINT PK_B_INTRANET_SHAREPOINT_LOG PRIMARY KEY (ID)
)
GO
CREATE UNIQUE INDEX UI_B_INTRANET_SHAREPOINT_LOG ON B_INTRANET_SHAREPOINT_LOG (IBLOCK_ID, ELEMENT_ID)
GO
CREATE TABLE B_RATING_SUBORDINATE
(
	ID int NOT NULL IDENTITY (1, 1),
	RATING_ID int NOT NULL,
	ENTITY_ID int NOT NULL,
	VOTES decimal(18, 4) NULL
)
GO
ALTER TABLE B_RATING_SUBORDINATE ADD CONSTRAINT PK_B_RAT_SUB PRIMARY KEY (ID)
GO
CREATE INDEX IX_B_RAT_SUB ON B_RATING_SUBORDINATE(RATING_ID, ENTITY_ID)
GO
ALTER TABLE B_RATING_SUBORDINATE ADD CONSTRAINT DF_B_RAT_SUB_VOTES DEFAULT '0.0000' FOR VOTES
GO

CREATE TABLE B_INTRANET_USTAT_HOUR
(
  USER_ID int NOT NULL,
  HOUR datetime NOT NULL,
  TOTAL smallint NOT NULL DEFAULT '0',
  SOCNET smallint NOT NULL DEFAULT '0',
  LIKES smallint NOT NULL DEFAULT '0',
  TASKS smallint NOT NULL DEFAULT '0',
  IM smallint NOT NULL DEFAULT '0',
  DISK smallint NOT NULL DEFAULT '0',
  MOBILE smallint NOT NULL DEFAULT '0',
  CRM smallint NOT NULL DEFAULT '0',
  CONSTRAINT PK_B_INTRANET_USTAT_HOUR PRIMARY KEY (USER_ID,HOUR)
)
GO

CREATE INDEX IX_B_INTRANET_USTAT_HOUR_DATE ON B_INTRANET_USTAT_HOUR (HOUR)
GO

CREATE TABLE B_INTRANET_USTAT_DAY
(
  USER_ID int NOT NULL,
  DAY date NOT NULL,
  TOTAL smallint NOT NULL DEFAULT '0',
  SOCNET smallint NOT NULL DEFAULT '0',
  LIKES smallint NOT NULL DEFAULT '0',
  TASKS smallint NOT NULL DEFAULT '0',
  IM smallint NOT NULL DEFAULT '0',
  DISK smallint NOT NULL DEFAULT '0',
  MOBILE smallint NOT NULL DEFAULT '0',
  CRM smallint NOT NULL DEFAULT '0',
  CONSTRAINT PK_B_INTRANET_USTAT_DAY PRIMARY KEY (USER_ID,DAY)
)
GO

CREATE INDEX IX_B_INTRANET_USTAT_DAY_DATE ON B_INTRANET_USTAT_DAY (DAY)
GO

CREATE TABLE B_INTRANET_DSTAT_HOUR
(
  DEPT_ID int NOT NULL,
  HOUR datetime NOT NULL,
  TOTAL int NOT NULL DEFAULT '0',
  SOCNET int NOT NULL DEFAULT '0',
  LIKES int NOT NULL DEFAULT '0',
  TASKS int NOT NULL DEFAULT '0',
  IM int NOT NULL DEFAULT '0',
  DISK int NOT NULL DEFAULT '0',
  MOBILE int NOT NULL DEFAULT '0',
  CRM int NOT NULL DEFAULT '0',
  CONSTRAINT PK_B_INTRANET_DSTAT_HOUR PRIMARY KEY (DEPT_ID,HOUR)
)
GO

CREATE INDEX IX_B_INTRANET_DSTAT_HOUR_DATE ON B_INTRANET_DSTAT_HOUR (HOUR)
GO

CREATE TABLE B_INTRANET_DSTAT_DAY
(
  DEPT_ID int NOT NULL,
  DAY date NOT NULL,
  ACTIVE_USERS int NOT NULL DEFAULT '0',
  INVOLVEMENT tinyint NOT NULL DEFAULT '0',
  TOTAL int NOT NULL DEFAULT '0',
  SOCNET int NOT NULL DEFAULT '0',
  LIKES int NOT NULL DEFAULT '0',
  TASKS int NOT NULL DEFAULT '0',
  IM int NOT NULL DEFAULT '0',
  DISK int NOT NULL DEFAULT '0',
  MOBILE int NOT NULL DEFAULT '0',
  CRM int NOT NULL DEFAULT '0',
  CONSTRAINT PK_B_INTRANET_DSTAT_DAY PRIMARY KEY (DEPT_ID,DAY)
)
GO

CREATE INDEX IX_B_INTRANET_DSTAT_DAY_DATE ON B_INTRANET_DSTAT_DAY (DAY)
GO

CREATE TABLE B_INTRANET_USERSUBORD
(
  ID int NOT NULL IDENTITY (1, 1),
  DIRECTOR_ID int not null,
  SUBORDINATE_ID int not null,
  CONSTRAINT PK_B_INTRANET_USERSUBORD PRIMARY KEY (ID)
)
GO

CREATE INDEX IX_INTRANET_USERSUBORD_DS ON B_INTRANET_USERSUBORD (DIRECTOR_ID, SUBORDINATE_ID)
GO

CREATE TABLE B_INTRANET_USER2DEP
(
  ID int NOT NULL IDENTITY (1, 1),
	USER_ID int NOT NULL,
	DEPARTMENT_ID int,
	WEIGHT tinyint DEFAULT '1',
  CONSTRAINT PK_B_INTRANET_USER2DEP PRIMARY KEY (ID)
)
GO

CREATE INDEX IX_INTRANET_USER2DEP_UWD ON B_INTRANET_USER2DEP (USER_ID, WEIGHT, DEPARTMENT_ID)
GO