CREATE TABLE B_MEETING
(
	ID int NOT NULL IDENTITY(1,1),
	TIMESTAMP_X datetime NOT NULL, 
	EVENT_ID int NULL,
	DATE_START datetime NULL,
	DATE_FINISH datetime NULL,
	CURRENT_STATE CHAR(1) NULL,
	DURATION int NULL,
	TITLE varchar(255) NOT NULL,
	GROUP_ID int NULL,
	PARENT_ID int NULL,
	DESCRIPTION text NULL,
	PLACE varchar(255) NULL,
	PROTOCOL_TEXT text NULL,
	CONSTRAINT PK_B_MEETING PRIMARY KEY (ID)
)
GO
ALTER TABLE B_MEETING ADD CONSTRAINT DF_B_MEETING_CURRENT_STATE DEFAULT 'P' FOR CURRENT_STATE
GO
CREATE INDEX IX_B_MEETING_1 ON B_MEETING (GROUP_ID)
GO
ALTER TABLE B_MEETING ADD CONSTRAINT DF_B_MEETING_TIMESTAMP_X DEFAULT GETDATE() FOR TIMESTAMP_X
GO
CREATE TABLE B_MEETING_FILES
(
	MEETING_ID int NOT NULL,
	FILE_ID int NOT NULL,
	FILE_SRC int NULL,
	CONSTRAINT PK_B_MEETING_FILE PRIMARY KEY (MEETING_ID, FILE_ID)
)
GO
ALTER TABLE B_MEETING_FILES ADD CONSTRAINT FK_B_MEETING_FILE_1 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO

CREATE TABLE B_MEETING_USERS
(
	MEETING_ID int NOT NULL,
	USER_ID int NOT NULL,
	USER_ROLE CHAR(1) NULL,
	CONSTRAINT PK_B_MEETING_USERS PRIMARY KEY (MEETING_ID, USER_ID)
)
GO
ALTER TABLE B_MEETING_USERS ADD CONSTRAINT FK_B_MEETING_USERS_1 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO
ALTER TABLE B_MEETING_USERS ADD CONSTRAINT FK_B_MEETING_USERS_2 FOREIGN KEY (USER_ID) REFERENCES B_USER(ID)
GO
ALTER TABLE B_MEETING_USERS ADD CONSTRAINT DF_B_MEETING_USERS_USER_ROLE DEFAULT 'M' FOR USER_ROLE
GO

CREATE TABLE B_MEETING_ITEM
(
	ID int NOT NULL IDENTITY(1,1),
	TITLE varchar(255) NULL,
	DESCRIPTION text NULL,
	CONSTRAINT PK_B_MEETING_ITEM PRIMARY KEY (ID)
)
GO
CREATE TABLE B_MEETING_ITEM_FILES
(
	ITEM_ID INT NOT NULL,
	FILE_ID INT NOT NULL,
	FILE_SRC INT NULL,
	CONSTRAINT PK_B_MEETING_ITEM_FILES PRIMARY KEY (ITEM_ID, FILE_ID)
)
GO
ALTER TABLE B_MEETING_ITEM_FILES ADD CONSTRAINT FK_B_MEETING_ITEM_FILES_1 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO
CREATE INDEX IX_B_MEETING_ITEM_FILES_1 ON B_MEETING_ITEM_FILES(FILE_SRC)
GO

CREATE TABLE B_MEETING_INSTANCE
(
	ID int NOT NULL IDENTITY(1,1),
	ITEM_ID int NOT NULL,
	MEETING_ID int NOT NULL,
	INSTANCE_PARENT_ID int NULL,
	INSTANCE_TYPE CHAR(1) NULL,
	ORIGINAL_TYPE CHAR(1) NULL,
	SORT int DEFAULT 500 NULL,
	DURATION int NULL,
	DEADLINE datetime NULL,
	TASK_ID int NULL,
	CONSTRAINT PK_B_MEETING_INSTANCE PRIMARY KEY (ID)
)
GO
ALTER TABLE B_MEETING_INSTANCE ADD CONSTRAINT FK_B_MEETING_INSTANCE_1 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO
ALTER TABLE B_MEETING_INSTANCE ADD CONSTRAINT FK_B_MEETING_INSTANCE_2 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO
ALTER TABLE B_MEETING_INSTANCE ADD CONSTRAINT DF_B_MEETING_INSTANCE_INSTANCE_TYPE DEFAULT 'A' FOR INSTANCE_TYPE
GO
ALTER TABLE B_MEETING_INSTANCE ADD CONSTRAINT DF_B_MEETING_INSTANCE_ORIGINAL_TYPE DEFAULT 'A' FOR ORIGINAL_TYPE
GO

CREATE TABLE B_MEETING_ITEM_TASKS
(
	ITEM_ID int NOT NULL,
	INSTANCE_ID int NULL,
	TASK_ID int NOT NULL,
	CONSTRAINT PK_B_MEETING_ITEM_TASKS PRIMARY KEY (ITEM_ID, TASK_ID)
)
GO

ALTER TABLE B_MEETING_ITEM_TASKS ADD CONSTRAINT FK_B_MEETING_ITEM_TASKS_1 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO

ALTER TABLE B_MEETING_ITEM_TASKS ADD CONSTRAINT FK_B_MEETING_ITEM_TASKS_2 FOREIGN KEY (INSTANCE_ID) REFERENCES B_MEETING_INSTANCE(ID)
GO
CREATE INDEX IX_B_MEETING_ITEM_TASKS_1 ON B_MEETING_ITEM_TASKS(INSTANCE_ID)
GO

CREATE TABLE B_MEETING_INSTANCE_USERS
(
	USER_ID int NOT NULL,
	INSTANCE_ID int NOT NULL,
	ITEM_ID int NOT NULL,
	MEETING_ID int NOT NULL,
	CONSTRAINT PK_B_MEETING_INSTANCE_USERS PRIMARY KEY (INSTANCE_ID, USER_ID)
)
GO

ALTER TABLE B_MEETING_INSTANCE_USERS ADD CONSTRAINT FK_B_MEETING_INSTANCE_USERS_1 FOREIGN KEY (USER_ID) REFERENCES B_USER(ID)
GO

ALTER TABLE B_MEETING_INSTANCE_USERS ADD CONSTRAINT FK_B_MEETING_INSTANCE_USERS_2 FOREIGN KEY (INSTANCE_ID) REFERENCES B_MEETING_INSTANCE(ID)
GO
ALTER TABLE B_MEETING_INSTANCE_USERS ADD CONSTRAINT FK_B_MEETING_INSTANCE_USERS_3 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO
ALTER TABLE B_MEETING_INSTANCE_USERS ADD CONSTRAINT FK_B_MEETING_INSTANCE_USERS_4 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO

CREATE TABLE B_MEETING_REPORTS
(
	ID int NOT NULL IDENTITY(1,1),
	USER_ID int NOT NULL,
	INSTANCE_ID int NOT NULL,
	ITEM_ID int NOT NULL,
	MEETING_ID int NOT NULL,
	REPORT text,
	CONSTRAINT PK_B_MEETING_REPORTS PRIMARY KEY (ID)
)
GO
ALTER TABLE B_MEETING_REPORTS ADD CONSTRAINT FK_B_MEETING_REPORTS_1 FOREIGN KEY (USER_ID) REFERENCES B_USER(ID)
GO
ALTER TABLE B_MEETING_REPORTS ADD CONSTRAINT FK_B_MEETING_REPORTS_2 FOREIGN KEY (INSTANCE_ID) REFERENCES B_MEETING_INSTANCE(ID)
GO
ALTER TABLE B_MEETING_REPORTS ADD CONSTRAINT FK_B_MEETING_REPORTS_3 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO
ALTER TABLE B_MEETING_REPORTS ADD CONSTRAINT FK_B_MEETING_REPORTS_4 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO
CREATE UNIQUE INDEX IX_B_MEETING_REPORTS_1 ON B_MEETING_REPORTS (INSTANCE_ID, USER_ID)
GO

CREATE TABLE B_MEETING_REPORTS_FILES
(
	FILE_ID int NOT NULL,
	FILE_SRC int NULL,
	REPORT_ID int NOT NULL,
	INSTANCE_ID int NOT NULL,
	ITEM_ID int NOT NULL,
	MEETING_ID int NOT NULL,
	CONSTRAINT pk_b_meeting_reports_files PRIMARY KEY (REPORT_ID, FILE_ID)
)
GO

ALTER TABLE B_MEETING_REPORTS_FILES ADD CONSTRAINT FK_B_MEETING_REPORTS_FILES_1 FOREIGN KEY (FILE_ID) REFERENCES B_FILE(ID)
GO
ALTER TABLE B_MEETING_REPORTS_FILES ADD CONSTRAINT FK_B_MEETING_REPORTS_FILES_2 FOREIGN KEY (REPORT_ID) REFERENCES B_MEETING_REPORTS(ID)
GO
ALTER TABLE B_MEETING_REPORTS_FILES ADD CONSTRAINT FK_B_MEETING_REPORTS_FILES_3 FOREIGN KEY (INSTANCE_ID) REFERENCES B_MEETING_INSTANCE(ID)
GO
ALTER TABLE B_MEETING_REPORTS_FILES ADD CONSTRAINT FK_B_MEETING_REPORTS_FILES_4 FOREIGN KEY (ITEM_ID) REFERENCES B_MEETING_ITEM(ID)
GO
ALTER TABLE B_MEETING_REPORTS_FILES ADD CONSTRAINT FK_B_MEETING_REPORTS_FILES_5 FOREIGN KEY (MEETING_ID) REFERENCES B_MEETING(ID)
GO

