CREATE TABLE B_PULL_STACK
(
	ID int NOT NULL IDENTITY (1, 1),
	CHANNEL_ID varchar(50) NOT NULL,
	MESSAGE text NOT NULL,
	DATE_CREATE datetime NOT NULL
)
ALTER TABLE B_PULL_STACK ADD CONSTRAINT PK_B_PSI PRIMARY KEY (ID)
GO
CREATE INDEX IX_PULL_STACK_CID ON B_PULL_STACK(CHANNEL_ID)
GO
CREATE INDEX IX_PULL_STACK_D ON B_PULL_STACK(DATE_CREATE)
GO

CREATE TABLE B_PULL_CHANNEL
(
	ID int NOT NULL IDENTITY (1, 1),
	USER_ID int NOT NULL,
	CHANNEL_TYPE varchar(50) NULL,
	CHANNEL_ID varchar(50) NOT NULL,
	LAST_ID int NULL,
	DATE_CREATE datetime NOT NULL
)
ALTER TABLE B_PULL_CHANNEL ADD CONSTRAINT PK_B_PCI PRIMARY KEY (ID)
GO
CREATE INDEX IX_PULL_CN_CID ON B_PULL_CHANNEL(CHANNEL_ID)
GO
CREATE INDEX IX_PULL_CN_D ON B_PULL_CHANNEL(DATE_CREATE)
GO
CREATE UNIQUE INDEX IX_PULL_CN_UID ON B_PULL_CHANNEL(USER_ID, CHANNEL_TYPE)
GO

CREATE TABLE B_PULL_PUSH
(
	ID int NOT NULL IDENTITY (1, 1),
	USER_ID int NOT NULL,
	DEVICE_TYPE varchar(50) NOT NULL,
	APP_ID varchar(50) NOT NULL,
	UNIQUE_HASH varchar(50) NOT NULL,
	DEVICE_ID varchar(255) NULL,
	DEVICE_NAME varchar(50) NULL,
	DEVICE_TOKEN varchar(255) NOT NULL,
	DATE_CREATE datetime NOT NULL,
	DATE_AUTH datetime NULL
)
ALTER TABLE B_PULL_PUSH ADD CONSTRAINT PK_B_PPI PRIMARY KEY (ID)
GO
CREATE INDEX IX_PULL_PSH_UID ON B_PULL_PUSH(USER_ID)
GO
CREATE INDEX IX_PULL_PSH_UA ON B_PULL_PUSH(UNIQUE_HASH)
GO

CREATE TABLE B_PULL_PUSH_QUEUE
(
	ID int NOT NULL IDENTITY (1, 1),
	USER_ID int NOT NULL,
	APP_ID varchar(50) NOT NULL,
	TAG varchar(255) NULL,
	SUB_TAG varchar(255) NULL,
	MESSAGE text NULL,
	PARAMS text NULL,
	ADVANCED_PARAMS text NULL,
	BADGE int NULL,
	DATE_CREATE datetime NULL
)
ALTER TABLE B_PULL_PUSH_QUEUE ADD CONSTRAINT PK_B_PPQI PRIMARY KEY (ID)
GO
CREATE INDEX IX_PULL_PSHQ_UT ON B_PULL_PUSH_QUEUE (USER_ID, TAG)
GO
CREATE INDEX IX_PULL_PSHQ_UST ON B_PULL_PUSH_QUEUE (USER_ID, SUB_TAG)
GO
CREATE INDEX IX_PULL_PSHQ_UID ON B_PULL_PUSH_QUEUE (USER_ID)
GO
CREATE INDEX IX_PULL_PSHQ_DC ON B_PULL_PUSH_QUEUE (DATE_CREATE)
GO
CREATE INDEX IX_PULL_PSHQ_AID ON B_PULL_PUSH_QUEUE (APP_ID)
GO

CREATE TABLE B_PULL_WATCH
(
	ID int NOT NULL IDENTITY (1, 1),
	USER_ID int NOT NULL,
	CHANNEL_ID varchar(50) NOT NULL,
	TAG varchar(255) NOT NULL,
	DATE_CREATE datetime NOT NULL
)
ALTER TABLE B_PULL_WATCH ADD CONSTRAINT PK_B_PWI PRIMARY KEY (ID)
GO
CREATE INDEX IX_PULL_W_UT ON B_PULL_WATCH(USER_ID, TAG)
GO
CREATE INDEX IX_PULL_W_D ON B_PULL_WATCH(DATE_CREATE)
GO
CREATE INDEX IX_PULL_W_T ON B_PULL_WATCH(TAG)
GO


