<?php
if(!defined("B_PROLOG_INCLUDED") || B_PROLOG_INCLUDED !== true)
{
	die();
}

/**
 * Bitrix vars
 *
 * @var array $arParams
 * @var array $arResult
 * @var CBitrixComponent $this
 * @global CMain $APPLICATION
 * @global CUser $USER
 */

use Bitrix\Main\Loader;

if(!Loader::includeModule('rest'))
{
	return;
}

$request = \Bitrix\Main\Context::getCurrent()->getRequest();

$clientId = $request['client_id'];
if(!$clientId)
{
	ShowError(\Bitrix\Main\Localization\Loc::getMessage('REST_APP_NOT_FOUND'));
	return;
}

if($USER->IsAuthorized())
{
	if(isset($request['state']))
	{
		$state = $request['state'];
	}
	else
	{
		$state = '';
	}

	$authResult = \Bitrix\Rest\OAuth\Auth::authorizeApplication($clientId, $USER->GetID(), $state);

	if($authResult['error'])
	{
		ShowError($authResult['error'].': '.$authResult['error_description']);
	}
	elseif($authResult['redirect_uri'])
	{
		$redirectUri = $authResult['redirect_uri'];

		unset($authResult['redirect_uri']);

		$fragment = '';
		if(array_key_exists('fragment', $authResult))
		{
			$fragment = $authResult['fragment'];
			unset($authResult['fragment']);
		}

		$authResult['server_domain'] = $authResult['domain'];
		$authResult['domain'] = $request->getHttpHost();

		$redirectUri .= (strpos($redirectUri, '?') !== false) ? '&' : '?';
		$redirectUri .= http_build_query($authResult);

		if(strlen($fragment) > 0)
		{
			$redirectUri .= '#'.$fragment;
		}

		LocalRedirect($redirectUri, true);
	}
	else
	{
		$arResult['OAUTH_PARAMS'] = $authResult;
		$this->includeComponentTemplate();
	}
}
else
{
	if(isset($request['client_id']))
	{
		$dbRes = \Bitrix\Rest\AppTable::getList(array(
			'filter' => array(
				'=CLIENT_ID' => $request['client_id'],
				'=ACTIVE' => \Bitrix\Rest\AppTable::ACTIVE,
			),
			'select' => array('CODE'),
		));
		$appInfo = $dbRes->fetch();

		if($appInfo)
		{
			$APPLICATION->AuthForm(\Bitrix\Main\Localization\Loc::getMessage('REST_NEED_AUTHORIZE_A', array(
				'#APP_ID#' => $appInfo['CODE']
			)));
			return;
		}
	}

	ShowError(\Bitrix\Main\Localization\Loc::getMessage('REST_APP_NOT_FOUND'));
}
