CREATE TABLE b_sender_list
(
  ID		INT		NOT NULL IDENTITY(1,1),
  NAME		VARCHAR(100)	NULL,
  CODE		VARCHAR(60)	NULL,
  SORT		INT		DEFAULT 100 NOT NULL,
)
GO
ALTER TABLE b_sender_list ADD CONSTRAINT PK_B_SENDER_LIST PRIMARY KEY (ID)
GO

CREATE TABLE b_sender_contact
(
  ID		INT		NOT NULL IDENTITY(1,1),
  DATE_INSERT	DATETIME	NOT NULL,
  DATE_UPDATE	DATETIME	NULL,
  NAME		VARCHAR(255)	NULL,
  EMAIL		VARCHAR(255)	NULL,
  PHONE		VARCHAR(20)	NULL,
  USER_ID		INT	NULL
)
GO
ALTER TABLE b_sender_contact ADD CONSTRAINT PK_B_SENDER_CONTACT PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_CONTACT_EMAIL on b_sender_contact(EMAIL)
GO

CREATE TABLE b_sender_contact_list
(
  CONTACT_ID	INT NOT NULL,
  LIST_ID	INT NOT NULL,
)
GO
CREATE UNIQUE INDEX UK_SENDER_CONTACT_LIST on b_sender_contact_list(CONTACT_ID, LIST_ID)
GO

CREATE TABLE b_sender_group
(
  ID		INT		NOT NULL IDENTITY(1,1),
  NAME		VARCHAR(100)	NULL,
  DESCRIPTION	TEXT		NULL,
  SORT		INT		DEFAULT 100 NOT NULL,
  ACTIVE		CHAR(1)		DEFAULT 'Y' NOT NULL,
  ADDRESS_COUNT	INT		DEFAULT 0 NOT NULL
)
GO
ALTER TABLE b_sender_group ADD CONSTRAINT PK_B_SENDER_GROUP PRIMARY KEY (ID)
GO

CREATE TABLE b_sender_group_connector
(
  GROUP_ID INT NOT NULL,
  NAME		VARCHAR(100)	NULL,
  ENDPOINT		VARCHAR(2000)	NULL,
  ADDRESS_COUNT	INT		DEFAULT 0 NOT NULL
)
GO
CREATE INDEX IX_SENDER_GROUP_CONNECTOR on b_sender_group_connector(GROUP_ID)
GO

CREATE TABLE b_sender_mailing
(
  ID		INT		NOT NULL IDENTITY(1,1),
  DATE_INSERT	datetime	NULL,
  NAME		varchar(100)		 NULL,
  DESCRIPTION	TEXT		NULL,
  ACTIVE	CHAR(1)		DEFAULT 'Y' NOT NULL,
  SITE_ID		CHAR(2)		NOT NULL,
  SORT		INT		DEFAULT 100 NOT NULL,
  IS_PUBLIC CHAR(1) DEFAULT 'Y' NOT NULL,
  TRACK_CLICK		CHAR(1)	DEFAULT 'N' NOT NULL,
  TRIGGER_FIELDS TEXT NULL,
  IS_TRIGGER char(1) default 'N'  NOT NULL,
  EMAIL_FROM VARCHAR(255) NULL
)
GO
ALTER TABLE b_sender_mailing ADD CONSTRAINT PK_B_SENDER_MAILING PRIMARY KEY (ID)
GO

CREATE TABLE b_sender_mailing_chain
(
  ID		INT		NOT NULL IDENTITY(1,1),
  MAILING_ID	INT NOT NULL,
  STATUS		CHAR(1)		NOT NULL,
  POSTING_ID	INT NULL,
  CREATED_BY	INT NULL,
  PARENT_ID INT NULL,
  IS_TRIGGER char(1) default 'N'  NOT NULL,
  DATE_INSERT DATETIME NULL,
  TIME_SHIFT INT default 0 NOT NULL,
  LAST_EXECUTED	datetime	NULL,
  AUTO_SEND_TIME	datetime	NULL,
	TITLE varchar(255) NULL,
	EMAIL_FROM varchar(255) NOT NULL,
	SUBJECT varchar(255) NULL,
  PRIORITY varchar(60) NULL,
  LINK_PARAMS varchar(255) NULL,
	MESSAGE text NULL,
  TEMPLATE_TYPE varchar(30) NULL,
  TEMPLATE_ID varchar(60) NULL,
  REITERATE	CHAR(1)		DEFAULT 'N' NOT NULL,
  DAYS_OF_MONTH	VARCHAR(100)	NULL,
  DAYS_OF_WEEK	VARCHAR(15)	NULL,
  TIMES_OF_DAY	VARCHAR(255)	NULL
)
GO
ALTER TABLE b_sender_mailing_chain ADD CONSTRAINT PK_B_SENDER_MAILING_CHAIN PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_MAILING_CHAIN_MAILING on b_sender_mailing_chain(MAILING_ID, STATUS)
GO
CREATE INDEX IX_SENDER_MAILING_CHAIN_REITERATE on b_sender_mailing_chain(REITERATE, STATUS)
GO

CREATE TABLE b_sender_mailing_group
(
  MAILING_ID	INT NOT NULL,
  GROUP_ID	INT NOT NULL,
  INCLUDE	INT		DEFAULT 0 NOT NULL
)
GO
CREATE UNIQUE INDEX UK_B_SENDER_MAILING_GROUP on b_sender_mailing_group(MAILING_ID, GROUP_ID, INCLUDE)
GO
CREATE TABLE b_sender_mailing_subscription
(
  MAILING_ID int not null,
  CONTACT_ID int not null,
  DATE_INSERT DATETIME NULL,
  IS_UNSUB CHAR(1) DEFAULT 'N'  NOT NULL
)
GO
CREATE UNIQUE INDEX UK_B_SENDER_MAILING_SUB on b_sender_mailing_subscription(MAILING_ID, CONTACT_ID, IS_UNSUB)
GO
CREATE TABLE b_sender_posting
(
  ID		INT		NOT NULL IDENTITY(1,1),
  DATE_UPDATE	datetime	NULL,
  MAILING_ID	INT NOT NULL,
  MAILING_CHAIN_ID	INT NOT NULL,
  STATUS		CHAR(1)		DEFAULT 'D' NOT NULL,
  DATE_SENT	DATETIME	NULL,
  DATE_CREATE	DATETIME	NULL,
  COUNT_SEND_ALL INT DEFAULT 0  NOT NULL,
  COUNT_SEND_NONE INT DEFAULT 0  NOT NULL,
  COUNT_SEND_ERROR INT DEFAULT 0  NOT NULL,
  COUNT_SEND_SUCCESS INT DEFAULT 0  NOT NULL,
  COUNT_SEND_DENY INT DEFAULT 0  NOT NULL,
  COUNT_READ INT DEFAULT 0  NOT NULL,
  COUNT_CLICK INT DEFAULT 0  NOT NULL,
  COUNT_UNSUB INT DEFAULT 0  NOT NULL
)
GO
ALTER TABLE b_sender_posting ADD CONSTRAINT PK_B_SENDER_POSTING PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_POSTING_MAILING_CHAIN on b_sender_posting(MAILING_ID, STATUS)
GO
CREATE INDEX IX_SENDER_POSTING_MAILING on b_sender_posting(MAILING_CHAIN_ID, STATUS)
GO

CREATE TABLE b_sender_posting_recipient
(
  ID INT NOT NULL IDENTITY(1,1),
  POSTING_ID INT NOT NULL,
  STATUS CHAR(1) NOT NULL,
  DATE_SENT	DATETIME	NULL,
  NAME		VARCHAR(255)	NULL,
  EMAIL		VARCHAR(255)	NULL,
  PHONE		VARCHAR(20)	NULL,
  USER_ID INT NULL,
  DATE_DENY DATETIME  NULL,
  FIELDS VARCHAR(2000) NULL,
  ROOT_ID INT NULL,
  IS_READ CHAR(1) DEFAULT 'N'  NOT NULL,
  IS_CLICK CHAR(1) DEFAULT 'N'  NOT NULL,
  IS_UNSUB CHAR(1) DEFAULT 'N'  NOT NULL
)
GO
ALTER TABLE b_sender_posting_recipient ADD CONSTRAINT PK_B_SENDER_POSTING_RECIPIENT PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_POSTING_RECIP_STATUS on b_sender_posting_recipient(STATUS, POSTING_ID)
GO
CREATE INDEX IX_SENDER_POSTING_RECIP_EMAIL on b_sender_posting_recipient(EMAIL)
GO

CREATE TABLE b_sender_posting_read
(
  ID INT NOT NULL IDENTITY(1,1),
  POSTING_ID INT NOT NULL,
  RECIPIENT_ID INT,
  DATE_INSERT	datetime	NULL
)
GO
ALTER TABLE b_sender_posting_read ADD CONSTRAINT PK_B_SENDER_POSTING_READ PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_POSTING_READ on b_sender_posting_read(POSTING_ID, RECIPIENT_ID)
GO
CREATE INDEX ix_b_sender_posting_read_recip_id ON b_sender_posting_read (RECIPIENT_ID)
GO

CREATE TABLE b_sender_posting_click
(
  ID INT NOT NULL IDENTITY(1,1),
  POSTING_ID INT NOT NULL,
  RECIPIENT_ID INT,
  DATE_INSERT	datetime	NULL,
  URL	VARCHAR(2000) 	NULL
)
GO
ALTER TABLE b_sender_posting_click ADD CONSTRAINT PK_B_SENDER_POSTING_CLICK PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_POSTING_CLICK on b_sender_posting_click(POSTING_ID, RECIPIENT_ID)
GO

CREATE TABLE b_sender_posting_unsub
(
  ID INT NOT NULL IDENTITY(1,1),
  RECIPIENT_ID INT NOT NULL,
  POSTING_ID INT NOT NULL,
  DATE_INSERT	datetime	NULL
)
GO
ALTER TABLE b_sender_posting_unsub ADD CONSTRAINT PK_B_SENDER_POSTING_UNSUB PRIMARY KEY (ID)
GO
CREATE INDEX IX_SENDER_POSTING_UNSUB on b_sender_posting_unsub(POSTING_ID, RECIPIENT_ID)
GO
CREATE TABLE B_SENDER_POSTING_LOCK
(
  ID		INT		NOT NULL,
  TIMESTAMP_X	DATETIME	NOT NULL
)
GO
ALTER TABLE B_SENDER_POSTING_LOCK ADD CONSTRAINT PK_B_SENDER_POSTING_LOCK PRIMARY KEY (ID)
GO
CREATE TABLE b_sender_preset_template
(
  ID          INT      NOT NULL IDENTITY(1,1),
  ACTIVE	CHAR(1)		DEFAULT 'Y' NOT NULL,
  NAME        VARCHAR(50) NOT NULL,
  CONTENT TEXT         NULL
);
GO
ALTER TABLE b_sender_preset_template ADD CONSTRAINT PK_B_SENDER_PRESET_TEMPLATE PRIMARY KEY (ID)
GO
CREATE TABLE b_sender_mailing_attachment
(
  CHAIN_ID int not null,
  FILE_ID int not null
)
GO
ALTER TABLE b_sender_mailing_attachment ADD CONSTRAINT PK_B_SENDER_MAILING_ATTACHMENT PRIMARY KEY (CHAIN_ID, FILE_ID)
GO
CREATE TABLE b_sender_mailing_trigger
(
  MAILING_CHAIN_ID int not null,
  IS_TYPE_START int default 1 not null,
  NAME VARCHAR(255) NULL,
  EVENT VARCHAR(255) NOT NULL,
  ENDPOINT TEXT NOT NULL
)
GO