CREATE TABLE b_sender_list
(
  ID		int(11)		NOT NULL auto_increment,
  NAME		VARCHAR(100)	NULL,
  CODE		VARCHAR(60)	NULL,
  SORT		int(11)		DEFAULT 100 NOT NULL,
  PRIMARY KEY (ID)
);
CREATE TABLE b_sender_contact
(
	ID		int(11)		NOT NULL auto_increment,
	DATE_INSERT	DATETIME	NOT NULL,
  DATE_UPDATE	DATETIME	NULL,
  NAME		VARCHAR(255)	NULL,
  EMAIL		VARCHAR(255)	NULL,
  PHONE		VARCHAR(20)	NULL,
  USER_ID		INT(11)	NULL,
	PRIMARY KEY (ID),
	UNIQUE UK_SENDER_CONTACT_EMAIL (EMAIL)
);
CREATE TABLE b_sender_contact_list
(
	CONTACT_ID	int(11) NOT NULL,
	LIST_ID	int(11) NOT NULL,
	UNIQUE UK_SENDER_CONTACT_LIST (CONTACT_ID,LIST_ID)
);
CREATE TABLE b_sender_group
(
  ID		int(11)		NOT NULL auto_increment,
  NAME		VARCHAR(100)	NULL,
  DESCRIPTION	TEXT		NULL,
  SORT		int(11)		DEFAULT 100 NOT NULL,
  ACTIVE		CHAR(1)		DEFAULT 'Y' NOT NULL,
  ADDRESS_COUNT	INT(11)		DEFAULT 0 NOT NULL,
  PRIMARY KEY (ID)
);
CREATE TABLE b_sender_group_connector
(
  GROUP_ID int(11) NOT NULL,
  NAME		VARCHAR(100)	NULL,
  ENDPOINT		VARCHAR(2000)	NULL,
  ADDRESS_COUNT	INT(11)		DEFAULT 0 NOT NULL
);
CREATE INDEX IX_SENDER_GROUP_CONNECTOR on b_sender_group_connector(GROUP_ID);

CREATE TABLE b_sender_mailing
(
	ID		INT(11)		NOT NULL auto_increment,
  DATE_INSERT	datetime	NULL,
  NAME		varchar(100)		 NULL,
  DESCRIPTION	TEXT		NULL,
  ACTIVE	CHAR(1)		DEFAULT 'Y' NOT NULL,
  SITE_ID		CHAR(2)		NOT NULL,
  SORT		int(11)		DEFAULT 100 NOT NULL,
  IS_PUBLIC CHAR(1) DEFAULT 'Y' NOT NULL,
	TRACK_CLICK		CHAR(1)		DEFAULT 'N' NOT NULL,
  TRIGGER_FIELDS TEXT NULL,
  EMAIL_FROM VARCHAR(255) NULL,
  IS_TRIGGER char(1) default 'N'  NOT NULL,
	PRIMARY KEY (ID)
);
CREATE TABLE b_sender_mailing_chain
(
  ID		INT(11)		NOT NULL auto_increment,
  MAILING_ID	int(11) NOT NULL,
  STATUS		CHAR(1)		NOT NULL,
  POSTING_ID	INT(11) NULL,
  CREATED_BY	INT(11) NULL,
  PARENT_ID int(11) NULL,
  IS_TRIGGER char(1) default 'N'  NOT NULL,
  DATE_INSERT DATETIME NULL,
  TIME_SHIFT int(11) default 0 NOT NULL,
  LAST_EXECUTED	datetime	NULL,
  AUTO_SEND_TIME	datetime	NULL,
	TITLE varchar(255) null,
	EMAIL_FROM varchar(255) not null,
	SUBJECT varchar(255),
	MESSAGE longtext,
  PRIORITY varchar(60),
  LINK_PARAMS varchar(255),
  TEMPLATE_TYPE varchar(30) NULL,
  TEMPLATE_ID varchar(60) NULL,
  REITERATE	CHAR(1)		DEFAULT 'N' NOT NULL,
  DAYS_OF_MONTH	VARCHAR(100)	NULL,
  DAYS_OF_WEEK	VARCHAR(15)	NULL,
  TIMES_OF_DAY	VARCHAR(255)	NULL,
  PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_MAILING_CHAIN_MAILING on b_sender_mailing_chain(MAILING_ID, STATUS);
CREATE INDEX IX_SENDER_MAILING_CHAIN_REITERATE on b_sender_mailing_chain(REITERATE, STATUS);

CREATE TABLE b_sender_mailing_group
(
  MAILING_ID	INT(11) NOT NULL,
  GROUP_ID	INT(11) NOT NULL,
  INCLUDE	INT(1)		DEFAULT 0 NOT NULL,
  UNIQUE UK_SENDER_MAILING_GROUP (MAILING_ID,GROUP_ID, INCLUDE)
);
CREATE TABLE b_sender_mailing_subscription
(
  MAILING_ID int(11) not null,
  CONTACT_ID INT(11) not NULL,
  DATE_INSERT	DATETIME NULL,
  IS_UNSUB CHAR(1) DEFAULT 'N'  NOT NULL,
  PRIMARY KEY (MAILING_ID, CONTACT_ID, IS_UNSUB)
);
CREATE TABLE b_sender_posting
(
  ID		INT(11)		NOT NULL auto_increment,
  DATE_UPDATE	datetime	NULL,
  MAILING_ID	int(11) NOT NULL,
  MAILING_CHAIN_ID	int(11) NOT NULL,
  STATUS		CHAR(1)		DEFAULT 'D' NOT NULL,
  DATE_SENT	DATETIME	NULL,
  DATE_CREATE	DATETIME	NULL,
  COUNT_SEND_ALL int(11) DEFAULT 0 NOT NULL,
  COUNT_SEND_NONE int(11) DEFAULT 0 NOT NULL,
  COUNT_SEND_ERROR int(11) DEFAULT 0 NOT NULL,
  COUNT_SEND_SUCCESS int(11) DEFAULT 0 NOT NULL,
  COUNT_SEND_DENY int(11) DEFAULT 0 NOT NULL,
  COUNT_READ int(11) DEFAULT 0 NOT NULL,
  COUNT_CLICK int(11) DEFAULT 0 NOT NULL,
  COUNT_UNSUB int(11) DEFAULT 0 NOT NULL,
  PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_POSTING_MAILING_CHAIN on b_sender_posting(MAILING_ID, STATUS);
CREATE INDEX IX_SENDER_POSTING_MAILING on b_sender_posting(MAILING_CHAIN_ID, STATUS);

CREATE TABLE b_sender_posting_recipient
(
	ID INT(11) NOT NULL auto_increment,
	POSTING_ID INT(11) NOT NULL,
	STATUS CHAR(1) NOT NULL,
  DATE_SENT	DATETIME	NULL,
  NAME		VARCHAR(255)	NULL,
  EMAIL		VARCHAR(255)	NULL,
  PHONE		VARCHAR(20)	NULL,
  USER_ID INT(11) NULL,
  DATE_DENY DATETIME  NULL,
  FIELDS VARCHAR(2000) NULL,
  ROOT_ID int(11) NULL,
  IS_READ   char(1) DEFAULT 'N' NOT NULL,
  IS_CLICK  char(1) DEFAULT 'N' NOT NULL,
  IS_UNSUB  char(1) DEFAULT 'N' NOT NULL,
	PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_POSTING_RECIP_STATUS on b_sender_posting_recipient(STATUS, POSTING_ID);
CREATE INDEX IX_SENDER_POSTING_RECIP_EMAIL on b_sender_posting_recipient(EMAIL);

CREATE TABLE b_sender_posting_read
(
  ID INT(11) NOT NULL auto_increment,
  POSTING_ID INT(11) NOT NULL,
  RECIPIENT_ID INT(11),
  DATE_INSERT	datetime	NULL,
  PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_POSTING_READ on b_sender_posting_read(POSTING_ID, RECIPIENT_ID);
CREATE INDEX ix_b_sender_posting_read_recip_id ON b_sender_posting_read (RECIPIENT_ID);

CREATE TABLE b_sender_posting_click
(
  ID INT(11) NOT NULL auto_increment,
  POSTING_ID INT(11) NOT NULL,
  RECIPIENT_ID INT(11),
  DATE_INSERT	datetime	NULL,
  URL	VARCHAR(2000) 	NULL,
  PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_POSTING_CLICK on b_sender_posting_click(POSTING_ID, RECIPIENT_ID);

CREATE TABLE b_sender_posting_unsub
(
  ID INT(11) NOT NULL auto_increment,
  RECIPIENT_ID int(11) NOT NULL,
  POSTING_ID INT(11) NOT NULL,
  DATE_INSERT	datetime	NULL,
  PRIMARY KEY (ID)
);
CREATE INDEX IX_SENDER_POSTING_UNSUB on b_sender_posting_unsub(POSTING_ID, RECIPIENT_ID);

CREATE TABLE b_sender_preset_template
(
  ID          INT(11)      NOT NULL AUTO_INCREMENT,
  ACTIVE	CHAR(1)		DEFAULT 'Y' NOT NULL,
  NAME        VARCHAR(50) NOT NULL,
  CONTENT TEXT         NULL,
  PRIMARY KEY (ID)
);
CREATE TABLE b_sender_mailing_attachment
(
  CHAIN_ID int(18) not null,
  FILE_ID int(18) not null,
  PRIMARY KEY (CHAIN_ID, FILE_ID)
);
CREATE TABLE b_sender_mailing_trigger
(
  MAILING_CHAIN_ID int(11) not null,
  IS_TYPE_START int(1) default 1 not null,
  NAME VARCHAR(255) NULL,
  EVENT VARCHAR(255) NOT NULL,
  ENDPOINT TEXT NOT NULL
);