<?
if (!defined("B_PROLOG_INCLUDED") || B_PROLOG_INCLUDED !== true) die();

use Bitrix\Main\Localization\Loc;
use Bitrix\Tasks\Util\UserField;
use Bitrix\Tasks\Util\Result;
use Bitrix\Tasks\Util\User;
use Bitrix\Tasks\Util\UserField\Restriction;

Loc::loadMessages(__FILE__);

CBitrixComponent::includeComponentClass("bitrix:tasks.base");

class TasksUserFieldPanelComponent extends TasksBaseComponent
{
	protected $state = null;
	protected $ctrl = null;
	protected $stateCtrl = null;

	protected function checkParameters()
	{
		static::tryParseEnumerationParameter($this->arParams['ENTITY_CODE'], array('TASK', 'TASK_TEMPLATE'), false);
		if(!$this->arParams['ENTITY_CODE'])
		{
			$this->errors->add('INVALID_PARAMETER.ENTITY_CODE', 'Unknown entity code');
		}
		static::tryParseArrayParameter($this->arParams['EXCLUDE']);

		return $this->errors->checkNoFatals();
	}

	protected function getData()
	{
		$fields = array();
		$types = array();
		$state = array();

		$this->ctrl = static::getControllerByEntity($this->arParams['ENTITY_CODE']);
		if($this->ctrl)
		{
			if(!Restriction::canUse($this->ctrl->getEntityCode()))
			{
				$this->errors->add('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_USAGE_RESTRICTED'));
			}
			else
			{
				$fields = $this->ctrl->getScheme();
				// restore uf values from task data
				foreach($fields as $ufCode => $ufDesc)
				{
					if(!empty($this->arParams['DATA']) && isset($this->arParams['DATA'][$ufCode]))
					{
						$fields[$ufCode]['VALUE'] = $this->arParams['DATA'][$ufCode];
					}

					$fields[$ufCode]['LABEL'] = (string) $ufDesc['EDIT_FORM_LABEL'] != '' ? $ufDesc['EDIT_FORM_LABEL'] : $ufDesc['FIELD_NAME_ORIG'];
				}

				$types = $this->ctrl->getTypes();

				$this->stateCtrl = static::getStateController($this->ctrl);
				if($this->stateCtrl)
				{
					$state = $this->stateCtrl->get();
					// filter state through exclude
					foreach($state as $ufCode => $ufState)
					{
						if(in_array($ufCode, $this->arParams['EXCLUDE']))
						{
							unset($state[$ufCode]);
						}
					}
				}
			}
		}

		$this->arResult['DATA']['FIELDS'] = $fields;
		$this->arResult['AUX_DATA']['FIELD_TYPE'] = $types;
		$this->arResult['DATA']['STATE'] = $state;

		$this->arResult['COMPONENT_DATA']['RESTRICTION'] = array(
			'USE' => Restriction::canUse($this->ctrl->getEntityCode(), $this->userId),
			'MANAGE' => Restriction::canManage($this->ctrl->getEntityCode(), $this->userId),
			'CREATE_MANDATORY' => Restriction::canCreateMandatory($this->ctrl->getEntityCode(), $this->userId),
		);
	}

	protected static function getControllerByEntity($entityCode)
	{
		$className = UserField::getControllerClassByEntityCode($entityCode);
		if($className)
		{
			return new $className();
		}

		return null;
	}

	protected static function getItemControllerByEntity($entityCode, $entityId = 0)
	{
		// todo: hardcoded by now, later auto-search may be implemented

		if($entityCode == 'TASK')
		{
			return new \Bitrix\Tasks\Item\Task($entityId);
		}
		elseif($entityCode == 'TASK_TEMPLATE')
		{
			return new \Bitrix\Tasks\Item\Task\Template($entityId);
		}
		else
		{
			return null;
		}
	}

	protected static function getStateController($ufController)
	{
		return new TasksUserFieldPanelComponentState($ufController);
	}

	public static function getAllowedMethods()
	{
		return array(
			'saveField',
			'deleteField',
			'setState',
			'getFieldHTML',
		);
	}

	public static function saveField($id = 0, array $data, array $parameters = array())
	{
		$result = new Result();

		$ufController = static::getControllerByEntity($data['ENTITY_CODE']);
		if(!$ufController)
		{
			$result->addError('ILLEGAL_ARGUMENT.ENTITY_CODE', 'Unknown entity code');
		}

		$type = $data['USER_TYPE_ID'];
		$label = trim((string) $data['LABEL']);

		if(!\Bitrix\Tasks\Util\User::isSuper())
		{
			$result->addError('ACTION_NOT_ALLOWED', 'Only admin can do field managing');
		}
		if($label == '')
		{
			$result->addError('ILLEGAL_ARGUMENT.LABEL', Loc::getMessage('TASKS_TUFE_EMPTY_LABEL'));
		}
		if(!Restriction::canManage($ufController->getEntityCode()))
		{
			$result->addError('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_MANAGING_RESTRICTED'));
		}
		if($data['MANDATORY'] == 'Y' && !Restriction::canCreateMandatory($ufController->getEntityCode()))
		{
			$result->addError('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_USAGE_RESTRICTED_MANDATORY'));
		}

		if($id) // update existing field
		{
			if($result->isSuccess())
			{
				$field = array(
					'MANDATORY'     => $data['MANDATORY'] ? 'Y' : 'N',
					'EDIT_FORM_LABEL' => array(
						LANGUAGE_ID => $label,
					),
				);

				$updateResult = $ufController->updateField($id, $field);
				if(!$updateResult->getErrors()->isEmpty())
				{
					$result->getErrors()->load($updateResult->getErrors());
				}
			}
		}
		else // create a new one
		{
			if(!in_array($type, array('string', 'double', 'boolean', 'datetime')))
			{
				$result->addError('ILLEGAL_ARGUMENT.USER_TYPE_ID', 'Unknown field type');
			}

			if($result->isSuccess())
			{
				$field = array(
					'USER_TYPE_ID'  => $type,
					'XML_ID'        => '',
					'MULTIPLE'      => $data['MULTIPLE'] ? 'Y' : 'N',
					'MANDATORY'     => $data['MANDATORY'] ? 'Y' : 'N',
					'SHOW_FILTER'   => 'Y',
					'SHOW_IN_LIST'  => 'Y',
					'EDIT_IN_LIST'  => 'Y',
					'IS_SEARCHABLE' => 'Y',
					'EDIT_FORM_LABEL' => array(
						LANGUAGE_ID => $label,
					),
				);

				$addResult = $ufController->addField($field);
				if(!$addResult->getErrors()->isEmpty())
				{
					$result->getErrors()->load($addResult->getErrors());
				}

				if($result->isSuccess())
				{
					$data['ID'] = $id = $addResult->getData();

					// also get html
					$fieldData = array();
					$scheme = $ufController->getScheme();
					foreach($scheme as $code => $field)
					{
						if($field['ID'] == $id)
						{
							$fieldData = $field;
						}
					}

					$inputPrefix = trim((string) $parameters['INPUT_PREFIX']);
					if($inputPrefix)
					{
						$fieldData['FIELD_NAME'] = $inputPrefix.'['.$fieldData['FIELD_NAME'].']';
					}

					$data['FIELD_HTML'] = static::getFieldUIHtml($fieldData);
				}
			}
		}

		$result->setData($data);

		return $result;
	}

	public static function deleteField($id)
	{
		$result = new Result();

		if(!intval($id))
		{
			$result->addError('ILLEGAL_ARGUMENT.ID', 'Illegal field id');
		}
		else
		{
			if(!\Bitrix\Tasks\Util\User::isSuper())
			{
				$result->addError('ACTION_NOT_ALLOWED', 'Only admin can do field managing');
			}

			$data = \CUserTypeEntity::GetByID($id);
			if(!$data)
			{
				$result->addError('FIELD_NOT_FOUND', 'Field not found');
			}
			else
			{
				$ufController = static::getControllerByEntity($data['ENTITY_ID']);
				if(!$ufController)
				{
					$result->addError('ILLEGAL_ARGUMENT.ENTITY_CODE', 'Unknown entity code');
				}
				else
				{
					if(!Restriction::canManage($ufController->getEntityCode()))
					{
						$result->addError('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_MANAGING_RESTRICTED'));
					}
				}

				if($result->isSuccess())
				{
					$deleteResult = $ufController->deleteField($id);
					if(!$deleteResult->getErrors()->isEmpty())
					{
						$result->getErrors()->load($deleteResult->getErrors());
					}
				}
			}
		}

		return $result;
	}

	public static function getFieldHTML($id, $entityCode, $entityId = 0, array $parameters = array())
	{
		$result = new Result();
		$html = '';

		$item = static::getItemControllerByEntity($entityCode, $entityId);
		$ufController = static::getControllerByEntity($entityCode);

		if(!$item || !$ufController)
		{
			$result->addError('ILLEGAL_ARGUMENT.ENTITY_CODE', 'Unknown entity code');
		}
		else
		{
			if(!Restriction::canUse($ufController->getEntityCode()))
			{
				$result->addError('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_USAGE_RESTRICTED'));
			}
		}

		if($result->isSuccess())
		{
			$scheme = $ufController->getScheme();
			$code = '';
			foreach($scheme as $ufCode => $ufField)
			{
				if($ufField['ID'] == $id)
				{
					$code = $ufCode;
					break;
				}
			}

			if($code == '')
			{
				$result->addError('ILLEGAL_ARGUMENT.ID', 'Unknown field id');
			}
			else
			{
				// todo: $item->setUserFieldController($ufController) should be used here, when implemented
				// todo: $item[$code] leads to *, UF_* in select query. Optimize it by calling getData() in un-greedy mode (when implemented)
				$value = $item[$code]; // we need to use item, because we have to check rights

				$fieldData = $scheme[$code];
				$fieldData['VALUE'] = $value;
				$inputPrefix = trim((string) $parameters['INPUT_PREFIX']);
				if($inputPrefix)
				{
					$fieldData['FIELD_NAME'] = $inputPrefix.'['.$fieldData['FIELD_NAME'].']';
				}

				$html = static::getFieldUIHtml($fieldData, !$entityId);
			}
		}

		$result->setData($html);
		return $result;
	}

	public static function setState(array $state, $entityCode, $dropAll = false)
	{
		$result = new Result();

		$ufController = static::getControllerByEntity($entityCode);
		if(!$ufController)
		{
			$result->addError('ILLEGAL_ARGUMENT.ENTITY_CODE', 'Unknown entity code');
		}
		else
		{
			if(!Restriction::canUse($ufController->getEntityCode()))
			{
				$result->addError('ACTION_RESTRICTED', Loc::getMessage('TASKS_TUFE_UF_USAGE_RESTRICTED'));
			}
			else
			{
				if($dropAll && !\Bitrix\Tasks\Util\User::isSuper())
				{
					$result->addError('ACTION_NOT_ALLOWED', 'Easier...');
				}
				else
				{
					$ctrl = static::getStateController($ufController);
					if($dropAll)
					{
						$ctrl->removeForAllUsers();
					}
					$ctrl->set($state);
				}
			}
		}

		return $result;
	}

	private static function getFieldUIHtml($fieldData, $preferDefault = false)
	{
		if(empty($fieldData))
		{
			return '';
		}

		ob_start();
		\Bitrix\Tasks\Util\UserField\UI::showEdit($fieldData, array(
			'PREFER_DEFAULT' => $preferDefault
		));
		return ob_get_clean();
	}
}

if(CModule::IncludeModule('tasks'))
{
	final class TasksUserFieldPanelComponentState extends Bitrix\Tasks\Util\Type\ArrayOption
	{
		protected $controller = null;

		public function __construct($controller)
		{
			$this->controller = $controller;
		}

		protected static function getFilterOptionName()
		{
			return 'tasks_component_ufp_state';
		}

		public function check($value)
		{
			$value = parent::check($value);

			// resort by S, for sure
			uasort($value, function($a, $b){

				$sA = intval($a['S']);
				$sB = intval($b['S']);

				return $sA > $sB ? 1 : ($sA == $sB ? 0 : -1);
			});

			// now reset sort indexes, to avoid holes
			$i = 0;
			foreach($value as $k => $v)
			{
				$value[$k]['S'] = ++$i;
			}

			return $value;
		}

		protected function getRules()
		{
			$ufs = $this->controller->getScheme();

			$rules = array();
			foreach($ufs as $k => $v)
			{
				if(UserField\UI::isSuitable($v))
				{
					$rules[$v['ID']] = array('VALUE' => array(
						'D' => array('VALUE' => 'boolean', 'DEFAULT' => true),
						'S' => array('VALUE' => 'integer', 'DEFAULT' => 0)
					), 'DEFAULT' => array());
				}
			}

			return $rules;
		}

		protected function fetchOptionValue()
		{
			$value = User::getOption(static::getFilterOptionName());
			if(empty($value))
			{
				$value = User::getOption(static::getFilterOptionName(), User::getAdminId());
			}

			return $value;
		}
	}
}