BX.namespace('BX.Tasks.Integration');

/**
 * This adapter provides suitable way of interaction with less-intuitive BX.SocNetLogDestination
 */

(function(){

	BX.Tasks.Integration.Socialnetwork = {};

	var dataCache = false;
	var dataFetchingInProgress = false;
	var popupOpenedId = false;

	BX.Tasks.Integration.Socialnetwork.NetworkSelector = BX.Tasks.Util.Widget.extend({
		sys: {
			code: 'network-selector'
		},
		options: {
			mode: 'user', // could be also "group" and "all" (the last means users, groups and departments selection)
            query: false,
			useSearch: false,
			useAdd: false
		},
		methods: {
			construct: function()
			{
				this.callConstruct(BX.Tasks.Util.Widget);

				if(!("SocNetLogDestination" in BX))
				{
					throw new ReferenceError('No BX.SocNetLogDestination detected. Forgot to include socialnetwork module and/or its assets?');
				}

				this.vars.snldId = false;
				this.vars.intendSearch = '';
				this.vars.last = {};
				this.vars.intendOpen = false;
				this.vars.changed = false;
			},

			initialize: function()
			{
				if(this.dialogInitialized())
				{
					this.fireInitEvent();
				}
				else
				{
					if(dataCache === false)
					{
						// no data loaded previously
						this.fetchDestinationData();
					}
					else
					{
						this.initializeDialog();
					}
				}
			},

			dialogInitialized: function()
			{
				return this.vars.snldId !== false;
			},

			open: function()
			{
				if(!this.dialogInitialized())
				{
					this.vars.intendOpen = true;
					this.initialize();
				}
				else if(this.vars.snldId != popupOpenedId)
				{
					this.vars.intendOpen = false;

					if(popupOpenedId != false)
					{
						BX.SocNetLogDestination.openDialog(this.vars.snldId); // close
					}
					BX.SocNetLogDestination.openDialog(this.vars.snldId); // re-open
					popupOpenedId = this.vars.snldId;
				}
			},

			close: function()
			{
				this.vars.intendOpen = false;

				if(this.vars.snldId == popupOpenedId)
				{
					BX.SocNetLogDestination.closeDialog();
				}
			},

			addLast: function(entity)
			{
				this.vars.changed = true;
				this.vars.last[entity.id] = {
					id: entity.entityId,
					type: this.getEntityType(entity.id)
				};
			},

			deleteLast: function(entity)
			{
				this.vars.changed = true;
				delete(this.vars.last[entity.id]);
			},

			updateLast: function()
			{
				if(!this.vars.changed)
				{
					return;
				}

				var items = this.vars.last;
				this.vars.last = {};

				var result = {};
				var i = 0;
				for(var k in items)
				{
					if(items.hasOwnProperty(k))
					{
						if(typeof result[items[k].type] == 'undefined')
						{
							result[items[k].type] = [];
						}

						result[items[k].type].push(items[k].id);
						i++;
					}
				}

				if(i > 0)
				{
					// send with delay, using query
                    this.getQuery().add('integration.socialnetwork.setdestinationlast', {items: result});
				}

				this.vars.changed = false;
			},

			fetchDestinationData: function()
			{
				if(!dataFetchingInProgress)
				{
					dataFetchingInProgress = true;
                    // send immediately, regardless the delay
                    this.getQuery().add('integration.socialnetwork.getdestinationdata', [], {code: 'get_destination_data'}).execute();
				}
			},

			getQuery: function()
			{
				if(typeof this.instances.query == 'undefined')
				{
                    if(this.option('query'))
                    {
                        this.instances.query = this.option('query');
                    }
                    else
                    {
                        this.instances.query = new BX.Tasks.Util.Query({
                            autoExec: true
                        });
                    }
					this.instances.query.bindEvent('executed', BX.delegate(this.onQueryExecuted, this));
				}

				return this.instances.query;
			},

			onQueryExecuted: function(result)
			{
				dataFetchingInProgress = false;

				if(result.success)
				{
					if(typeof result.data != 'undefined' && typeof result.data['get_destination_data'] != 'undefined')
					{
						if(result.data['get_destination_data'].SUCCESS)
						{
							dataCache = result.data['get_destination_data'].RESULT;
							this.initializeDialog();
						}
					}
				}
			},

			onSelectDestination: function(entity)
			{
				this.addLast(entity);

				entity.params = entity.params || {};

                var type = {
	                extranet: entity.isExtranet == 'Y',
	                email: entity.isEmail == 'Y'
                };

				var id = parseInt(entity.entityId);

				this.fireEvent('item-selected', [{
					id: isNaN(id) ? 0 : id,
					entityType: this.getEntityType(entity.id),
					nameFormatted: entity.name || '',
                    description: entity.desc || '',
                    avatar: entity.avatar || '',
					name: entity.params.name || '',
					lastName: entity.params.lastName || '',
					email: entity.email || '',
					type: type
				}]);
			},

			getEntityType: function(id)
			{
				var type = '?';

				if(!id)
				{
					return type;
				}

				var found = id.toString().trim().match(/^[a-z]+/i);
				if(found && found[0])
				{
					type = found[0];
				}

				return type;
			},

			onUnSelectDestination: function(entity)
			{
				this.deleteLast(entity);

				this.fireEvent('item-deselected', [{
					id: entity.entityId,
					entityType: this.getEntityType(entity.id),
					name: entity.name
				}]);
			},

			onOpenDialogDestination: function(id)
			{
				popupOpenedId = id;
			},

			onCloseDialogDestination: function(id)
			{
                if(id == popupOpenedId) // last opened dialog was ours
                {
                    this.fireEvent('close');
                    this.updateLast();
                }

				popupOpenedId = false;
			},

			onOpenSearchDestination: function(id)
			{
				popupOpenedId = id;
			},

			onCloseSearchDestination: function(id)
			{
				if(id == popupOpenedId) // last opened dialog was ours
				{
					this.fireEvent('close');
					this.updateLast();
				}

				popupOpenedId = false;
			},

			onOpenEmailDestination: function(id)
			{
				popupOpenedId = id;
			},

			onCloseEmailDestination: function(id)
			{
				if(id == popupOpenedId) // last opened dialog was ours
				{
					this.fireEvent('close');
					this.updateLast();
				}

				popupOpenedId = false;
			},

			checkIsOpened: function()
			{
				return popupOpenedId == this.vars.snldId;
			},

			deselectItem: function(id)
			{
				if(this.vars.snldId != false)
				{
					var u = this.option('mode') == 'user';

					// access to a low-level function, may fall here one day
					BX.SocNetLogDestination.deleteItem((u ? 'U' : 'SG')+id, u ? 'users' : 'groups', this.vars.snldId);
				}
			},

			selectItem: function()
			{
				if(this.vars.snldId != false)
				{
					var u = this.option('mode') == 'user';

					// access to a low-level function, may fall here one day
					BX.SocNetLogDestination.selectItem(this.vars.snldId, null, null, (u ? 'U' : 'SG')+id, u ? 'users' : 'groups');// = function(name, element, template, itemId, type, search)
				}
			},

			initializeDialog: function()
			{
				if(this.vars.snldId == false)
				{
					this.vars.snldId = BX.util.hashCode(Math.random().toString());
					var scope = this.scope();
					var inputName = 'name-'+this.id();
					var input = this.control('search');

					if(input)
					{
						BX.adjust(input, {
							attrs: {input: inputName, id: inputName}
						});
					}

					var modeAll = this.option('mode') == 'all';
					var modeUser = this.option('mode') == 'user';
					var modeGroup = this.option('mode') == 'group';

					var parameters = {
						name : this.vars.snldId,
						searchInput : input || null,
						bindMainPopup : { 'node' : scope, 'offsetTop' : '0px', 'offsetLeft': '0px'},
						bindSearchPopup : { 'node' : scope, 'offsetTop' : '0px', 'offsetLeft': '0px'},

						sendAjaxSearch: modeUser || modeAll, // as we pre-load groups, ajax is required only in "user" mode
						departmentSelectDisable: !modeAll,

						// set if we can add new entities in the selector
						allowAddUser: this.option('useAdd'),
						allowAddSocNetGroup: false,

						callback : {
							select : BX.proxy(this.onSelectDestination, this),
							unSelect : BX.proxy(this.onUnSelectDestination, this),
							openDialog : BX.proxy(this.onOpenDialogDestination, this),
							closeDialog : BX.proxy(this.onCloseDialogDestination, this),
							openSearch : BX.proxy(this.onOpenSearchDestination, this),
							closeSearch : BX.proxy(this.onCloseSearchDestination, this),
							openEmailAdd: BX.proxy(this.onOpenEmailDestination, this),
							closeEmailAdd: BX.proxy(this.onCloseEmailDestination, this)
						}
					};

					if (this.option('useSearch'))
					{
						parameters.showSearchInput = true;
					}

					parameters.items = {
						users: modeUser || modeAll ? (dataCache.USERS || {}) : {},
						groups: {}, // always empty
						department: modeUser || modeAll ? (dataCache.DEPARTMENT || {}) : {},
						departmentRelation: modeUser || modeAll ? (dataCache.DEPARTMENT_RELATION || {}) : {},
						sonetgroups: modeGroup || modeAll ? (dataCache.SONETGROUPS || {}) : {}
					};
					parameters.itemsLast = {
						users: modeUser || modeAll ? (dataCache.LAST.USERS || {}) : {},
						groups: {}, // always empty
						department: modeAll ? dataCache.LAST.DEPARTMENT : {},
						sonetgroups: modeGroup || modeAll ? (dataCache.LAST.SONETGROUPS || {}) : {}
					};
					parameters.itemsSelected = dataCache.SELECTED || {};

					BX.SocNetLogDestination.init(parameters);

					if (input)
					{
						var params = {
							formName: this.vars.snldId,
							inputName: 'name-'+this.id(),
							sendAjax: this.option('mode') == 'user'
						};

						BX.bind(input, "keyup", BX.proxy(BX.SocNetLogDestination.BXfpSearch, params));
						BX.bind(input, "keydown", BX.proxy(BX.SocNetLogDestination.BXfpSearchBefore, params));
						BX.bind(input, "click", BX.delegate(this.open, this)); //re-open when occasionly closed
					}
				}

				this.fireInitEvent();
				if(this.vars.intendOpen)
				{
					this.open();
				}
			},

			fireInitEvent: function()
			{
				this.fireEvent('initialized');
			},

			clearDataCache: function()
			{
				dataCache = false;
			}
		}
	});

})();
