BX.namespace('Tasks.Util');

/**
 * This class is a prototype for the future-planned "view" class
 */

(function(){

	BX.Tasks.Util.ItemSet = BX.Tasks.Util.Widget.extend({
		options: {
			min: 			    false,
			max: 			    false,
            preRendered:        false,
            autoSync:           false,
            itemFx:             'none', // also 'vertical' and 'horizontal',
			itemAppearFxSpeed:      200,
			itemDisappearFxSpeed:   200,
            itemFxHoverDelete:  false
		},
		sys: {
			code: 'item-set'
		},
		methods: {
			construct: function()
			{
				this.callConstruct(BX.Tasks.Util.Widget);

				BX.mergeEx(this.vars, {
					items: {},
					order: [],
					checkRestrictions: true,
                    syncLock: true,
                    idOffset: 1,
					readonly: false
				});

				this.bindEvents();
                this.load(this.option('data'), {
                    dontRender: this.option('preRendered'),
                    loadInitial: true
                });

				this.checkConstraints();
			},

			checkConstraints: function()
			{
				var min = parseInt(this.option('min'));
				var max = parseInt(this.option('max'));

				if(isNaN(min))
				{
					min = 0;
				}
				if(isNaN(max))
				{
					max = Number.POSITIVE_INFINITY;
				}

				if(min > max)
				{
					throw new TypeError('Min constraint cannot be greater than max. Check options.');
				}

				this.vars.constraint = {min: min, max: max};
			},

			bindEvents: function()
			{
                this.bindItemActions();
                this.bindFx();
				this.bindDelegateControl('open-form', 'click', this.passCtx(this.openAddForm));

				// for each new instance it should be a different function
				this.fireChangeDeferredEvent = BX.debounce(this.fireChangeDeferredEvent, 5, this);
			},

            bindItemActions: function()
            {
                // standard item actions, like delete, etc...
                this.bindDelegateControl('item-delete', 'click', this.passCtx(this.onItemDeleteClicked));
            },

            bindFx: function()
            {
                if(this.option('itemFxHoverDelete'))
                {
                    this.bindDelegateControl('item-delete', 'mouseover', this.passCtx(this.onItemDeleteOver));
                    this.bindDelegateControl('item-delete', 'mouseout', this.passCtx(this.onItemDeleteOut));
                }
            },

            getItemClass: function()
            {
                return BX.Tasks.Util.ItemSet.Item;
            },

			createItem: function(data, parameters)
			{
                var cl = this.getItemClass();
                var itemScope = false;

				parameters = parameters || {};

                if(parameters.dontRender)
                {
                    itemScope = 'item-'+data.VALUE; // it will be searched later
                }
                else
                {
                    // generate scope here...
                    itemScope = this.getNodeByTemplate('item', data)[0];
                }

				return new cl({
                    scope: itemScope,
                    data: data,
					controlBind: this.option('controlBind'),
                    parent: this
                });
			},

			openAddForm: function()
			{
				// open some form or just add a new item
			},

			addItem: function(data, parameters)
			{
                parameters = parameters || {};
				if(typeof parameters.itemFx == 'undefined')
				{
					parameters.itemFx = true; // do animation by default
				}

				// backward compatibility, prepare VALUE
                if('extractItemValue' in this)
                {
                    data.VALUE = this.extractItemValue(data);
                }
				// backward compatibility, prepare DISPLAY
				if('extractItemDisplay' in this)
				{
					data.DISPLAY = this.extractItemDisplay(data);
				}

                if('prepareData' in this)
                {
                    data = this.prepareData(data);
                }
                else
                {
                    var cl = this.getItemClass();
                    if('prepareData' in cl)
                    {
                        data = cl.prepareData.apply(this, [data]);
                    }
                }

				if(this.has(data.VALUE) || parameters.checkRestrictions !== false && !this.checkCanAddItems())
				{
					return false;
				}

				if(typeof data.VALUE == 'undefined' || typeof data.DISPLAY == 'undefined')
				{
					return false;
				}

                if(this.option('itemFx') != 'none')
                {
                    data.ITEM_SET_INVISIBLE = parameters.load ? '' : 'invisible';
                }

				var itemInst = this.createItem(data, parameters);
				if(itemInst == null)
				{
					return false;
				}

				return this.registerItem(itemInst, parameters);
			},

			registerItem: function(itemInst, parameters)
			{
				parameters = parameters || {};

				if(BX.type.isElementNode(itemInst.option('scope')))
				{
					// todo: when items are massively added, buffierize this and then append for all items at once
					BX.append(itemInst.scope(), this.control('items'));
				}

				// backward compatibility
				itemInst.bindEvent('delete', BX.delegate(this.deleteItem, this));

				this.vars.items[itemInst.value()] = itemInst;
				this.vars.order.push(itemInst.value());

				this.processItemAfterCreate(itemInst.value(), parameters);

				// todo: call appear() anyway, use promise
				if('appear' in itemInst && parameters.useAppear !== false) // commented out, need to have "instant" fx to set opacity to 1 instantly
				{
					itemInst.appear(parameters);
				}

				if(!parameters.load) // .fireChangeEvent() will be called inside .load() instead
				{
					this.fireChangeEvent(parameters);
				}

				return true;
			},

			hasItem: function(value)
			{
				return typeof this.vars.items[value] != 'undefined';
			},

			getItem: function(value)
			{
				var item = this.vars.items[value];
				if(typeof item == 'undefined')
				{
					return null;
				}

				return item;
			},

			getItemFirst: function()
			{
				if(this.vars.order.length == 0)
				{
					return null;
				}

				var value = this.vars.order[0];
				return this.getItem(value);
			},

			getItemLast: function()
			{
				if(this.vars.order.length == 0)
				{
					return null;
				}

				var value = this.vars.order[this.vars.order.length - 1];
				return this.getItem(value);
			},

            itemCount: function()
            {
                return this.vars.order.length;
            },

            onItemDestroy: function()
            {
            },

			deleteItem: function(value, parameters)
			{
                parameters = parameters || {};
				if(typeof parameters.itemFx == 'undefined')
				{
					parameters.itemFx = true; // do animation by default
				}

				if(parameters.checkRestrictions !== false && !this.checkCanDeleteItems())
				{
					return false;
				}

                var itemInst = false;
                if(typeof value == 'object')
                {
                    itemInst = value;
                    value = itemInst.value();
                }
                else
                {
                    itemInst = this.vars.items[value];
                }

				if(typeof itemInst != 'undefined')
				{
                    // remove item from item set

					this.vars.items[value] = null;
					delete(this.vars.items[value]);

					for(var k in this.vars.order)
					{
						if(this.vars.order[k] == value)
						{
							this.vars.order.splice(k, 1);
							break;
						}
					}

                    // remove item itself

                    var ctx = this;

                    var itemDestroyer = function(){
                        var scope = this.scope();
                        var value = this.value();
                        this.destroy();
                        BX.remove(scope);

                        ctx.onItemDestroy(value);
                    };

					// todo: call disappear() anyway (even if does nothing), use promise
                    if('disappear' in itemInst && parameters.itemFx)
                    {
                        itemInst.disappear(itemDestroyer, parameters);
                    }
                    else
                    {
                        itemDestroyer.call(itemInst);
                    }

					//todo: if(parameters.unload)
					this.fireChangeEvent(parameters);
					this.fireEvent('item-delete', [value]);

					return true;
				}

				return false;
			},

			replaceItem: function(value, data, parameters)
			{
				var replaceParameters = {itemFx: false};
				BX.merge(replaceParameters, parameters || {});

				replaceParameters.checkRestrictions = false;

				if(this.deleteItem(value, replaceParameters))
				{
					this.addItem(data, replaceParameters);
				}
			},

			updateItem: function(value, data)
			{
			},

			load: function(data, parameters)
			{
				if(BX.type.isPlainObject(data) || BX.type.isArray(data))
				{
					data = BX.clone(data);
                    parameters = BX.clone(parameters) || {};
                    parameters.load = true;

                    this.vars.syncLock = true;

					var i = 0;
					for(var k in data)
					{
						if(BX.type.isPlainObject(data[k]))
						{
							if(this.addItem(data[k], parameters))
							{
								i++;
							}
						}
					}

					if(i > 0)
					{
						this.fireChangeEvent(parameters);
					}

                    this.vars.syncLock = false;
				}
			},

            processItemAfterCreate: function()
            {
            },

			// do not use this every time, it repaints the whole item set which is generally not necessary
			redraw: function()
			{
				var pool = BX.create('div');
				for(var k in this.vars.order)
				{
					var id = this.vars.order[k];

					BX.append(this.vars.items[id].scope(), pool);
				}

				this.moveNodePool(pool, this.control('items'));

				this.updateInstant();
			},

			// utils
			checkCanAddItems: function()
			{
				var max = this.option('max');

				if(max === false)
				{
					return true;
				}

				return this.vars.order.length < parseInt(max);
			},

			checkCanDeleteItems: function()
			{
				var min = this.option('min');

				if(this.vars.order.length == 0 || (min !== false && this.vars.order.length <= parseInt(min)))
				{
					return false;
				}

				return true;
			},

			updateDelayed: function()
			{
			},

			updateInstant: function(items)
			{
				this.setCSSFlagEmpty();
				this.setCSSFlagLimits();
			},

			setCSSFlagLimits: function()
			{
				var canAdd = this.checkCanAddItems();
				var canDelete = this.checkCanDeleteItems();

				this.changeCSSFlag('t-min', !canDelete);
				this.changeCSSFlag('t-max', !canAdd);

				// todo: left for compatibility, need to be removed
				this.changeCSSFlag(this.getFullBxId('min'), !canDelete);
				this.changeCSSFlag(this.getFullBxId('max'), !canAdd);
			},

			setCSSFlagEmpty: function()
			{
				var filled = this.vars.order.length > 0;

				this.changeCSSFlag('t-filled', filled);
				this.changeCSSFlag('t-empty', !filled);

				// todo: left for compatibility, need to be removed
				var className = this.getFullBxId('empty');
				this.dropCSSFlags(className+'-*');
				this.setCSSFlag(className+'-'+(filled ? 'false' : 'true'));
			},

			moveNodePool: function(from, to)
			{
				while(from.childNodes.length > 0)
				{
					BX.append(from.childNodes[0], to);
				}
			},

			fireChangeEvent: function(parameters)
			{
				this.fireEvent('change', [this.vars.order, parameters]);
				this.fireChangeDeferredEvent(parameters);

				this.updateInstant();
			},

			fireChangeDeferredEvent: function(parameters)
			{
				this.fireEvent('change-deferred', [this.vars.order, parameters]);
			},

			isEnter: function(e)
			{
				e = e || window.event;

				return e.keyCode == 13;
			},

			doOnItem: function(node, callback, callbackNew)
			{
				if(this.vars.readonly)
				{
					return false;
				}

                // find item value
				var itemValue = BX.data(node, 'item-value');
                if(!itemValue) // try to find item scope
                {
                    var scope = this.controlP('item', node, this.scope());
                    if(scope)
                    {
                        itemValue = BX.data(scope, 'item-value');
                    }
                }

				var args = Array.prototype.slice.call(arguments);
				args.shift();
				args.shift();
				args.shift();

				if(typeof itemValue != 'undefined' && itemValue !== null)
				{
					if(itemValue == 'new')
					{
						if(BX.type.isFunction(callbackNew))
						{
							callbackNew.apply(this, args);
						}
					}
					else
					{
						BX.data(node, 'item-value', itemValue);

						var item = this.vars.items[itemValue];
						if(typeof item != 'undefined') // item may be in process of destruction
						{
							args.unshift(this.vars.items[itemValue]);

							callback.apply(this, args);
						}
					}

					return itemValue;
				}

                return false;
			},

			bindOnItem: function(cb, cbNew)
			{
				var this_ = this;
				return function()
				{
					var args = Array.prototype.slice.call(arguments);
					args.unshift(cbNew);
					args.unshift(cb);
					args.unshift(this); // this is a ctx of the node event happened on
					return this_.doOnItem.apply(this_, args);
				}
			},

            onItemDeleteClicked: function(node)
            {
                this.doOnItem(node, this.deleteItem);
            },

            onItemDeleteOver: function(node)
            {
                this.doOnItem(node, function(item){
                    BX.addClass(item.scope(), 'hover-delete');
                });
            },

            onItemDeleteOut: function(node)
            {
                this.doOnItem(node, function(item){
                    BX.removeClass(item.scope(), 'hover-delete');
                });
            },

            getQuery: function()
            {
                if(!this.instances.query)
                {
                    this.instances.query = new BX.Tasks.Util.Query({
                        autoExec: true
                    });
                }

                return this.instances.query;
            },

            checkCanSync: function()
            {
                return this.option('autoSync') && !this.vars.syncLock;
            },

            syncAllIfCan: function()
            {
                if(this.checkCanSync())
                {
                    this.syncAll();
                }
            },

            syncAll: function(items)
            {
                //var q = this.getQuery();
                // do smth
            },

            extractItemValue: function(data)
            {
                return data.ID;
            },

			readonly: function(way)
			{
				if(typeof way == 'undefined')
				{
					return this.vars.readonly;
				}
				else
				{
					way = !!way;

					this.changeCSSFlag('readonly', way);
					this.vars.readonly = way;
				}
			},

			value: function()
			{
				return this.vars.order;
			},

			// iterator functions

			first: function()
			{
				return this.getItemFirst();
			},

			last: function()
			{
				return this.getItemLast();
			},

			has: function(value)
			{
				return this.hasItem(value);
			},

			get: function(value)
			{
				return this.getItem(value);
			},

			each: function(cb)
			{
				if(!BX.type.isFunction(cb))
				{
					return;
				}

				for(var k = 0; k < this.vars.order.length; k++)
				{
					if(cb.apply(this, [this.vars.items[this.vars.order[k]]]) === false)
					{
						break;
					}
				}
			},

			count: function()
			{
				return this.vars.order.length;
			},

			/**
			 * Returns a collection of item data objects
			 * @param clone
			 * @returns {BX.Tasks.Util.Collection|*}
			 */
			exportItemData: function(clone)
			{
				var data = [];

				this.each(function(item){
					var iData = item.data();
					data.push(clone ? BX.clone(iData) : iData);
				});

				var c = new BX.Tasks.Util.Collection({keyField: 'VALUE'});
				c.load(data);

				return c;
			},

			/**
			 * Returns a collection of item instances
			 */
			export: function()
			{
				// todo
			}
		}
	});

    /////////////////////
    // predefined items
    /////////////////////

    // standard item
    BX.Tasks.Util.ItemSet.Item = BX.Tasks.Util.Widget.extend({
        sys: {
            code: 'item-set-item'
        },
        methods: {
            value: function(value)
            {
	            if(typeof value != 'undefined')
	            {
		            this.option('data').VALUE = value;
		            BX.data(this.scope(), 'item-value', value);
		            this.addId(value);
	            }
	            else
	            {
		            return this.option('data').VALUE;
	            }
            },
            display: function()
            {
                return this.option('data').DISPLAY;
            },
            data: function(data)
            {
	            if(BX.type.isPlainObject(data))
	            {
		            this.vars.data = data;
	            }
	            else
	            {
		            if(typeof this.vars.data != 'undefined')
		            {
			            return this.vars.data;
		            }

		            return this.option('data');
	            }
            },

            // use this to show item gracefully
            appear: function(parameters)
            {
	            parameters = parameters || {};

                if(!parameters.dontRender && !parameters.load) // no fx on pre-rendered and .load()-ed blocks
                {
	                var fx = this.optionP('itemFx');

                    if(fx != 'none')
                    {
	                    var fn = 'fadeToggleByClass';
	                    if(fx == 'horizontal' || fx == 'vertical')
	                    {
							fn = fx == 'horizontal' ? 'fadeSlideHToggleByClass' : 'fadeSlideToggleByClass';
	                    }

                        return BX.Tasks.Util[fn](this.scope(), this.optionP('itemAppearFxSpeed'));
                    }
                }
            },
            // use this to hide item gracefully
            disappear: function(complete, parameters)
            {
                var fx = this.optionP('itemFx');

                if(fx != 'horizontal' && fx != 'vertical')
                {
                    complete.apply(this);
                }
                else
                {
                    return BX.Tasks.Util[fx == 'horizontal' ? 'fadeSlideHToggleByClass' : 'fadeSlideToggleByClass'](this.scope(), this.optionP('itemDisappearFxSpeed'), BX.delegate(complete, this));
                }
            },

	        isShown: function()
	        {
		        return !BX.hasClass(this.scope(), 'invisible');
	        }
        }
    });
    // the following function must define VALUE and DISPLAY inside data structure, basing on data itself
    BX.Tasks.Util.ItemSet.Item.prepareData = function(data)
    {
        return data;
    };

})();

/**
 Usage memo

 <div id="widget-scope">
     <div data-bx-id="item-set-items">
        <script type="text/html" data-bx-id="item-set-item">
            <div data-bx-id="item-set-item" data-item-value="{{VALUE}}">
                Item named "{{DISPLAY}}" with value={{VALUE}}
                <div data-bx-id="item-set-item-delete">Delete</div>
            </div>
        </script>
     </div>
     <div data-bx-id="item-set-open-form">Change or add</div>
 </div>

 If you got items pre-rendered on server side, you may add "item-set-item-{{VALUE}}" to "data-bx-id" attribute, and
 then pass preRendered:true in options
 */