<?
/**
 * Bitrix Framework
 * @package bitrix
 * @subpackage tasks
 * @copyright 2001-2016 Bitrix
 *
 * @access private
 */

namespace Bitrix\Tasks\Util\Type;

final class StructureChecker
{
	private $standard = array();

	public function __construct(array $standard)
	{
		$this->standard = $standard;
	}

	public function check($structure, $initial = false)
	{
		return $this->walk($structure, $this->standard, array('initial' => $initial));
	}

	private function walk($structure, array $standard, array $params = array(), $depth = 1)
	{
		if($depth >= 10) // protection
		{
			return false;
		}

		if(!is_array($structure))
		{
			$structure = array();
		}

		// remove illegal keys
		$structure = array_intersect_key($structure, $standard);

		foreach($standard as $k => $value)
		{
			if(!array_key_exists($k, $structure))
			{
				if(isset($value['INITIAL']) && $params['initial']) // pre-create initial
				{
					$structure[$k] = $this->evaluateValue($value['INITIAL']);
				}
				elseif(isset($value['DEFAULT'])) // pre-create absent fields with default values
				{
					$structure[$k] = $this->evaluateValue($value['DEFAULT']);
				}
			}
		}

		// check value of existed legal keys
		foreach($structure as $k => $value)
		{
			$sValue = $standard[$k]['VALUE'];

			if(is_array($sValue))
			{
				$value = $this->walk($value, $sValue, $params, $depth + 1);
			}
			elseif($sValue == 'integer')
			{
				$value = intval($value);
			}
			elseif($sValue == 'boolean')
			{
				$value = !!$value;
			}
			elseif($sValue == 'unique integer[]') // array of unique integer
			{
				if(!is_array($value))
				{
					$value = array();
				}
				$value = array_unique(array_filter($value, 'intval'));
			}
			elseif(is_callable($sValue))
			{
				$value = call_user_func_array($sValue, array($value));
			}
			else
			{
				unset($structure[$k]); // we can not let unchecked alive
				continue;
			}

			$structure[$k] = $value;
		}

		return $structure;
	}

	protected function evaluateValue($value)
	{
		if(is_callable($value))
		{
			return call_user_func_array($value, array());
		}

		return $value;
	}
}