<?
IncludeModuleLangFile(__FILE__);

use Bitrix\Voximplant as VI;

class CVoxImplantTransfer
{
	const TYPE_PHONE = 'phone';
	const TYPE_USER = 'user';
	const DEVICE_WEBRTC = 'WEBRTC';
	const DEVICE_PHONE = 'PHONE';
	const DEVICE_EXTERNAL = 'EXTERNAL';

	public static function Invite($callId, $transferType, $transferUserId = 0, $transferPhone = '')
	{
		CVoxImplantHistory::WriteToLog(print_r(func_get_args(), true), 'CVoxImplantTransfer::Invite');

		if($transferType !== self::TYPE_USER && $transferType !== self::TYPE_PHONE)
			throw new \Bitrix\Main\ArgumentException('unsupported parameter value', 'transferType');

		$transferUserId = intval($transferUserId);
		$transferPhone = CVoxImplantPhone::Normalize($transferPhone);

		if($transferType == self::TYPE_PHONE && $transferPhone == '')
			throw new \Bitrix\Main\ArgumentException('transferPhone is empty', 'transferPhone');

		if($transferType == self::TYPE_USER && $transferUserId == 0)
			throw new \Bitrix\Main\ArgumentException('transferUserId is empty', 'transferUserId');

		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		if ($call['TRANSFER_TYPE'] != '')
			self::Cancel($callId);


		$call['TRANSFER_USER_ID'] = $transferUserId;

		VI\CallTable::update(
			$call['ID'],
			array(
				'TRANSFER_TYPE' => $transferType,
				'TRANSFER_USER_ID' => $transferUserId,
				'TRANSFER_PHONE' => $transferPhone
			)
		);

		$call['USER_HAVE_PHONE'] = 'N';

		if($transferType === self::TYPE_USER)
		{
			$res = CVoxImplantUser::GetList(array(
				'select' => Array('ID', 'IS_ONLINE_CUSTOM', 'UF_VI_PHONE', 'ACTIVE'),
				'filter' => Array('=ID' => $call['TRANSFER_USER_ID'], '=ACTIVE' => 'Y'),
			));
			if ($userData = $res->fetch())
			{
				$call['USER_HAVE_PHONE'] = $userData['UF_VI_PHONE'];
			}

			$crmData = Array();
			if ($call['CRM'] == 'Y')
				$crmData = CVoxImplantCrmHelper::GetDataForPopup($call['CALL_ID'], $call['CALLER_ID'], $transferUserId);

			self::SendPullEvent(Array(
				'COMMAND' => 'inviteTransfer',
				'USER_ID' => $transferUserId,
				'CALL_ID' => $call['CALL_ID'],
				'CALLER_ID' => $call['CALLER_ID'],
				'CRM' => $crmData,
			));
		}
		else if($transferType === self::TYPE_PHONE)
		{
			$outgoingConfig = CVoxImplantOutgoing::GetConfigByUserId($call['USER_ID']);
		}

		$command['COMMAND'] = 'inviteTransfer';
		$command['OPERATOR_ID'] = $call['USER_ID'];
		$command['TRANSFER_TYPE'] = $transferType;
		$command['TRANSFER_USER_ID'] = $call['TRANSFER_USER_ID'];
		$command['TRANSFER_PHONE'] = $transferPhone;
		$command['USER_HAVE_PHONE'] = $call['USER_HAVE_PHONE'];
		$command['CONFIG'] = $outgoingConfig;

		$http = new \Bitrix\Main\Web\HttpClient();
		$http->waitResponse(false);
		$http->post($call['ACCESS_URL'], json_encode($command));

		return true;
	}

	public static function Cancel($callId)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		VI\CallTable::update(
			$call['ID'],
			array(
				'TRANSFER_USER_ID' => 0,
				'TRANSFER_TYPE' => null,
				'TRANSFER_PHONE' => null
			)
		);

		$command['COMMAND'] = 'cancelTransfer';
		$command['OPERATOR_ID'] = $call['USER_ID'];
		$command['TRANSFER_USER_ID'] = $call['TRANSFER_USER_ID'];

		$http = new \Bitrix\Main\Web\HttpClient();
		$http->waitResponse(false);
		$http->post($call['ACCESS_URL'], json_encode($command));

		self::SendPullEvent(Array(
			'COMMAND' => 'cancelTransfer',
			'USER_ID' => $call['TRANSFER_USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		return true;
	}

	public static function Wait($callId)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		$command['COMMAND'] = 'waitTransfer';
		$command['OPERATOR_ID'] = $call['USER_ID'];

		$http = new \Bitrix\Main\Web\HttpClient();
		$http->waitResponse(false);
		$http->post($call['ACCESS_URL'], json_encode($command));

		self::SendPullEvent(Array(
			'COMMAND' => 'waitTransfer',
			'USER_ID' => $call['USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		return true;
	}

	public static function Answer($callId)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		$command['COMMAND'] = 'waitTransfer';
		$command['OPERATOR_ID'] = $call['USER_ID'];

		$http = new \Bitrix\Main\Web\HttpClient();
		$http->waitResponse(false);
		$http->post($call['ACCESS_URL'], json_encode($command));

		self::SendPullEvent(Array(
			'COMMAND' => 'waitTransfer',
			'USER_ID' => $call['USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		self::SendPullEvent(Array(
			'COMMAND' => 'timeoutTransfer',
			'USER_ID' => $call['TRANSFER_USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		return true;
	}

	public static function Ready($callId)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		$answer['COMMAND'] = 'transferConnect';
		$answer['OPERATOR_ID'] = $call['USER_ID'];

		$http = new \Bitrix\Main\Web\HttpClient();
		$http->waitResponse(false);
		$http->post($call['ACCESS_URL'], json_encode($answer));

		return true;
	}

	public static function Complete($callId, $device)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		VI\CallTable::update(
			$call['ID'],
			array(
				'USER_ID' => ($call['TRANSFER_USER_ID'] > 0 ? $call['TRANSFER_USER_ID'] : $call['USER_ID']),
				'TRANSFER_USER_ID' => 0,
				'TRANSFER_TYPE' => null,
				'TRANSFER_PHONE' => null
			)
		);

		CVoxImplantHistory::TransferMessage($call['USER_ID'], $call['TRANSFER_USER_ID'], $call['CALLER_ID'], $call['TRANSFER_PHONE']);

		self::SendPullEvent(Array(
			'COMMAND' => 'completeTransfer',
			'USER_ID' => $call['USER_ID'],
			'TRANSFER_USER_ID' => $call['TRANSFER_USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		$crmDataSend = false;
		if ($call['CRM'] == 'Y' && $call['CONFIG_ID'] > 0)
		{
			$config = CVoxImplantConfig::GetConfig($call['CONFIG_ID']);
			if (isset($config['CRM_TRANSFER_CHANGE']) && $config['CRM_TRANSFER_CHANGE'] == 'Y')
			{
				if ($call['CRM_LEAD'] > 0)
				{
					$crmData = Array(
						'LEAD_DATA' => Array(
							'ID' => $call['CRM_LEAD'],
							'ASSIGNED_BY_ID' => 0,
						)
					);
				}
				else
				{
					$crmData = CVoxImplantCrmHelper::GetDataForPopup($call['CALL_ID'], $call['CALLER_ID']);
				}

				if (isset($crmData['LEAD_DATA']) && $crmData['LEAD_DATA']['ASSIGNED_BY_ID'] >= 0 && $call['TRANSFER_USER_ID'] > 0 && $crmData['LEAD_DATA']['ASSIGNED_BY_ID'] != $call['TRANSFER_USER_ID'])
				{
					CVoxImplantCrmHelper::UpdateLead($crmData['LEAD_DATA']['ID'], Array('ASSIGNED_BY_ID' => $call['TRANSFER_USER_ID']));
					$crmDataSend = CVoxImplantCrmHelper::GetDataForPopup($call['CALL_ID'], $call['CALLER_ID'], $call['TRANSFER_USER_ID']);
				}
			}
		}

		if($device !== self::DEVICE_EXTERNAL)
		{
			self::SendPullEvent(Array(
				'COMMAND' => 'completeTransfer',
				'USER_ID' => $call['TRANSFER_USER_ID'],
				'TRANSFER_USER_ID' => $call['TRANSFER_USER_ID'],
				'CALL_DEVICE' => $device,
				'CALL_ID' => $call['CALL_ID'],
				'CRM' => $crmDataSend
			));
		}

		return true;
	}

	public static function Decline($callId, $send = true)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		VI\CallTable::update(
			$call['ID'],
			array(
				'TRANSFER_USER_ID' => 0,
				'TRANSFER_TYPE' => null,
				'TRANSFER_PHONE' => null
			)
		);

		if ($send)
		{
			$command['COMMAND'] = 'declineTransfer';
			$command['OPERATOR_ID'] = $call['USER_ID'];

			$http = new \Bitrix\Main\Web\HttpClient();
			$http->waitResponse(false);
			$http->post($call['ACCESS_URL'], json_encode($command));
		}

		self::SendPullEvent(Array(
			'COMMAND' => 'declineTransfer',
			'USER_ID' => $call['USER_ID'],
			'CALL_ID' => $call['CALL_ID']
		));

		if($call['TRANSFER_TYPE'] === self::TYPE_USER)
		{
			self::SendPullEvent(Array(
				'COMMAND' => 'timeoutTransfer',
				'USER_ID' => $call['TRANSFER_USER_ID'],
				'CALL_ID' => $call['CALL_ID']
			));
		}

		return true;
	}

	public static function Timeout($callId)
	{
		$call = VI\CallTable::getByCallId($callId);
		if (!$call)
			return false;

		VI\CallTable::update(
			$call['ID'],
			array(
				'TRANSFER_USER_ID' => 0,
				'TRANSFER_TYPE' => null,
				'TRANSFER_PHONE' => null
			)
		);

		if($call['TRANSFER_TYPE'] === self::TYPE_USER)
		{
			self::SendPullEvent(array(
				'COMMAND' => 'timeoutTransfer',
				'USER_ID' => $call['TRANSFER_USER_ID'],
				'CALL_ID' => $call['CALL_ID']
			));

			self::SendPullEvent(Array(
				'COMMAND' => 'declineTransfer',
				'USER_ID' => $call['USER_ID'],
				'CALL_ID' => $call['CALL_ID']
			));
		}

		return true;
	}

	public static function completePhoneTransfer($fromCallId, $toCallId)
	{
		$callFrom = VI\CallTable::getList(Array(
			'filter' => array(
				'=CALL_ID' => $fromCallId,
			),
		))->fetch();
		if (!$callFrom)
			return false;

		$callTo = VI\CallTable::getList(Array(
			'filter' => array(
				'=CALL_ID' => $toCallId,
			),
		))->fetch();
		if (!$callTo)
			return false;

		CVoxImplantHistory::TransferMessage($callFrom['USER_ID'], $callTo['USER_ID'], $callFrom['CALLER_ID']);

		VI\CallTable::update(
			$callFrom['ID'],
			array(
				'USER_ID' => $callTo['USER_ID']
			)
		);

		$crmData = false;
		if($callFrom['CRM'] === 'Y')
		{
			$crmData = CVoxImplantCrmHelper::GetDataForPopup($callFrom['CALL_ID'], $callFrom['CALLER_ID'], $callFrom['USER_ID']);
			if ($callFrom['CONFIG_ID'] > 0)
			{
				$config = CVoxImplantConfig::GetConfig($callFrom['CONFIG_ID']);
				if (isset($config['CRM_TRANSFER_CHANGE']) && $config['CRM_TRANSFER_CHANGE'] == 'Y')
				{
					if ($callFrom['CRM_LEAD'] > 0 && $callTo['USER_ID'] > 0)
					{
						CVoxImplantCrmHelper::UpdateLead(
							$callFrom['CRM_LEAD'],
							array('ASSIGNED_BY_ID' => $callTo['USER_ID'])
						);
					}
					else if (
						isset($crmData['LEAD_DATA'])
						&& $crmData['LEAD_DATA']['ASSIGNED_BY_ID'] >= 0
						&& $callTo['USER_ID'] > 0
						&& $crmData['LEAD_DATA']['ASSIGNED_BY_ID'] != $callTo['USER_ID']
					)
					{
						CVoxImplantCrmHelper::UpdateLead(
							$crmData['LEAD_DATA']['ID'],
							array('ASSIGNED_BY_ID' => $callTo['USER_ID'])
						);
					}
				}
			}
		}

		self::SendPullEvent(Array(
			'COMMAND' => 'replaceCallerId',
			'USER_ID' => $callTo['USER_ID'],
			'CALL_ID' => $callTo['CALL_ID'],
			'CALLER_ID' => $callFrom['CALLER_ID'],
			'CRM' => $crmData
		));

		return true;
	}

	public static function SendPullEvent($params)
	{
		if (!CModule::IncludeModule('pull') || !CPullOptions::GetQueueServerStatus() || $params['USER_ID'] <= 0)
			return false;

		if (empty($params['COMMAND']))
			return false;

		$config = Array();
		if ($params['COMMAND'] == 'inviteTransfer')
		{
			$config = Array(
				"callId" => $params['CALL_ID'],
				"callerId" => $params['CALLER_ID'],
				"phoneNumber" => $params['PHONE_NAME'],
				"chatId" => 0,
				"chat" => array(),
				"application" => $params['APPLICATION'],
				"CRM" => $params['CRM'],
			);
		}
		else if ($params['COMMAND'] == 'completeTransfer')
		{
			$config = Array(
				"callId" => $params['CALL_ID'],
				"transferUserId" => $params['TRANSFER_USER_ID'],
				"callDevice" => $params['CALL_DEVICE'],
				"CRM" => $params['CRM']? $params['CRM']: false,
			);
		}
		else if ($params['COMMAND'] == 'replaceCallerId')
		{
			$config = Array(
				"callId" => $params['CALL_ID'],
				"callerId" => $params['CALLER_ID'],
				"CRM" => $params['CRM']? $params['CRM']: false,
			);
		}
		else
		{
			$config["callId"] = $params['CALL_ID'];
		}
		if (isset($params['MARK']))
		{
			$config['mark'] = $params['MARK'];
		}
		CPullStack::AddByUser($params['USER_ID'],
			Array(
				'module_id' => 'voximplant',
				'command' => $params['COMMAND'],
				'params' => $config
			)
		);

		return true;
	}
}
?>