CREATE TABLE B_VOXIMPLANT_PHONE
(
	ID number(11),
	USER_ID number(11) NOT NULL,
	PHONE_NUMBER varchar(20 char) NOT NULL,
	PHONE_MNEMONIC varchar(20 char) NULL,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_PH_1 ON B_VOXIMPLANT_PHONE (USER_ID, PHONE_NUMBER)
/
CREATE INDEX IX_VI_PH_2 ON B_VOXIMPLANT_PHONE (USER_ID, PHONE_MNEMONIC)
/
CREATE INDEX IX_VI_PH_3 ON B_VOXIMPLANT_PHONE (PHONE_NUMBER)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_PHONE INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_PHONE_insert
BEFORE INSERT
ON B_VOXIMPLANT_PHONE
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_PHONE.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_STATISTIC
(
	ID number(11),
	ACCOUNT_ID number(11) NOT NULL,
	APPLICATION_ID number(11) NOT NULL,
	APPLICATION_NAME varchar(80 char) NOT NULL,
	PORTAL_USER_ID number(11) NULL,
	PORTAL_NUMBER varchar(20 char) NULL,
	PHONE_NUMBER varchar(20 char) NOT NULL,
	INCOMING varchar(50 char) DEFAULT '1' not null,
	CALL_ID varchar(255 char) NOT NULL,
	CALL_CATEGORY varchar(20 char) DEFAULT 'external' NULL,
	CALL_LOG varchar(255 char) NULL,
	CALL_DIRECTION varchar(255 char) NULL,
	CALL_DURATION number(11) DEFAULT '0' NOT NULL,
	CALL_START_DATE date not null,
	CALL_STATUS number(11) DEFAULT '0' NULL,
	CALL_FAILED_CODE varchar(255 char) NULL,
	CALL_FAILED_REASON varchar(255 char) NULL,
	CALL_RECORD_ID number(11) NULL,
	CALL_WEBDAV_ID number(11) NULL,
	CALL_VOTE NUMBER(1) DEFAULT '0' NULL,
	COST NUMBER(11, 4) DEFAULT '0' NULL,
	COST_CURRENCY varchar(50 char) NULL,
	CRM_ENTITY_TYPE varchar(50 char) NULL,
	CRM_ENTITY_ID number(11) NULL,
	CRM_ACTIVITY_ID number(11) NULL,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_ST_1 ON B_VOXIMPLANT_STATISTIC (PORTAL_USER_ID)
/
CREATE INDEX IX_VI_ST_2 ON B_VOXIMPLANT_STATISTIC (CALL_START_DATE)
/
CREATE INDEX IX_VI_ST_3 ON B_VOXIMPLANT_STATISTIC (CALL_FAILED_CODE)
/
CREATE INDEX IX_VI_ST_4 ON B_VOXIMPLANT_STATISTIC (CALL_CATEGORY)
/
CREATE INDEX IX_VI_ST_5 ON B_VOXIMPLANT_STATISTIC (CALL_VOTE)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_STATISTIC INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_STATISTIC_insert
BEFORE INSERT
ON B_VOXIMPLANT_STATISTIC
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_STATISTIC.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_CALL
(
	ID number(11),
	CONFIG_ID number(11) NULL,
	USER_ID number(11) NULL,
	TRANSFER_TYPE varchar(50 char) NULL,
	TRANSFER_USER_ID number(11) NULL,
	TRANSFER_PHONE varchar(255 char) NULL,
	PORTAL_USER_ID number(11) NULL,
	CALL_ID varchar(255 char) NOT NULL,
	CALLER_ID varchar(255 char) NULL,
	STATUS varchar(50 char) NULL,
	CRM char(1 CHAR) DEFAULT 'Y' not null,
	CRM_LEAD number(11) NULL,
	CRM_ENTITY_TYPE varchar(50 char) NULL,
	CRM_ENTITY_ID number(11) NULL,
	CRM_ACTIVITY_ID number(11) NULL,
	ACCESS_URL varchar(255 char) NOT NULL,
	DATE_CREATE date,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_I_1 ON B_VOXIMPLANT_CALL (CALL_ID)
/
CREATE INDEX IX_VI_I_2 ON B_VOXIMPLANT_CALL (DATE_CREATE)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_CALL INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_CALL_insert
BEFORE INSERT
ON B_VOXIMPLANT_CALL
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_CALL.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_SIP
(
	ID number(11),
	APP_ID varchar(128 char) NULL,
	CONFIG_ID number(11) NOT NULL,
	TYPE VARCHAR2(255 CHAR) DEFAULT 'office' NULL,
	REG_ID number(11) DEFAULT 0 NULL,
	SERVER varchar(255 char) NULL,
	LOGIN varchar(255 char) NULL,
	PASSWORD varchar(255 char) NULL,
	INCOMING_SERVER varchar(255 char) NULL,
	INCOMING_LOGIN varchar(255 char) NULL,
	INCOMING_PASSWORD varchar(255 char) NULL,
	AUTH_USER varchar(255 char) NULL,
	OUTBOUND_PROXY varchar(255 char) NULL,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_SIP_1 ON B_VOXIMPLANT_SIP (CONFIG_ID)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_SIP INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_SIP_insert
BEFORE INSERT
ON B_VOXIMPLANT_SIP
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_SIP.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_CONFIG
(
	ID number(11),
	PORTAL_MODE varchar(50 char),
	SEARCH_ID varchar(255 char) NULL,
	PHONE_NAME varchar(255 char) NULL,
	PHONE_COUNTRY_CODE varchar(50 char) NULL,
	PHONE_VERIFIED char(1 CHAR) DEFAULT 'Y' null,
	CRM char(1 CHAR) DEFAULT 'Y' not null,
	CRM_RULE varchar(50 char),
	CRM_CREATE varchar(50 char) DEFAULT 'none' null,
	CRM_SOURCE varchar(50 char) DEFAULT 'CALL' null,
	CRM_FORWARD char(1 CHAR) DEFAULT 'Y' not null,
	CRM_TRANSFER_CHANGE char(1 CHAR) DEFAULT 'N' null,
	QUEUE_TIME NUMBER(3) DEFAULT 0,
	QUEUE_TYPE varchar(255 char) DEFAULT 'evenly' not null,
	DIRECT_CODE char(1 CHAR) DEFAULT 'Y' not null,
	DIRECT_CODE_RULE varchar(50 char),
	RECORDING char(1 CHAR) DEFAULT 'Y' not null,
	RECORDING_NOTICE char(1 CHAR) DEFAULT 'N' null,
	RECORDING_TIME NUMBER(1) DEFAULT 0,
	VOTE char(1 CHAR) DEFAULT 'N' null,
	FORWARD_NUMBER varchar(20 char) NULL,
	FORWARD_LINE varchar(255 char) DEFAULT 'default' not null,
	TIMEMAN char(1 CHAR) DEFAULT 'N' not null,
	VOICEMAIL char(1 CHAR) DEFAULT 'Y'  not null,
	MELODY_LANG char(2 CHAR) DEFAULT 'EN' not null,
	MELODY_WELCOME number(11) null,
	MELODY_WELCOME_ENABLE char(1 CHAR) DEFAULT 'Y' not null,
	MELODY_VOICEMAIL number(11) null,
	MELODY_WAIT number(11) null,
	MELODY_HOLD number(11) null,
	MELODY_RECORDING number(11) null,
	MELODY_VOTE number(11) null,
	MELODY_VOTE_END number(11) null,
	TO_DELETE char(1 CHAR) DEFAULT 'N' null,
	DATE_DELETE date null,
	NO_ANSWER_RULE varchar(50 char) DEFAULT 'voicemail' null,
	WORKTIME_ENABLE char(1 char) DEFAULT 'N' null,
	WORKTIME_FROM varchar(5 char) null,
	WORKTIME_TO varchar(5 char) null,
	WORKTIME_TIMEZONE varchar(50 char) null,
	WORKTIME_HOLIDAYS varchar2(2000 char) null,
	WORKTIME_DAYOFF varchar(20 char) null,
	WORKTIME_DAYOFF_RULE varchar(50 char) DEFAULT 'voicemail' null,
	WORKTIME_DAYOFF_NUMBER varchar(20 char) null,
	WORKTIME_DAYOFF_MELODY number(11) null,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_PC_1 ON B_VOXIMPLANT_CONFIG (SEARCH_ID)
/
CREATE INDEX IX_VI_PC_2 ON B_VOXIMPLANT_CONFIG (PORTAL_MODE)
/
CREATE INDEX IX_VI_PC_3 ON B_VOXIMPLANT_CONFIG (TO_DELETE, DATE_DELETE)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_CONFIG INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_CONFIG_insert
BEFORE INSERT
ON B_VOXIMPLANT_CONFIG
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_CONFIG.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_QUEUE
(
	ID number(11),
	SEARCH_ID varchar(255 char) NULL,
	CONFIG_ID number(11) NOT NULL,
	USER_ID number(11) NOT NULL,
	STATUS varchar(50 char) NULL,
	LAST_ACTIVITY_DATE date,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_PQ_1 ON B_VOXIMPLANT_QUEUE (CONFIG_ID)
/
CREATE INDEX IX_VI_PQ_2 ON B_VOXIMPLANT_QUEUE (SEARCH_ID, STATUS, LAST_ACTIVITY_DATE)
/
CREATE INDEX IX_VI_PQ_3 ON B_VOXIMPLANT_QUEUE (USER_ID)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_QUEUE INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_QUEUE_insert
BEFORE INSERT
ON B_VOXIMPLANT_QUEUE
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_QUEUE.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_BLACKLIST
(
	ID number(11),
	PHONE_NUMBER varchar(20 char) NULL,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VI_BL_1 ON B_VOXIMPLANT_BLACKLIST (PHONE_NUMBER)
/

CREATE TABLE B_VOXIMPLANT_ROLE
(
	ID number(11),
	NAME varchar(255 char) NOT NULL,
	PRIMARY KEY (ID)
)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_ROLE INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_ROLE_INSERT
BEFORE INSERT
ON B_VOXIMPLANT_ROLE
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_ROLE.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_ROLE_PERMISSION
(
	ID number(11),
	ROLE_ID number(11) NOT NULL,
	ENTITY varchar(50 char) NOT NULL,
	ACTION varchar(50 char) NOT NULL,
	PERMISSION char(1 char) NULL,
	PRIMARY KEY (ID)
)
/
CREATE INDEX IX_VOX_PERM_ROLE_ID ON B_VOXIMPLANT_ROLE_PERMISSION (ROLE_ID)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_ROLE_PERM INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_ROLE_PERM_INSERT
BEFORE INSERT
ON B_VOXIMPLANT_ROLE_PERMISSION
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_ROLE_PERM.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/

CREATE TABLE B_VOXIMPLANT_ROLE_ACCESS
(
	ID number(11),
	ROLE_ID number(11),
	ACCESS_CODE varchar(100 char) NOT NULL,

	PRIMARY KEY(ID)
)
/
CREATE INDEX IX_VOX_ACCESS_ROLE_ID ON B_VOXIMPLANT_ROLE_ACCESS (ROLE_ID)
/
CREATE SEQUENCE SQ_B_VOXIMPLANT_ROLE_ACC INCREMENT BY 1 NOMAXVALUE NOCYCLE NOCACHE NOORDER
/
CREATE OR REPLACE TRIGGER B_VOXIMPLANT_ROLE_ACC_INSERT
BEFORE INSERT
ON B_VOXIMPLANT_ROLE_ACCESS
FOR EACH ROW
BEGIN
	IF :NEW.ID IS NULL THEN
		SELECT SQ_B_VOXIMPLANT_ROLE_ACC.NEXTVAL INTO :NEW.ID FROM dual;
	END IF;
END;
/
